;
putkeys proc			;jj
;
	;This routine acts as a screen between the receiver handler and the
	;input buffer routines.  It routes bytes designated for the command
	;buffer and keyboard buffer to their respective routines.
	;ENTRY: ds is data segment of current port
	;	ah & al hold the status code and input byte respectively
	;EXIT: byte is inserted into proper buffer
	;RETURN CODES: Z = buffer full, error, NZ = byte inserted in buffer
	;SUBROUTINES USED: Putkey, Getcomt
;
;	Is port in command mode or normal input mode?
	mov	di,flag2
	and	byte ptr [di],0feh	;discard the special package bit
	cmp	byte ptr [di],0 	;mode bits set?
	je	putkeys2	;jmp if not in command mode
;
;				;Command mode
	test	byte ptr [di],dwnldbit
	jnz	cnt		;jmp if counting zero input
putkeys1:
	mov	di,flag2
	test	byte ptr [di],modebit	;command mode? 1 -> command mode, 0 -> normal
	jz	putkeys2	;put byte to keybuf if not command mode
	call	put_comt	;store byte in command string with al as entry
	xor	al,al		;set error return
putkeysret:
	ret
cnt:		;Count number of occurances of zero bytes for the download routine
	cmp	al,0
	jne	cnt001
	xor	al,ah		;both byte zero?
	jnz	cnt001		;reset download mode if non-zero
	mov	di,dwncnt
	inc	byte ptr [di]	;both zeros, means byte is input of zero.
	jmp	putkeys1	;go put byte into buffer
cnt001:
		;error - non-zero input byte - reset download command and count
	mov	di,dwncnt
	mov	byte ptr [di],0 		;reset count
	mov	di,flag2
	and	byte ptr [di],not dwnldbit	;get out of download sequence
	jmp	putkeys1			;try to put byte into buffer
;
putkeys2:
				;Normal input mode - store byte in keyboard buffer
	call	putkey		;store byte
	jz	putkeysret
	mov	ah,al		;ah=input byte, fall through putkey for 2nd byte
;
putkey: 			;j+ Enclosed within putkeys,
;	********************************************
;	Routine to put byte in the keyboard buffer.
;	ah holds the byte to be stored.
;	Return: Z = buffer is full, NZ = successful
;	********************************************
				;entry code is in ah
	cli			;disable inturrupt
	mov	di,key_count
	cmp	byte ptr [di],96		;end of buffer?
	jb	putkey1 			;jmp if not full
	xor	di,di				;set error code, buffer full
	ret
putkey1:
	inc	byte ptr [di]
	mov	di,key_wrt
	mov	bx,[di]
	xor	bh,bh
	mov	di,key_buf
	mov	[di][bx],ah			;ah?
	inc	bl				;inc write pointer
	cmp	bl,key_size - 1
	jbe	putkey2
	mov	bl,0
putkey2:
	mov	di,key_wrt
	mov	word ptr [di],bx
	mov	di,sts_byte
	or	byte ptr [di],01h		;0/empty, 1/occupied
	or	di,0ffh 			;set return code NZ
	ret
putkeys endp
;
getkey	proc			;j+
;	***********************************************
;	Routine gets (a) bytes from key_buf
;	Exit: al holds char, while ah get status code.
;	***********************************************
;	Bytes were stored in key_buf as 2-byte groups. The first byte states
;	whether or not the ENDI bit should be sent along with the second byte
;	when writing it to 386. Both bytes are passed as they are in buffer.
	mov	di,key_count
	sub	byte ptr [di],2        ;we will get 2 code at one time
	jnz	getkey2 	       ;zero count?
	mov	di,txbyte
	and	byte ptr [di],not keybit       ;Count = 0, reset keybit
	mov	di,sts_byte
	and	byte ptr [di],0feh	;0/empty, 1/occupied
getkey2:
	mov	di,key_rd
	mov	bx,word ptr [di]       ;get the keybuf read index
	xor	bh,bh
	mov	di,key_buf
	mov	ah,[di][bx]	       ;get the status code
	inc	bx		       ;increment index
	mov	al,[di][bx]	       ;get the character code
	inc	bx
	cmp	bl,key_size - 1        ;up to the max?
	jb	getkey3 	       ;no,
	mov	bl,0		       ;reset to 0
getkey3:
	mov	di,key_rd
	mov	word ptr [di],bx       ;update the index
	ret			       ;ah hold status code, al hold char.
getkey	endp
;
getkeyt proc
;	**********************************************************************
	;This routine gets a byte from the keyboard buffer and translate it
	;for keyin mode.  Due to history and the number of devices this board
	;must support a number of anommolies exist in the definition of keys
	;on the keyboards. Following is a list of transformations.
;	--------------------------------------------------------------------
;	 Please note that control codes have already been stripped from the
;	 input stream 'RXIN' and should not be found in the keyboard buffer!
;	--------------------------------------------------------------------
;		INPUT				OUTPUT
;	     -----------		     ------------
;	'A0' 'Any byte'                output with ENDI (underscore byte)
;	'FC' 'Any byte < 80'           output without ENDI
;	'FC' 'Any byte > 80'           output with ENDI (foriegn characters)
;	'FD' 'Any byte < 80'           output with ENDI (special function)
;	'FD' 'SOME bytes > 80'         output without ENDI (text atoms, control keys)
;	     Exceptions:
;		'BD' =>                'F0' with ENDI  (edit key)
;		'BE' or 'BF' =>        '7E' or '7F' without ENDI (function keys)
;	'14' ... '7F'                  output without ENDI (ASCII char's)
;	'80' ... 'FF'                  output without ENDI (atoms, controls, etc)
;
;	NOTE: Many codes above 80 are not defined and should NEVER be
;	found in the keyboard buffer (should never be received from the
;	terminal).  But, for consistency I handle them as if they are
;	defined for KEYIN mode.  Also, some keyboards generate different
;	codes for the same key!!  The major difference is that some may
;	send 'FD' before and ATOM while others may not (and some keyboards
;	are mixed) so the atoms must be treated the same, with or without
;	an 'FD'.
;	Please see the edit routines for a complete description of TEXT ATOMS.
;ENTRY: Want a byte from the keyboard buffer
;EXIT:	al= byte gotten, ah= attribute to indicate with/without endi
;
	call	getkey		;get code, ah=status al=char
;
	mov	bx,ax
	mov	bh,01		;bh temp. store the attribute with endi
	cmp	ah,0a0h 	;is preceeded by 0a0h?
	je	getkeytret	;output with endi
	cmp	ah,0fch 	;is preceeded by 0fch?
	je	precfch 	;jmp if so
	cmp	ah,0fdh 	;is preceeded by 0fdh?
	je	precfdh
;	@@
;	for the c0 - df of special function, apr/30/92
	cmp	al,0c0h 	;between 0c0h and 0dfh
	jb	getkeyt0
	cmp	al,0dfh
	ja	getkeyt0
	sub	al,0c0h
	mov	bl,al
	jmp	getkeytret
getkeyt0:
;	for the c0 - df of special function, apr/30/92, end
;
	cmp	al,0bdh 	;edit key?
	je	precxx1 	;output with endi
	cmp	al,0beh
	je	precxx2 	;output 7eh and without endi
	cmp	al,0bfh
	je	precxx3 	;output 7fh and without endi
;	@@@
;	cmp	al,014h 	;is range 14h to 7fh?
	jmp	getnotendi	;output without endi if so
;	jmp	getkeytret
precfch:
	cmp	al,80h
	jb	getnotendi	;< 80h output without endi
	jmp	getkeytret
precfdh:
	cmp	al,80h
	jb	getkeytret	;< 80h output with endi
	cmp	al,0bdh 	;edit key?
	jne	precfdh1
precxx1:
	mov	bl,0f0h 	;output 0f0h with endi
	jmp	getkeytret
precfdh1:
	cmp	al,0beh
	jne	precfdh2
precxx2:
	mov	bl,7eh
	jmp	getnotendi
precfdh2:
	cmp	al,0bfh
	jne	getnotendi
precxx3:
	mov	bl,7fh
getnotendi:
	mov	bh,0		;without endi
getkeytret:
	mov	ax,bx		;restore to ax
	ret
getkeyt endp
;
putprt	proc			;j+
;	************************************
;	   Put a byte into the prt_buffer.
;	   Insert d0 if fb is found.
;	   Entry: al holds incoming data
;	************************************
	cli
	mov	di,prt_wrt
	mov	bx,[di]
putprt3:
	mov	di,prt_buf
	mov	[di][bx],al
	inc	bx
	cmp	bx,03ffh
	jbe	putprtv
	mov	bx,0
putprtv:
	mov	di,prt_count
	inc	word ptr [di]
;	cmp	bl,prt_size
;	jbe	putprt1
;	mov	bl,0
;putprt1:
	cmp	al,esccode
	jnz	putprt10
	mov	al,0d0h
	jmp	putprt3
putprt10:
	mov	di,prt_wrt
	mov	[di],bx
	mov	di,prt_count
	cmp	word ptr [di],03c0h	;200 byte to warn of overflow
	jbe	putprt2
	mov	di,rbi
	and	byte ptr [di],not prtrb ;if prtrb=1 then okay to put string
putprt2:				;prtrb=0 then stop printing string
	mov	di,txbyte
	test	byte ptr [di],prtbit
	jz	putprt30
	sti
	ret
putprt30:
	or	byte ptr [di],prtbit
	call	entx
	sti
	ret
putprt	endp
;
putprtg proc			;j+
;	************************************
;	   Put a byte into the prt_buffer.
;	   Entry: al holds incoming data
;	************************************
	cli
	mov	di,prt_wrt
	mov	bx,[di]
	mov	di,prt_buf
	mov	[di][bx],al
	inc	bx
	cmp	bx,03ffh
	jbe	putprtgv
	mov	bx,0
putprtgv:
	mov	di,prt_count
	inc	word ptr [di]
	mov	di,prt_wrt
	mov	[di],bx
	mov	di,prt_count
	cmp	word ptr [di],03c0h	;200 byte to warn of overflow
	jbe	putprtg2
	mov	di,rbi
	and	byte ptr [di],not prtrb ;if prtrb=1 then okay to put string
	jmp	putprtg7
putprtg2:				 ;prtrb=0 then stop printing string
;	For testing of gio command
	mov	di,rbi
	or	byte ptr [di],prtrb
;	End of testing gio command
putprtg7:
	mov	di,txbyte
	test	byte ptr [di],prtbit
	jz	putprtg30
	sti
	ret
putprtg30:
	or	byte ptr [di],prtbit
	call	entx
	sti
	ret
putprtg endp
;
gprt	proc			;j+
;	*********************************
;	 Get only one byte from prtbuf
;	 Exit: bx holds the read pointer
;	       dx is the counter
;	       al holds the data
;	*********************************
	cmp	dx,0
	jne	gprt1
;	xor	dx,dx		;Z flag to indicate no data in buffer
	ret
gprt1:				;Not empty
	mov	di,prt_buf
	mov	al,[di][bx]
	inc	bx
	cmp	bx,03ffh
	jbe	gprt2
	mov	bx,0
gprt2:
	dec	dx
	cmp	dx,1c0h
	ja	gprt66
	mov	si,rbi
	or	byte ptr [si],prtrb
	mov	si,curnt_sts
	or	byte ptr [si],prtint
gprt66:
	or	si,0ffh 	 ;Set ZF
	ret
gprt	endp
;
getprt	proc			;j+
;	*******************************************************
;	  Routine to get (a) byte(s) from the prt buffer
;	  And Check compression conditions.
;	  Note: This routine look-aheads, and return pointer
;		if the data is not compressioned.
;	  Exit: (1) Byte gotten from al
;		(2) al=data, ch=esccode, cl=count of compress
;	  Return: Z = empty buffer, NZ = byte returned
;	*******************************************************
;	cli			;disable interrupt
	mov	di,prt_rd
	mov	bx,[di] 	;get prtbuf read index
	mov	di,prt_count
	mov	dx,[di]
	call	gprt
	jnz	getprt1 	;Return if receiving Z.
	mov	di,txbyte
	and	byte ptr [di],not prtbit       ;reset prt buffer flag
;	call	distx
;	sti
	xor	di,di
	ret
getprt1:
	mov	ah,al
	mov	si,prtcomp
	cmp	byte ptr [si],0
	jz	getprt101
	dec	byte ptr [si]
	jmp	getprt6
getprt101:
	push	bx		;save the index
	push	dx
	call	gprt		;get the next
	jz	getprt5 	;jump if no more data coming
	cmp	ah,al		;over 3 char same, we do compress procedure
	jne	getprt5 	;jump if different of two data
	call	gprt
	jz	getprt5 	;jump if no data
	cmp	ah,al
	jne	getprt5 	;jump if the third one is different
	pop	cx		;dummy pop, just for stack balance
	pop	cx		;dummy pop, just for stack balance
	mov	cx,3		;now, we get 3 same char at least
;
getprt2:			;Keep on looking for the same data, cx = counter
	push	bx		;bx is the most current read-pointer
	push	dx		;Most current prt_count
	call	gprt
	jz	getprt3 	;No more data in buffer, counter and bx unchanged
	cmp	ah,al
	jne	getprt3
	inc	cx
	cmp	cx,5eh		;exceed the max of count that crt allowed
	ja	getprt3
	pop	si		;dummy pop, just for stack balance
	pop	si		;dummy pop, just for stack balance
	jmp	getprt2
getprt3:
	pop	dx
	add	dx,2
	mov	di,prt_count
	mov	[di],dx 	;restore the prtcount
	pop	bx		;restore index
	mov	si,prt_buf
;	mov	[di][bx],ah	;Store back the different data
;	inc	byte ptr [di]	;now, we add one char to prtbuff
	dec	bx		;back one byte
	cmp	bx,0ffffh
	jne	getprtc
	mov	bx,03ffh
getprtc:
	mov	[si][bx],ah
;	jmp	getprt45
;	jnc	getprt4
;	mov	bl,prt_size
;getprt4:
;	pop	dx
;	pop	dx
;getprt45:
	dec	bx
	cmp	bx,0ffffh
	jne	getprtd
	mov	bx,03ffh
getprtd:
	mov	[si][bx],cl	;store compress count
	mov	si,prtcomp
	mov	byte ptr [si],2 ;two more to send
;
	mov	ah,esccode	;now output the esc code
	jmp	getprt6
getprt5:
	pop	dx
	mov	di,prt_count
	mov	[di],dx
	pop	bx
getprt6:
	mov	di,prt_rd
	mov	[di],bx 	 ;restore the prtbuf index
;
	mov	di,prt_count
	mov	[di],dx
	cmp	dx,0
	jnz	getprt10
;	Buffer will be empty after this byte, reset prtbit and set ready bit
	mov	di,txbyte
	and	byte ptr [di],not prtbit
getprt10:
	cmp	dx,03c0h	;level?
	jnc	getprt11
	mov	di,rbi
	or	byte ptr [di],prtrb
;
getprt11:
	mov	al,ah
	or	dx,0ffh 	 ;NF was set
	ret
getprt	endp
;
; ********************************************************************
;
;putcrt  proc			 ;j+
;	************************************
;	   Put a byte into the crt_buffer.
;	   Insert d0 if fb is found.
;	   Entry: al holds incoming data
;	************************************
;	 cli
;	 mov	 di,crt_wrt
;	 mov	 bx,[di]
;putcrt3:
;	 mov	 di,crt_buf
;	 mov	 [di][bx],al
;	 inc	 bx
;	 cmp	 bx,03ffh
;	 jbe	 putcrtv
;	 mov	 bx,0
;putcrtv:
;	 mov	 di,crt_count
;	 inc	 word ptr [di]
;	 cmp	 al,esccode
;	 jnz	 putcrt10
;	 mov	 al,0d0h
;	 jmp	 putcrt3
;putcrt10:
;	 mov	 di,crt_wrt
;	 mov	 [di],bx
;	 mov	 di,crt_count
;	 cmp	 word ptr [di],03c0h	 ;200 byte to warn of overflow
;	 jbe	 putcrt2
;	 mov	 di,rbi
;	 and	 byte ptr [di],not crtrb ;if crtrb=1 then okay to put string
;putcrt2:				 ;crtrb=0 then stop printing string
;	 mov	 di,txbyte
;	 test	 byte ptr [di],crtbit
;	 jz	 putcrt30
;	 sti
;	 ret
;putcrt30:
;	 or	 byte ptr [di],crtbit
;	 call	 entx
;	 sti
;	 ret
;putcrt  endp
;
putcrtg proc			;j+
;	************************************
;	   Put a byte into the crt_buffer.
;	   Entry: al holds incoming data
;	************************************
	cli
	mov	di,crt_wrt
	mov	bx,[di]
	mov	di,crt_buf
	mov	[di][bx],al
	inc	bx
	cmp	bx,03ffh
	jbe	putcrtgv
	mov	bx,0
putcrtgv:
	mov	di,crt_count
	inc	word ptr [di]
	mov	di,crt_wrt
	mov	[di],bx
	mov	di,crt_count
	cmp	word ptr [di],03c0h	;200 byte to warn of overflow
	jbe	putcrtg2
	mov	di,rbi
	and	byte ptr [di],not crtrb ;if crtrb=1 then okay to put string
	jmp	putcrtg7
putcrtg2:				 ;crtrb=0 then stop printing string
;	For testing of gio command
	mov	di,rbi
	or	byte ptr [di],crtrb
;	End of testing gio command
;
putcrtg7:
	mov	di,txbyte
	test	byte ptr [di],crtbit
	jz	putcrtg30
	sti
	ret
putcrtg30:
	or	byte ptr [di],crtbit
	call	entx
	sti
	ret
putcrtg endp
;
putcrtm proc			;j+
;	********************************************
;	Routine puts a compressed string into crt.
;	Compressed format is:
;		<esccode> + <count> + <char>
;	Entry: bl/count, bh/char
;	********************************************
	push	bx
	mov	al,esccode
	call	putcrtg
	pop	bx
	mov	al,bl
	push	bx
	call	putcrtg
	pop	bx
	mov	al,bh
	call	putcrtg
	ret
putcrtm endp
;
;	****************************************************
;
putstr	proc			;j+
	mov	al,es:[si]
	inc	si
;	lodsb			;get data to al
	push	cx
	push	si
	call	putcrtg
	pop	si
	pop	cx
	loop	putstr
	ret
putstr	endp
;
gcrt	proc			;j+
;	*********************************
;	 Get only one byte from crtbuf
;	 Exit: bx holds the read pointer
;	       dx holds the counter
;	       al holds the data
;	*********************************
	cmp	dx,0
	jne	gcrt1
;	call	putcrt		;if command buffer still has some chars to crt
;	jnz	gcrt1		;if so, move to crt buffer and return NZ
;	xor	dx,dx		;Z flag to indicate no data in buffer
	ret
gcrt1:				;Not empty
	mov	di,crt_buf
	mov	al,[di][bx]
	inc	bx
	cmp	bx,03ffh
	jbe	gcrt2
	mov	bx,0
gcrt2:
	dec	dx
	cmp	dx,1c0h
	ja	gcrt66
	mov	si,rbi
	or	byte ptr [si],crtrb
	mov	si,curnt_sts
;	mov	si,flag9
	or	byte ptr [si],crtint
gcrt66:
	or	si,0ffh 	 ;Set ZF
	ret
gcrt	endp
;
getcrt	proc			;j+
;	*******************************************************
;	  Routine to get (a) byte(s) from the crt buffer
;	  And Check compression conditions.
;	  Note: This routine look-aheads, and return pointer
;		if the data is not compressioned.
;	  Exit: (1) Byte gotten from al
;		(2) al=data, ch=esccode, cl=count of compress
;	  Return: Z = empty buffer, NZ = byte returned
;	*******************************************************
;	cli			;disable interrupt
	mov	di,crt_rd
	mov	bx,[di] 	;get crtbuf read index
	mov	di,crt_count
	mov	dx,[di]
	call	gcrt
	jnz	getcrt1 	;Return if receiving Z.
	mov	di,txbyte
	and	byte ptr [di],not crtbit       ;reset crt buffer flag
;	Try to find if lr needs service? crtalert in MXE code originally.
	call	crtalert
;	call	distx
;	sti
	xor	di,di
	ret
getcrt1:
	mov	ah,al
;
;	may/12/92
	cmp	al,esccode
	jnz	getcrt2z
	mov	si,flag13
	cmp	byte ptr [si],0ffh
	jnz	getcrt2z
	mov	byte ptr [si],0
	mov	si,crtcomp
	mov	byte ptr [si],3
;
getcrt2z:
;	may/12/92, end
;
	mov	si,crtcomp
	cmp	byte ptr [si],0
	jz	getcrt101	;no compressed occurs, check
	dec	byte ptr [si]
	jmp	getcrt6 	;go send directly
getcrt101:
	push	bx		;save the index
	push	dx
	call	gcrt		;get the next
	jz	getcrt5 	;jump if no more data coming
	cmp	ah,al		;over 3 char same, we do compress procedure
	jne	getcrt5 	;jump if different of two data
	call	gcrt
	jz	getcrt5 	;jump if no data
	cmp	ah,al
	jne	getcrt5 	;jump if the third one is different
	pop	cx		;dummy pop, just for stack balance
	pop	cx		;dummy pop, just for stack balance
	mov	cx,3		;now, we get 3 same char at least
;
getcrt2:			;Keep on looking for the same data, cx = counter
	push	bx		;bx is the most current read-pointer
	push	dx		;Most current crt_count
	call	gcrt
	jz	getcrt3 	;No more data in buffer, counter and bx unchanged
	cmp	ah,al
	jne	getcrt3
	inc	cx
	cmp	cx,5eh		;exceed the max of count that crt allowed
	ja	getcrt3
	pop	si		;dummy pop, just for stack balance
	pop	si		;dummy pop, just for stack balance
	jmp	getcrt2
getcrt3:
	pop	dx
	add	dx,2
	mov	di,crt_count
	mov	[di],dx 	;restore the crtcount
	pop	bx		;restore index
	mov	si,crt_buf
;	mov	[di][bx],ah	;Store back the different data
;	inc	byte ptr [di]	;now, we add one char to crtbuff
	dec	bx		;back one byte
	cmp	bx,0ffffh
	jne	getcrtc
	mov	bx,03ffh
getcrtc:
	mov	[si][bx],ah
;	jmp	getcrt45
;	jnc	getcrt4
;	mov	bl,crt_size
;getcrt4:
;	pop	dx
;	pop	dx
;getcrt45:
	dec	bx
	cmp	bx,0ffffh
	jne	getcrtd
	mov	bx,03ffh
getcrtd:
	mov	[si][bx],cl
	mov	si,crtcomp
	mov	byte ptr [si],2 ;two more to send
;
	mov	ah,esccode	;now output the esc code
	jmp	getcrt6
getcrt5:
	pop	dx
	mov	di,crt_count
	mov	[di],dx
	pop	bx
getcrt6:
	mov	di,crt_rd
	mov	[di],bx 	 ;restore the crtbuf index
;
	mov	di,crt_count
	mov	[di],dx
	cmp	dx,0
	jnz	getcrt10
;	Buffer will be empty after this byte, reset crtbit and set ready bit
	mov	di,txbyte
	and	byte ptr [di],not crtbit
getcrt10:
	cmp	dx,03c0h	;level?
	jnc	getcrt11
	mov	di,rbi
	or	byte ptr [di],crtrb
;
getcrt11:
	mov	al,ah
	or	dx,0ffh 	 ;NF was set
	ret
getcrt	endp
;
; ********************************************************************
putrx	proc			;j+
;	This routine put a byte to Rxbuf, and set the Rx service flag.
;	ENTRY: ds is data segment of current port, es is data
;	       al hold the input byte from RXin
;
	mov	di,rx_wrt
	mov	bx,[di]
	xor	bh,bh
	mov	di,rx_buf
	mov	[di][bx],al
	inc	bl
	mov	di,rx_wrt
	mov	[di],bx
	mov	di,rx_count
	inc	byte ptr [di]
;
;	In order to protect against buffer overflow by sending a stop bit when
;	the buffer gets more than half
;
;	mov	di,rx_count
;	cmp	byte ptr [di],80h	;Length=256=100h, Half=80h
	cmp	byte ptr [di],0d0h	 ;aug/28/91 for windowing
	jb	putrx3
;					 Buffer too full, send out stop bit
;					 to terminal if in RSD mode!
	mov	di,rsdbyte
	test	byte ptr [di],rsdwaitb
	jnz	putrx3			;rsdstop bit had been set
;					 Set the stop bit
	or	byte ptr [di],rsdwaitb
	mov	al,esccode
	call	putdt
;
;	aug/28/91, for windowing
	mov	di,windw
	cmp	byte ptr [di],0
	jz	putrx6
	mov	al,0dbh
	call	putdt
	jmp	putrx3
;	aug/28/91, for windowing
;
putrx6:
	mov	al,rsdstop
	call	putdt
;
putrx3: 				;Set the snapshot in (rxbyte)
	mov	di,usermask
	mov	bx,[di]
	or	es:rxbyte,bx
	ret
putrx	endp
;
getrx	proc			;j+
;	*****************************************************
;	Routine gets a byte from rx_buf and reset the service
;	flag if it is the last byte in the buffer.
;	Exit: al will holds the data byte, Z if no data
;	Note: cl holds the data in MXE. Beware!
;	*****************************************************
	cli			;Make operation atomic
	mov	di,rx_count
	cmp	byte ptr [di],0      ;Any data?
	jnz	getrx2
	mov	di,usermask
	mov	bx,[di]
	xor	es:rxbyte,bx	;turn off the service request
	xor	dx,dx		;Set Z flag
	sti
	ret			;No data, return
;				 Make sure that terminal is in go mode if
;				 the buffer is less than 1/4 full.
getrx2:
	mov	di,rx_count
	cmp	byte ptr [di],40h
	ja	getrx3		;jump if more than 1/4 full
;				 set go mode
	mov	di,rsdbyte
	test	byte ptr [di],rsdwaitb
	jz	getrx3		;jump if already in go mode
	sti
	and	byte ptr [di],not rsdwaitb    ;reset the waitbit to go mode
	mov	al,esccode
	call	putdt
;
;	aug/28/91, for windowing
	mov	di,windw
	cmp	byte ptr [di],0
	jz	getrx6
	mov	al,0d9h
	call	putdt
	jmp	putrx3
;	aug/28/91, for windowing
;
getrx6:
	mov	al,rsdgo
	call	putdt
	cli
getrx3:
	mov	di,rx_count
	dec	byte ptr [di]
	mov	di,rx_rd
	mov	bx,[di]
	xor	bh,bh
	mov	di,rx_buf
	mov	al,[di][bx]
	inc	bl
	mov	di,rx_rd
	mov	[di],bx
;	mov	cl,al
	sti			;enable int
	or	dx,0ffh 	;set NF flag
	ret
getrx	endp
;
getdt	proc			;j+
;	*****************************************************
;	Routine that gets a byte from data transfer buffer
;	Exit: al stored the desired data if any
;	Return: Z = no data gotten, NZ = received data
;	*****************************************************
	mov	di,dt_count
	cmp	byte ptr [di],0
	jz	getdtz		;jump if the buffer is empty
	dec	byte ptr [di]	;update buffer
	mov	di,dt_rd
	mov	bx,[di]
	xor	bh,bh
	mov	di,dt_buf
	mov	al,[di][bx]	;receive the data
	inc	bl		;update read-pointer
	mov	di,dt_rd
	mov	[di],bx
	or	dx,0ffh 	 ;NZ flag
getdtz:
;	sti			;Enable interrupt
	ret
getdt	endp
;
putdt	proc			;j+
;	****************************************************************
;	Routine that put a byte into the data transfer buffer
;	Entry: al holds the byte, ds point the port's corresponding seg.
;	       es = data seg.
;	Return: Z = Overflow, NZ = enter buffer successfully
;	****************************************************************
	mov	di,dt_count
	mov	bl,[di]
	inc	bl
	jz	putdtz			;jump if overflow
;	cmp	dt_count,01h		;check? MXE had it meaninglessly.
	mov	si,dt_wrt
	mov	bx,[si]
;	inc	bl
	xor	bh,bh
	mov	di,dt_buf
	mov	[di][bx],al		;enter the data from al
;	mov	di,dt_wrt
	inc	bl
	mov	[si],bx 		;update dt_wrt
;
	mov	di,dt_count
	inc	byte ptr [di]
	call	entx
	or	al,0ffh 		;Set return flag NZ
putdtz:
	ret
putdt	endp
;***************************************************************************
chkfrfc proc
;
;	Routine looks at the last byte stored in RXBUF and returns a Z if it
;	was FC, and an NZ otherwise.
;	push	ax
	mov	di,rx_wrt
	mov	bx,[di]
	xor	bh,bh
	dec	bx
	mov	di,rx_buf
	mov	ah,[di][bx]
	cmp	ah,trans
;	pop	ax
	ret
chkfrfc endp
;
crtalert	proc			;j+
;	Show CS386 that CRT buffer empty and available, on termstat?
	mov	di,crt_count
	cmp	word ptr [di],0
	jnz	crtaltz 		;return if data left on buffer
;
	mov	di,lrbyte
	cmp	byte ptr [di],0
	jz	crtalt2 		;return if need no service in LR
;
	test	byte ptr [di],dtdelay
	jz	crtalt1
	test	byte ptr [di],rsdgob
	jz	crtalt1
					;set up rsd
	and	byte ptr [di],not dtdelay
	mov	di,dsrbyte
	or	byte ptr [di],dtb	      ;output rsd command
	call	entx
	ret
crtalt1:
	mov	di,usermask
	mov	bx,[di]
	or	es:commode,bx
	ret
crtalt2:
	mov	di,dsrbyte
	test	byte ptr [di],timeoutb
	jnz	crtaltz 		;Don't notice if timeout on
	mov	di,intbyte
	test	byte ptr [di],crtint	      ;int bit set?
	jz	crtaltz
	and	byte ptr [di],not crtint      ;reset crtint bit
	mov	di,curnt_sts
;	mov	di,flag9
	or	byte ptr [di],crtint
;	or	rbibus,statrb		;report 02 ready (MXE), not on MXF
crtaltz:
	ret
crtalert	endp
;
prtalert	proc			;j+
;	Show CS386 that PRT buffer empty and available, on termstat?
	mov	di,dsrbyte
	test	byte ptr [di],timeoutb
	jnz	prtaltz 		;Don't notice if timeout on
	mov	di,intbyte
	test	byte ptr [di],prtint	      ;in bit set?
	jz	prtaltz
	and	byte ptr [di],not prtint      ;reset prtint bit
	mov	di,curnt_sts
	or	byte ptr [di],prtint
;	or	rbibus,statrb		;report 02 ready (MXE), not on MXF
prtaltz:
	ret
prtalert	endp
;
putlr	proc		;j+
;	*******************************************************************
;	Routine puts one byte into lr_buf at the current cursor position
;	(lrwr) and updates buffer  pointer, flags as needed.
;	Entry: al holds the byte to be stored
;	Exit: Byte is stored, screen also updated or beep is out of screen.
;	Return: Z = buffer full, NZ = byte into buffer
;	*******************************************************************
;	Test for insert mode - insert byte if in insert mode
;	Note: should be moved to lredit so, underscore is handled properly.
	mov	di,lrbyte
	test	byte ptr [di],inbit	       ;insert bit set?
	jz	putlr0			;jump if in normal condition
	call	shftlr			;shift lr_buf to make room for byte
	jz	putlr22 		;beep, buffer is full!!

;		     dx  byte will be printed pointing at lrwr
	mov	di,lrwr
	mov	bx,[di] 		     ;point at lrwr
;	mov	bx,dx
	mov	di,lr_buf
	mov	[di][bx],al		;into buffer
	mov	di,lrwre		;point at lrwre
	mov	bx,[di]
	mov	di,lrwrp2
	mov	[di],bx
	call	lrout
	or	ax,0ffh 		;set NZ flag
	ret
putlr0: 				;Is lrwr = lrwre?
	mov	di,lrwr
	mov	bx,[di]
	add	di,2			;point at lrwre
	cmp	[di],bx
	jz	putlr2
putlr1: 		;Update print pointers and wr pointer, screen, buffer.
;	mov	di,lrwr
;	mov	bx,[di]
	mov	di,lr_buf
	mov	[di][bx],al
	push	bx
	call	putclr
	pop	bx
	inc	bx
	mov	di,lrwr
	mov	[di],bx
	mov	di,lrwrp1
	mov	[di],bx
	add	di,2		;point at lrwrp2
	mov	[di],bx
	mov	dx,bx
	call	findcur
	cmp	al,0		;is it column 0?
	jnz	putlrz
	call	nxtline
putlrz:
	or	ax,0ffh 	;set NZ flag
	ret
putlr2: 		;Update lrwre, lrcnt if not at end of buffer, quit else
	mov	di,lrcnt
	cmp	word ptr [di],0
	ja	putlr3		;if lrcnt=0 then quit
putlr22:			;buffer full, beep terminal.
	mov	al,beep
	call	putcrtg
	xor	ax,ax		;set Z flag
	ret			;error exit
putlr3: 			;update lrcnt
	mov	di,lrcnt
	dec	word ptr [di]
;	mov	di,lrwr
;	mov	bx,[di]
	inc	bx
	mov	di,lrwre	;point at lrwre
	mov	[di],bx 	;update lewre
	dec	bx
	jmp	putlr1		;go to enter byte
putlr	endp
;
putlrstr	proc	;j+
;	**************************************************************
;	Routine puts string pointed at by si into lr_buf, and screen.
;	Entry: si = string pointer, bx = length of string
;	**************************************************************
	mov	al,es:[si]
	inc	si
	push	si
	push	cx
;	lodsb		;load memory pointed by si to al, si increments
	call	putlr
	pop	cx
	pop	si
	loop	putlrstr
	ret
putlrstr	endp
;
putclr	proc		;j+
;	********************************************************************
;	Routine translates bytes destined for the crt_buf as part of line
;	request. Rules as: Bytes less than 10h must be turned into a period
;	(2eh), All bytes must be or'ed with 80h if in underline mode.
;	entry: al holds byte need to be translated and stored.
;	********************************************************************
	cmp	al,10h
	jnc	putclr2
	mov	al,2eh		;period, codes not to be seen on screen
putclr2:			;need to underline?
	mov	di,lrbyte
	test	byte ptr [di],underbit
	jz	putclr3
	or	al,80h
putclr3:
	push	ax		;store ax
	call	putcrtg
	pop	ax
;	Note: This routine should never output a 0fbh to the screen without
;	a 0d0h following it! 0fbd0h is the char 0fbh, but fbh by itself is the
;	start of a command to the terminal.
	cmp	al,0fbh 	;0fbh?
	jnz	putclrz 	;exit if not
	mov	al,0d0h
	call	putcrtg
putclrz:
	ret
putclr	endp
;
putrsd	proc		;j+
;	******************************************************************
;	Routine puts a byte into RSD buffer. Stop is sent to terminal thru
;	DT buffer when buffer level reach the designated level.
;	Entry : al holds the data byte
;	******************************************************************
	mov	di,rsdcomp
	test	byte ptr [di],escbit
	jz	putrsd3 		;escape not found
;	Escape code was found
	test	byte ptr [di],compbit
	jnz	putrsd2
;	May be a count or an escaped escape
	cmp	al,rsddesc
	jnz	putrsd15		;jump if is count
	and	byte ptr [di],not escbit
	mov	di,rsd_wrt
	mov	bx,[di]
	mov	si,rsd_buf
	mov	byte ptr [si][bx],rsddesc
	inc	bx
;
;	window 16/july/91
	mov	di,windw
	test	byte ptr [di],wndw		;under window mode?
	jz	putrsd1a
	mov	di,rsd_wrt
	mov	[di],bx
	mov	di,rsd_count
	add	word ptr [di],1
;	ret
	jmp	putrsd6
;
putrsd1a:		;rsd mode
	mov	byte ptr [si][bx],rsddesc
	inc	bx
	mov	di,rsd_wrt
	mov	[di],bx
	mov	di,rsd_count
	add	word ptr [di],2
;	window 16/july/91
;	ret
	jmp	putrsd6
;
putrsd15:				;Count found
	or	byte ptr [di],compbit
	inc	di			;point at rsdp
	mov	[di],al 		;store count in temp variable
	ret
putrsd2:				;Last byte of the compression sequence
					;store entire sequence
	and	byte ptr [di],not escbit
	and	byte ptr [di],not compbit
	mov	di,rsd_wrt
	mov	bx,[di]
	mov	si,rsd_buf
	mov	byte ptr [si][bx],rsddesc
	inc	bx
	mov	di,rsdp
	mov	cl,[di]
	mov	[si][bx],cl
	inc	bx
	mov	[si][bx],al
	inc	bx
	mov	di,rsd_wrt
	mov	[di],bx
	mov	di,rsd_count
	add	word ptr [di],3
;	window 16/july/91
;	ret
	jmp	putrsd6
;
putrsd3:				;No flag, test for rsddesc
	cmp	al,rsddesc
	jnz	putrsd4
	mov	di,rsdcomp
	or	byte ptr [di],escbit	      ;Escape byte, set and exit
	ret
putrsd4:
	mov	di,rsd_wrt
	mov	bx,[di]
	mov	si,rsd_buf
	mov	[si][bx],al
	inc	bx
	mov	[di],bx
	mov	di,rsd_count
	inc	word ptr [di]
	mov	di,rbi
	or	byte ptr [di],rsdrb
;
;	RSD testing Dec. 12th
;	mov	di,curnt_sts
;	or	byte ptr [di],rsdint
;	RSD Dec. 12th end
;
;	window 16/july/91
;	ret
putrsd6:
;	window 16/july/91
	mov	di,windw
	test	byte ptr [di],wndw		;under window mode?
	jnz	putrsd6a
	ret
putrsd6a:			;yes, under window mode
	mov	di,rsd_count
	cmp	word ptr [di],6a0h
	jb	putrsd6b
	mov	di,windw
	or	byte ptr [di],stopbit
;
;	aug/16/91, limit the following under once
	mov	di,fbdb
	cmp	byte ptr [di],0ffh
	jz	putrsd77
;	aug/16/91, limit the following under once, end
;
;	call	disrx		;disable receiver
	mov	al,esccode
	call	putdt
	mov	al,0dbh
	call	putdt
	mov	di,fbdb
	mov	byte ptr [di],0ffh
;
;	aug/15/91
;	mov	di,xorbyte
;	mov	al,[di]
;	mov	di,rsd_wrt
;	mov	bx,[di]
;	mov	si,rsd_buf
;	mov	[si][bx],al
;	inc	bx
;	mov	[di],bx
;	mov	di,rsd_count
;	inc	word ptr [di]
;
;	mov	di,cmd_sts
;	mov	byte ptr [di],0f0h
;	mov	di,cmd_parm
;	mov	byte ptr [di],midpage
;	mov	di,windw
;	and	byte ptr [di],not in_prcs
;	or	byte ptr [di],cntncmd
;
	mov	di,dsrbyte
	or	byte ptr [di],dtb	   ;let dt go
	call	entx
;	call	enrx
putrsd77:
	ret
;
putrsd6b:
	ret
putrsd	endp
;
getrsd	proc
;	**********************************************
;	Routine that reads a byte from rsd buffer for
;	the purpose of window/WRITE.
;	Exit : Z = buffer empty, NZ = otherwise
;	**********************************************
	mov	si,rsd_count
	cmp	word ptr [si],0
	jz	getrsdz 		;no more data in window/write
	dec	word ptr [si]
;
	mov	di,rsd_rd
	mov	bx,word ptr [di]
	mov	si,rsd_buf
	mov	al,[si][bx]
	inc	word ptr [di]
	or	si,0ffh 		;NZ flag
	ret
getrsdz:
	xor	si,si			;Z flag
	ret
getrsd	endp
;
disrx	proc
;	*******************************
;	Routine disables the RX action.
;	*******************************
	pushf
	cli
	xor	dh,dh
	mov	di,crport
	mov	dl,[di]
	mov	al,02h		;disable the rx
	out	dx,al
	popf
	ret
disrx	endp
;
enrx	proc
;	*******************************
;	Routine enables the RX action.
;	*******************************
	pushf
	cli
	xor	dh,dh
	mov	di,crport
	mov	dl,[di]
	mov	al,01h		;disable the rx
	out	dx,al
	popf
	ret
enrx	endp
;
