;
;	**************************************************
;	Routines that handles the buffers for TC functions
;	**************************************************
;
puttx	proc
;	*************************************************
;	Routine puts a byte, which in al, into the tx_buf
;	*************************************************
	mov	si,prt_wrt
	mov	bx,[si]
	xor	bh,bh
	mov	di,prt_buf
	mov	[di][bx],al	;Store byte into tx_buf (prt_buf)
	inc	word ptr [si]
;
	mov	di,prt_count
	inc	word ptr [di]	;Bring out the counter
	mov	ax,[di]
;	cmp	al,prtbusy	;above busy level
	cmp	al,200		;above busy level
	jb	puttxz		;return
;	Buffer almost full
	mov	si,rbi
	and	byte ptr [si],not prtrb
puttxz:
	ret
puttx	endp
;
gettx	proc
;	*********************************************************
;	Routine gets a byte from tx_buf (prt_buf), and put in al.
;	Return: Z is buffer empty, NZ buffer not empty.
;	*********************************************************
	mov	si,prt_count
	cmp	word ptr [si],0
	jne	gettx1			;jump if not empty
;	xor	dx,dx			;Z flag
	ret
gettx1:
	dec	word ptr [si]		;decrease the counter
;	Looking for the flags and buffer level. check??
;
;	call	int_386 		;notice OS check???
	mov	si,prt_rd
	mov	bx,[si]
	mov	di,prt_buf
	mov	al,[di][bx]
	inc	word ptr [si]
	or	dx,0ffh 		;NZ flag
	ret
gettx	endp
;
rsttx	proc
;	*************************
;	Routine clears out tx_buf
;	*************************
	cli				;stop the interrupts
	mov	di,prt_wrt
	mov	word ptr [di],0
	add	di,2
	mov	word ptr [di],0 	;Clear prt_rd
	mov	di,prt_count
	mov	word ptr [di],0
	sti
	ret
rsttx	endp
;
putrxp	proc
;	***************************************************************
;	Routine puts a byte, which in al, into the rxp_buf (preprocess)
;	Also, rxp_buf utilizes crt_buf of terminal ports.
;	***************************************************************
;	test count and drop byte if buffer is full
	mov	di,crt_count
	cmp	word ptr [di],03ffh	;Is it full?
	jne	putrxp1
;	Buffer overflow! Set error bit
	mov	di,statvec+2
	or	byte ptr [di],02h
	ret
putrxp1:
	mov	di,crt_wrt
	mov	bx,[di]
	mov	si,crt_buf
	mov	[si][bx],al
	inc	word ptr [di]
	ret
putrxp	endp
;
getrxp	proc
;	*********************************************************
;	Routine gets a byte from rxpbuf (crt_buf), returns in al.
;	Return: Z buffer empty, NZ successes.
;	*********************************************************
	mov	si,crt_count
	cmp	word ptr [si],0
	jne	getrxpz 		;jump if not empty
;	xor	dx,dx			;Z flag
	ret
getrxpz:
	dec	word ptr [si]		;decrease the counter
	mov	di,convec+20		;Flow control options byte
	test	byte ptr [di],0ah	;flow control from MXF?
	jz	getrxp1 		;no, get byte
	mov	di,crt_count
	mov	ax,[di] 		;Beware!!! we uses AL over here only
	mov	di,rx_count
	add	al,[di] 		;add count of preprocessed rcv buffer
	mov	di,convec+27		;point at Xon threshold level
	cmp	al,[di] 		;Has this level reached
	jnc	getrxp1 		;jmup if no,
	mov	di,convec+7		;yes
	test	byte ptr [di],80h	;Xoff pending?
	jz	getrxp1 		;no, get byte
	mov	di,convec+20		;flow control
	test	byte ptr [di],08h	;Xon/Xoff selected?
	jz	getrxfc1		;no, see of DTR is selected
;	Send Xon to connected device
	mov	si,fcmxf		;yes send Xon code
	or	byte ptr [si],fcmode	;set flow control mode
	test	byte ptr [si],mxfp	;1 or 2 byte sequence?
	jz	getrxfc 		;jmup if it is two
	or	byte ptr [si],escsnt	;set escape send flag for 1 byte sequence
getrxfc:
	mov	di,convec+25		;get Xon code
	mov	al,[di]
	mov	di,shiftmsk
	and	al,[di]
	mov	di,fcbyte
	mov	[di],al 		;store Xon code
	call	entx
;	Raise DTR if signal not lock
getrxfc1:
	mov	di,convec+20
	test	byte ptr [di],02h	;DTR select?
	jz	getrxp1 		;no, get byte
	mov	di,lockstat
	test	byte ptr [di],10h	;DTR lock?
	jnz	getrxp1 		;jump if yes, get byte only
	mov	si,dartreg+dr5		;point at dart table
	or	byte ptr [si],80h	;turn on dtr
	call	dtr_enb
;	Trun on the DTR signal! 	@check?????
;
;	call	signlset		;output from port???
	mov	di,convec+20
	test	byte ptr [di],08h
	jnz	getrxp1 		;Xon/Xoff selected?
	mov	di,statvec+7		;no, DTR
	and	byte ptr [di],not 80h	;reset Xoff sent flag
getrxp1:
	mov	si,crt_rd
	mov	bx,[si]
	mov	di,crt_buf
	mov	al,[di][bx]
	inc	word ptr [si]
	or     dx,0ffh			;NZ
	ret
getrxp	endp
;
rstp	proc
;	***************************************************
;	Reset RX translation buffer, and int's are enabled.
;	***************************************************
	cli				;disable
	mov	di,crt_wrt
	mov	word ptr [di],0
	add	di,2
	mov	word ptr [di],0 	;Clear crt_rd
	mov	di,crt_count
	mov	word ptr [di],0
	mov	di,statvec+4
	mov	byte ptr [di],0
	sti
	ret
rstp	endp
;
putrxtc 	proc
;	***************************************************************
;	Routine puts a byte into rx buffer (preprocessed). Byte in al.
;	If flow control is enabled, the Xoff buffer level is checked.
;	and Xoff code is sent if the buffer has been increased to this
;	level. Also, rx_buf utilizes rx_buf of terminal ports.
;	Exit: Byte is stored or overflow condition is flagged in status
;	      vector. Poll loop service bit is set.
;	***************************************************************
;	Test count and drop byte if buffer is full
	push	ax			;Save the incomming byte ??
	mov	di,convec+20
	test	byte ptr [di],0ah	;flow control?
	jz	putrxtc1		;test for overflow
	mov	di,rx_count
	inc	byte ptr [di]		;preprocessed rcv buffer
	mov	al,byte ptr [di]
	mov	si,crt_count
	mov	dx,[si]
	add	al,dl
	mov	si,convec+28		;Xoff threshold level
	cmp	al,[si] 		;Reach its level
	jc	putrxjj 		;no, store byte if count < level
	cmp	al,0feh 		;almost overflow, Send!
	jz	putrxfc0
	mov	di,statvec+7
	test	byte ptr [di],80h	;Xon pending
	jnz	putrxjj
putrxfc0:
	mov	di,convec+20
	test	byte ptr [di],08h
	jz	putrxfc1
;	Send Xoff to connected device
	mov	di,fcmxf
	or	byte ptr [di],fcmode
	test	byte ptr [di],mxfp	;1 or 2 byte sequence
	jz	putrxfc 		;two
	or	byte ptr [di],escsnt	;one
putrxfc:
	mov	di,convec+26		;get Xoff code
;
	mov	al,[di]
	mov	di,shiftmsk
	and	al,[di]
	mov	di,fcbyte
	mov	[di],al 		;store Xon code
	call	entx
;	Raise DTR if signal not lock
putrxfc1:
	mov	di,convec+20
	test	byte ptr [di],02h	;DTR select?
	jz	putrxtc2		;no, store byte
	mov	di,lockstat
	test	byte ptr [di],10h	;DTR lock?
	jnz	putrxtc2		;jump if yes,
	mov	si,dartreg+dr5
	and	byte ptr [si],07fh
	call	dtr_dis 		;disable dtr
;	Trun off the DTR signal!	@ckeck?????
;	call	signlset		;output from port???
	mov	di,convec+20
	test	byte ptr [di],08h
	jnz	putrxtc2		;Xon/Xoff selected?
	mov	di,statvec+7		;no, DTR
	or	byte ptr [di],80h	;set Xoff sent flag
putrxjj:
	jmp	putrxtc2
;
putrxtc1:
	mov	di,rx_count
	inc	byte ptr [di]
	jnz	putrxtc2
;	Overflow! set error bit, send Xoff code if Xon/Xoff is enabled
	mov	di,convec+20
	test	byte ptr [di],08h
	jz	putrxt1b		;no, go set error bit
	mov	di,fcmxf
	or	byte ptr [di],fcmode
	test	byte ptr [di],mxfp	;1 or 2
	jz	putrxt1a		;two, jump
	or	byte ptr [di],escsnt	;one,
putrxt1a:
	mov	di,convec+26		;get Xoff code
;
	mov	al,[di]
	mov	di,shiftmsk
	and	al,[di]
	mov	di,fcbyte
	mov	[di],al 		;store Xon code
	call	entx
putrxt1b:
	mov	si,rx_count
	dec	byte ptr [si]
	mov	di,statvec+2
	or	byte ptr [di],02h
	pop	ax
	jmp	putrxtc3
putrxtc2:
	pop	ax
	mov	si,rx_wrt
	mov	bx,[si]
	mov	di,rx_buf
	mov	[di][bx],al
	inc	word ptr [si]
putrxtc3:
	mov	di,usermask
	mov	cx,[di]
	or	es:rxbyte,cx
	ret
putrxtc 	endp
;
;signlset	 proc		 ;@check ???
;	*******************************************
;	Routine returns a new control byte to dart
;
;	*******************************************
;
;
;	ret
;signlset	 endp
;
getrxtc 	proc
;	******************************************************
;	Routine gets a byte from rx_buf, returns in al.
;	Return: Z empty, NZ return okay.
;	Poll loop service bit is reset when buffer goes empty.
;	******************************************************
	mov	di,rx_count
	cmp	byte ptr [di],0 	;empty?
	jnz	getrxtc2
;	Error, buffer is empty! reset service flag.
	mov	di,usermask
	cli
	mov	cx,[di]
	not	cx
	and	es:rxbyte,cx		;check???
	sti
	xor	cx,cx			;Z flag
	ret
getrxtc2:
	dec	byte ptr [di]		;update the count
;
	mov	si,rx_rd
	mov	bx,[si]
	mov	di,rx_buf
	mov	al,[di][bx]
	inc	word ptr [si]		;update read count
	or	cx,0ffh 		;NZ flag
	ret
getrxtc 	endp
;
rsttc	proc
;	********************************
;	Clear rx_buf, int's are enabled.
;	********************************
	cli
	mov	di,rx_wrt
	mov	word ptr [di],0
	add	di,2
	mov	word ptr [di],0
	mov	di,rx_count
	mov	byte ptr [di],0
	ret
rsttc	endp
;
