;
;	************************
;	General routines for TC.
;	************************
dummy	proc
	cli
	mov	di,usermask
	mov	cx,[di]
	not	cx
	and	es:rxbyte,cx
	sti
	ret
dummy	endp
;
trantx	proc
;	****************************************************************************
;	Routine translates a byte waiting to be outputted and sets up delay
;	after specified bytes. Byte in al.
;	Return: Z no byte to be sent, NZ byte in al is okay.
;	Exit: Translated byte is in al.
;	Note: How to process a byte before transmitted:
;	1) Translate the byte by using it as an index into the TX translation table.
;	2) If the byte size is translated is 5 or 6 bits/Char then look for the
;	   need to send a shift byte before transmitting this byte. This is done
;	   by interpreting bits 6 and 7 of translated byte as follows:
;	       00 = downshifted character
;	       01 = upshifted character
;	       10 = do not care
;	       11 = do not care
;	   Byte 6 and 7 of the control vector contain the upshift and downshift
;	   bytes respectively.
;	3) The byte to finally be transmitted must now be matched against the
;	   delay bytes to see if a time delay must be setup after transmission
;	   of this byte. Byte 10 to 17 are 4 two byte delay groups with the byte
;	   to be delayed after followed by the delay in 10ms intervals (4 pairs).
;	****************************************************************************
;
	mov	di,savetx	;Is there a byte stacked up waiting for sending
	mov	al,[di] 	;al is saved byte
	mov	byte ptr [di],0 	;clear save byte
	cmp	al,0
	jnz	trantx4 	;byte is already translated, put it out
	call	gettx		;get byte
	jnz	trantx1 	;jump if received
	ret			;no byte in buffer!
trantx1:
	mov	di,lr_buf	;First page of lr_buf working as tx buffer
;				 Second page of lr_buf as rx buffer
;	add	di,al		;al is the index of table
	mov	bl,al
	xor	bh,bh
	mov	al,[di][bx]	;al holds translated byte
;	Test shift status of byte and insert shift byte as needed
	add	al,0
	js	trantx4 		;jump if bit 7 is set. (sign flag)
;
;	Is shift enabled?
	mov	di,convec+2
	test	byte ptr [di],20h	;greater than 6-bit character
	jnz	trantx4
;	Test shift value of this byte and last byte
	mov	di,shifttx		;shift byte
	mov	ah,al			;save this byte
	xor	ah,[di] 		;compare bits with shift byte
	and	ah,40h			;just need shift bit
	jz	trantx4 		;jump if no shift difference
	mov	si,savetx
	mov	[si],al 		;save byte for later transmission
;	Set new shift bit and send out shift byte
	and	al,40h			;get shift bit from current byte
	mov	si,shifttx
	mov	[si],al 		;store new shift mask
;	Send out shift byte in accordance with new shift status
	jz	trantx3 		;jump if downshift
	mov	si,convec+6
	mov	al,[si]
	jmp	trantx4
trantx3:
	mov	si,convec+7
	mov	al,[si]
trantx4:			;Look for delay byte, does byte match on of the
;				 specified bytes which must be delayed?
	mov	cx,4		;counter is four
	mov	si,convec+10
trantx45:
	cmp	al,[si] 	;compare with byte
	jnz	trantx6 	;jump if not matched
;	Byte matched here! Set delay if not zero
	push	ax
	inc	si		;point at time out value
	mov	al,[si]
	cmp	al,0
	jz	trantx5 	;jump if no timeout value
	mov	si,offset entxtc;Point at tx int enable. entxtc will be executed.
	call	time_01 	;set timer value
	call	distx		;dis tx of this port
	mov	di,tranbyte		;Set delay flag to prevent int's
	or	byte ptr [di],txdelay	;being enable.
trantx5:
	pop	ax
	or	dx,0ffh 	;NZ
	ret
trantx6:
	add	si,2
	loop	trantx45
	xor	dx,dx		;Z flag
	ret
trantx	endp
;
entxtc	proc
;	*********************************************************************
;	Routine works as an interface between entx and a routine being called
;	with control passed in bx rather than that port.
;	*********************************************************************
	push	ds
	mov	si,dataseg
	mov	ax,es:[si][bx]		;looking for the data segment
	mov	ds,ax
;	reset transmission delay flag
	mov	si,tranbyte
	and	byte ptr [si],not txdelay
	call	entx
	pop	ds
	ret
entxtc	endp
;
tranrx	proc
;	*******************************************************
;	Routine translates a byte which has just been inputted.
;	Note that data segment is held by bx. check?
;	Exit:  returned byte in al.
;	*******************************************************
;	push	ds
;	mov	si,dataseg
;	mov	ax,es:[si][bx]		;looking for the data segment
;	mov	ds,ax
;
	call	getrxtc
	jnz	tranrx1
;	pop	ds
	ret				;Exit if no byte with Z flag
;
tranrx1:	;mask off top bits which are illegal size of the byte set
	mov	di,shiftmsk		;point at size mask
	and	al,[di]
	mov	di,convec+2		;get size byte
	test	byte ptr [di],20h	;5 or 6 bit/character?
	jnz	tranrx3 		;jump if 7 or 8 bit/character
;	Look for shift byte - upshift first
	mov	di,convec+6
	cmp	al,[di]
	jnz	tranrx2
;	upshift character!
	mov	di,shiftmsk+1		;2nd byte
	or	byte ptr [di],80h	;set shift bit
	ret				;discard byte check???
tranrx2:				;look for downshift
	inc	di			;point at down shift
	cmp	al,[di]
	jnz	tranrx3
;	downshift character!
	mov	di,shiftmsk+1
	and	byte ptr [di],07fh	;reset the bit 7 (MSB)
	ret				;discard byte
tranrx3:	;Translate byte - look in the table
	mov	si,lr_buf+256		;2nd page of lr_buf is for RX
	mov	di,shiftmsk+1		;point at shift mask
	or	al,[di] 		;set shift bit if upshifted
	xor	ah,ah			;check ???????
	add	si,ax			;add the index (al)
	mov	al,[si] 		;get the byte by looking the table
	cmp	al,0
	jnz	tranrx4
;	Null byte!
	mov	si,convec+1
	test	byte ptr [si],10h	;check null discard flag
	jz	tranrx4
	ret				;Yes, discard
tranrx4:				;look for end of record byte
	add	al,0
	jns	tranrx5 		;jump if bit 7 is not set
;	Could be EOR byte, check EOR flag?
	mov	si,convec+5
	test	byte ptr [si],01h	;test EOR detection flag
	jz	tranrx5 		;jump if no flag
;	EOR is on
	mov	si,convec+4
	inc	byte ptr [si]		;increase EOR count
tranrx5:				;save byte and return
	call	putrxp			;store byte
;	pop	ds
	ret
tranrx	endp
;
err04	proc
;	**************************************
;	Bring back to async with OS
;	OS is outputing, then it gets a OBS
;	OS is inputing, then it sends ENDI(00)
;	**************************************
	jmp	cretvp
	ret
err04	endp
;
setcomm 	proc
;	*****************************************************
;	Routine gets the command vector from OS and store it.
;	*****************************************************
	mov	al,12h
	call	setlastc
	mov	si,convec
	mov	cx,29			;counter
	jmp	setcomm2
;
setcomm1:
	mov	di,nxtcomp
	mov	cl,[di]
	xor	ch,ch
	mov	di,nxtcom
	mov	si,[di]
;
setcomm2:
;setcomm7:
	push	si
	call	two386i
	jz	setcommk
;	mov	al,[di]
	pop	si
	mov	[si],al
	inc	si
;	inc	di
	loop	setcomm2
	jmp	cretvp
;
setcommk:
	pop	si
	mov	di,nxtcom
	mov	[di],si
	mov	di,nxtcomp
	mov	[di],cl
	jmp	cretvp
;
setcommc:
	mov	al,0
	call	setlastc
	call	initcont		;initial the device
	call	testtc0 		;test dsr status
	call	rxtcs			;set up status vector flags
;	mov	di,cmd_sts
;	mov	byte ptr [di],0f1h	;complete without value
	jmp	cretvp
	ret
setcomm 	endp
;
rdstat	proc
;	********************************************
;	Routine outputs the status vector to OS/386.
;	********************************************
	mov	al,13h
	call	setlastc
;	Set dsr flag before outputting status vector to 2200.
;	A changed in dsr does not produce an int!!!
	mov	si,statvec+1		;point into status vector
	mov	dx,ip_1ab
	mov	di,tc_who		;Note tc_who=1 the port1a, =4/1c
	cmp	es:byte ptr [di],1	;Is it port 1a? Note tc_who=0 then 1a,
	jz	rdst_ys 		;jump if it is
	mov	dx,ip_1cd		;Port 1c!
rdst_ys:
	in	al,dx
	test	al,04h
	jz	rdstat01		;jump if dsr is active
;	dsr is not set (sense of bit n portstat is inverted)
	and	byte ptr [si],not 04h	;reset dsr flag in status vector
	mov	si,statvec+7		;point to control signals in status vector
	and	byte ptr [si],not 04h
	jmp	rdstap1
rdstat01:				;rsd is active, set vectors
	or	byte ptr [si],04h
	mov	si,statvec+7		;point to control signals in status vector
	or	byte ptr [si],04h
rdstap1:	;Set rts before give OS status vectors
	mov	di,dartreg+dr5
	mov	si,statvec+7
	test	byte ptr [di],02h	;get rts status
	jnz	rdstap2 		;jump if set
;	reset rts status bit in status vector
	and	byte ptr [si],not 20h	;reset rts flag
	jmp	rdstap3
rdstap2:
	or	byte ptr [si],20h	;set rts
rdstap3:	;Set dtr flag before hand over to OS
	test	byte ptr [di],80h	;get dtr bit
	jnz	rdstap4 		;jump if dtr set
;	reset dtr status bit in status vector
	and	byte ptr [si],not 10h	;reset dtr flag
	jmp	rdstat02
rdstap4:
	or	byte ptr [si],10h	;set dtr
rdstat02:	;Set pointers and counters for output operation
	mov	si,statvec
	mov	cx,8			;# bytes to give
	jmp	rdstat22
;
rdstat1:		;restart entry
	mov	di,nxtcomp
	mov	cl,[di]
	xor	ch,ch
	mov	di,nxtcom
	mov	si,[di]
rdstat2:
	cmp	cl,0
	jz	err04k
rdstat22:
;
;rdstat03:
	mov	al,[si]
;	mov	[di],al
	push	si
	call	out386
	jz	rdstat3
	pop	si
	inc	si
;	inc	di
	loop	rdstat22
	jmp	rdstatc2
rdstat3:
	pop	si
	mov	di,nxtcom
	mov	[di],si
	mov	di,nxtcomp
	mov	[di],cl
	jmp	cretvp
err04k:
	jmp	err04
;
rdstatc:		;termination code
	mov	al,0
	call	setlastc
rdstatc2:
	mov	di,statvec
	mov	byte ptr [di],0
	add	di,2			;point at error byte
	mov	byte ptr [di],0
;	mov	di,cmd_sts
;	mov	byte ptr [di],0f0h	;complete with value
	jmp	cretvp
	ret
rdstat	endp
;
ldtxtab 	proc
;	**************************************************************
;	Routine inputs the TX translation table, note that TX table is
;	located in the first page of the terminal lr_buf.
;	**************************************************************
	mov	al,14h
	call	setlastc
	mov	si,lr_buf
	mov	cx,100h 	;256 byte to turn over
	jmp	ldtxtab2
ldtxtab1:			;restart entry
	mov	di,nxtcomp
	mov	cl,[di]
	xor	ch,ch
	cmp	cl,0
	jz	ldjj
	mov	di,nxtcom
	mov	si,[di]
;
ldtxtab2:			;point at buffer, beware no counter over this
				;command. The length is fixed -> 256 bytes
	push	si
	call	two386i
	jz	ldtxtabk
	pop	si
	mov	[si],al
;	inc	si
	inc	si
	loop	ldtxtab2
	jmp	cretvp
ldtxtabk:
	pop	si
	mov	di,nxtcom
	mov	[di],si
	mov	di,nxtcomp
	mov	[di],cl
	jmp	cretvp
;
ldtxtabc:		;termination entry
	mov	al,0
	call	setlastc
	jmp	cretvp
	ret
ldtxtab 	endp
;
ldrxtab 	proc			;@@
;	**************************************************************
;	Routine inputs the RX translation table, note that RX table is
;	located in the second page of the terminal lr_buf.
;	**************************************************************
	mov	al,15h
	call	setlastc
;
	mov	di,lr_buf+256	;second page of lr_buf
	mov	cx,100h 	;256 byte to turn over
	jmp	ldtxtab2
				;point at buffer, beware no counter over this
				;command. The length is fixed -> 256 bytes
ldrxtab1:		;restart entry
	mov	di,nxtcomp
	mov	cl,[di]
	xor	ch,ch
	cmp	cl,0
	jz	ldjj
	mov	di,nxtcom
	mov	si,[di]
	jmp	ldtxtab2
;
ldrxtabc:		;termination entry
	mov	al,0
	call	setlastc
ldjj:
	jmp	cretvp
	ret
ldrxtab 	endp
;
