;
;	***********************
;	INITIALIZATION ROUTINES
;	***********************
;
initcont	proc			;jj
;	**********************************************************************
;	Routine initializes a port to the specification of the control vector.
;	It is assumed that control vector has been already set by the user or
;	to the default values. See the storage block for a complete descrip-
;	tion of the control and status vectors.
;	However, owing to the differences of darts of new UART, masking table
;	of darts have to be reconsidered. Thus this routine will store
;	these maskings and set them in the new ways of 2698.
;	**********************************************************************
	mov	di,zerostrt	;clear pointers, start point
	mov	cx,zeros	;counter
initcon1:
	mov	byte ptr [di],0
	inc	di
	loop	initcon1
;
	mov	di,convec+2	;character size
	mov	ah,[di] 	;set the table (default)
	and	ah,30h		;4,5 bits only
;
	mov	si,dartreg+dr5
	mov	al,[si]
	and	al,0fch 	;reset current setting
	or	al,ah
	mov	[si],al
;	Set second dart register (RX char bit size)
	mov	si,dartreg+dr3
	mov	al,[si] 	;al = current setting
	and	al,0fch 	;reset current setting
	or	al,ah		;set new size bits
;
	mov	di,convec+20	;flow control option
	test	byte ptr [di],01	;cts/dcd selected?
	jnz	initco2a	;yes, set auto enable option
	and	al,not 20h	;no, reset auto enable
	jmp	initco2b
initco2a:
	or	al,20h
initco2b:
	mov	[si],al 	;store in dartreg+dr3
;	Set parity choice
	mov	di,convec+2	;control
	mov	ah,[di]
	and	ah,03h		;only parity spec bits
;	Note: bit 0 = parity enable, bit 1 = parity (1=odd). The sense of
;	parity bit in the dart control register is inverted from CONVEC.
;	mov	ah,al		;save
	mov	si,dartreg+dr4
	mov	al,[si]
	and	al,0cch 	;reset parity and stop bits
	or	al,ah		;set new selection
	mov	[si],al
;	Set stop bit selection
	mov	di,convec
	mov	al,[di]
	and	al,30h		;stop bit only
	jnz	initcon3	;jump if is 1, 1.5, 2
;	Illegal stop bit selection - select one stop bit
	mov	al,10h		;select default of one stop bit
initcon3:	;Shift bits to proper location for dart register
;	or	al,[si] 	;set stop bit selection
	or	[si],al 	;save in dartreg+dr4 table
;
;	Now initialize baud rate
	mov	al,[di]
	and	al,0fh		;get lower nibble, baud rate selection
	call	initbaud	;@
;
initcon4:			;External entry. Write every thing to 2698!!!
	call	dart4		;write out data 4
	call	dart3		;data 3
	call	dart5		;data 5
;	call	dart1		;data 1
;	Set up timer interrupt counter, MXF sets in inittc
;	Set bit in porttype to designate TC port instead of terminal
	mov	di,usermask
	mov	ax,[di]
	mov	si,porttype
	or	[si],al
;	Clear buffer
	call	rsttx		;TX buffer
	call	rsttc		;RX TC buffer
	call	rstp		;RX 2200 buffer
;	Setup default translation table
	mov	di,lr_buf
	mov	al,0
	mov	cx,200h 	;512 bytes
	cld
	rep	stosb
;	set shift mask - this mask is used to prepare a byte for
;	translation in reception mode
	mov	di,convec+2		;# char per byte
	mov	al,[di]
	and	al,0f0h 		;only size spec
	mov	si,shiftmsk+1		;shift register
	mov	byte ptr [si],0 	;clear to unshifted
	dec	si			;size mask
	mov	byte ptr [si],1fh	;set to 5 bytes/char
	cmp	al,0			;5 bytes/char?
	jz	initcon7
;	6 bytes/char?
	mov	byte ptr [si],3fh	;6 bytes/char
	cmp	al,10h
	jz	initcon7
	mov	byte ptr [si],7fh	;7 bytes?
	cmp	al,20h
	jz	initcon7
;	Must be 8 bytes/char
	mov	byte ptr [si],0ffh	;8 bytes
;	Set control option
initcon7:
	mov	di,convec+20
	mov	al,[di]
	and	al,0ch			;Xon/Xoff selected in either direction?
	jz	initco12		;no, check buffer level
	mov	di,convec+1
	test	byte ptr [di],20h	;half duplex selected?
	jnz	initcon8		;no, full duplex
	mov	al,[di] 		;force full duplex
	and	al,0fh
	or	al,20h
	mov	[di],al
initcon8:				;set 1/2 byte escape sequence
	mov	di,convec+20
	mov	al,[di]
	and	al,04			;Xon/Xoff selected?
	jz	initco10		;no, check if selected from MXF
	inc	di			;point at +21
	cmp	byte ptr [di],0 	;escape byte
	jz	initcon9		;one byte sequence
	mov	di,fccdev
	and	byte ptr [di],not cdevp ;set 2 byte sequence
	jmp	initc9a 		;check Xon/Xoff bytes are not same
initcon9:
	mov	di,fccdev
	or	byte ptr [di],cdevp
;	set Xon/Xoff to default if selected to the same value
initc9a:
	mov	si,convec+22		;Xon value
	mov	al,[si]
	inc	si			;Xoff value
	cmp	al,[si] 		;same?
	jnz	initco10
	mov	byte ptr [si],13h	;default of Xoff
	dec	si
	mov	byte ptr [si],11h	;default of Xon
initco10:				;check if Xon/Xoff from MXF
	mov	si,convec+20		;flow control spec
	test	byte ptr [si],08h	;from MXF?
	jz	initco12		;no, go check buffer level
	mov	si,convec+24
	cmp	byte ptr [si],0 	;get escape byte
	jz	initco11
	mov	di,fcmxf
	and	byte ptr [di],not mxfp	;set 2 bytes sequence
	jmp	initc11a
initco11:
	mov	di,fcmxf
	or	byte ptr [di],mxfp	;set 1 byte
initc11a:				;set Xon/Xoff to default if not same
	mov	si,convec+25		;Xon value
	mov	al,[si]
	inc	si			;Xoff value
	cmp	al,[si]
	jnz	initco12
	mov	byte ptr [si],13h	;default of Xoff
	dec	si
	mov	byte ptr [si],11h	;default of Xon
initco12:				;check buffer threshold is valid?
	mov	si,convec+20		;flow control option
	test	byte ptr [si],0eh	;buffer level if Xon/Xoff or dtr
	jz	initco14		;neither then exit
	mov	si,convec+27		;Xon level
	mov	al,[si]
	inc	si			;Xoff level
	cmp	al,[si] 		;Xon - Xoff
	ja	initco13		;Xon > Xoff then jmp to set default
	dec	si			;back to Xon
	cmp	byte ptr [si],1
	jb	initco13		;Xon must > = 1
	inc	si			;Xoff level
	cmp	byte ptr [si],1
	jb	initco13		;Xoff must > = 1
	mov	si,convec+27		;Xon
	cmp	byte ptr [si],254	;Xon must be <= 254 (0feh)
	ja	initco13		;jmp if > 254
	inc	si			;Xoff
	cmp	byte ptr [si],254	;Xoff must be <= 254 (0feh)
	ja	initco13		;jmp if > 254
	jmp	initco14		;exit
initco13:				;set default Xon/32, Xoff/200
	mov	si,convec+27
	mov	byte ptr [si],20h	;32
	inc	si
	mov	byte ptr [si],0c8h	;200
initco14:
	ret
initcont	endp
;
initbaud	proc			;jj
;	*****************************
;	Routine inits baud rate of TC
;	*****************************
	sub	al,03			;2/100bps -> become 0, will be set default
	jb	initbb1 		;-in fd_baud
	xor	al,0fh			;complement low nibble
	call	fd_baud 		;Find baud, and set of BR
	jmp	initbb2
initbb1:
	cmp	al,0
	jnz	initbb3
;
;	mov	si,crdata
;	mov	al,[si]
	mov	ah,7fh		;1st set
	mov	al,0		;50 baud rate
	mov	si,ratetbl + 105
	jmp	initbb4
initbb3:
	cmp	al,2		;100, not support set default -> 19200
	jnz	initbb6
	mov	ah,80h
	call	fd_baud1	;19200
	jmp	initbb2
initbb6:			;must be 75 baud rate
;	mov	si,crdata
;	mov	al,[si]
	mov	ah,80h
	mov	al,0		;75 baud rate
	mov	si,ratetbl + 98
initbb4:
	call	fd_baud0
initbb2:
;
;	For fixed and added baud rate 3/june, 1991
	push	ax
	mov	al,ah
	mov	di,acrport
	xor	dh,dh
	mov	dl,[di]
	mov	di,acrdata
	mov	ah,[di] 	;acr value
	cmp	al,80h
	jz	june03z
;	1st set of baud rate
	and	al,ah
	jmp	june03az
june03z:			;2nd set
	or	al,ah
june03az:
	mov	[di],al 	;store back to acr value
	out	dx,al
	pop	ax
;	End bugging of 3/June, 1991
;
	mov	di,csrport
	mov	dl,[di] 	;csrport out
;	xor	dh,dh
	out	dx,al
;
	ret
initbaud	endp
;
dart4	proc			;jj
;	************************************************************
;	Routine writes the I/O configurations to UARTs for 4th data.
;	Including : X16, parity, stop bit.
;	************************************************************
	xor	al,al
	xor	dh,dh		;clear dx for out port
;
	mov	di,crport
	mov	dl,[di]
	out	dx,al		;reset rm register to mr1
;
	mov	di,mrport
	mov	dl,[di]
	mov	si,dartreg+dr4	;point at 4th dart
	mov	ah,[si]
	mov	si,dartreg+dr3	;check the bits/char
	mov	bh,[si]
	and	bh,03h		;low nibble, 2 low bits
	or	al,bh		;bits/char
;
	and	ah,03h		;lowest 2 bits
	test	ah,01h
	jnz	dart41
;
	or	al,10h		;no parity
	and	al,0f7h
	jmp	dart49
dart41:
	test	ah,02h		;even or odd
	jnz	dart42
;	Even parity
	and	al,0fbh 	;even
	jmp	dart49
dart42: 			;odd
	or	al,04h
dart49:
	out	dx,al
	inc	di
	mov	dl,[di] 	;mr2
	xor	al,al
	mov	si,dartreg+dr4	;point at 4th dart
	mov	ah,[si]
	and	ah,030h 	;high nibble
	jz	dart4a		;illegal, set 1 stop
	cmp	ah,30h
	jnz	dart4b
	mov	al,0fh		;stop = 2
	jmp	dart4k
;
dart4b:
	cmp	ah,20h
	jnz	dart4a
	mov	al,08h		;stop = 1.563 ( 1.5 ? )
	jmp	dart4k
;
dart4a:
	mov	al,07h		;stop = 1
dart4k:
	out	dx,al
	ret
dart4	endp
;
dart3	proc
;	***************************************************************
;	Routine writes the I/O configurations to UARTs for 3rd data.
;	This is for RX, including : bit/char, auto enable, rx disabled.
;	***************************************************************
	xor	al,al
	xor	dh,dh		;clear dx for out port
;
	mov	di,crport
	mov	dl,[di]
	out	dx,al		;reset rm register to mr1
;
	mov	di,mrport
	mov	dl,[di]
	mov	si,dartreg+dr4	;point at 4th dart
	mov	ah,[si]
	mov	si,dartreg+dr3	;check the bits/char
	mov	bh,[si]
	and	bh,03h		;low nibble, 2 low bits
	or	al,bh		;bits/char
;
	and	ah,03h		;lowest 2 bits
	test	ah,01h
	jnz	dart31
;
	or	al,10h		;no parity
	and	al,0f7h
	jmp	dart39
dart31:
	test	ah,02h		;even or odd
	jnz	dart32
;	Even parity
	and	al,0fbh 	;even
	jmp	dart39
dart32: 			;odd
	or	al,04h
dart39:
	out	dx,al		;output mr1
;
	mov	si,dartreg+dr3	;check the RX en/dis-able bit
	mov	al,[si]
	test	al,10h
	jnz	dart3t
	call	disrx		;Disable RX interrupt
	ret
dart3t:
	call	enrx
	ret			;beware "auto enable" is not done here sep/9/91
dart3	endp
;
dart5	proc			;jj
;	***************************************************************
;	Routine writes the I/O configurations to UARTs for 5th data.
;	This is for TX, including : bit/char, not RTS, DTR, tx enabled.
;	Check the 'break' enable bit
;	***************************************************************
	xor	al,al
	xor	dh,dh		;clear dx for out port
;
	mov	di,crport
	mov	dl,[di]
	out	dx,al		;reset rm register to mr1
;
	mov	di,mrport
	mov	dl,[di]
	mov	si,dartreg+dr4	;point at 4th dart
	mov	ah,[si]
	mov	si,dartreg+dr5	;check the bits/char
	mov	bh,[si]
	and	bh,03h		;low nibble, 2 low bits
	or	al,bh		;bits/char
;
	and	ah,03h		;lowest 2 bits
	test	ah,01h
	jnz	dart51
;
	or	al,10h		;no parity
	and	al,0f7h
	jmp	dart59
dart51:
	test	ah,02h		;even or odd
	jnz	dart52
;	Even parity
	and	al,0fbh 	;even
	jmp	dart59
dart52: 			;odd
	or	al,04h
dart59:
	out	dx,al		;output mr1
;
	mov	si,dartreg+dr5	;check the RX en/dis-able bit
	mov	al,[si]
	test	al,10h
	jnz	dart5t
	call	distx		;Disable TX interrupt
	ret
dart5t:
	call	entx
	ret
dart5	endp
;
;disrx	 proc
;	*******************************
;	Routine disables the RX action.
;	*******************************
;	 pushf
;	 cli
;	 xor	 dh,dh
;	 mov	 di,crport
;	 mov	 dl,[di]
;	 mov	 al,02h 	 ;disable the rx
;	 out	 dx,al
;	 popf
;	 ret
;disrx	 endp
;
;enrx	 proc
;	*******************************
;	Routine enables the RX action.
;	*******************************
;	 pushf
;	 cli
;	 xor	 dh,dh
;	 mov	 di,crport
;	 mov	 dl,[di]
;	 mov	 al,01h 	 ;disable the rx
;	 out	 dx,al
;	 popf
;	 ret
;enrx	 endp
;
