;
time1	proc
;	********************************************************************
;	Routine sets up a timeout whose resolution is 1 second.
;	Note: Only one timeout per port is allowed. If a timeout is already
;	in effect for this port then it is immediately ended by performing
;	the action it required.
;	Entry: al is timeout interval in 1 sec. increments (0=256 sec.)
;	       si holds the routine to call at the end of timeout
;	********************************************************************
;	mov	ah,al		;store the interval
	mov	di,intt1_0
	cmp	byte ptr [di],0 ;timeout clear
	jz	time13
;	Beware, unfinished timeout!
	push	ax
	push	si		;save the routine
	inc	di
	mov	si,[di] 	;get routine pointer
	call	si
	pop	si
	pop	ax
time13:
	mov	di,intt1_0	;save counter
	mov	[di],al
	inc	di		;point at routine pointer
	mov	[di],si 	;save it!
	ret
time1	endp
;
time_01 	proc
;	********************************************************************
;	Routine sets up a timeout whose resolution is .01 second.
;	Note: Only one timeout per port is allowed. If a timeout is already
;	in effect for this port then it is immediately ended by performing
;	the action it required.
;	Entry: al is timeout interval in .01 sec. increments (0=256 sec.)
;	       si holds the routine to call at the end of timeout
;	********************************************************************
;	mov	ah,al		;store the interval
	mov	di,intt_01
	cmp	byte ptr [di],0 ;timeout clear
	jz	time_013
;	Beware, unfinished timeout!
	push	ax
	push	si		;save the routine
	inc	di
	mov	si,[di] 	;get routine pointer
	call	si
	pop	si
	pop	ax
time_013:
	mov	di,intt1_0	;save counter
	mov	[di],al
	inc	di		;point at routine pointer
	mov	[di],si 	;save it!
;	start the timer of 0.01 sec. interrupt
;	call	on_01		;make start automatically
	ret
time_01 	endp
;
testtc	proc
;	********************************************************
;	Routine is called once per second to test TC conditions.
;	********************************************************
	mov	si,flag2		;special package byte, termbyte
	cmp	byte ptr [si],01h	;TC selected?
	jnz	testtc0 		;jump if not
;	check dsr status
	mov	dx,ip_1ab
	mov	di,tc_who		;Note tc_who=1 the port1a, =4/1c
	cmp	es:byte ptr [di],1	;Is it port 1a? Note tc_who=1 then 1a,
	jz	test_ys 		;jump if it is
	mov	dx,ip_1cd		;Port 1c!
test_ys:
	in	al,dx
	test	al,04h
	jz	txtcj2			;dsr not ready! jump
	jmp	txtcb2
txtcj2:
	mov	di,statvec+7
	test	byte ptr [di],40h	;Xoff condition pending
	jnz	testtc0 		;go if yes
	call	entx			;enable TX interrupt of Xon condition
testtc0:
;	Get current timer value and decrement if not already zero.
;	Call specified routine if counter decrements to zero
	mov	di,intt1_0
	cmp	byte ptr [di],0
	jz	testtcz 		;exit if no timer
	dec	byte ptr [di]		;update the counter
	jnz	testtcz 		;exit if not zero yet
;	call specified routine here
	inc	di			;point to routine pointer
	mov	si,[di]
	call	si			;check???
testtcz:
	ret
testtc	endp
;
testbrk 	proc
;	*********************************************************************
;	Routine tests for break completion. Note if break is set then set the
;	break detection bit in the status vector. check???  update breakmk !!
;	*********************************************************************
	mov	di,breakmk
	mov	al,[di]
	test	al,01h		;test delta break from variable breakmk
	jnz	testbrk1	;jmp if break is no longer on
	ret
testbrk1:
	mov	di,statvec	;break still set, set flags
	mov	byte ptr [di],01
	ret
testbrk 	endp
;
discon	proc
;	**********************************************************************
;	Routine disconnects a port. Data Terminal Ready (DTR) is held inactive
;	for 3 seconds and then returned to active.
;	Exit: DTR, RTS are inactive. TX is also disabled, and buffer-cleared.
;	      While al holds the counter of 3 seconds.
;	**********************************************************************
	mov	al,0
	call	setlastc		;clear current TC command
;	Set up 3 seconds timeout counter
	mov	si,offset contc 	;routine pointer
	mov	al,4			;3.0 to 3.1 second timeout
	call	time1
	mov	di,dartreg+dr5
	cli
	and	byte ptr [di],0fh	;turn off TX, RTS, DTR
	call	dtr_dis
	call	rts_dwn
	call	distx			;turn off tx too here!
	sti
;	mov	si,cmd_sts
;	mov	byte ptr [si],0f1h	;complete without value
	jmp	cretvp
	ret
discon	endp
;
contc	proc
;	****************************************
;	Routine reconnects a port after timeout.
;	Exit: DTR is active again.
;	****************************************
	mov	si,dartreg+dr5
	cli
	or	byte ptr [si],20h
	call	dtr_enb
	sti
	jmp	cretvp
	ret
contc	endp
;
txbreak 	proc
;	********************************************************
;	Routine puts break on the TC line for the time specified
;	in the control vector.
;	Exit: TC in break state and timeout is set to Ms.?
;	********************************************************
	mov	al,0
	call	setlastc
;	Do not send break if break is not enabled
	mov	di,convec+1
	mov	al,[di]
	and	al,0fh			;low nibble
	cmp	al,01
	jz	txbreak2		;exit if not set
	ret
txbreak2:
;	Okay to send break
	mov	si,offset breakoff	;routine to be called after timeout
	mov	di,convec+8		;timeout value
	mov	al,[di]
	cmp	al,0
	jnz	txbreak3		;timeout counter=0 -> exit
	ret
txbreak3:
	call	time_01
;
	mov	si,dartreg+dr5
	cli
	or	byte ptr [si],40h
	mov	di,crdata
	mov	al,[di]
	and	al,0fh			;keep low nibble
	or	al,60h			;start break
	mov	[di],al
	mov	di,crport
	mov	dl,[di]
	xor	dh,dh
	out	dx,al
	sti
;	mov	si,cmd_sts
;	mov	byte ptr [si],0f1h	;complete without value
	jmp	cretvp
	ret
txbreak 	endp
;
breakoff	proc
;	**********************************
;	Routine resets the break condition
;	**********************************
	mov	si,dartreg+dr5
	cli
	and	byte ptr [si],0fbh	;00000100
	mov	di,crdata
	mov	al,[di]
	and	al,0fh			;keep low nibble
	or	al,70h			;start break
	mov	[di],al
	mov	di,crport
	mov	dl,[di]
	xor	dh,dh
	out	dx,al
	sti
	jmp	 cretvp
	ret
breakoff	endp
;
rxon	proc
;	***********************
;	Routine enables the RX.
;	***********************
	mov	al,0
	call	setlastc
;
	mov	di,convec+20
	test	byte ptr [di],0eh	;flow control?
	jnz	rxon0g			;no, process as usual
	jmp	rxon0f
rxon0g:
;	check if buffer level <= Xon level
	mov	di,rx_count		;pre-processed rcvd buffer
	mov	al,[di]
	mov	di,crt_count		;translated rcvd buffer
	add	al,[di]
	mov	di,convec+27		;Xon level
	cmp	al,[di] 		;compare
	jnc	rxon0c			;count > level, continue
;	If Xon/Xoff selected, send Xon to device.
	mov	di,convec+20
	test	byte ptr [di],08h
	jz	rxon0b			;jump if no
	mov	di,fcmxf
	or	byte ptr [di],fcmode
	test	byte ptr [di],mxfp	;1 or 2 sequence?
	jz	rxon0a			;2
	or	byte ptr [di],escsnt	;send escape sent flag for 1 byte sequence
rxon0a:
	mov	di,convec+25		;Xon code
	mov	al,[di]
	mov	di,shiftmsk		;size mask
	and	al,[di]
	mov	di,fcbyte		;flow control
	mov	[di],al 		;store Xon code
	call	entx
rxon0b:
	mov	di,convec+20
	test	byte ptr [di],02h	;DTR selected?
	jz	rxon0f
	mov	di,lockstat
	test	byte ptr [di],10h	;DTR locked?
	jnz	rxon0f
;
	mov	si,dartreg+dr5
	or	byte ptr [si],20h
	call	dtr_enb
	jmp	rxon0d			;DTR active
;
rxon0c:
	mov	di,convec+20		;flow control option
	test	byte ptr [di],02h	;DTR selected?
	jz	rxon0f			;no, jump
	mov	di,lockstat		;yes
	test	byte ptr [di],10h	;lock?
	jnz	rxon0f			;yes, jump
	mov	si,dartreg+dr5
	and	byte ptr [si],0dfh
;
	call	dtr_dis
rxon0d:
	mov	di,convec+20		;flow control option
	test	byte ptr [di],08h	;Xon/Xoff?
	jnz	rxon0f			;yes, setting of Xoff sent flag is determined
	mov	si,dartreg+dr5
	test	byte ptr [si],020h
	mov	si,statvec+7
	jz	rxon0e			;DTR set or reset?
	and	byte ptr [si],not 80h	;reset Xoff sent
	jmp	rxon0f
rxon0e:
	or	byte ptr [si],80h	;set Xoff
;	Clear buffer if in half duplex and reset RX timeout counter
rxon0f:
	mov	di,convec+1
	test	byte ptr [di],20h	;half duplex?
	jnz	rxon1			;jump if full duplex
	call	rsttx			;clear tx buffer
rxon1:					;clear RX buffer and pointers
	call	rsttc			;clear rx buffer
	call	rstp			;clear after translation buffer
	mov	di,convec+4		;reset time counter
	mov	al,[di] 		;time limit
	mov	di,statvec+5
	mov	[di],al 		;set status counter
;	Modify the table and enable the RX
rxon2:
	mov	si,dartreg+dr3
	or	byte ptr [si],10h
	mov	si,crdata
	mov	di,crport
	mov	dl,[di]
	xor	dh,dh
	mov	al,[si]
	or	al,01h			;enable RX
	and	al,not 02h
	mov	[si],al
;	cli
	out	dx,al
;	sti
;	mov	si,cmd_sts
;	mov	byte ptr [si],0f1h	;complete without value
	jmp	cretvp
	ret
rxon	endp
;
stoptx	proc
;	*****************************************************************
;	Routine disable TX, which should drop RTS if not in locked state,
;	unless this command is the terminator of another command in which
;	case the proper termination routine will be executed.
;	*****************************************************************
;	First check is this a stop TX command or the end of a previous command?
	mov	di,lastcom
	cmp	byte ptr [di],0
	jz	stoptx5 		;jmp if not end of command
	jmp	closec
stoptx5:
;
	mov	si,dartreg+dr5
	mov	di,lockstat
	test	byte ptr [di],20h	;RTS lock?
	jnz	stoptx1
	and	byte ptr [si],0afh
;	disable tx, drop rts
	call	rts_dwn
	jmp	stoptx2
stoptx1:
	and	byte ptr [si],0efh
stoptx2:
	mov	si,crdata
	mov	di,crport
	mov	dl,[di]
	xor	dh,dh
	mov	al,[si]
	or	al,80h			;disable TX
	and	al,not 04h
	mov	[si],al
;	cli
	out	dx,al
;	sti
;	mov	si,cmd_sts
;	mov	byte ptr [si],0f1h	;complete without value
	jmp	cretvp
	ret
stoptx	endp
;
starttx 	proc
;	************************************************************
;	Routine enables the TX, and sets RTS if not in locked state.
;	************************************************************
	mov	al,0
	call	setlastc
starttx2:
	mov	si,dartreg+dr5
	mov	di,lockstat
	test	byte ptr [di],20h
	jnz	starttx3		;RTS lock?
;	enable tx, raise rts
	or	byte ptr [si],50h
	call	rts_up
	jmp	starttx4
starttx3:
	or	byte ptr [si],10h	;TX enabled only
starttx4:
	mov	si,crdata
	mov	di,crport
	mov	dl,[di]
	xor	dh,dh
	mov	al,[si]
	or	al,40h			;enable TX
	and	al,not 08h
	mov	[si],al
;	cli
	out	dx,al
;	sti
;	mov	si,cmd_sts
;	mov	byte ptr [si],0f1h	;complete without value
	jmp	cretvp
	ret
starttx 	endp
;
input	proc
;	*******************************************
;	Routine translates data from MXF to OS/386.
;	*******************************************
	mov	al,19h
	call	setlastc
;	jmp	input2
input1:
;	Send byte to commom memory
input2:
	call	getrxp			;get a byte, in al
	jz	input4			;jp if end of buffer
;	look for End Of Record (EOR) delimiters if EOR detection is enabled
	test	al,80h			;test MSB, 7th bit
	jz	input3			;jump if not set
	mov	di,convec+5
	test	byte ptr [di],01h	;EOR bit set?
	jz	input3			;jump if not set
;	EOR is set, this is an EOR byte, reset bit 7 and decrease EOR count
	and	al,not 80h		;reset bit7 and send EOR out!
;	mov	si,cmd_parm
;	inc	byte ptr [si]		;increase counter
;	inc	si			;start of buffer
;	mov	byte ptr [si],al
	call	out386
;
;	Beware! check the buffer in case of over flow!
	mov	di,statvec+4		;point at EOR count
	cmp	byte ptr [di],0
	jz	input25 		;zero count? don't dec it
	dec	byte ptr [di]
input25:
	jmp	input4
input3:
	call	out386
;
;	Beware! check the buffer in case of over flow! No impletement here.
	jmp	input2
input4:
;	call	out386
	mov	di,gio_now2
	or	byte ptr [di],gio_ib9	;endi
	jmp	cretvp
inputc:
	mov	al,0
	call	setlastc
	jmp	cretvp
	ret
input	endp
;
prtpage 	proc
;	***********************************************************************
;	Routine passes back a page of memory to OS/386 for DEBUGGING?
;	Warning! Assumption: Users must proceed the $GIO command by a $OPEN and
;	a $BREAK to allow the first part of the command to be unbroken. check??
;	***********************************************************************
	mov	al,1eh
	call	setlastc
	jmp	prtpage2
prtpage1:				;restart entry
	cli
	mov	di,savepp
	mov	dx,[di] 		;get memory pointer
	add	di,2
	mov	si,[di] 		;get restart pointer
	add	di,2
	mov	cx,[di] 		;get count
	jmp	si			;jmp to restart point
;
prtpage2:
pp2_2:
;	call	obs04
	call	two386i 		;get a byte from 386
	jnz	pp2_4
	mov	si,offset pp2_2 	;restart point
	jmp	prtpg3_2
pp2_4:
	mov	dh,al			;first pointer
pp2_5:
;	call	obs04
	call	two386i 		;get a byte from OS
	jnz	pp2_6
	mov	si,offset pp2_5
	jmp	prtpg3_2
pp2_6:					;page pointer
	mov	dl,al			;dx page pointer???
	mov	si,dx
	mov	cl,0			;cl = one page
;	Send data until OS go away
pp2_7:
	push	si			;stored pointer
;	call	cpb04tc 		;wait for cpb (and look for lockup)
;	jnz	prtpg3
;	Send a byte, put into the buffer for OS/386
	mov	al,byte ptr [si]	;check?
	call	out386
	pop	si
	jz	prtpg3			;leave if not ready
	inc	si
	dec	cl
	jnz	pp2_7			;not
;
;	Last byte to send, send out endi
;	call	cpb04tc
	mov	di,gio_now2
	or	byte ptr [di],gio_ib9	;endi
;
	mov	al,0
	call	setlastc
	jmp	cretvp
;	ret
;
prtpg3:
	mov	dx,si			;store pointer
	mov	al,cl			;al got cl temporarily
	mov	si,offset pp2_7
prtpg3_2:				;save pointers for restart
	cli
	mov	di,savepp
	mov	[di],dx
	add	di,2
	mov	[di],si
	add	di,2
	mov	[di],al
	sti
	jmp	cretvp
;	Beware, a lock around this command must be done to
;	avoid the corruption of the data
	ret
prtpage 	endp
;
output	proc
;	********************************************************************
;	Routine opens the output channel and puts byte into the TX's buffer.
;	********************************************************************
	mov	ah,1ah
	call	setlastc
;	Clear Rx buffer if in half duplex
	mov	di,convec+1
	test	byte ptr [di],20h	;half duplex?
	jnz	output2 		;jump if full
	call	rsttc			;clear int RX buffer
	call	rstp			;clear translation RX buffer
;	jmp	output2 		;go get bytes
output1:
output2:
;
	mov	di,convec+4		;timeout value
	mov	al,[di] 		;get value in seconds
	mov	di,statvec+5
	mov	[di],al 		;set counter to Max value
;	check ready and reset at OS/386
	mov	di,rbi			;notice os
	test	byte ptr [di],prtrb
	jnz	output3 		;ready at 04?
;	need to reset ready bit at OS/386
	mov	si,rbibus
	mov	al,[si] 		;get system ready
	or	al,[di] 		;get port ready
	mov	si,rbimask
	and	al,[si] 		;mask off unwanted addresses
;	mov	si,rbireg
;	mov	[si],al 		;for rbireg
;
output3:				;get bytes as long as OS will send it
;	call	obs04			;wait for byte
	call	two386i
	jz	output4 		;go enable tx if no more byte
	call	puttx			;put into buffer (prt_buf)
	jmp	output2 		;try next
output4:
;	Getting bytes done, set TX enabled if no delay in effect currently
	mov	si,tranbyte
	test	byte ptr [si],txdelay
	jz	output5 		;delay, exit
;	ret
	jmp	cretvp
output5:
	call	entx
	jmp	starttx2		;check??
outputc:
;	terminate entry point
	mov	al,0
	call	setlastc
	jmp	cretvp
;
	ret
output	endp
;
outputh 	proc
;	*******************************************************
;	Routine does the line turnaround to occur at the end of
;	transmission in the half duplex mode.
;	*******************************************************
	mov	al,1bh
	call	setlastc
;	At the end of the command the turnaround bit must be set.
	jmp	output2
outputhc:				;termination entry point
	mov	al,0
	call	setlastc		;clear restart point
;	set turnaround flag
	mov	di,txflag
	or	byte ptr [di],taround
;
	jmp	cretvp
	ret
outputh 	endp
;
stsign	proc
;	*************************************************************
;	Routine sets the states of RTS and DTR, and also to set these
;	states to a locked condition if requested.
;	*************************************************************
	mov	al,1fh
	call	setlastc		;restart pointer
;
	mov	cl,3			;check??? 2 or 3
	xor	ch,ch
	mov	si,sgnbuf		;point at the signal buffer
	jmp	stsign2
stsign1:				;restart entry point
	xor	ch,ch
	mov	di,nxtcomp
	mov	cl,[di] 		;cl = # of bytes left to be read
	cmp	cl,0
	jz	stsignk 		;exit if all bytes gotten
	mov	di,nxtcom		;restart command pointer
	mov	si,[di] 		;dx holds the buffer pointer
stsign2:				;get and store byte
;
	push	si			;push counter
	call	two386i 		;get bytes until OS stop
	jz	stsign7
	pop	si
;	Compute restart values
	mov	[si],al 		;store the remaining bytes
	inc	si
	loop	stsign2
stsign7:
	jmp	cretvp
;
stsignk:
	pop	si
	mov	di,nxtcom
	mov	[di],si
	mov	di,nxtcomp
	mov	[di],cl
	jmp	cretvp
;
stsignc:				;termination entry point
	mov	al,0
	call	setlastc
	mov	di,dartreg+dr5
	cli
	mov	ah,[di]
	sti
	and	ah,60h			;need only current DTR/RTS
	mov	di,sgnbuf+1
	test	byte ptr [di],10h	;check DTR first
	jz	stsc02			;don't change, check RTS
	mov	si,sgnbuf
	test	byte ptr [si],10h	;RTS?
	jnz	stsc01
;	Reset DTR setting
	and	ah,not 20h
	call	dtr_dis
	jmp	stsc02
stsc01: 				;set DTR
	or	ah,20h
	call	dtr_enb
stsc02: 				;RTS's turn
	mov	di,sgnbuf+1
	test	byte ptr [di],20h
	jz	stsc04			;don't change, exit
	dec	di
	test	byte ptr [si],20h
	jnz	stsc03
	and	ah,not 40h		;reset RTS
	call	rts_dwn
	jmp	stsc04
stsc03:
	or	ah,40h			;set RTS
	call	rts_up
stsc04:
	mov	si,dartreg+dr5
	cli
	and	byte ptr [si],77h
	or	byte ptr [si],ah
;	mov	si,cmd_sts
;	mov	byte ptr [si],0f1h	;command completes without value
	jmp	cretvp
	ret
stsign	endp
;
dtr_dis 	proc
;	******************************************
;	Routine disables the dtr, and send to 2698
;	******************************************
	mov	al,0
	cli
	mov	dx,0100h
	mov	di,tc_who		;Note tc_who=1 the port1a, otherwise 1c
	cmp	es:byte ptr [di],1	;Is it port 1a? Note tc_who=1 then 1a,
	jz	dtrc_ys 		;jump if it is
	mov	dx,0110h		;Port 1c!
dtrc_ys:
	out	dx,al
	sti
	ret
dtr_dis 	endp
;
dtr_enb 	proc
;	*****************************************
;	Routine enables the dtr, and send to 2698
;	*****************************************
;
	mov	al,1
	cli
	mov	dx,010ah
	mov	di,tc_who		;Note tc_who=1 the port1a, =4/1c
	cmp	es:byte ptr [di],1	;Is it port 1a? Note tc_who=1 then 1a,
	jz	dtrc_yss		;jump if it is
	mov	dx,011eh		;Port 1c!
dtrc_yss:
	out	dx,al
	sti
	ret
dtr_enb 	endp
;
rts_dwn 	proc
;	*****************************************
;	Routine negates the rts, and send to 2698
;	*****************************************
	mov	si,crdata
	mov	di,crport
	mov	dl,[di]
	xor	dh,dh
	mov	al,[si]
	or	al,0fh			;keep the low nibble
	or	al,90h
	mov	[si],al
	cli
	out	dx,al
	sti
	ret
rts_dwn 	endp
;
rts_up		proc
;	*****************************************
;	Routine asserts the rts, and send to 2698
;	*****************************************
	mov	si,crdata
	mov	di,crport
	mov	dl,[di]
	xor	dh,dh
	mov	al,[si]
	or	al,0fh			;keep the low nibble
	or	al,80h
	mov	[si],al
	cli
	out	dx,al
	sti
	ret
rts_up		endp
;
;obs04	 proc
;	**********************************************************************
;	Routine get a byte from OS at 04 (command buffer, not real address 04)
;	If no byte being gotten in 5 tries or the OS switches the address or
;	deselects or sets cbs then an error return will be made.
;	Exit: al holds data or address is changed
;	Return: Z = no data, NZ = data fetched
;	**********************************************************************
;	 mov	 cx,4			 ;# of tries
;obs041:				 ;Do some talks with OS
;	 mov
;	 jnz	 obs043 		 ;jmp if byte is not ready
;obs0411:				 ;check obs is in 04
;
;	 jz	 obs045 		 ;go set error flag
;obs042:
;	 or	 si,0ffh		 ;set NZ flag
;					 ;get byte here!!!
;	 ret
;obs043:				 ;will MXF still select address 04?
;
;
;	 jnz	 obs044 		 ;check the obs leave if not 04
;	 loop	 obs041 		 ;try another byte
;obs044:				 ;error exit
;
;	 jz	 obs0411		 ;address at 06? Try one last time
;obs045:				 ;no byte
;	 xor	 si,si			 ;set Z flag
;	 ret
;obs04	 endp
;
;obs04r  proc
;	*************************************************************
;	Routine gets N bytes from OS at 04, Bytes are gotten until
;	either N bytes were gotten or OS stopped sending. N is passed
;	over in cl. si points at buffer location.
;	Exit: Some number bytes were gotten and stored at buffer. cl
;	      held the number of bytes read.
;	*************************************************************
;	 cli
;	 push	 ax
;	 push	 cx			 ;save initial cl (counter)
;obs04r1:
;	 push	 cx			 ;again
;	 call	 obs04
;	 push	 si
;	 call	 two386i
;	 pop	 si
;	 pop	 cx
;	 jz	 obs04r3		 ;jmp if byte gotten
;	 mov	 [si],al		 ;store the byte
;	 inc	 si
;	 loop	 obs04r1		 ;go try another byte
;obs04r3:				 ;end of routine, calculate bytes
;	 pop	 ax			 ;original count
;	 sub	 al,cl
;	 mov	 cl,al			 ;store in cl
;	 pop	 ax
;	 sti
;	 ret
;obs04r  endp
;
setlastc	proc
;	***********************************************************
;	Routine set restart pointer for this port using al as index
;	***********************************************************
	mov	di,lastcom		;restart index in lastcom
	mov	[di],al
	ret
setlastc	endp
;
;cbs04	 proc
;	*******************************************************************
;	Routine gets a byte from OS at 04, if no byte is gotten in 5 tries
;	or OS switches addresses or deselects or sets cbs then error return
;	Exit: byte in al, or address was changed
;	Return: Z = no byte gotten, NZ = byte gotten
;	*******************************************************************
;	 mov	 cl,4			 ;# times through loop til timeout
;cbs04	 endp
;
;cpb04tc proc
;	*********************************************************************
;	Routine wait for cpb to go ready at 04. If OS does not set cpb at 04
;	then routine will check for an input byte. If input byte is available
;	then lastcom will reset to zero to prevent lockout.
;	Return: NZ = cpb oaky, Z = error
;	*********************************************************************
;	 mov	 cl,5			 ;# tries if not select
;cpb04tc endp
;
out386	 proc
;	****************************************************
;	Routine handles the transmission of a byte to OS/386
;	which were stored in register ah.
;	****************************************************
;	%%% testing for disable GIO Dec/16
	mov	si,gio_over
	cmp	byte ptr [si],0ffh	;OS disable?
	jz	out386z 		;yes, jump out of GIO
;	%%% End %%%
;
;	Routine to check the OS's next input
	mov	di,gio_now1
	mov	al,[di]
	cmp	al,0
	jz	out386				;not yet
;
	test	al,gio_ibs
	jnz	out86vl
	test	al,gio_abs
	jnz	outabs				;new start
;
	mov	di,gio_now1
	mov	byte ptr [di],0 		;point at gio_now1
;
	jmp	out386
outabs:
	mov	di,gio_data
	mov	al,[di]
	mov	di,gio_now1
	mov	byte ptr [di],0 		;point at gio_now1
;
out386z:
	xor	al,al				;Z flag, not sent yet,
	ret					;new abs or disabled coming in
;	jmp	topabs
;
out86vl:			;Okay to send 386 a byte
	test	byte ptr [di],gio_cpb	;CPU ready?
	jnz	out86vl
;
	mov	si,gio_data		;Yes
	mov	[si],ah
	or	byte ptr [di],gio_cpb	;set busy
;
;	mov	di,intbyte
;	or	byte ptr [di],keyint
	or	al,0ffh 		;set NZ to note sent complete
	ret
;
out386	endp
;
closec	proc
;	************************************************
;	This block is used to select a code block to end
;	a command which requires termination
;	************************************************
	mov	di,lastcom
	mov	al,[di] 		;index of jump table
	sub	al,tc04difc
;
	xor	ah,ah
	shl	ax,1
	mov	si,ax
;
	jmp	cs:tctab04c[si]
	ret
closec	endp
;
