;
init	proc			;j+
;	***********************************************************************
;	Routine initializes all devices and memory to their starting (default)
;	status. Any data will be cleared, and devices are reset, baud rates
;	will be effect upon completion of this routine.
;	***********************************************************************
;
;	Init general storage area
	call	gen_init
;	Initialize the data segments for ports
	call	datasg_init
;	Initialize the I/O registers and usermask
	call	port_init
;	Init the registers of UART
	call	dart_init
;	Initialize each terminal (port)
	call	term_init
	ret
init	endp
;
gen_init	proc		;j+
;	*********************************************
;	Routine initializes the general storage area
;	*********************************************
;
	cli			;disable interrupt
;	Initialize the interrupt vector
	call	int_init
;	Initialize the memory of variables in general storage
	cld				;set direct flag
	mov	al,0
	mov	cx,gen_size
	mov	di,reset_byte	;starting point
gen_init1:
	stosb			;Store zeros to all the general storage
	loop	gen_init1
;
;	Move all the messages maskings to general store segment
	push	ds
	mov	ax,cs
	mov	ds,ax
	mov	cx,lengthl
	mov	si,offset ratetblx
	mov	di,ratetbl
	rep	movsb
;
	pop	ds
;
	mov	di,rbibus
	mov	byte ptr [di],30h
	mov	di,fairmask
	mov	byte ptr [di],0eeh
;
;	Masking for mapping in some address before 600p, June/10, 1991
	push	ds
	push	es
	cld				;set direct flag
	mov	ax,40h
	mov	es,ax
	mov	ax,cs
	mov	ds,ax
	mov	si,04h
	and	byte ptr es:[si],0fh	   ;high nibble = 1, RAM Mode
	or	byte ptr es:[si],10h
	mov	si,18h
	mov	word ptr es:[si],0001
;
;	masking june/11, 1991
;
	mov	di,30h
	mov	si,offset termbase
	mov	cx,22h
	rep	movsw
;
	pop	es
;
	pop	ds
;
	ret
gen_init	endp
;
; -----------------------------------------------------------------------
rstbuf		proc	;j+
;	************************************************
;	Routine that clear all buffer area, and pointers
;	to their original state.
;	************************************************
	cli				;disable int
	mov	di,txbyte
	and	byte ptr [di],72h	;txbyte/turn off crt, rsd bits
	nop				;For what? delay?
	nop
	mov	di,rbi
	and	byte ptr [di],0fbh		  ;get rid of rsd bit
	or	byte ptr [di],40h		  ;crt ready bit
	inc	di			;rbimask
	mov	byte ptr [di],0ffh	;rest rbimask so that all addresses
					;may go ready
	mov	di,outblock
	mov	byte ptr [di],0 	     ;reset crt blocker,
;	******************************************************************
;	Here were a set of instructions to reset printer buffer, but they
;	had been masked in the later version of MXE?
;	******************************************************************
	mov	di,crt_wrt
	mov	word ptr [di],0
	inc	di
	inc	di			     ;crt_rd
	mov	word ptr [di],0
	mov	di,crt_count
	mov	word ptr [di],0
	mov	di,comt_len
	mov	byte ptr [di],0
	inc	di			     ;comt_wrt
	mov	word ptr [di],0
	inc	di
	inc	di			     ;comt_rd
	mov	word ptr [di],0 	     ;commandt read pointer
;
	mov	di,dsrbyte
	mov	byte ptr [di],01h
	and	byte ptr [di],09fh	     ;reset rsdb and dtb
	mov	al,02h
	mov	di,rx_wrt
	mov	word ptr [di],0
	inc	di
	inc	di
	mov	word ptr [di],0 	;rx_rd
	mov	di,rx_count
	mov	byte ptr [di],0
	inc	di			;pstopcnt
	mov	byte ptr [di],0
	inc	di			;cstopcnt
	mov	byte ptr [di],0
	ret
rstbuf		endp
;********************************************************
rstbufk 	proc	;j+
;	Routine clear key buffer
	mov	di,txbyte
	and	byte ptr [di],not keybit       ;reset the key bit in txbyte
	mov	di,key_count
	mov	byte ptr [di],0
	mov	di,key_wrt
	mov	word ptr [di],0
	inc	di
	inc	di
	mov	word ptr [di],0 	;key_rd
	ret
rstbufk 	endp
;********************************************************
rstbufc 	proc			;j+
;	Routine clear crt buffer
	mov	di,txbyte
	and	byte ptr [di],not crtbit
	mov	di,crt_count
	mov	word ptr [di],0
	mov	di,crt_wrt
	mov	word ptr [di],0
	inc	di
	inc	di			;crt_rd
	mov	word ptr [di],0
	ret
rstbufc 	endp
;********************************************************
rstbufp 	proc			;j+
;	Routine clear prt buffer
	mov	di,txbyte
	and	byte ptr [di],not prtbit
	mov	di,prt_count
	mov	word ptr [di],0
	mov	di,prt_wrt
	mov	word ptr [di],0
	inc	di
	inc	di
	mov	word ptr [di],0 	;prt_rd
	ret
rstbufp 	endp
;********************************************************
rstbufdt	proc			;j+
;	Routine clear data transfer buffer
	mov	di,dt_count
	mov	byte ptr [di],0
	mov	di,dt_wrt
	mov	word ptr [di],0
	inc	di
	inc	di
	mov	word ptr [di],0 	;dt_rd
	ret
rstbufdt	endp
;********************************************************
rstbfrsd	proc			;j+
;	Routine clear remote screen down buffer
	mov	di,rsd_count
	mov	word ptr [di],0
rstrsd1:
	mov	di,rsd_wrt
	mov	word ptr [di],0
	inc	di
	inc	di
	mov	word ptr [di],0 	    ;rsd_rd
	inc	di
	inc	di
	mov	byte ptr [di],0 	    ;check? must or not
	ret
rstbfrsd	endp
;
;********************************************************
rstlr		proc			;j+
;	Routine reinitialize the LR buf to clear state
	mov	di,lrbyte
	and	byte ptr [di],dton
	mov	di,lr_count
	mov	word ptr [di],0
	inc	di
	inc	di
	mov	byte ptr [di],0 	;lrinit
	inc	di
	mov	byte ptr [di],0 	      ;lrstart = column 0
	inc	di
	mov	word ptr [di],0 	;lrstop
	inc	di
	inc	di
	mov	word ptr [di],0 	;lrwrp1
	inc	di
	inc	di
	mov	word ptr [di],0 	;lrwrp2
	inc	di
	inc	di
	mov	word ptr [di],0 	;lrwr
	inc	di
	inc	di
	mov	word ptr [di],0 	;lrwre
	inc	di
	inc	di
	mov	word ptr [di],0 	;lrcnt
	inc	di
	inc	di
	mov	word ptr [di],0ff00h	;Beginning of buffer
	inc	di
	inc	di
	mov	byte ptr [di],0 	;lrquery
					;Clear content of buffer to blank
	mov	bx,0
	mov	al,blank
	mov	cx,0200h
	mov	di,lr_buf
rstlr1:
	mov	[di][bx],al
	inc	bx
	loop	rstlr1
;
	mov	di,outblock
	and	byte ptr [di],not lrblock
	jnz	rstlrz			;Any other blocking output
	mov	di,rbimask
	or	byte ptr [di],crtrb
rstlrz:
	ret
rstlr		endp
;
rstterm 	proc			;j+
;	***************************************************
;	Routine reset a terminal crt -> 'esccode''termrst'
;	is sent to terminal to act the reset.
;	MXF: A string of bytes which would not affect the
;	terminal is put out to cause a waiting condition.
;	The reason is that terminal needs some time to get
;	out of a race condition after a reset.
;	***************************************************
	mov	al,clearsc
	call	putcrtg
rstterm0:
	mov	al,esccode
	call	putcrtg
	mov	al,crtrst
	call	putcrtg
rstterm1:
	mov	cx,005fh
rstterm2:
	mov	al,nopz
	call	putcrtg
	mov	al,home
	call	putcrtg
	loop	rstterm2
	ret
rstterm 	endp
;
;*********************************************************
moderst 	proc	;j+
;	Routine reset command mode flag and buffer area
	mov	ah,0
	mov	di,rbimask
	mov	byte ptr [di],crtrb
	inc	di
	inc	di		     ;position of flag2
	mov	[di],ah
	inc	di
	mov	[di],ah 	     ;clear the counter of modebyte
	mov	di,comt_len
	mov	[di],ah 	     ;command length -> 0
	inc	di		     ;point at comt_wrt
	xor	al,al
	mov	[di],ax
				     ;comt_wrt = word, commandt =byte
	mov	di,usermask
	mov	ax,[di]
	xor	ax,0ffffh	     ;invert user bits
	and	es:commode0,ax	     ;clear command mode flag
;
;	Allow input to crt_buf from CS386 if no other function is block it???
	mov	di,outblock
	and	byte ptr [di],not mxfblock
	jnz	moderstz
	mov	di,rbimask
	mov	byte ptr [di],0ffh
moderstz:
	ret
moderst 	endp
;
int_init	proc		;j+
;***********************************************************
;	This subroutine will initiate the interrupt vectors
;	that the mxf286 needs, which diferentiates from the
;	those by diagnostic purposes.
;***********************************************************
	cli
	cld
	push	es
	mov	ax,0
	mov	es,ax
	mov	di,60h*4
	mov	ax,offset int_vec60
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 60h
;				 which is interrupted from port 1a,1b
	mov	ax,offset int_vec61
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 61h, from 1c,1d
;
	mov	ax,offset int_vec62
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 62h, from 1e,1f
;
	mov	ax,offset int_vec63
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 63h, from 1g,1h
;
	mov	ax,offset int_vec64
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 64h, from 2a,2b
;
	mov	ax,offset int_vec65
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 65h, from 2c,2d
;
	mov	ax,offset int_vec66
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 66h, from 2e,2f
;
	mov	ax,offset int_vec67
	stosw
	mov	ax,01d60h
	stosw			;Set up interrupt vector 67h, from 2g,2h
;
	pop	es
	ret
int_init	endp
;
;***********************************************************
datasg_init	proc		;j+
	mov	ax,060h
	mov	es,ax
	mov	ds,ax
	cld
	mov	di,dataseg
	mov	ax,0160h		;port0
	stosw
;	mov	ax,0320h		;port1
	mov	ax,04e0h		;--- testing --- exchange 2,3
	stosw
;	mov	ax,04e0h		;port2
	mov	ax,0320h		;--- testing --- exchange 2,3
	stosw
	mov	ax,06a0h		;port3
	stosw
	mov	ax,0860h		;port4
	stosw
	mov	ax,0a20h		;port5
	stosw
	mov	ax,0be0h		;port6
	stosw
	mov	ax,0da0h		;port7
	stosw
	mov	ax,0f60h		;port8
	stosw
	mov	ax,1120h		;port9
	stosw
	mov	ax,12e0h		;porta
	stosw
	mov	ax,14a0h		;portb
	stosw
	mov	ax,1660h		;portc
	stosw
	mov	ax,1820h		;portd
	stosw
	mov	ax,19e0h		;porte
	stosw
	mov	ax,1ba0h		;portf
	stosw
	ret
datasg_init	endp
;
;	***************************
;	Initial all the channels.
;	***************************
dart_init	proc		;j+
;
;	Initialize the dart channels, ds=es=data
;
;	Disable all RX/TX, 28/nov/91
	mov	cx,10h
	mov	di,crport
	mov	bx,0
	mov	dx,0
d_init00:
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	mov	al,20h
	mov	dl,[di]
	out	dx,al
	mov	al,30h
	out	dx,al
	mov	al,1ah
	out	dx,al
	add	bx,2
	loop	d_init00
;	Disable all RX/TX, 28/nov/91, end
;
;	First, the channels 1a, 1b
	mov	cx,08h
	mov	bx,0		;clear index
d_initz1:
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	port_init_2
	call	port_init_1
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	port_init_1
	add	bx,2
	loop	d_initz1
;
;	Set up the timer interrupt of 0.1 second each
;	mov	dx,csr_1a	;only need last channel pair
;	mov	al,0ddh
;	out	dx,al		;turn on the timer interrupt
;	mov	dx,csr_1b
;	out	dx,al		;turn on the timer interrupt
;
;	mov	dx,imr_2gh	;only need last channel pair
;	mov	al,3bh
;	out	dx,al		;turn on the timer interrupt
;	mov	al,02dh 	;set 0.1 sec. interrupt
;	mov	dx,0ech 	;cturport out, only from block (ctur_2gh)
;	out	dx,al
;	mov	al,0		;low byte is zero
;	mov	dx,0eeh
;	out	dx,al
;
;	26/nov/91, start timer bit
;	mov	dx,start_1ab
;	in	ax,dx
;	26/nov/91, start timer bit, end
;
;	Set up the timer interrupt of 0.1 second each, end
;				 Enable all ports here.
;
;	Enable the RTS, DTR of port 1a, 1c 9/jan/92
	mov	dx,010ah
	mov	al,01
	out	dx,al		;enable DTR1
	mov	dx,011eh
	out	dx,al		;enable DTR2
	mov	dx,cr_1a
	mov	al,80h
	out	dx,al		;enable RTS1
	mov	dx,cr_1c
	out	dx,al		;enable RTS2
;	Enable the RTS, DTR of port 1a, 1c 9/jan/92, end
;
	mov	cx,10h
	mov	bx,0
	xor	dh,dh
;	mov	si,crdata
	mov	di,crport
d_initz2:
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	mov	al,011h
;	mov	byte ptr [si],011h
	mov	dl,[di]
	out	dx,al
	add	bx,2
	loop	d_initz2
;
;	26/nov/91, start timer bit
	mov	ax,19e0h
	mov	ds,ax
	mov	dx,acr_2gh
	mov	di,acrdata
	mov	al,[di]
	out	dx,al
;
	mov	dx,start_2gh
	in	ax,dx
;	26/nov/91, start timer bit, end
;
	ret
dart_init	endp
;
;	****************************************************
;	Routine for initialization of each port
;	Registers used: mr_1,mr_2,csr,cr,acr,imr,ctur,ctlr
;	****************************************************
port_init_1	proc		;j+ Registers for single port
	xor	dh,dh
	mov	al,07h
	mov	di,mrport
	mov	dl,[di]
	out	dx,al		;RxInt=RxRdy, 8 bit data, odd parity
	mov	al,07h		;Normal mode, 1 stop bit
	out	dx,al
;
;	Changes for the sake of the switch of 2nd and 3rd port
;
	mov	di,usermask
	mov	ax,[di]
	test	ax,06h		;port 2nd or 3rd?
	jz	baud_pas
	test	ax,02h		;is it port 1b?
	jz	baud_nps
	mov	dx,baud1cd	;read in the next dip switch
	in	al,dx
;;	shl	al,4		;shift to high nibble
	shr	al,4		;shift to low nibble
	jmp	baud_go
baud_nps:			;port 1c!
	test	ax,04h		;is it port 1c?
	jz	baud_pas	;neither 2 or 3, maybe error
	mov	dx,baud1ab	;read in the next dip switch
	in	al,dx
;;	shr	al,4		;shift to low nibble
	shl	al,4		;shift to high nibble
	jmp	baud_go
;	-------- End of switching of 3, 2 ports -------
baud_pas:
	mov	di,baudport
	mov	dx,[di]
	in	al,dx
baud_go:
	call	read_baud
;
;	For fixed and added baud rate 3/june, 1991
	push	ax
	mov	al,ah
	mov	di,acrport
	xor	dh,dh
	mov	dl,[di]
	mov	di,acrdata
	mov	ah,[di] 	;acr value
	cmp	al,80h
	jz	june03
;	1st set of baud rate
	and	al,ah
	jmp	june03a
june03: 			;2nd set
	or	al,ah
june03a:
	mov	[di],al 	;store back to acr value
	out	dx,al
	pop	ax
;	End bugging of 3/June, 1991
;
	mov	di,csrport
	mov	dl,[di] 	;csrport out
;	xor	dh,dh
	out	dx,al
	mov	al,1ah		;Return point to mr_1, disable Tx/Rx; They
				;should be enable as soon as initialization ends.
	inc	di		;crport out
	mov	dl,[di]
	out	dx,al
	ret
port_init_1	endp
;*******************************************************************************
port_init_2	proc		;j+ Registers for a pair of ports
	xor	dh,dh
	mov	al,0f0h 	;2nd set baud rate, clock /16, change-of-state?
	mov	di,acrport
	mov	dl,[di]
;	bugging 3/June, 1991
	mov	si,acrdata
	mov	[si],al
;	End bugging 3/June, 1991
	out	dx,al
;	mov	al,033h 	;Rx int, and C/T int, 27/nov/91
	mov	al,03bh 	;Rx int, and C/T int, 27/nov/91
	inc	di
	mov	dl,[di] 	;imrport out
	out	dx,al
	mov	al,02dh 	;set 0.1 sec. interrupt
	inc	di
	inc	di		;skip rthr
	mov	dl,[di] 	;cturport out
	out	dx,al
	mov	al,0		;low byte is zero.
	inc	di
	mov	dl,[di] 	;ctlrport out
	out	dx,al
	ret
port_init_2	endp
;
term_init	proc		;j+
;	****************************************************
;	Routine resets all terminals and associated buffers
;	****************************************************
;	Don't clear screen as error message may be on screen
;
	mov	cx,10h
	mov	bx,0		;start from 1st data segment(port)
;
term_set2:
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	push	cx		;save both counters, they might be destroyed.
	push	bx
;
;	Test the dtr over here
	in	al,ip_2ab
	mov	ah,al
	in	al,ip_2cd
	mov	di,connct
	mov	es:[di],ax		;record connection condition
	mov	di,usermask
	mov	bx,[di]
	test	ax,[di]
	jnz	term_z		;Disconnected! go away without reset terminal
;
	call	rstterm 	;puts out 'esccode' 'termrst' to terminal
	call	moderst 	;reset command pointers
term_z:
	pop	bx
	pop	cx
	add	bx,2
	mov	di,cmd_sts
	mov	byte ptr [di],0f0h
	mov	di,dsrbyte
	mov	byte ptr [di],01h
	mov	di,rbi
	mov	byte ptr [di],48h
	mov	di,rbimask
	mov	byte ptr [di],0ffh
	mov	di,txbyte
	or	byte ptr [di],prtstopb	;set prt stop in case no printer
;	For LR width, july/27/91
	mov	di,lwidth
	mov	byte ptr [di],linelen	;default width = 50h
	loop	term_set2
	ret
term_init	endp
;
port_init	proc			;j+
;	**********************************************************
;	Routine that sets the IO addresses and the user masks...
;	**********************************************************
	mov	bx,0		;start from zero data segment(port)
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_1a
	mov	[di],al
	inc	di
	mov	al,csr_1a
	mov	[di],al
	inc	di
	mov	al,cr_1a
	mov	[di],al
	inc	di
	mov	al,acr_1ab
	mov	[di],al
	inc	di
	mov	al,imr_1ab
	mov	[di],al
	inc	di
	mov	al,06h
	mov	[di],al
;
	inc	di
	mov	al,ctur_1ab
	mov	[di],al
	inc	di
	mov	al,ctlr_1ab
	mov	[di],al
;
	inc	di
	mov	ax,baud1ab
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0001h
	mov	[di],ax 	;point at usermask
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1a
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1ab
	mov	[di],al
	mov	di,dsrbyte
	mov	byte ptr [di],01h	;make port 0 current connected.

;	*************************      1st user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
	mov	di,mrport
	mov	al,mr_1b
	mov	[di],al
	inc	di
	mov	al,csr_1b
	mov	[di],al
	inc	di
	mov	al,cr_1b
	mov	[di],al
	inc	di
	mov	al,acr_1ab
	mov	[di],al
	inc	di
	mov	al,imr_1ab
	mov	[di],al
	inc	di
	mov	al,16h		;rhtr
	mov	[di],al
;
	inc	di
	mov	al,ctur_1ab
	mov	[di],al
	inc	di
	mov	al,ctlr_1ab
	mov	[di],al
;
	inc	di
	mov	ax,baud1ab
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0002h	;usermask
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1b
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1ab
	mov	[di],al
;
;	*************************      2nd users
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
	mov	di,mrport
	mov	al,mr_1c
	mov	[di],al
	inc	di
	mov	al,csr_1c
	mov	[di],al
	inc	di
	mov	al,cr_1c
	mov	[di],al
	inc	di
	mov	al,acr_1cd
	mov	[di],al
	inc	di
	mov	al,imr_1cd
	mov	[di],al
	inc	di
	mov	al,26h		;rhtr
	mov	[di],al
;
	inc	di
	mov	al,ctur_1cd
	mov	[di],al
	inc	di
	mov	al,ctlr_1cd
	mov	[di],al
;
	inc	di
	mov	ax,baud1cd
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0004h	;usermask
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1c
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1cd
	mov	[di],al
;
;	*************************      3rd user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_1d
	mov	[di],al
	inc	di
	mov	al,csr_1d
	mov	[di],al
	inc	di
	mov	al,cr_1d
	mov	[di],al
	inc	di
	mov	al,acr_1cd
	mov	[di],al
	inc	di
	mov	al,imr_1cd
	mov	[di],al
	inc	di
	mov	al,36h
	mov	[di],al
;
	inc	di
	mov	al,ctur_1cd
	mov	[di],al
	inc	di
	mov	al,ctlr_1cd
	mov	[di],al
;
	inc	di
	mov	ax,baud1cd
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0008h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1d
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1cd
	mov	[di],al

;	*************************      4th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_1e
	mov	[di],al
	inc	di
	mov	al,csr_1e
	mov	[di],al
	inc	di
	mov	al,cr_1e
	mov	[di],al
	inc	di
	mov	al,acr_1ef
	mov	[di],al
	inc	di
	mov	al,imr_1ef
	mov	[di],al
	inc	di
	mov	al,46h		;rhtr
	mov	[di],al
;
	inc	di
	mov	al,ctur_1ef
	mov	[di],al
	inc	di
	mov	al,ctlr_1ef
	mov	[di],al
;
	inc	di
	mov	ax,baud1ef
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0010h	;usermask
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1e
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1ef
	mov	[di],al

;	*************************      5th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_1f
	mov	[di],al
	inc	di
	mov	al,csr_1f
	mov	[di],al
	inc	di
	mov	al,cr_1f
	mov	[di],al
	inc	di
	mov	al,acr_1ef
	mov	[di],al
	inc	di
	mov	al,imr_1ef
	mov	[di],al
	inc	di
	mov	al,56h		;rhtr
	mov	[di],al
;
	inc	di
	mov	al,ctur_1ef
	mov	[di],al
	inc	di
	mov	al,ctlr_1ef
	mov	[di],al
;
	inc	di
	mov	ax,baud1ef
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0020h	;usermask
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1f
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1ef
	mov	[di],al

;	*************************      6th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_1g
	mov	[di],al
	inc	di
	mov	al,csr_1g
	mov	[di],al
	inc	di
	mov	al,cr_1g
	mov	[di],al
	inc	di
	mov	al,acr_1gh
	mov	[di],al
	inc	di
	mov	al,imr_1gh
	mov	[di],al
	inc	di
	mov	al,66h		;rhtr
	mov	[di],al
;
	inc	di
	mov	al,ctur_1gh
	mov	[di],al
	inc	di
	mov	al,ctlr_1gh
	mov	[di],al
;
	inc	di
	mov	ax,baud1gh
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0040h	;usermask
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1g
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1gh
	mov	[di],al

;	*************************      7th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_1h
	mov	[di],al
	inc	di
	mov	al,csr_1h
	mov	[di],al
	inc	di
	mov	al,cr_1h
	mov	[di],al
	inc	di
	mov	al,acr_1gh
	mov	[di],al
	inc	di
	mov	al,imr_1gh
	mov	[di],al
	inc	di
	mov	al,76h		;rhtr
	mov	[di],al
;
	inc	di
	mov	al,ctur_1gh
	mov	[di],al
	inc	di
	mov	al,ctlr_1gh
	mov	[di],al
;
	inc	di
	mov	ax,baud1gh
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0080h	;usermask
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_1h
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_1gh
	mov	[di],al

;	*************************      8th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2a
	mov	[di],al
	inc	di
	mov	al,csr_2a
	mov	[di],al
	inc	di
	mov	al,cr_2a
	mov	[di],al
	inc	di
	mov	al,acr_2ab
	mov	[di],al
	inc	di
	mov	al,imr_2ab
	mov	[di],al
	inc	di
	mov	al,86h
	mov	[di],al
;
	inc	di
	mov	al,ctur_2ab
	mov	[di],al
	inc	di
	mov	al,ctlr_2ab
	mov	[di],al
;
	inc	di
	mov	ax,baud2ab
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0100h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2a
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2ab
	mov	[di],al

;	*************************      9th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2b
	mov	[di],al
	inc	di
	mov	al,csr_2b
	mov	[di],al
	inc	di
	mov	al,cr_2b
	mov	[di],al
	inc	di
	mov	al,acr_2ab
	mov	[di],al
	inc	di
	mov	al,imr_2ab
	mov	[di],al
	inc	di
	mov	al,96h
	mov	[di],al
;
	inc	di
	mov	al,ctur_2ab
	mov	[di],al
	inc	di
	mov	al,ctlr_2ab
	mov	[di],al
;
	inc	di
	mov	ax,baud2ab
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0200h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2b
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2ab
	mov	[di],al

;	*************************      10th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2c
	mov	[di],al
	inc	di
	mov	al,csr_2c
	mov	[di],al
	inc	di
	mov	al,cr_2c
	mov	[di],al
	inc	di
	mov	al,acr_2cd
	mov	[di],al
	inc	di
	mov	al,imr_2cd
	mov	[di],al
	inc	di
	mov	al,0a6h
	mov	[di],al
;
	inc	di
	mov	al,ctur_2cd
	mov	[di],al
	inc	di
	mov	al,ctlr_2cd
	mov	[di],al
;
	inc	di
	mov	ax,baud2cd
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0400h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2c
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2cd
	mov	[di],al

;	*************************      11th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2d
	mov	[di],al
	inc	di
	mov	al,csr_2d
	mov	[di],al
	inc	di
	mov	al,cr_2d
	mov	[di],al
	inc	di
	mov	al,acr_2cd
	mov	[di],al
	inc	di
	mov	al,imr_2cd
	mov	[di],al
	inc	di
	mov	al,0b6h
	mov	[di],al
;
	inc	di
	mov	al,ctur_2cd
	mov	[di],al
	inc	di
	mov	al,ctlr_2cd
	mov	[di],al
;
	inc	di
	mov	ax,baud2cd
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,0800h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2d
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2cd
	mov	[di],al

;	*************************      12th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2e
	mov	[di],al
	inc	di
	mov	al,csr_2e
	mov	[di],al
	inc	di
	mov	al,cr_2e
	mov	[di],al
	inc	di
	mov	al,acr_2ef
	mov	[di],al
	inc	di
	mov	al,imr_2ef
	mov	[di],al
	inc	di
	mov	al,0c6h
	mov	[di],al
;
	inc	di
	mov	al,ctur_2ef
	mov	[di],al
	inc	di
	mov	al,ctlr_2ef
	mov	[di],al
;
	inc	di
	mov	ax,baud2ef
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,1000h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2e
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2ef
	mov	[di],al

;	*************************      13th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2f
	mov	[di],al
	inc	di
	mov	al,csr_2f
	mov	[di],al
	inc	di
	mov	al,cr_2f
	mov	[di],al
	inc	di
	mov	al,acr_2ef
	mov	[di],al
	inc	di
	mov	al,imr_2ef
	mov	[di],al
	inc	di
	mov	al,0d6h
	mov	[di],al
;
	inc	di
	mov	al,ctur_2ef
	mov	[di],al
	inc	di
	mov	al,ctlr_2ef
	mov	[di],al
;
	inc	di
	mov	ax,baud2ef
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,2000h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2f
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2ef
	mov	[di],al

;	*************************      14th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2g
	mov	[di],al
	inc	di
	mov	al,csr_2g
	mov	[di],al
	inc	di
	mov	al,cr_2g
	mov	[di],al
	inc	di
	mov	al,acr_2gh
	mov	[di],al
	inc	di
	mov	al,imr_2gh
	mov	[di],al
	inc	di
	mov	al,0e6h 	;rthr
	mov	[di],al
;
	inc	di
	mov	al,ctur_2gh
	mov	[di],al
	inc	di
	mov	al,ctlr_2gh
	mov	[di],al
;
	inc	di
	mov	ax,baud2gh
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,4000h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2g
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2gh
	mov	[di],al

;	*************************      15th user
	add	bx,2
	mov	ax,es:dataseg[bx]
	mov	ds,ax
	call	zero_out
;
	mov	di,mrport
	mov	al,mr_2h
	mov	[di],al
	inc	di
	mov	al,csr_2h
	mov	[di],al
	inc	di
	mov	al,cr_2h
	mov	[di],al
	inc	di
	mov	al,acr_2gh
	mov	[di],al
	inc	di
	mov	al,imr_2gh
	mov	[di],al
	inc	di
	mov	al,0f6h 	;rthr
	mov	[di],al
;
	inc	di
	mov	al,ctur_2gh
	mov	[di],al
	inc	di
	mov	al,ctlr_2gh
	mov	[di],al
;
	inc	di
	mov	ax,baud2gh
	mov	[di],ax
	inc	di
	inc	di
	mov	ax,8000h
	mov	[di],ax
;
	inc	di
	inc	di		;point at status register
	mov	al,sr_2h
	mov	[di],al
	inc	di		;point at interrupt status register
	mov	al,isr_2gh
	mov	[di],al
;
	ret
port_init	endp
;
zero_out	proc	;j+
	cld				;set direct flag
	push	es
	mov	dx,ds
	mov	es,dx
	mov	al,0
	mov	cx,total_bk
	mov	di,int_event	;starting point
zero_out1:
	stosb			;Store zeros to all the general storage
	loop	zero_out1
	pop	es
	ret
zero_out	endp
;
read_baud	proc
;	**************************************************
;	Routine reads in the baud rate set by dip switches
;	Entry: al holds the baud rate by H/W.
;	Exit : al holds the data sent to csrport
;	**************************************************
	push	bx
	ror	bx,2
	jc	rd_baudc	;carry, so go to check high nibble
;;	No carry, so they are 0,2,4,6..  ports, check low nibble
;	No carry, so they are 0,2,4,6..  ports, check high nibble
	and	al,0f0h
	shr	al,4

;;	and	al,0fh
;	call	fd_baud
;	mov	di,csr_baud
;	mov	[di],al
	jmp	rdz
rd_baudc:
;	Carry, so they are 1,3,5,7..  ports, check low nibble
	and	al,0fh
;
;;	and	al,0f0h
;;	shr	al,4
rdz:
	call	fd_baud
	mov	di,csr_baud
	mov	[di],al
	pop	bx
	ret
read_baud	endp
;
fd_baud proc
;	***********************************************
;	Find the acr value according to the input value
;	al will hold csr value, while ah holds baud
;	***********************************************
	mov	ah,80h
	cmp	al,01h
	jnz	fd_baud1
	mov	al,22h		;38.4k baud rate
;	mov	ah,80h
;
	mov	si,ratetbl
;
	jmp	fd_baud0
fd_baud1:
	cmp	al,02h
	jnz	fd_baud2
	mov	al,0cch 	;19.2k baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 7
;
	jmp	fd_baud0
fd_baud2:
	cmp	al,03h
	jnz	fd_baud2a
	mov	al,0bbh 	;9600 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 14
;
	jmp	fd_baud0
fd_baud2a:
	cmp	al,04h
	jnz	fd_baud3
	mov	al,0aah 	;7200 baud rate
	mov	ah,7fh
;
	mov	si,ratetbl  + 21
;
	jmp	fd_baud0
fd_baud3:
	cmp	al,05h
	jnz	fd_baud4
	mov	al,99h		;4800 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 28
;
	jmp	fd_baud0
fd_baud4:
	cmp	al,07h
	jnz	fd_baud4a
	mov	al,88h		;2400 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 35
;
	jmp	fd_baud0
fd_baud4a:
	cmp	al,08h
	jnz	fd_baud5
	mov	al,0aah 	 ;1800 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 42
;
	jmp	fd_baud0
fd_baud5:
	cmp	al,09h
	jnz	fd_baud6
	mov	al,66h		;1200 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 49
;
	jmp	fd_baud0
fd_baud6:
	cmp	al,0ah
	jnz	fd_baud7
	mov	al,55h		;600 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 56
;
	jmp	fd_baud0
fd_baud7:
	cmp	al,0bh
	jnz	fd_baud7a
	mov	al,44h		;300 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 63
;
	jmp	fd_baud0
fd_baud7a:
	cmp	al,0ch
	jnz	fd_baud7b
	mov	al,33h		;200 baud rate
	mov	ah,7fh
;
	mov	si,ratetbl  + 70
;
	jmp	fd_baud0
fd_baud7b:
	cmp	al,0dh
	jnz	fd_baud7c
	mov	al,33h		;150 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 77
;
	jmp	fd_baud0
fd_baud7c:
	cmp	al,0eh
	jnz	fd_baud7d
	mov	al,22h		;134.5 baud rate
	mov	ah,7fh
;
	mov	si,ratetbl  + 84
;
	jmp	fd_baud0
fd_baud7d:
	cmp	al,0fh
	jnz	fd_baud8
	mov	al,11h		;110 baud rate
;	mov	ah,80h
;
	mov	si,ratetbl  + 91
;
	jmp	fd_baud0
fd_baud8:
	mov	al,0cch 	;error, set baud rate as 19.2k
;
	mov	si,ratetbl  + 7
;
fd_baud0:
;
	mov	di,swbrate
	mov	dx,si
	mov	[di],dx
	add	di,2		;point at hwbrate
	mov	[di],dx
	ret
fd_baud endp
;
