
         PAGE      60,132
;---------------------------------------------------------------------------+
;                                                                           !
;  (c) Copyright, Wang Laboratories, Inc.,  1991                            !
;        All rights reserved.                                               !
;                                                                           !
;                                                                           !
;    Program name :  22C11/SCSI Power-up Diagnostic Program                 !
;    File name :     SCSIDIAG.ASM                                           !
;    Support group : Enginerring Service Department                         !
;                    Wang Computer (Taiwan) Ltd.                            !
;                    Taiwan, R.O.C.                                         !
;                                                                           !
;                                                                           !
;    Revision History :                                                     !
;      Date       Programmer       Description               Revision       !
;    08/15/91    Milton Chen        Creation                   5180         !
;                                                                           !
;                                                                           !
;    TEST FLOW :                                                            !
;                [ 1 ]  80286 CPU test                                      !
;                [ 2 ]  Test first 4K byte for Semaphore Area               !
;                [ 3 ]  SRAM data bus test                                  !
;                [ 4 ]  SRAM address line test                              !
;                [ 5 ]  SRAM write/read test                                !
;                [ 6 ]  HMAC 52C61 P-Channel test                           !
;                [ 7 ]  HMAC 52C61 A-Channel test                           !
;                [ 8 ]  PARITY test                                         !
;                [ 9 ]  SCSI 5390A Controller Test                          !
;                [ A ]  System interrupt Test                               !
;                [ B ]  82C542 Chip test under run-in test                  !
;                                                                           !
;---------------------------------------------------------------------------+

;CODE     SEGMENT                            ; @@ include DSK/OS del
;         ASSUME    CS:CODE,DS:CODE          ; @@


START:   CLI
         JMP       AFTER_COPY

COPYRIGHT          DB        0DH,0AH,'Copyright, Wang Laboratories, Inc., 1991'
                   DB        20H,20H,20H,20H,20H
REVISION           DB        'Rev 5180', 0DH,0AH,00H         ; R0 Version

LOPCUNT  DB        4 DUP(20H),'22C11-SCSI   Run-in test      '
         DB        '  LOOP COUNT: ',00H
ERRCUNT  DB        '     ERROR COUNT: ',00H

PATN     DW        55AAH                    ; memory test pattern
         DW        0AA55H
         DW        0000H

BOARDMS  DB        '22C11-SCSI  '           ; description, 12 chars
BOARDMSL EQU       $-BOARDMS

BOARDST  EQU       404H                     ; pass store 'F0'
BOARDID  EQU       405H                     ; 22C11-SCSI => 02 ID code

DESADDR  EQU       406H                     ; description store addr. 406H-415H
REV      EQU       416H                     ; 416-417 EPROM rev.

PARITY   EQU       41EH                     ; parity occur store 11H

INTBIT   EQU       41FH                     ; interrupt semaphore bit
RESULT   EQU       420H                     ; test status bit

TSCOM    EQU       42AH                     ; common memory test
DG       EQU       42BH                     ; for @DG2

LOPCNT   EQU       500H                     ; loop count
ERRCNT   EQU       502H                     ; error count

N1       EQU       505H                     ; LSB
N2       EQU       506H
N3       EQU       507H
N4       EQU       508H
N5       EQU       509H                     ; MSB


;==========================    PORT ADDR.     ===========================

CLPORT   EQU       8002H                    ; clear 386 interrupt port
INTPORT  EQU       8000H                    ; intrupt 386 port
SWID     EQU       1C00H                    ; DIP SW for diag. run in test
;DMAPORT  EQU       1000H                    ; DMA port
PNTPORT  EQU       1A00H                    ; print port
PARPORT  EQU       1E00H                    ; b0=0 : disable parity error interrupt
                                            ;   =1 : enable parity error interrupt
HMACINT  EQU       1E02H                    ; b0=0 : HMAC interrupt disable
                                            ;   =1 : HMAC interrupt enable
SCSINT   EQU       1E04H                    ; b0=0 : SCSI interrupt disable
                                            ;   =1 : SCSI interrupt enable
SCBK     EQU       1E06H                    ; b0=0 : SCSI RAM access bank 0, HMAC-B
                                            ;   =1 : SCSI RAM access bank 1, HMAC-B
CPUBK    EQU       1E08H                    ; b0=0 : CPU RAM access bank 0, HMAC-A & P
                                            ;   =1 : CPU RAM access bank 1, HMAC-A & P
LEDPORT  EQU       1E0AH                    ; b0=0 : LED on
                                            ;   =1 : LED off
ENPAR    EQU       1E0CH                    ; b0=0 : disable parity error check
                                            ;   =1 : enable parity error check

;----------------------    HMAC PORT ADDR.    ---------------------------

;HMCR     EQU       1200H                    ; contol reg.
HMSR     EQU       1H*2+HMCR                ; status reg.
HMREF    EQU       2H*2+HMCR                ; refresh timer

;HMACR    EQU       3H*2+HMCR                ; channel-A control reg.

;HMA_AR1  EQU       4H*2+HMCR                ; CH-A start address, LSB1
;HMA_AR2  EQU       5H*2+HMCR                ; middle
;HMA_AR3  EQU       6H*2+HMCR                ; MSB, addr. regs

;HMA_LR1  EQU       7H*2+HMCR                ; CH-A transfer length, LSB
;HMA_LR2  EQU       8H*2+HMCR                ; middle
;HMA_LR3  EQU       9H*2+HMCR                ; MSB, length regs

HMA_AC1  EQU       0AH*2+HMCR               ; addr. counter
HMA_AC2  EQU       0BH*2+HMCR               ;
HMA_AC3  EQU       0CH*2+HMCR               ;

HMA_LC   EQU       0DH*2+HMCR

HMTR     EQU       10H*2+HMCR
HMACTE   EQU       11H*2+HMCR
HMCHT    EQU       12H*2+HMCR               ; Timer/Processor Channel Rest Reg.

HMBCR    EQU       13H*2+HMCR               ; channel-B control reg.
HMB_AR1  EQU       14H*2+HMCR               ; CH-A start address, LSB1
HMB_AR2  EQU       15H*2+HMCR               ; middle
HMB_AR3  EQU       16H*2+HMCR               ; MSB, addr. regs

HMB_LR1  EQU       17H*2+HMCR               ; CH-B transfer length, LSB
HMB_LR2  EQU       18H*2+HMCR               ; middle
HMB_LR3  EQU       19H*2+HMCR               ; MSB, length regs

HMB_AC1  EQU       1AH*2+HMCR               ; addr. counter
HMB_AC2  EQU       1BH*2+HMCR               ;
HMB_AC3  EQU       1CH*2+HMCR               ;

HMB_LC   EQU       1DH*2+HMCR

HMPCH    EQU       2DH*2+HMCR               ; data,p-channel
HMP_1    EQU       2EH*2+HMCR               ; LSB
HMP_2    EQU       2FH*2+HMCR               ; Middle
HMP_3    EQU       30H*2+HMCR               ; MSB, CTRL

HM_TAS   EQU       31H*2+HMCR
HM_TAL   EQU       32H*2+HMCR
HM_TBS   EQU       33H*2+HMCR
HM_TBL   EQU       34H*2+HMCR
HMRAMSP  EQU       39H*2+HMCR               ; RAM speed control reg.
HM_SCAL  EQU       3AH*2+HMCR

SERPORT1 EQU       1600H
SERINT1  EQU       1602H                    ; interrupt enable regs. disable
SERMCA1  EQU       1608H                    ; MCR int Enable

SERPORT2 EQU       1800H
SERINT2  EQU       1802H                    ; interrupt enable regs. disable
SERMCA2  EQU       1808H                    ; MCR int Enable

;PRTRW    EQU       1A00H                    ; R/W port
;PRTST    EQU       1A02H                    ; read status
;PRTCRL   EQU       1A04H                    ; R/W control


;reg_BASE               EQU       1400H
;reg_XFER_CNT_LO        EQU       reg_base
;reg_XFER_CNT_HI        EQU       reg_base + 01H*2
;reg_FIFO               EQU       reg_base + 02H*2
;reg_CMD                EQU       reg_base + 03H*2
;reg_STATUS             EQU       reg_base + 04H*2
;reg_BUS_ID             EQU       reg_base + 04H*2
;reg_INTR               EQU       reg_base + 05H*2
;reg_TIMEOUT            EQU       reg_base + 05H*2
;reg_SEQ                EQU       reg_base + 06H*2
;reg_SYNC_XFER          EQU       reg_base + 06H*2
;reg_FIFOFLAG           EQU       reg_base + 07H*2
;reg_SYNC_OFF           EQU       reg_base + 07H*2
;reg_CONFIG1            EQU       reg_base + 08H*2
;reg_CCF                EQU       reg_base + 09H*2
;reg_TEST_MODE          EQU       reg_base + 0AH*2
;reg_CONFIG2            EQU       reg_base + 0BH*2


;----------------------   TEST 1   80286 CPU TEST   -----------------------+
;                                                                          |
;        CPU 80286 : Flag - SF,CF,ZF,AF,PF test                            |                                   |
;                    General Reg, & Segment test         error code: 01    |
;                                                                          |
;--------------------------------------------------------------------------+

AFTER_COPY:
         MOV       AL,0                     ; LED on
         MOV       DX,LEDPORT
         OUT       DX,AL

         MOV       AX,0
         MOV       ES,AX
         MOV       AL,ES:[DG]
         CMP       AL,80H                   ; DG2 set warm boot
         JNE       NORM
         JMP       INIVECT
NORM:
         CLI                                ; disable interrupt
         MOV       AH,0D5H                  ; set SF,CF,ZF,AF flags on
         SAHF                               ; store AH to flag reg.
         JNC       FATAL_ERROR              ; error if CF not set
         JNZ       FATAL_ERROR              ; error if ZF not set
         JNP       FATAL_ERROR              ; error if PF not set
         JNS       FATAL_ERROR              ; error if SF not set
         LAHF                               ; load flag to AH
         MOV       CL,5                     ; shift count
         SHR       AH,CL                    ; shift AF into CARRY bit
         JNC       FATAL_ERROR              ; error if no carry
         MOV       AL,40H                   ; set flag on
         SHL       AL,1                     ; set up for test
         JNO       FATAL_ERROR              ; error if overflow
         XOR       AH,AH                    ; AH=0
         SAHF
         JBE       FATAL_ERROR              ; error if CF on
         JS        FATAL_ERROR              ; error if SIGN flag set
         JP        FATAL_ERROR              ; error if EVEN Parity set
         LAHF                               ; load flag to AH
         MOV       CL,5                     ; shift count
         SHR       AH,CL
         JC        FATAL_ERROR              ; error if CF on
         SHL       AH,1                     ; check OF is clear
         JO        FATAL_ERROR              ; error if OF on

         XOR       AX,AX                    ; test reset condition & flags
         JNZ       FATAL_ERROR              ; check zero flag off
         OR        AX,1
         JZ        FATAL_ERROR              ; check zero flag on
         AND       AX,AX
         JC        FATAL_ERROR              ; test carry, error if carry flag not cleared
         STC                                ; set carry flag
         JNC       FATAL_ERROR              ; error if not carry
         CLC                                ; clear carry flag
         JC        FATAL_ERROR              ; error if carry flag is not clear
         CMC                                ; complement carry flag
         JNC       FATAL_ERROR              ; error if no carry

         MOV       AX,1                     ; store 1 to EAX Reg.
         MOV       CX,7                     ; count times
LP1:     AND       AX,AX                    ; set flag
         JZ        FATAL_ERROR              ; error if ZERO flag set
         JS        FATAL_ERROR              ; error if SIGN flag set
         JPE       FATAL_ERROR              ; error if EVEN Parity, b0 - b7
         JC        FATAL_ERROR              ; error if CARRY
         RCL       AX,1
         LOOP      LP1


;---------------- R/W GENERAL AND SEGMENT REGS. ---------------


         MOV       AX,55AAH                 ; AX <= FFFFH, pattern
         STC                                ; set carry flag
LP2:     MOV       DS,AX                    ; write to all regs.
         MOV       BX,DS
         MOV       ES,BX
         MOV       CX,ES
         MOV       SS,CX
         MOV       DX,SS
         MOV       SP,DX
         MOV       BP,SP
         MOV       SI,BP
         MOV       DI,SI
         JNC       LP3
         XOR       AX,DI
         JNZ       FATAL_ERROR
         CLC
         JMP       LP2

LP3:
         OR        AX,DI                    ; zero pattern thru all regs.
         JZ        TST1


FATAL_ERROR:
         MOV       AL,21H                   ; error code=01
         MOV       BP,01H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
         JMP       $





;---------------------  TEST 2  4K byte For Semaphore  ----------------------+
;                                                                            |
;      Seg 0 : 400-403  => 386 identify 286 interface card. 'CRAD'           |
;          0 : 421,422 => Seg.                                               |
;          0 : 420 => 20H -- error, 21-2A                                    |
;                     40H -- run-in mode, pass                               |
;                     80H -- pass                                            |
;                     60H -- run-in mode, printer not ready                  |
;          0 : 41F => 11H -- 386 interrupt 286 flag '11'                     |
;                                                                            |
;          BEGIN                                                             |
;                FOR  J = 3  ( four patterns : 55AA,AA55,0000H )             |
;                      WRITE memory flood 2K words                           |
;                      READ  verify content data                             |
;                NEXT J   ( next pattern )                                   |
;          END                                            error code=02      |
;                                                                            |
;----------------------------------------------------------------------------+


TST1:    MOV       AX,CS                    ; DS <= CS
         MOV       DS,AX

         MOV       DX,0                     ; ES <= 0
         MOV       ES,DX

         LEA       BX,PATN                  ; test pattern
         MOV       SI,3                     ; three pattern
NXTPN1:  XOR       DI,DI
         CLD
         MOV       CX,800H                  ; 2k words
         MOV       AX,[BX]
   REP   STOSW

         XOR       DI,DI
         MOV       CX,800H
RDATA1:
         MOV       AX,ES:[DI]
         XOR       AX,[BX]
         JZ        NXTWORD1
         JMP       ERROR

NXTWORD1:
         INC       DI
         INC       DI
         LOOP      RDATA1

         INC       BX
         INC       BX
         DEC       SI                       ; next pattern
         JNZ       NXTPN1

         JMP       INIVECT

ERROR:
         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                  ; DIP switch all off
         JE        RUN1

         MOV       AL,22H
         MOV       BP,02H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         JMP       $

RUN1:    MOV       AL,22H
         MOV       BP,02H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H


;-----------------------------------------------------------

INIVECT:

         MOV       AL,10H                   ; printer  82C454
         MOV       DX,PRTCRL
         OUT       DX,AL

         MOV       DX,PRTCRL                ;
         MOV       AL,08H
         OUT       DX,AL

         MOV       AL,0CH                   ;
         OUT       DX,AL

         XOR       SI,SI
         MOV       DS,SI

         MOV       AL,02H                   ; ID: 2 22C11-SCSI
         MOV       BYTE PTR DS:[BOARDID],AL
         MOV       BYTE PTR DS:[REV],1      ; REV 01.00
         MOV       BYTE PTR DS:[REV+1],0

         MOV       ES,SI                    ; store description
         MOV       DI,DESADDR               ; ES:DI <= DS:SI
         LEA       SI,BOARDMS
         MOV       AX,CS
         MOV       DS,AX
         MOV       CL,BOARDMSL
         REP     MOVSB

         XOR       SI,SI
         MOV       DS,SI

         MOV       CX,256
         MOV       BX,OFFSET NULRT
LP:      MOV       WORD PTR DS:[SI], BX
         ADD       SI,2
         MOV       WORD PTR DS:[SI],0F000H
         ADD       SI,2
         ADD       BX,5
         LOOP      LP

         JMP       TST4

NULRT:   MOV       AL,0
         JMP       HLTPTR
         MOV       AL,1
         JMP       HLTPTR
         MOV       AL,2
         JMP       HLTPTR
         MOV       AL,3
         JMP       HLTPTR
         MOV       AL,4
         JMP       HLTPTR
         MOV       AL,5
         JMP       HLTPTR
         MOV       AL,6
         JMP       HLTPTR
         MOV       AL,7
         JMP       HLTPTR
         MOV       AL,8
         JMP       HLTPTR
         MOV       AL,9
         JMP       HLTPTR
         MOV       AL,10
         JMP       HLTPTR
         MOV       AL,11
         JMP       HLTPTR
         MOV       AL,12
         JMP       HLTPTR
         MOV       AL,13
         JMP       HLTPTR
         MOV       AL,14
         JMP       HLTPTR
         MOV       AL,15
         JMP       HLTPTR
         MOV       AL,16
         JMP       HLTPTR
         MOV       AL,17
         JMP       HLTPTR
         MOV       AL,18
         JMP       HLTPTR
         MOV       AL,19
         JMP       HLTPTR
         MOV       AL,20
         JMP       HLTPTR
         MOV       AL,21
         JMP       HLTPTR
         MOV       AL,22
         JMP       HLTPTR
         MOV       AL,23
         JMP       HLTPTR
         MOV       AL,24
         JMP       HLTPTR
         MOV       AL,25
         JMP       HLTPTR
         MOV       AL,26
         JMP       HLTPTR
         MOV       AL,27
         JMP       HLTPTR
         MOV       AL,28
         JMP       HLTPTR
         MOV       AL,29
         JMP       HLTPTR
         MOV       AL,30
         JMP       HLTPTR
         MOV       AL,31
         JMP       HLTPTR
         MOV       AL,32
         JMP       HLTPTR
         MOV       AL,33
         JMP       HLTPTR
         MOV       AL,34
         JMP       HLTPTR
         MOV       AL,35
         JMP       HLTPTR
         MOV       AL,36
         JMP       HLTPTR
         MOV       AL,37
         JMP       HLTPTR
         MOV       AL,38
         JMP       HLTPTR
         MOV       AL,39
         JMP       HLTPTR
         MOV       AL,30
         JMP       HLTPTR
         MOV       AL,41
         JMP       HLTPTR
         MOV       AL,42
         JMP       HLTPTR
         MOV       AL,43
         JMP       HLTPTR
         MOV       AL,44
         JMP       HLTPTR
         MOV       AL,45
         JMP       HLTPTR
         MOV       AL,46
         JMP       HLTPTR
         MOV       AL,47
         JMP       HLTPTR
         MOV       AL,48
         JMP       HLTPTR
         MOV       AL,49
         JMP       HLTPTR
         MOV       AL,50
         JMP       HLTPTR
         MOV       AL,51
         JMP       HLTPTR
         MOV       AL,52
         JMP       HLTPTR
         MOV       AL,53
         JMP       HLTPTR
         MOV       AL,54
         JMP       HLTPTR
         MOV       AL,55
         JMP       HLTPTR
         MOV       AL,56
         JMP       HLTPTR
         MOV       AL,57
         JMP       HLTPTR
         MOV       AL,58
         JMP       HLTPTR
         MOV       AL,59
         JMP       HLTPTR
         MOV       AL,60
         JMP       HLTPTR
         MOV       AL,61
         JMP       HLTPTR
         MOV       AL,62
         JMP       HLTPTR
         MOV       AL,63
         JMP       HLTPTR
         MOV       AL,64
         JMP       HLTPTR
         MOV       AL,65
         JMP       HLTPTR
         MOV       AL,66
         JMP       HLTPTR
         MOV       AL,67
         JMP       HLTPTR
         MOV       AL,68
         JMP       HLTPTR
         MOV       AL,69
         JMP       HLTPTR
         MOV       AL,70
         JMP       HLTPTR
         MOV       AL,71
         JMP       HLTPTR
         MOV       AL,72
         JMP       HLTPTR
         MOV       AL,73
         JMP       HLTPTR
         MOV       AL,74
         JMP       HLTPTR
         MOV       AL,75
         JMP       HLTPTR
         MOV       AL,76
         JMP       HLTPTR
         MOV       AL,77
         JMP       HLTPTR
         MOV       AL,78
         JMP       HLTPTR
         MOV       AL,79
         JMP       HLTPTR
         MOV       AL,80
         JMP       HLTPTR
         MOV       AL,81
         JMP       HLTPTR
         MOV       AL,82
         JMP       HLTPTR
         MOV       AL,83
         JMP       HLTPTR
         MOV       AL,84
         JMP       HLTPTR
         MOV       AL,85
         JMP       HLTPTR
         MOV       AL,86
         JMP       HLTPTR
         MOV       AL,87
         JMP       HLTPTR
         MOV       AL,88
         JMP       HLTPTR
         MOV       AL,89
         JMP       HLTPTR
         MOV       AL,90
         JMP       HLTPTR
         MOV       AL,91
         JMP       HLTPTR
         MOV       AL,92
         JMP       HLTPTR
         MOV       AL,93
         JMP       HLTPTR
         MOV       AL,94
         JMP       HLTPTR
         MOV       AL,95
         JMP       HLTPTR
         MOV       AL,96
         JMP       HLTPTR
         MOV       AL,97
         JMP       HLTPTR
         MOV       AL,98
         JMP       HLTPTR
         MOV       AL,99
         JMP       HLTPTR
         MOV       AL,100
         JMP       HLTPTR
         MOV       AL,101
         JMP       HLTPTR
         MOV       AL,102
         JMP       HLTPTR
         MOV       AL,103
         JMP       HLTPTR
         MOV       AL,104
         JMP       HLTPTR
         MOV       AL,105
         JMP       HLTPTR
         MOV       AL,106
         JMP       HLTPTR
         MOV       AL,107
         JMP       HLTPTR
         MOV       AL,108
         JMP       HLTPTR
         MOV       AL,109
         JMP       HLTPTR
         MOV       AL,110
         JMP       HLTPTR
         MOV       AL,111
         JMP       HLTPTR
         MOV       AL,112
         JMP       HLTPTR
         MOV       AL,113
         JMP       HLTPTR
         MOV       AL,114
         JMP       HLTPTR
         MOV       AL,115
         JMP       HLTPTR
         MOV       AL,116
         JMP       HLTPTR
         MOV       AL,117
         JMP       HLTPTR
         MOV       AL,118
         JMP       HLTPTR
         MOV       AL,119
         JMP       HLTPTR
         MOV       AL,120
         JMP       HLTPTR
         MOV       AL,121
         JMP       HLTPTR
         MOV       AL,122
         JMP       HLTPTR
         MOV       AL,123
         JMP       HLTPTR
         MOV       AL,124
         JMP       HLTPTR
         MOV       AL,125
         JMP       HLTPTR
         MOV       AL,126
         JMP       HLTPTR
         MOV       AL,127
         JMP       HLTPTR
         MOV       AL,128
         JMP       HLTPTR
         MOV       AL,129
         JMP       HLTPTR
         MOV       AL,130
         JMP       HLTPTR
         MOV       AL,131
         JMP       HLTPTR
         MOV       AL,132
         JMP       HLTPTR
         MOV       AL,133
         JMP       HLTPTR
         MOV       AL,134
         JMP       HLTPTR
         MOV       AL,135
         JMP       HLTPTR
         MOV       AL,136
         JMP       HLTPTR
         MOV       AL,137
         JMP       HLTPTR
         MOV       AL,138
         JMP       HLTPTR
         MOV       AL,139
         JMP       HLTPTR
         MOV       AL,140
         JMP       HLTPTR
         MOV       AL,141
         JMP       HLTPTR
         MOV       AL,142
         JMP       HLTPTR
         MOV       AL,143
         JMP       HLTPTR
         MOV       AL,144
         JMP       HLTPTR
         MOV       AL,145
         JMP       HLTPTR
         MOV       AL,146
         JMP       HLTPTR
         MOV       AL,147
         JMP       HLTPTR
         MOV       AL,148
         JMP       HLTPTR
         MOV       AL,149
         JMP       HLTPTR
         MOV       AL,150
         JMP       HLTPTR
         MOV       AL,151
         JMP       HLTPTR
         MOV       AL,152
         JMP       HLTPTR
         MOV       AL,153
         JMP       HLTPTR
         MOV       AL,154
         JMP       HLTPTR
         MOV       AL,155
         JMP       HLTPTR
         MOV       AL,156
         JMP       HLTPTR
         MOV       AL,157
         JMP       HLTPTR
         MOV       AL,158
         JMP       HLTPTR
         MOV       AL,159
         JMP       HLTPTR
         MOV       AL,160
         JMP       HLTPTR
         MOV       AL,170
         JMP       HLTPTR
         MOV       AL,171
         JMP       HLTPTR
         MOV       AL,172
         JMP       HLTPTR
         MOV       AL,173
         JMP       HLTPTR
         MOV       AL,174
         JMP       HLTPTR
         MOV       AL,175
         JMP       HLTPTR
         MOV       AL,176
         JMP       HLTPTR
         MOV       AL,177
         JMP       HLTPTR
         MOV       AL,178
         JMP       HLTPTR
         MOV       AL,179
         JMP       HLTPTR
         MOV       AL,180
         JMP       HLTPTR
         MOV       AL,181
         JMP       HLTPTR
         MOV       AL,182
         JMP       HLTPTR
         MOV       AL,183
         JMP       HLTPTR
         MOV       AL,184
         JMP       HLTPTR
         MOV       AL,185
         JMP       HLTPTR
         MOV       AL,186
         JMP       HLTPTR
         MOV       AL,187
         JMP       HLTPTR
         MOV       AL,188
         JMP       HLTPTR
         MOV       AL,189
         JMP       HLTPTR
         MOV       AL,190
         JMP       HLTPTR
         MOV       AL,191
         JMP       HLTPTR
         MOV       AL,192
         JMP       HLTPTR
         MOV       AL,193
         JMP       HLTPTR
         MOV       AL,194
         JMP       HLTPTR
         MOV       AL,195
         JMP       HLTPTR
         MOV       AL,196
         JMP       HLTPTR
         MOV       AL,197
         JMP       HLTPTR
         MOV       AL,198
         JMP       HLTPTR
         MOV       AL,199
         JMP       HLTPTR
         MOV       AL,200
         JMP       HLTPTR
         MOV       AL,201
         JMP       HLTPTR
         MOV       AL,202
         JMP       HLTPTR
         MOV       AL,203
         JMP       HLTPTR
         MOV       AL,204
         JMP       HLTPTR
         MOV       AL,205
         JMP       HLTPTR
         MOV       AL,206
         JMP       HLTPTR
         MOV       AL,207
         JMP       HLTPTR
         MOV       AL,208
         JMP       HLTPTR
         MOV       AL,209
         JMP       HLTPTR
         MOV       AL,210
         JMP       HLTPTR
         MOV       AL,211
         JMP       HLTPTR
         MOV       AL,212
         JMP       HLTPTR
         MOV       AL,213
         JMP       HLTPTR
         MOV       AL,214
         JMP       HLTPTR
         MOV       AL,215
         JMP       HLTPTR
         MOV       AL,216
         JMP       HLTPTR
         MOV       AL,217
         JMP       HLTPTR
         MOV       AL,218
         JMP       HLTPTR
         MOV       AL,219
         JMP       HLTPTR
         MOV       AL,220
         JMP       HLTPTR
         MOV       AL,221
         JMP       HLTPTR
         MOV       AL,222
         JMP       HLTPTR
         MOV       AL,223
         JMP       HLTPTR
         MOV       AL,224
         JMP       HLTPTR
         MOV       AL,225
         JMP       HLTPTR
         MOV       AL,226
         JMP       HLTPTR
         MOV       AL,227
         JMP       HLTPTR
         MOV       AL,228
         JMP       HLTPTR
         MOV       AL,229
         JMP       HLTPTR
         MOV       AL,230
         JMP       HLTPTR
         MOV       AL,231
         JMP       HLTPTR
         MOV       AL,232
         JMP       HLTPTR
         MOV       AL,233
         JMP       HLTPTR
         MOV       AL,234
         JMP       HLTPTR
         MOV       AL,235
         JMP       HLTPTR
         MOV       AL,236
         JMP       HLTPTR
         MOV       AL,237
         JMP       HLTPTR
         MOV       AL,238
         JMP       HLTPTR
         MOV       AL,239
         JMP       HLTPTR
         MOV       AL,230
         JMP       HLTPTR
         MOV       AL,231
         JMP       HLTPTR
         MOV       AL,232
         JMP       HLTPTR
         MOV       AL,233
         JMP       HLTPTR
         MOV       AL,234
         JMP       HLTPTR
         MOV       AL,235
         JMP       HLTPTR
         MOV       AL,236
         JMP       HLTPTR
         MOV       AL,237
         JMP       HLTPTR
         MOV       AL,238
         JMP       HLTPTR
         MOV       AL,239
         JMP       HLTPTR
         MOV       AL,240
         JMP       HLTPTR
         MOV       AL,241
         JMP       HLTPTR
         MOV       AL,242
         JMP       HLTPTR
         MOV       AL,243
         JMP       HLTPTR
         MOV       AL,244
         JMP       HLTPTR
         MOV       AL,245
         JMP       HLTPTR
         MOV       AL,246
         JMP       HLTPTR
         MOV       AL,247
         JMP       HLTPTR
         MOV       AL,248
         JMP       HLTPTR
         MOV       AL,249
         JMP       HLTPTR
         MOV       AL,250
         JMP       HLTPTR
         MOV       AL,251
         JMP       HLTPTR
         MOV       AL,252
         JMP       HLTPTR
         MOV       AL,253
         JMP       HLTPTR
         MOV       AL,254
         JMP       HLTPTR
         MOV       AL,255
         JMP       HLTPTR

HLTPTR: JMP     $


;------------------  TEST 3  SRAM DATA BUS TEST ---------------------------+
;                                                                          |
;       BEGIN                                                              |
;          FOR DX:= Memory size seg.                                       |
;            FOR I:= 2 ( two pattern : 0000-8000, FFFF - 7FFF )            |
;               FOR SI:= 0001                                              |
;                  FOR CX:= 16 times                                       |
;                     BX: current test pattern, AX: next test pattern      |
;                     XCHG DS:SI,test pattern                              |
;                  NEXT CX  ( next pattern )                               |
;               NEXT SI ( next address )                                   |
;            NEXT I                                                        |
;          NEXT DX ( next 64k bank )                                       |
;       END.                                           error code = 03     |
;                                                                          |
;--------------------------------------------------------------------------+


TST4:
         MOV       SI,SI                    ; start address = 0:0001h
         MOV       DI,100H

T4L1:    MOV       AX,DI                    ; start test from the first bank
         MOV       DS,AX

         MOV       BX,0000H                 ; BX = current test pattern
         MOV       AX,0001H                 ; AX = next test pattern
         MOV       DX,AX                    ; DX = save next test pattern
         MOV       CX,16                    ; CX = pattern counter (0000h - 8000h) = 15
         MOV       DS:[SI],BX               ; write current pattern to current address
T4L2:
         XCHG      DS:[SI],AX               ; read current pattern/write next pattern from/to
         CMP       AX,BX                    ; current address then verify the pattern
         JNE       ERROR4
         MOV       BX,DX                    ; update BX (current pattern) = next pattern
         SHL       DX,1                     ; upgrade the next pattern
         MOV       AX,DX                    ; upgrade AX (next pattern)
         DEC       CX                       ; decrease the pattern counter
         JNZ       T4L2                     ; loop back to test next pattern

         ADD       DI,100H                  ; point to next bank
         CMP       DI,4000H                 ; if 64K memory be tested
         JNZ       T4L1

T3_3:
         XOR       SI,SI                    ; start address = 0:0001h
         MOV       DI,100H

T4L4:    MOV       AX,DI                    ; start test from the first bank
         MOV       DS,AX

         MOV       BX,0FFFFH                ; BX = current test pattern
         MOV       AX,0FFFEH                ; AX = next test pattern
         MOV       DX,AX                    ; DX = save next test pattern
         MOV       CX,16                    ; CX = pattern counter (0FFFFH - 7FFFh) = 15
         MOV       [SI],BX                  ; write current pattern to current address

T4L5:
         XCHG      [SI],AX                  ; read current pattern/write next pattern from/to
         CMP       AX,BX                    ; current address then verify the pattern
         JNE       ERROR4
         MOV       BX,DX                    ; update BX (current pattern) = next pattern
         SHL       DX,1                     ; upgrade the next pattern
         MOV       AX,DX                    ; upgrade AX (next pattern)
         DEC       CX                       ; decrease the pattern counter
         JNZ       T4L5                     ; loop back to test next pattern

         ADD       DI,100H                  ; point to next bank
         CMP       DI,4000H                 ; if 64K memory be tested
         JNZ       T4L4

         JMP       TST5                     ; TST5

ERROR4:

         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                     ; DIP switch all off
         JE        RUN2

         MOV       AL,23H                   ; error code=03
         MOV       BP,03H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
         JMP       $

RUN2:
         MOV       AL,23H
         MOV       BP,03H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H


;----------    TEST 4  SRAM Invalid Memory Address Line (IMA) Test  ---------+
;                                                                            |
;       BEGIN                                                                |
;            FOR DX:= 4K bank                                                |
;               Flood background data '55' to bank                           |
;               write a data '3C' at address ( 0100:003C )                   |
;               FOR  J = 11 ( 100:01, 0:0002, 0004, 0008 .. 400 )            |
;                   read/verify content of current address = '55h'           |
;                   if not then error                                        |
;               NEXT J ( next address )                                      |
;            NEXT DX ( next 4k )                                             |
;       END                                               error code = 04    |
;                                                                            |
;----------------------------------------------------------------------------+


TST5:    MOV       DX,100H
NBK:     MOV       ES,DX
         XOR       DI,DI

         MOV       AL,55H                   ; setup the first pattern = 55h
         MOV       CX,1000H                 ; will test 4kb memory
         CLD
         REP       STOSB                    ; fill memory 0000:0000 - ffff with 55h

         MOV       DI,3CH
         MOV       BL,0AAH
         MOV       ES:[DI],BL               ; fill location 0100:0001 with AAH

         MOV       DI,0001H                 ; point to next location for Ao detect
         MOV       CX,11
LP4:
         MOV       AL,ES:[DI]               ; read data to detect A0 address first
         CMP       AL,55H                   ; if not equal then error occurs
         JNZ       ERROR5
         SHL       DI,1                     ; detect next address line
         LOOP      LP4

         ADD       DX,100H
         CMP       DX,4000H
         JL        NBK

         JMP       TST6

ERROR5:
         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                     ; DIP switch all off
         JE        RUN3

         MOV       AL,24H                  ; error code=04
         MOV       BP,04H                  ; store 0:0E => 24H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
         JMP       $

RUN3:
         MOV       AL,24H
         MOV       BP,04H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H


;--------------------   TEST 5 SRAM WRITE/READ TEST   -----------------------+
;                                                                            |
;          BEGIN                                                             |
;             FOR  I = 6  ( six banks, bank = 64k bytes )                    |
;                FOR  J = 7  ( seven patterns )                              |
;                      WRITE memory flood 32K words                          |
;                      READ  verify content data                             |
;                NEXT J   ( next pattern )                                   |
;             NEXT I  ( next bank  0:0000 - 3:FFFF )                         |
;          END                                            error code=05      |
;                                                                            |
;                                                                            |
;----------------------------------------------------------------------------+


TST6:    PUSH      CS                       ; DS <= CS
         POP       DS

         MOV       DX,100H                  ; 0100:0000
NTBK:    MOV       ES,DX

         MOV       BX,OFFSET PATN
         MOV       SI,3

NXPN:    XOR       DI,DI
         CLD
         MOV       CX,800H
         MOV       AX,DS:[BX]
     REP       STOSW

         XOR       DI,DI
         MOV       CX,800H
RDA:     MOV       AX,ES:[DI]
         XOR       AX,DS:[BX]
         JZ        NEXTWD
         JMP       ERROR6

NEXTWD:  INC       DI
         INC       DI
         LOOP      RDA

         INC       BX
         INC       BX
         DEC       SI
         JNZ       NXPN

         ADD       DX,100H                  ; next 4K
         CMP       DX,4000H                 ; SP <= Seg.
         JL        NTBK

         JMP       INISTAK

ERROR6:
         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                  ; DIP switch all off
         JE        RUN4

         MOV       AL,25H                  ; error code=05
         MOV       BP,05H                  ; store 0:0E => 25H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
         JMP       $

RUN4:
         MOV       AL,25H
         MOV       BP,05H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H


;--------------  Initial Stack & Set up interrupt vector  -----------------


INISTAK:
         MOV       AX,0H                    ;  stack segment: SS:SP => 0:550
         MOV       SS,AX
         MOV       AX,550H
         MOV       SP,AX

         MOV       AX,0                     ; point to RAM to set up interrupt vector
         MOV       DS,AX

         MOV       AX,CS

         MOV       BX,40H*4                 ;  286 to 386 interrupt vector
         MOV       WORD PTR [BX],OFFSET H286_INT
         MOV       [BX+2],AX

         MOV       BX,30H*4                 ; parity error interrupt vector
         MOV       WORD PTR [BX],OFFSET PARITYERR
         MOV       [BX+2],AX

         MOV       BX,31H*4                 ; HMAC interrupt vector
         MOV       WORD PTR [BX],OFFSET HMACSR
         MOV       [BX+2],AX

         MOV       BX,32H*4                 ; SCSI interrupt vector
         MOV       WORD PTR [BX],OFFSET SCSISR
         MOV       [BX+2],AX

         MOV       BX,33H*4                 ; Serial1 Interrupt vector
         MOV       WORD PTR [BX],OFFSET SER1
         MOV       [BX+2],AX

         MOV       BX,34H*4                 ; Serial2 Interrupt vector
         MOV       WORD PTR [BX],OFFSET SER2
         MOV       [BX+2],AX

         MOV       BX,35H*4                 ; printer interrupt vector
         MOV       WORD PTR [BX],OFFSET PRTSR
         MOV       [BX+2],AX



;-------------------   Chip 52C61 HMAC Initial Procedure   -----------------+
;                                                                           |
;    High-performance Memory Array Controller Initial :                     |
;                                                                           |
;                1. Config Control Reg.                                     |
;                2. RAM speed Reg.                                          |
;                3. Refresh Timer                                           |
;                4. Timer/Processor Reg.                                    |
;                5. Channel Addr. and Length reg.                           |
;                                                                           |
;---------------------------------------------------------------------------+


         MOV       DS,AX                    ; initial data segment: DS <= CS
         MOV       ES,AX                    ; initial extra segment: ES <= CS

         MOV       DX,HMCR                  ; contol regs.
         MOV       AL,30H                   ; b7:0 non fly-by mode, b6:0 parity through disable
         OUT       DX,AL                    ; b5,b4:1 enable, b3:0, disable SCSI & HMAC parity
                                            ; b2,b1:0

         MOV       DX,HMREF                 ; refresh timer
         MOV       AL,91                    ; 500*24*8/1024-2=91
         OUT       DX,AL

         MOV       DX,HMCHT                 ; timer/processor test reg.
         MOV       AL,10H                   ; b4:1 processor enable
         OUT       DX,AL                    ; b3:0 odd

         MOV       DX,HMRAMSP               ; RAM speed control
         MOV       AL,01H                   ; b0=1, 24MHz, 85ns
         OUT       DX,AL


;--------------------   TEST 9  PARITY TEST    --------------------------+
;                                                                        |
;   Test Procedure:                                                      |
;        1. enable parity error check                                    |
;        2. P-channel write even parity.                                 |
;        3. A-channel read odd parity.                                   |
;        4. int 30 occur                                                 |
;        5  INTSR 30 : disable parity , mark flag and enable parity.     |
;                                                                        |
;                                                       error code: 06   |
;------------------------------------------------------------------------+


         MOV       AL,0
         MOV       DX,SERINT1               ; disable serial port 0 INT regs.
         OUT       DX,AL

         MOV       DX,SERINT2               ; disable serial port 1 INT regs.
         OUT       DX,AL

         MOV       AL,08H                   ; MCR b3=1, INT enable, otherwise
         MOV       DX,SERMCA1               ; signal floating, int occur
         OUT       DX,AL

         MOV       DX,SERMCA2               ; MCR b3=1
         OUT       DX,AL

         XOR       SI,SI
         MOV       ES,SI

         STI

         MOV       AL,1H                    ; enable parity error check
         MOV       DX,ENPAR
         OUT       DX,AL

         MOV       DX,HMCHT                 ; timer/processor test reg.
         MOV       AL,18H                   ; b4:1 processor enable, b3:1
         OUT       DX,AL                    ; b3:1 even parity

         MOV       AL,0
         MOV       DX,CPUBK                 ; CPU access bank0
         OUT       DX,AL

         MOV       AL,0                     ; start address 0
         MOV       DX,HMP_1
         OUT       DX,AL
         MOV       DX,HMP_2
         OUT       DX,AL

         MOV       AL,00100000B             ; addr.0, b5=1 auto inc, b4=0 WRITE
         MOV       DX,HMP_3
         OUT       DX,AL

         MOV       DX,HMPCH                 ; WRITE 2 BYTES
         MOV       AL,5AH
         MOV       CL,2
         REP       OUTSB

         MOV       AL,0                     ; start address 0
         MOV       DX,HMA_AR1
         OUT       DX,AL
         MOV       DX,HMA_AR2
         OUT       DX,AL
         MOV       DX,HMA_AR3
         OUT       DX,AL

         MOV       AL,2                     ;
         MOV       DX,HMA_LR1
         OUT       DX,AL

         MOV       AL,0H                    ;
         MOV       DX,HMA_LR2
         OUT       DX,AL

         MOV       DX,HMA_LR3
         OUT       DX,AL

         MOV       DX,HMACR                 ; b7:1, addr.& length counter to be
         MOV       AL,10000100B             ;
         OUT       DX,AL                    ; b2:1, addr counter to be copied to the addr regs


         MOV       DX,DMAPORT
         IN        AL,DX                    ; INT 30 occurence
         JMP       $+2
         JMP       $+2
         JMP       $+2
         JMP       $+2

;        MOV       AL,BYTE PTR ES:[PARITY]  ;for some problem, we remove parity
;        CMP       AL,08H                   ;testing
;        JNE       PARERR

;--------------------------------------------------------------

         MOV       AL,1                     ; interrupt enable
         MOV       DX,HMACINT
         OUT       DX,AL

         MOV       AL,0                     ; start address 0
         MOV       DX,HMA_AR1
         OUT       DX,AL
         MOV       DX,HMA_AR2
         OUT       DX,AL
         MOV       DX,HMA_AR3
         OUT       DX,AL

         MOV       AL,1                     ; length 1 bytes
         MOV       DX,HMA_LR1
         OUT       DX,AL

         MOV       AL,0
         MOV       DX,HMA_LR2
         OUT       DX,AL
         MOV       DX,HMA_LR3
         OUT       DX,AL

         MOV       DX,HMACR                 ; b7:1, addr.& length counter to be
         MOV       AL,10001100B             ;       loaded from their respective regs.
         OUT       DX,AL                    ; b6:0, addr counter to be copied to the addr regs
                                            ; b5:0, length counter NOT to be copied to the addr regs.
                                            ; b4:0, CTR Hold
                                            ; b3:1, write ; b2:1, enable
                                            ; b1:0, disable, b0:0 interrupt disable

         MOV       DX,DMAPORT               ; WRITE
         OUT       DX,AL




                                            ; READ 1M VERIFY
         MOV       AL,0                     ; start address 0
         MOV       DX,HMP_1
         OUT       DX,AL
         MOV       DX,HMP_2
         OUT       DX,AL
         MOV       DX,HMP_3
         MOV       AL,30H                   ; b4:1 read, b5:1 inc
         OUT       DX,AL

         MOV       DX,HMPCH                 ; dummy
         OUT       DX,AL                    ; dummy op
         JMP       $+2
         IN        AL,DX                    ; read data
         JMP       $+2

         JMP       TESTPCH

PARERR:

         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                     ; DIP switch all off
         JE        RUN5

         MOV       AL,26H                  ; error code=06
         MOV       BP,06H                  ; store 0:0E => 26H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
BK1:
         CALL      BLINK
         JMP       BK1

RUN5:
         MOV       AL,26H
         MOV       BP,06H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H



;------------------   TEST 7  HMAC  P-Channel TEST   ---------------------+
;                                                                         |
;                                                                         |
;     P Channel test : Bank0 1M write/read verify  DI:0                   |
;                      Bank1 1M write/read verify  DI:1                   |
;                                                                         |
;         FOR DI = 0 TO 1  two banks                                      |
;             FOR SI = 1 TO 16  ( 64K*16 = 1M )                           |
;                 FOR  J = 256  LOOP     64k                              |
;                    FOR I = 256 BYTES                                    |
;                      Write FF, FE,....., 01, 00 ---256 bytes            |
;                    NEXT I                                               |
;                 NEXT J                                                  |
;                                                                         |
;                 FOR  K = 256  LOOP                                      |
;                    FOR L = 256 BYTES                                    |
;                      Read FF, FE,....., 01, 00 ---256 bytes             |
;                    NEXT L                                               |
;                 NEXT K                                                  |
;             NEXT SI                                                     |
;         NEXT DI  ( next bank = 1M )                                     |
;                                                  error code : 07        |
;                                                                         |
;-------------------------------------------------------------------------+

TESTPCH: CLI
         XOR       DI,DI                    ; ##
         MOV       AL,0
         MOV       DX,CPUBK                 ; CPU access bank0
         OUT       DX,AL

PBK1:    MOV       AL,0                     ; start address 0
         MOV       DX,HMP_1
         OUT       DX,AL
         MOV       DX,HMP_2
         OUT       DX,AL

         MOV       AL,00100000B             ; addr.0, b5=1 auto inc, b4=0 WRITE
         MOV       DX,HMP_3
         OUT       DX,AL
                                            ; WRITE 125K
         MOV       DX,HMPCH                 ; P-CH data port
         MOV       SI,2                     ; write 4 * 64k
W64KP:   MOV       CX,256                   ; 256 loop
         MOV       AL,0FFH
W256P:   PUSH      CX                       ; 256 bytes
         MOV       CX,256
LL1:     OUT       DX,AL                    ; write 1M, 256*256
         DEC       AL
         LOOP      LL1
         POP       CX
         LOOP      W256P
         DEC       SI
         JNZ       W64KP


         MOV       DX,HMP_3
WATP:    IN        AL,DX                    ; buffer transfer complete ?
         TEST      AL,80H
         JZ        WATP

                                            ; READ 1M VERIFY
         MOV       AL,0                     ; start address 0
         MOV       DX,HMP_1
         OUT       DX,AL
         MOV       DX,HMP_2
         OUT       DX,AL
         MOV       DX,HMP_3
         MOV       AL,30H                   ; b4:1 read, b5:1 inc
         OUT       DX,AL


         MOV       SI,2                     ; read 128K
R64KP:   MOV       CX,256                   ; 256 loop, 64k
R256P:   PUSH      CX                       ; 256 bytes
         MOV       BL,0FFH
         MOV       CX,256
         MOV       DX,HMPCH                 ; dummy
LL2:     OUT       DX,AL                    ; dummy op
         JMP       $+2
         IN        AL,DX                    ; read data
         CMP       AL,BL
         JNE       PERR
         DEC       BL
         LOOP      LL2
         POP       CX
         LOOP      R256P
         DEC       SI
         JNZ       R64KP                    ; next 64K
         CMP       DI,01H                   ; next bank
         JE        TESTA
         INC       DI
         MOV       AL,01H                   ; bank1
         MOV       DX,CPUBK
         OUT       DX,AL
         JMP       PBK1
PERR:
         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                  ; DIP switch all off
         JE        RUN6

         MOV       AL,27H                   ; error code=07
         MOV       BP,07H                   ; store 0:0E => 27H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
BK2:
         CALL      BLINK
         JMP       BK2

RUN6:
         MOV       AL,27H
         MOV       BP,07H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H


;------------------   TEST 8  HMAC  A-Channel TEST   ---------------------+
;                                                                         |
;                                                                         |
;     A Channel test : Bank0  0-512k write/read verify  DI:0  BP:0        |
;                      Bank0 512k-1M write/read verify  DI:1  BP:0        |
;                      Bank1  0-512K write/read verify  DI:0  BP:1        |
;                      Bank1 512k-1M write/read verify  DI:1  BP:1        |
;                                                                         |
;       FOR BP=0 TO 1 two banks                                           |
;         FOR DI = 0 TO 1  two 512k block * 2 = 1M                        |
;             FOR SI = 1 TO 8  ( 64K*8 = 512k)                            |
;                 FOR  J = 256  LOOP     64k                              |
;                    FOR I = 256 BYTES                                    |
;                      Write 00, 01,....., FE, FF ---256 bytes            |
;                    NEXT I                                               |
;                 NEXT J                                                  |
;             NEXT SI                                                     |
;                                                                         |
;             FOR SI = 1 TO 8  ( 64K*8 = 512K )                           |
;                 FOR  K = 256  LOOP                                      |
;                    FOR L = 256 BYTES                                    |
;                      Read 00, 01,....., FE, FF ---256 bytes             |
;                    NEXT L                                               |
;                 NEXT K                                                  |
;             NEXT SI                                                     |
;         NEXT DI  ( next bank )                                          |
;                                                  error code : 08        |
;                                                                         |
;-------------------------------------------------------------------------+


TESTA:   MOV       AL,0
         MOV       DX,CPUBK                 ; CPU access bank0
         OUT       DX,AL

         XOR       BP,BP                    ; bank 0
PBKA1:   XOR       DI,DI                    ; upper half bank
         MOV       AL,0                     ; start address 0
         MOV       DX,HMA_AR1
         OUT       DX,AL
         MOV       DX,HMA_AR2
         OUT       DX,AL
         MOV       DX,HMA_AR3
         OUT       DX,AL

         MOV       AL,0                     ; length 512K, 08 00 00, bytes
         MOV       DX,HMA_LR1
         OUT       DX,AL
         MOV       DX,HMA_LR2
         OUT       DX,AL
         MOV       AL,08H                   ; 0-512k
         MOV       DX,HMA_LR3
         OUT       DX,AL

         MOV       DX,HMACR                 ; b7:1, addr.& length counter to be
         MOV       AL,10001100B             ;       loaded from their respective regs.
         OUT       DX,AL                    ; b6:0, addr counter to be copied to the addr regs
                                            ; b5:0, length counter NOT to be copied to the addr regs.
                                            ; b4:0, CTR Hold
                                            ; b3:1, write ; b2:1, enable
                                            ; b1:0, disable, b0:0 interrupt disable

STAD:    MOV       DX,DMAPORT
         MOV       SI,8                     ; write 8 * 64k = 512k
W64KA:   MOV       CX,256                   ; 256 loop
         MOV       AL,00
W256A:   PUSH      CX                       ; 256 bytes
         MOV       CX,256
LL3:     OUT       DX,AL
         INC       AL
         LOOP      LL3
         POP       CX
         LOOP      W256A
         DEC       SI
         JNZ       W64KA

         MOV       DX,HMACR
WAT:     IN        AL,DX
         TEST      AL,10H                   ; read b4:1 block transfer complete
         JZ        WAT

         MOV       DX,HMACR                 ; b7:1, addr.& length counter to be
         MOV       AL,10000100B             ;       loaded from their respective regs.
         OUT       DX,AL                    ; b6:1, addr counter to be copied to the addr regs


         MOV       SI,8                     ; read 8 * 64k
         MOV       DX,DMAPORT
R64KA:   MOV       CX,256                   ; 256 loop
R256A:   PUSH      CX                       ; 256 bytes
         MOV       BL,00
         MOV       CX,256
LL4:     IN        AL,DX
         CMP       AL,BL
         JNE       PERR1
         INC       BL
         LOOP      LL4
         POP       CX
         LOOP      R256A
         DEC       SI
         JNZ       R64KA
         CMP       DI,01H
         JE        TESTBK1
         INC       DI
         MOV       DX,HMACR
         MOV       AL,0CCH                  ; copy to start addr.
         OUT       DX,AL
         JMP       $+2
         JMP       STAD                    ; test lower bank di=1
TESTBK1: CMP       BP,01H
         JE        TESTSCSI
         INC       BP
         MOV       AL,01H                   ; select BANK 1
         MOV       DX,CPUBK
         OUT       DX,AL
         JMP       PBKA1

PERR1:
         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                  ; DIP switch all off
         JE        RUN7

         MOV       AL,28H                   ; error code=08
         MOV       BP,08H                   ; store 0:0E => 28H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
BK3:
         CALL      BLINK
         JMP       BK3

RUN7:
         MOV       AL,28H
         MOV       BP,08H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H



;------------------   TEST A  53C90A SCSI CHIP TEST  -----------------------+
;                                                                           |
;      Config. reg  self-test                   error code : 09             |
;      FIFO write/read test                                                 |
;                                                                           |
;                                                                           |
;---------------------------------------------------------------------------+

TESTSCSI:
         MOV       AL,02H
         MOV       DX,reg_CMD
         OUT       DX,AL

         MOV       AL,00H
         MOV       DX,reg_CMD
         OUT       DX,AL


         XOR       BP,BP
CR2:     MOV       ES,BP
         LEA       BX,PATN
         MOV       SI,3
NXB:     MOV       AL,BYTE PTR [BX]
         MOV       DX,reg_CONFIG1
         OUT       DX,AL                    ; write to control regs.1
         MOV       CL,BYTE PTR [BX]
         IN        AL,DX                    ; read verify
         CMP       AL,CL
         JNE       SCSIERR
         ADD       BX,2                     ; point next pattern
         DEC       SI
         JNZ       NXB
         CMP       BP,01H
         JE        SC1
         INC       BP
         MOV       DX,reg_CONFIG2           ; test CR2
         JMP       CR2


SC1:     MOV       AL,00H                   ; test 16 byte-FIFO
         MOV       DX,reg_FIFO
NXFIFO:  OUT       DX,AL                    ; write
         INC       AL
         CMP       AL,0FH                   ; 16 byte
         JNE       NXFIFO

         MOV       DX,reg_FIFOFLAG
         IN        AL,DX                    ; read
         CMP       AL,0FH                   ; equal 16 byte FIFO
         JE        FIFOR
         JMP       SCSIERR


FIFOR:   MOV       BL,00H                   ;
         MOV       DX,reg_FIFO
NXRFO:   IN        AL,DX
         CMP       AL,BL
         JNE       SCSIERR
         INC       BL
         CMP       BL,0FH
         JNE       NXRFO
         JMP       SC2

SCSIERR:
         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                     ; DIP switch all off
         JE        RUN8

         MOV       AL,29H                  ; error code=09
         MOV       BP,09H                  ; store 0:0E => 29H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL
BK4:
         CALL      BLINK
         JMP       BK4

RUN8:
         MOV       AL,29H
         MOV       BP,09H
         XOR       SI,SI
         MOV       ES,SI
         MOV       ES:[RESULT],AL

         ADD       WORD PTR ES:[ERRCNT],0001H



;---------------------    TEST B  PRINTER TEST    ----------------------------+
;                                                                             |
;       This test is executed under Run-in test, DIP Switch setting OFF(00)   |
;                                                                             |
;-----------------------------------------------------------------------------+

SC2:
         MOV       DX,SWID                  ; sence dip switch
         IN        AL,DX
         CMP       AL,0FFH                     ; DIP switch all off
         JE        RUNIN
         JMP       DSK

RUNIN:   MOV       AL,10H
         MOV       DX,PRTCRL
         OUT       DX,AL

         MOV       DX,PRTCRL                ; initial
         MOV       AL,08H
         OUT       DX,AL

         MOV       AL,0CH                   ; initial
         OUT       DX,AL

         MOV       CX,0
         MOV       DX,PRTST
NOREADY: IN        AL,DX
         TEST      AL,40H
         JNZ       PRT
         AAM
         AAM
         AAM
         LOOP      NOREADY

         MOV       SI,0
         MOV       ES,SI
         MOV       AH,60H
         MOV       ES:[RESULT],AH           ; printer not rea

PRT:
         XOR       SI,SI
         MOV       ES,SI

         MOV       BX,OFFSET COPYRIGHT
         CALL      PRTSR

         MOV       BX,OFFSET LOPCUNT
         CALL      PRTSR

         MOV       AX,WORD PTR ES:[LOPCNT]
         CALL      PRTCNT

         MOV       AL,BYTE PTR ES:[N5]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N4]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N3]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N2]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N1]
         CALL      PRTCH

         MOV       CX,5
         MOV       AL,0
         MOV       DI,505H
         REP      STOSB

         MOV       BX,OFFSET ERRCUNT
         CALL      PRTSR

         MOV       AX,WORD PTR ES:[ERRCNT]
         CALL      PRTCNT

         MOV       AL,BYTE PTR ES:[N5]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N4]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N3]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N2]
         CALL      PRTCH
         MOV       AL,BYTE PTR ES:[N1]
         CALL      PRTCH

         MOV       AL,0DH                   ; CR,LF
         CALL      PRTCH
         MOV       AL,0AH
         CALL      PRTCH

         MOV       SI,0
         MOV       ES,SI
         MOV       AH,40H                   ; run-in pass '40'
         MOV       ES:[RESULT],AH

         ADD       WORD PTR ES:[LOPCNT],0001H

NOPT:                                       ; LED       OFF,LEDPORT
         MOV       AL,1
         MOV       DX,LEDPORT
         OUT       DX,AL

         MOV       DX,5                     ; delay time
WTM:     XOR       CX,CX
WATIME:  AAM
         AAM
         AAM
         AAM
         LOOP      WATIME
         DEC       DX
         JNZ       WTM

         CLI

         MOV       AL,0                     ; LED on
         MOV       DX,LEDPORT
         OUT       DX,AL

         JMP       TST4

;-----------------   Interrup Service Routine   ------------------

H286_INT:                                   ; 386 int 286 service routine

         PUSH      ES
         PUSH      SI
         PUSH      DX
         PUSH      AX

         XOR       SI,SI
         MOV       ES,SI
         MOV       AL,11H
         MOV       ES:[INTBIT],AL           ; store 0:0E meaning int occur

         MOV       AL,0                     ; clear 386 int
         MOV       DX,CLPORT
         OUT       DX,AL

         MOV       AX,8000H                 ; int 386
         MOV       DX,INTPORT
         OUT       DX,AX

         POP       AX
         POP       DX
         POP       SI
         POP       ES

         IRET

;-------------------------------------------------------------------------

PARITYERR:                                  ; INT30SR
         MOV       AX,0                     ; disable
         MOV       DX,ENPAR
         OUT       DX,AX

         MOV       AL,08H                   ; '11' to indicate int occur
         MOV       BYTE PTR ES:[PARITY],AL
         MOV       AL,01H                   ; enable parity error check
         OUT       DX,AL

         IRET

;-----------------------------------------------------------------

HMACSR:                                     ; INT31SR
         JMP       SCSIERR                  ; ******
         MOV       DX,HMSR
         IN        AL,DX
         TEST      AL,04H
         JZ        SKIP

         MOV       AL,80H                   ; '81' to indicate int occur
         MOV       BYTE PTR ES:[PARITY],AL
         MOV       AL,01H                   ; enable parity error check
;        OUT       DX,AL

SKIP:    IRET

;-----------------------------------------------------------------

SCSISR:                                     ; INT32SR
         JMP       SCSIERR
         IRET
;----------------------------------------------------------------

SER1:                                       ; INT33SR

         IRET
;------------------------------------------------------------

SER2:                                       ; INT34SR
         IRET


;--------------  PRINTER SUBROUTINE  --------------------

PRTSR:                                      ; INT35SR
         MOV       AL,BYTE PTR CS:[BX]
         CMP       AL,0
         JE        PRTEND
         INC       BX
         CALL      PRTCH
         JMP       PRTSR
PRTEND:
         RET

;--------------------------------------------------------

PRTCH:                                      ; print char service routine
         PUSH      AX
         PUSH      BX
         PUSH      DX

         MOV       BL,AL

         MOV       DX,PRTST                 ; check busy ?
B2:      IN        AL,DX
         TEST      AL,80H
         JZ        B2                       ; out strobe

PTOUT:
         MOV       AL,BL
         MOV       DX,PRTRW
         OUT       DX,AL                    ; output char


         MOV       AL,1DH                   ;
         MOV       DX,PRTCRL
         OUT       DX,AL
         JMP       $+2
         MOV       AL,1CH
         OUT       DX,AL

         POP       DX
         POP       BX
         POP       AX
         RET

;--------------------------------------------------

BLINK:
         MOV       AL,1
         MOV       DX,LEDPORT               ; LED OFF
         OUT       DX,AL
         XOR       CX,CX
ON1:     AAM
         AAM
         AAM
         LOOP      ON1

         MOV       AL,0                     ; LED ON
         OUT       DX,AL
         XOR       CX,CX
OFF1:    AAM
         AAM
         LOOP      OFF1

         RET

;-------------------------------------------

PRTCNT:
         PUSH      AX
         MOV       BX,10
         MOV       CX,5                     ; 5 digit
NUM:
         MOV       DX,0
         DIV       BX
         ADD       DL,30H
         XCHG      AX,DX

         CMP       CX,5
         JNE       D4
         MOV       BYTE PTR ES:[N1],AL      ; CX=5

D4:      CMP       CX,4
         JNE       D3
         MOV       BYTE PTR ES:[N2],AL      ; CX=4

D3:      CMP       CX,3
         JNE       D2
         MOV       BYTE PTR ES:[N3],AL      ; CX=3

D2:      CMP       CX,2
         JNE       D1
         MOV       BYTE PTR ES:[N4],AL      ; CX=2

D1:      CMP       CX,1
         JNE       TT
         MOV       BYTE PTR ES:[N5],AL      ; CX=1

TT:      CMP       DX,0
         JE        RTN8

         MOV       AX,DX
         LOOP      NUM

RTN8:
         POP       AX
         RET

;------------------------------------------------

DUMP:    PUSH      SS
         PUSH      CS
         PUSH      DS
         PUSH      ES
         PUSH      DI
         PUSH      SI
         PUSH      BP
         PUSH      SP
         PUSH      DX
         PUSH      CX
         PUSH      BX
         PUSH      AX

         MOV       AH,'A'                   ; AX
         MOV       AL,'X'
         POP       DX                       ; DX<=AX
         CALL      WRITE_VALUE

         MOV       AH,'B'                   ; BX
         MOV       AL,'X'
         POP       DX                       ; DX<=BX
         CALL      WRITE_VALUE

         MOV       AH,'C'                   ; CX
         MOV       AL,'X'
         POP       DX                       ; DX<=CX
         CALL      WRITE_VALUE

         MOV       AH,'D'                   ; DX
         MOV       AL,'X'
         POP       DX                       ; DX<=DX
         CALL      WRITE_VALUE

         MOV       AH,'S'                   ; SP
         MOV       AL,'P'
         POP       DX                       ; DX<=SP
         CALL      WRITE_VALUE

         MOV       AH,'B'                   ; BP
         MOV       AL,'P'
         POP       DX                       ; DX<=BP
         CALL      WRITE_VALUE

         MOV       AH,'S'                   ; SI
         MOV       AL,'I'
         POP       DX                       ; DX<=SI
         CALL      WRITE_VALUE

         MOV       AH,'D'                   ; DI
         MOV       AL,'I'
         POP       DX                       ; DX<=DI
         CALL      WRITE_VALUE

         MOV       AL,0DH                   ; CR,LF
         CALL      PRTCH
         MOV       AL,0AH
         CALL      PRTCH

         MOV       AH,'E'                   ; ES
         MOV       AL,'S'
         POP       DX
         CALL      WRITE_VALUE

         MOV       AH,'D'                   ; DS
         MOV       AL,'S'
         POP       DX
         CALL      WRITE_VALUE

         MOV       AH,'C'                   ; CS
         MOV       AL,'S'
         POP       DX
         CALL      WRITE_VALUE

         MOV       AH,'S'                   ; SS
         MOV       AL,'S'
         POP       DX
         CALL      WRITE_VALUE

         MOV       AH,'I'                   ; IP
         MOV       AL,'P'
         POP       DX
         PUSH      DX
         SUB       DX,3
         CALL      WRITE_VALUE

         MOV       AL,0DH
         CALL      PRTCH
         MOV       AL,0AH
         CALL      PRTCH
         RET

WRITE_VALUE:

         XCHG      AH,AL
         CALL      PRTCH
         XCHG      AH,AL
         CALL      PRTCH

         MOV       AL,'='
         CALL      PRTCH

         MOV       AL,AH
         SHR       AL,1
         SHR       AL,1
         SHR       AL,1
         SHR       AL,1
         CALL      WRITE_HEX

         MOV       AL,DH
         AND       AL,0FH
         CALL      WRITE_HEX

         MOV       AL,DL
         SHR       AL,1
         SHR       AL,1
         SHR       AL,1
         SHR       AL,1
         CALL      WRITE_HEX

         MOV       AL,DL
         AND       AL,0FH
         CALL      WRITE_HEX
         MOV       AL,20H
         CALL      PRTCH
         RET

WRITE_HEX:
         ADD       AL,30H
         CMP       AL,39H
         JBE       WRITE_HEX1
         ADD       AL,07H
WRITE_HEX1:
         CALL      PRTCH
         RET


;==================    jmp to dsk    =====================

DSK:
         XOR       SI,SI
         MOV       ES,SI

         MOV       AL,80H
         CMP       AL,ES:[DG]
         JE        WAITDG

         MOV       AH,80H                   ; pass code for diag. '80'
         MOV       ES:[RESULT],AH

         STI
         MOV       CX,10H
DKOS:    PUSH      CX
         MOV       CX,0
INT386G: MOV       AL,ES:[TSCOM]            ;
         CMP       AL,11H
         JE        DSKSYS
         AAM
         AAM
         LOOP      INT386G
         POP       CX
         LOOP      DKOS
         JMP       DSKSYS

WAITDG:  MOV       AL,ES:[DG]
         CMP       AL,88H
         JNE       WAITDG


DSKSYS:
         MOV       AL,1
         MOV       DX,LEDPORT               ; LED OFF
         OUT       DX,AL

         XOR       DI,DI
         MOV       BL,0F0H                   ; pass code for os, 'F0'=> BOARDST
         MOV       ES,DI
         MOV       BYTE PTR ES:[BOARDST],BL

         MOV       AX,0
         MOV       BX,0FFFFH
         MOV       DI,430H
         SUB       BX,DI
         MOV       CX,BX
         INC       CX
         REP  STOSB

         MOV       DI,1000H
         MOV       ES,DI
         MOV       DI,0
         MOV       CX,8000H
         REP  STOSW

         MOV       DI,2000H
         MOV       ES,DI
         MOV       DI,0
         MOV       CX,8000H
         REP  STOSW

         MOV       DI,3000H
         MOV       ES,DI
         MOV       DI,0
         MOV       CX,8000H
         REP  STOSW


;        JMP       SCSI_INIT                ; @@ SCSI_INI OS del

