NAME    CS_SRC
$TITLE ('CS.source')
;PAGE 60,132
;-------------------------------------------------------------------------+
;                                                                         !
;   (c) Copyright, Wang Laboratories, Inc., 1991                          !
;       All rights reserved.                                              !
;                                                                         !
;    Program name: WANG 2200 CS386/II Power up diagnostics & Boot strap   !
;    File name: CPUDIAG.ASM                                               !
;    Support group: Engineering Service Department                        !
;                   Wang Computer (Taiwan) LTD.                           !
;                   2, Science Road One       ZIP 20077                   !
;                   Hsinchu, Taiwan, Republic of China                    !
;                                                                         !
;    Revision History:                                                    !
;       Date         Programmer      Description            Revision      !
;    07/25/1991     Milton Chen      CS386 TURBO              5170        !
;                                                                         !
;    Test Routine:                                                        !
;                                                                         !
;      [ TEST 0 ]  LED TEST                                               !
;      [ TEST 1 ]  CPU 80386 FLAG & REG. TEST                             !
;      [ TEST 2 ]  64K DRAM MEMORY TEST                                   !                  !
;      [ TEST 3 ]  DRAM DATA BUS TEST                                     !
;      [ TEST 4 ]  DRAM ADDRESS LINE TEST                                 !
;              ENTER PROTECTED MODE                                       !
;      [ TEST 5 ]  RTC CHIP INITIAL CHECK & USER RAM TEST                 !
;      [ TEST 6 ]  PARITY CHECK AND NMI TEST                              !
;      [ TEST 7 ]  DETECT SYSTEM INTERFACE CONTROL CARD                   !
;      [ TEST 8 ]  SYSTEM INTERRUPT TEST                                  !
;      [ TEST 9 ]  MEMORY SIZE TEST                                       !
;      [ TEST A ]  COMMON MEMORY TEST                                     !
;      [ TEST B ]  DRAM FAST EXCHANGE FUNCTION TEST                       !
;      [ TEST C ]  BAD SIMM DETECT                                        !
;      [ TEST D ]  TWO WAY SET ASSOCIATIVE CACHE TEST                     !
;               MOV EPROM TEST PROGRAM TO DRAM                            !
;      [ TEST E ]  DRAM TEST WITH CACHE                                   !
;               BOOT STRAP TASK SWITCH TO LOADER                          !
;                                                                         !
;-------------------------------------------------------------------------+


;****************************************************************************
;                        MEMORY  ALLOCATION MAP
;****************************************************************************
;
;   Diagnostic Memory Map :
;
;              +-------------------------------+
;     00000000 |  00     NULL_DES   GDT Table  |  00
;              |-------------------------------|
;              |  08     FLAT_DES              |  08  DS:base 00000000 lim:4K*FFFFF
;              |-------------------------------|
;              |  10      RAM_DES              |  10  CS:base 00002000 lim:4K*FFFFF
;              |-------------------------------|
;              |  18      ROM_DES              |  18  CS:base FFFF0000 lim:4K*FFFFF
;              |------------------------------|
;      0000300 |  HSC STATUS, 301: INT STATUS  |
;              |-------------------------------|
;      0000400 |          IDT Base             |
;              |-------------------------------|
;              |                               |
;     00002000 |-------------------------------|
;              |                               |
;              |   MOV TEST PROGRAM TO HERE    | <------+
;              |                               |        !
;     0003FF00 |-------------------------------|        !
;              |          STACK                |        !
;     00040000 |-------------------------------|        !
;              |                               |        !
;              |                               |        !
;              :      MEMORY TEST AREA         :        !
;              :                               :        !
;      EPROM   |                               |        !
;     FFFF0000 |-------------------------------|        !
;              |                               |        !
;              |         LOADER                |        !
;              |                               |        !
;     FFFF8000 |-------------------------------|        !
;              |    DIAGNOSTIC PROGRAM         |        !
;              |- - - - - - - - - - - - - - - -|        !
;              |   TEST PROGRAM (MOV TO DRAM)  |--------+
;              |-------------------------------|
;              |         BOOT STRAP            |
;     FFFFE240 |-------------------------------|
;              |    INT Service Routine        |
;     FFFFE550 |-------------------------------|
;              |        IDT table              |
;              +-------------------------------+
;
;
;     System memory map:
;
;    0 H |------------------------------------------|
;        |   TEMP_GDT (for bstrap copy TSS,LDT from ROM to RAM) reserve for $init
;   40 H |------------------------------------------|
;        |   GDT ( loader, sytem use)             9 |
;   A0 H |------------------------------------------|
;        |   LDT ( system)                        7 |
;   E0 H |------------------------------------------|
;        |   TSS (system)                         67|
;  160 H |------------------------------------------|
;        |                                          |
;        .                                          .
;        .   SYSTEM LOADED HERE(TOTAL 192K)         .
;        .                                          .
;
; 50160H |------------------------------------------|
;        .                                          .
;        .   SYSTEM RUNNING SPACE(begin)            .
;        .                                          .
;
; 51000H |------------------------------------------| ---
;        |   TSS (loader's TSS            )         |    |
; 51080H |------------------------------------------|    |
;        |   LDT (loader's LDT            )         |    |
; 510e0H |------------------------------------------|    |
;        |   DATA (loader's writable data ) + I/O BUFFER |  when RUNNING the system
; 51600H |------------------------------------------|    |  this area will be OVERWRTIE
;        |   STACK (loader's stack)                 |    |  no use anymore
; 51700H |------------------------------------------|    |
;        |   TEMP AREA (loader builds system use)   |    |
; 51780H |------------------------------------------| ----
;        .                                          .
;        .                                          .
;   2M H .   SYSTEM RUNNING SPACE(end)              .
;        .                                          .
;        .                                          .
;   8M H |__________________________________________|
;


;define the adjustment factor

ADJUST1  EQU        8000H
ADJUST2  EQU       0fff0h
adjust3  equ       0ffe9h
bootid   equ       0ffffffe0h

ADJIDT   EQU       0E550H

gdtbase  EQU       00000040H                ; phisical address for GDT Base
IDTBASE  EQU       00000400H                ; FOR IDT BASE

;define addresses to locate GDT  in RAM

; note: initial task's ldt will be put 00051080h
;                      tss will be put 00051000h
;                      writable data will be put 000510e0h
;                      writable stack will be put  051700h (expand down)


;I/O PORT

CPUST    EQU       80000002H                ; READ CPU/IO STATUS
                                            ; D0= 1, 5us pulse of ABS/, OBS/, CBS/
                                            ; D1= 1, CPU BUSY
                                            ; D2= 1, KEY BOARD FUNCTION
                                            ; D3= 1, I/O READY
                                            ; D4= 1, TIME OUTME OUT
                                            ; D5= 1, CPU HALT
                                            ; D6= 1, MEMORY PARITY ERROR
                                            ; D7= 1, SYSTEM ERROR
                                            ; D8= 1, primary reset in from 2200 bus
                                            ; D9= 1, nomal boot; =0, run-in
                                            ; D10=1, same as D2, but read cause clear
                                            ; D11=1, 4M SIMM INSTALLED; =0 1M SIMM
                                            ; D12=1, LAST ACCESS TIME OUT
                                            ; D13=1, same as D5, CPU HALT

CPUFLG1  EQU       8004H                    ; I/O PORT
CPUFLG   EQU       80000004H                ; WRITE IO PORT
                                            ; D0= 1, generate even parity bit for diag
                                            ;  = 0, clear force even parity condition
                                            ; D1= 1, clear parity error if occured anytime
                                            ;  = 0, not clear parity error, if occured
                                            ; D2= 1, SET CPU BUSY
                                            ; D3= 1, CPU LED on; =0 OFF
                                            ; D6= 0, disable parity error ( disable NMI )
                                            ;    =1, enable parity error ( enable NMI )

RTCLOCK  EQU       80000008H                ; realtime clock address strobe
RTCLOC2  EQU       8000000AH                ; realtime clock data strobe, data r/w port
                                            ; [0Eh]= the DRAM memory size in unit of MB
                                            ; [0Fh]= boot ID high byte
                                            ; [10H]= boot ID low byte
                                            ; [11h]= system config
                                            ; [12h]= interrupt test result
                                            ; [13h]= NMI test result
                                            ; [1Eh]= memory size low byte, 64K unit
                                            ; [1Fh]= memory size high byte, 64K unit
                                            ; [20h]= loop count low byte
                                            ; [21h]= loop count hi byte

CPURDY   EQU       80000006H
_ABS     EQU       80000012H                ; address bus strobe
_OBS     EQU       80000014H                ; output data to bus
NCA      EQU       80000108H                ; cache/non-cacheable memory bank & cache flush
IBGET    EQU       8000000CH

CBS      EQU          8018H                 ; command data bus strobe

ABS7     EQU       0FF07H
MXFBIT   EQU       01B                      ; mxf exist
MXFBASE  EQU       70400430H

;MXFSTS   EQU       70401602H
;MXFABS   EQU       70401603H
;MXFPAR   EQU       70401605H

MXFSTS   EQU       320H
MXFABS   EQU       324H
MXFPAR   EQU       328H

HSCOFG   EQU       300H
NMIBIT   EQU       302H
RUNFLG   EQU       304H                     ; 01H

DUMMY   SEGMENT RW                          ; only for ASM386 MAIN MODULE stack init
        DW      00                          ; w/o this dummy segment, you will get
DUMMY   ENDS                                ; Warning 21 during compiler and fatal


;------------------------------------------------------------------------------+
;  RESET_CODE must be USE16 because the 386 initially executes in real mode    |
;------------------------------------------------------------------------------+


RESET_CODE SEGMENT ER PUBLIC USE16
ASSUME  DS:NOTHING, ES:NOTHING

                                             ; 386 descriptor template
desc    struc
        lim_0_15        dw 0
        bas_0_15        dw 0
        bas_16_23       db 0
        access          db 0
        gran            db 0
        bas_24_31       db 0
desc    ends

; GDT[0].... null
; GDT[1].... gdt alias
; GDT[2].... idt alias
; GDT[3].... initial task tss ------base address point to rom for copy from
; GDT[4].... initial task tss alias-base address point to ram for copy to
; GDT[5].... initial task ldt ------base address ponit to rom for copy from
; GDT[6].... initial task ldt alias-base address ponit to ram for copy to
; GDT[7].... reserve for loaded task
; GDT[8].... reserve for loaded task

; the descriptors in gdt total 9

gdt_entries     equ 9

gdt_alias       equ 1*size desc
idt_alias       equ 2*size desc
init_tss        equ 3*size desc
init_tss_a      equ 4*size desc
init_ldt        equ 5*size desc
init_ldt_a      equ 6*size desc

init_ldt_alias  equ 1*size desc

; access right byte for data and tss descriptor

ds_access       equ 010010010b
tss_access      equ 010001001b

;this temporary gdt will be used to set up the real gdt in ram



        org     adjust1

temp_gdt        label byte                  ; gdt for bstrsp and loader

null_des        desc <>
flat_des        desc<0ffffh,0,0,92h,0cfh,0>
                                            ; the gdt address and limit in eprom
gdt_eprom       label pword
        dw      0047h
        dd      0ffff00e0h
                                            ; the temporary gdt address and limit
tgdt_pword      label pword
        dw      end_temp_gdt -1
        dd      0
                                            ; the gdt address and limit in ram
gdt_pword      label pword
        dw      gdt_entries * size desc -1
        dd      gdtbase

end_temp_gdt    label byte

gdt_des_flat    equ ds:gdt_alias + gdtbase

init_tss_a_offset       equ ds:init_tss_a
init_tss_offset         equ ds:init_tss

init_ldt_a_offset       equ ds:init_ldt_a
init_ldt_offset         equ ds:init_ldt

; define pointer for first task switch

entry_pointer   label  pword                ; loader entry
        dd 0
        dw ds:init_tss


;===============  Set up GDT,IDT for Diagnostic  ===================

; GDT[0].... null
; GDT[1].... flat_des      DS base 00000000
; GDT[2].... ram_des       CS base 00002000
; GDT[3].... rom_des       CS base FFFF0000
; GDT[4].... idt_des       CS base FFFF0000


  GDTENTRIES     equ 5                      ; 5 gdts
  IDTENTRIES     equ 112                    ; 112 idts


TEMPGDT      LABEL BYTE

  NULLDES        DESC <>
  FLATDES        DESC<0FFFFH,0,0,92H,0CFH,0>
  RAMDES         DESC<0FFFFH,2000H,00H,9AH,8FH,00H>
  ROMDES         DESC<0FFFFH,0000H,0FFH,9AH,00H,0FFH>
  IDTDES         DESC<0100H,0000H,0FFH,9AH,80H,0FFH>

  GDTEPROM    LABEL PWORD                   ; the gdt address and limit in eprom
         DW        004FH                    ; limit
         DD        0FFFFD400h               ; base addr.

  IDTEPROM    LABEL PWORD
         DW        04FFH                    ; limit
         DD        0FFFFE550H               ; base

  TGDTPWORD   LABEL PWORD
         DW        ENDTEMPGDT - TEMPGDT - 1
         DD        0
                                            ; the gdt address and limit in ram
  GDTPWORD    LABEL PWORD
         DW        GDTENTRIES * SIZE DESC -1
         DD        GDTBASE

  IDTPWORD    LABEL PWORD
         DW        IDTENTRIES * SIZE DESC -1
         DD        IDTBASE

ENDTEMPGDT    LABEL BYTE


BEGINPUP        LABEL   WORD                ; power-up entry point
                PUBLIC  BEGINPUP

START:   CLI                                ; disable interrupts
         JMP     AFTER_COPYRIGHT


;------------------------  display message  --------------------------

RUNLOOP         DB      0EH,'    DIAGNOSTIC RUN-IN TEST',0FH,32 DUP(20H)          ;#
                DB      'LOOP COUNT:', 5 DUP(20H),30H,0DH,0AH
                DB      56 DUP(20H),'ERROR COUNT:',5 DUP(20H),30H,0DH,0AH
RUNLOOPL  EQU   $-RUNLOOP

DRAMSIZE        DB      'DRAM', 5 DUP(20H),30H,20H,'KB', 08H, 08H, 08H, 08H
ENDDRAMSIZE:

LOOPCONT        DB      01h, 0Ah, 0Ah, 69 DUP(09h),6 DUP(20H)
ENDLOOPCONT:

BLANK           DB      01H,0AH,0AH,0AH,0AH,0AH
BLANKL          EQU     $-BLANK

BKLINE          DB      0DH,0AH,70 DUP (20H)
BKLINEL         EQU     $-BKLINE


ERRCONT         DB      01H,0AH,0AH,0AH, 69 DUP(09H), 6 DUP(20H)   ; #
ERRCONTL:

DRAMCLEN        DB      0Ah, 0Dh, 15 DUP(20h), 0Dh,0Ah     ;
ENDDRAMCLEN:

DRAMLOOP        DB      0Dh, 5 DUP(09h),4 DUP(20H)
ENDDRAMLOOP:

BADSIMM            DB        0EH,'   BAD SIMM :  L'
BADSIMML           EQU       $-BADSIMM

SM1      DB        0EH,'   BAD SIMM :  L3  OR  L5',0FH
SM1L     EQU       $-SM1

SM2      DB        0EH,'   BAD SIMM :  L10  OR  L15',0FH
SM2L     EQU       $-SM2

SM3      DB        0EH,'   BAD SIMM :  L18  OR  L24',0FH
SM3L     EQU       $-SM3

SM4      DB        0EH,'   BAD SIMM :  L29  OR  L35',0FH
SM4L     EQU       $-SM4

LF1      DB        0DH,0AH,0DH,0AH,0DH,0AH,0DH,0AH
LF1L     EQU       $-LF1

LF2      DB        12 DUP(0DH,0AH)
LF2L     EQU       $-LF2

KEYSF    DB        0DH,0AH,0DH,0AH,0DH,0AH,'   * PRESS',02H,04H,04H,00H,0EH,' SHIFT+RESET ',0FH,'KEY !'
KEYSFL   EQU       $-KEYSF

TSAER    DB        0DH,0AH,0DH,0AH,'    DRAM MEMORY TEST ERROR -->'
TSAERR:

CACERR   DB        0DH,0AH,0DH,0AH,0EH,'    CACHE MEMORY TEST FAIL !!',0FH,0DH,0AH
CACERRL  EQU       $-CACERR

RTCERR   DB        0DH,0AH,0DH,0AH,' FATAL ERROR - RTC FAIL !! ',0DH,0AH
RTCERRL  EQU       $-RTCERR

MEMF     DB        0DH,0AH,0DH,0AH,' FATAL ERROR - '
MEMFL    EQU       $-MEMF

COPYRIGHT          DB        03H,'Copyright, Wang Laboratories, Inc., 1991'
                   DB        20H, 20H, 20H, 20H, 20H, 06H
REVISION           DB        'CS/386 Turbo         Rev 5170', 0DH, 0AH, 0DH, 0AH

MSG      DB      0DH,0AH,0DH,0AH
         DB      ' Move EPROM Memory Test Program to DRAM and ',0EH,'Enable CACHE',0FH,'....'
         DB      0DH,0AH,0DH,0AH
MSGLEN   EQU        $-MSG

SYSMESS  DB        0DH,0AH,0DH,0AH,'System Interface Control Card Diagnostic Status:'
SYSLEN   EQU       $-SYSMESS

PB0      DB        0DH,0AH,'     * '
PB0L     EQU       $-PB0

PB1      DB        0DH,0AH,'     * 1st '
PB1L     EQU       $-PB1

PB2      DB        0DH,0AH,'     * 2nd '
PB2L     EQU       $-PB2

PB3      DB        0DH,0AH,'     * 3rd '
PB3L     EQU       $-PB3

PB4      DB        0DH,0AH,'     * 4th '
PB4L     EQU       $-PB4

GSS      DB        ' --> PASS                          '
GSSL     EQU       $-GSS

GRUN     DB        ' --> Run-in mode: PASS             '
GRUNL    EQU       $-GRUN

NOCRT    DB        ' --> Run-in mode: CRT NOT connect  '
NOCRTL   EQU       $-NOCRT

NOPTR    DB        ' --> Run-in mode: Printer NOT ready'
NOPTRL   EQU       $-NOPTR

GERROR   DB        ' --> ERROR '
GERRORL  EQU       $-GERROR

COMT     DB        0DH,0AH,0DH,0AH,'High Speed Channel Bus Test:'
COMTL    EQU       $-COMT

FMEM     DB        03H,0DH,0AH,0DH,0AH,0EH,'       FLOOD MEMORY, WAIT ...',0FH
FMEML    EQU       $-FMEM


;-----------------------------------------------------------------------------+
;                                                                             !
;      [ TEST 0 ]  LED TEST                                                   !
;                                                                             !
;     To check LED on/off, for diagnostic.                                    !
;                                                                             !
;-----------------------------------------------------------------------------+

AFTER_COPYRIGHT:
         MOV       DX,CPUFLG1               ; select I/O port, direct memory mapping
         MOV       AX,08H                   ; D3= 1, cpu led on; D3= 0, cpu led off
         OUT       DX,AX                    ; turn CPU LED on
         XOR       ECX,ECX                  ; set up counter for time delay
DIAGLED:
         AAM                                ; wait so CE can see LED is turned "on".
         AAM
         AAM
         AAM
         AAM
         AAM
         AAM
         AAM
         LOOP      DIAGLED                  ; 83 ck x .033us x 65536 x 8 = 1.4 sec

         MOV       AX,6                     ; D1=1 clear parity error,D2=1 clr 387
         OUT       DX,AX                    ; BX point to CPUFLG I/O port


;-----------------------------------------------------------------------------+
;                                                                             !
;    [ TEST 1 ]  80386 CPU FLAG & REGS. TEST                                  !
;                                                                             !
;       To check 80386 CPU flag on/off and all registers read/write test      !
;                                                                             !
;                                                  error code : 01 => BP      !
;-----------------------------------------------------------------------------+


         CLI                                ; disable interrupt
         MOV       AH,0D5H                  ; set SF,CF,ZF,AF flags on
         SAHF                               ; store AH to flag reg.
         JNC       FATAL_ERROR              ; error if CF not set
         JNZ       FATAL_ERROR              ; error if ZF not set
         JNP       FATAL_ERROR              ; error if PF not set
         JNS       FATAL_ERROR              ; error if SF not set
         LAHF                               ; load flag to AH
         MOV       CL,5                     ; shift count
         SHR       AH,CL                    ; shift AF into CARRY bit
         JNC       FATAL_ERROR              ; error if no carry
         MOV       AL,40H                   ; set flag on
         SHL       AL,1                     ; set up for test
         JNO       FATAL_ERROR              ; error if overflow
         XOR       AH,AH                    ; AH=0
         SAHF
         JBE       FATAL_ERROR              ; error if CF on

         JS        FATAL_ERROR              ; error if SIGN flag set
         JP        FATAL_ERROR              ; error if EVEN Parity set
         LAHF                               ; load flag to AH
         MOV       CL,5                     ; shift count
         SHR       AH,CL
         JC        FATAL_ERROR              ; error if CF on
         SHL       AH,1                     ; check OF is clear
         JO        FATAL_ERROR              ; error if OF on

         XOR       EAX,EAX                  ; test reset condition & flags
         JNZ       FATAL_ERROR              ; check zero flag off
         OR        EAX,1
         JZ        FATAL_ERROR              ; check zero flag on
         AND       EAX,EAX
         JC        FATAL_ERROR              ; test carry, error if carry flag not cleared
         STC                                ; set carry flag
         JNC       FATAL_ERROR              ; error if not carry
         CLC                                ; clear carry flag
         JC        FATAL_ERROR              ; error if carry flag is not clear
         CMC                                ; complement carry flag
         JNC       FATAL_ERROR              ; error if no carry

         MOV       EAX,1                    ; store 1 to EAX Reg.
         MOV       CX,7                     ; count times
LP1:     AND       EAX,EAX                  ; set flag
         JZ        FATAL_ERROR              ; error if ZERO flag set
         JS        FATAL_ERROR              ; error if SIGN flag set
         JPE       FATAL_ERROR              ; error if EVEN Parity, b0 - b7
         JC        FATAL_ERROR              ; error if CARRY
         RCL       EAX,1
         LOOP      LP1


;----------- R/W GENERAL AND SEGMENT REGS. WITH 1'S AND 0'S

         MOV       EAX,33CC55AAH            ; pattern data = 55AA
CLSET:   MOV       EBX,EAX                  ; copy to EBX
         MOV       ECX,EBX                  ; copy to ECX
         MOV       EDX,ECX                  ; copy to EDX
         MOV       ESI,EDX                  ; copy to ESI
         MOV       EDI,ESI                  ; copy to EDI
         MOV       EBP,EDI                  ; copy to EBP
         MOV       ESP,EBP                  ; copy to ESP
         CMP       EAX,ESP                  ; equal EAX ?
         JNZ       FATAL_ERROR              ; error if not equal
         XOR       EAX,EAX                  ; EAX <= 0
         CMP       EBX,EAX
         JNZ       CLSET                    ; set all reg. = 0

         JMP       CPUPASS                  ; test pass

FATAL_ERROR:
         MOV       BP,01H                   ; error code = 01
ERRFLSH: MOV       DX,CPUFLG1               ; point to I/O port
         MOV       AX,08H                   ; d3=1 CPU LED on
         OUT       DX,AX                    ; turn CPU LED on
         XOR       ECX,ECX                  ; set up counter for time delay
LEDLOOP: AAM
         AAM
         AAM
         AAM
         AAM
         AAM
         AAM
         LOOP      LEDLOOP

         MOV       AX,0
         OUT       DX,AX
         XOR       ECX,ECX
LP11:    AAM
         AAM
         AAM
         AAM
         LOOP      LP11

         JMP       ERRFLSH


;---------------------------------------------------------------------+
;                                                                     |
;   [ TEST 2 ]  64K DRAM MEMORY TEST                                  |
;                                                                     |
;         1. Double word test  pattern: 55AAAA55, AA5555AA            |
;         2. Word test         pattern: 3CC3,0C33C                    |
;         3. byte test         pattern: FF,00                         |
;                                                                     |
;                                              error code  02 <= BP   |
;---------------------------------------------------------------------+


CPUPASS:
         MOV       DI,2                     ; two patterns
         MOV       EAX,55AAAA55H
NXTDW:   MOV       CX,4000H                 ; 16K DW
         MOV       ESI,0
DWOD:    MOV       [ESI],EAX
         MOV       EBX,[ESI]
         ADD       ESI,4
         CMP       EAX,EBX
         JNE       ERR64K
         LOOP      DWOD
         DEC       DI
         JZ        WORDTST                    ; WORDTST
         NOT       EAX
         JMP       NXTDW


WORDTST:
         MOV       DI,2                     ; two patterns
         MOV       AX,3CC3H
NXTW:    MOV       CX,8000H                 ; 32k word
         MOV       ESI,0
WOD:     MOV       [ESI],AX
         MOV       BX,[ESI]
         ADD       ESI,2
         CMP       AX,BX
         JNE       ERR64K
         LOOP      WOD
         DEC       DI
         JZ        BYTETST
         NOT       AX
         JMP       NXTW

BYTETST:
         MOV       DI,2                     ; two patterns
         MOV       AL,0FFH
NXTB:    MOV       CX,0                     ; 64k byte
         MOV       ESI,0
BOD:     MOV       [ESI],AL
         MOV       BL,[ESI]
         INC       ESI
         CMP       AL,BL
         JNE       ERR64K
         LOOP      BOD
         DEC       DI
         JZ        TST3
         NOT       AL
         JMP       NXTB

ERR64K:  MOV       BP,02H                   ; error code =02
         JMP       ERRFLSH


;-------------------------------------------------------------------------+
;                                                                         !
;   [ TEST 3 ]  DRAM DATA BUS TEST                                        !                                     !
;                                                                         !
;   Description: Walking '0' & '1' data patterns (double word) are        !
;          written, then read/verified for correct data and NO parity     !
;          error on each bank of 1st addr.                                !
;                                                                         !
;      BEGIN                                                              !
;         FOR current bank(64K) of 1st addr.                              !
;             FOR pattern = FFFFFFFF, 01, 02, 04, ...80000000, 0,         !
;                           7FFFFFFF, BFFFFFFD, ......., FFFFFFFE         !
;                  write current pattern to current address               !
;                  read/verify current address                            !
;             NEXT pattern                                                !
;         NEXT bank                                                       !
;                                                                         !
;         FOR current bank(64K) of 1st addr.                              !
;             FOR pattern = 03, 06, 0C, 18, 30,...., C0000000, 80000001   !
;                  write current pattern to current address               !
;                  read/verify current address                            !
;             NEXT pattern                                                !
;         NEXT bank                                                       !
;      END                                                                !
;                                                error code : 03 <= BP    !
;                                                                         !
;-------------------------------------------------------------------------+

TST3:
         XOR       SI,SI                    ; start from address 0:0000
         MOV       DI,SI
TST3LOP1:
         MOV       AX,DI                    ; start from address 0:0000 to F000:0000
         MOV       DS,AX
         MOV       EBX,0FFFFFFFFH           ; current data pattern
         MOV       EAX,01                   ; next pattern
         MOV       EDX,EAX                  ; save next pattern
         MOV       CX,33                    ; 33 patterns from FFFFFFFF to 01h
         MOV       [SI],EBX                 ; write current pattern to current address
TST3LOP2:
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad (no jump is fast)
         MOV       EBX,EDX                  ; save current pattern
         SHL       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX                  ; ready to write data pattern from EAX reg
         DEC       CX
         JNZ       TST3LOP2                 ; loop for next pattern test

         MOV       EAX,7FFFFFFFH            ; next data pattern 7FFFFFFF
         MOV       EDX,EAX                  ; prepare for shift right
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad (no jump is fast)
         MOV       EBX,EDX                  ; save current pattern for compare later
         ROR       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX
         MOV       CX,32                    ; 32 patterns from 7FFFFFFF to FFFFFFFE
TST3LOP3:
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad (no jump is fast)
         MOV       EBX,EDX                  ; save current pattern for compare later
         ROR       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX
         DEC       CX
         JNZ       TST3LOP3                 ; loop for next pattern test

         ADD       EDI,1000H                ; point to next 64KB bank
         CMP       EDI,0F000H               ; last DRAM is F:0000h
         JLE       TST3LOP1                 ; otherwise go on next test


         XOR       ESI,ESI                  ; start from address 0:0000 for
         MOV       EDI,ESI                  ; even number bits patterns rotate
TST3LOP4:
         MOV       AX,DI                    ; start from address 0:0000 to F000:0000
         MOV       DS,AX
         MOV       EBX,03H                  ; current data pattern
         MOV       EAX,06H                  ; next pattern
         MOV       EDX,EAX                  ; save next pattern
         MOV       CX,32                    ; 32 patterns from 03 to 80000001h
         MOV       [SI],EBX                 ; write current pattern to current address
TST3LOP5:
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad
         MOV       EBX,EDX                  ; save current pattern
         ROL       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX                  ; ready to write data pattern from EAX reg
         DEC       CX
         JNZ       TST3LOP5                 ; loop for next pattern test

         ADD       EDI,1000H                ; point to next 64KB bank
         CMP       EDI,0F000H               ; last DRAM is F:0000h
         JLE       TST3LOP4                 ; otherwise go on next test

         JMP       ADDRTST

TST3_ERR:
         MOV       BP,03H
         JMP       ERRFLSH

;----------------------------------------------------------------------------+
;                                                                            !
;   [ TEST 4 ]  DRAM ADDRESS LINE TEST                                       !                                     !
;                                                                            !
;        At first, write 'FFFFFFFF' to address 4, write addr.1000 pattern    !     !
;   00000000H, and check addr.4 = FFFFFFFFH. that is check A2 address line   !
;   the A3 is check addr.=8 & addr.= 2000, addr.=10 & addr.=4000,... and so  !
;   on.                                                                      !
;                                                                            !
;----------------------------------------------------------------------------+



ADDRTST: XOR       ESI,ESI                  ; address 0
         MOV       DS,SI                    ; ds <= 0
         MOV       EBP,4
         MOV       ESI,1000H
         MOV       CX,4
NXADDR:  MOV       EAX,0FFFFFFFFH
         MOV       DWORD PTR [EBP],EAX                ; addr. 0 <= FF
         MOV       DWORD PTR [ESI],0
         CMP       DWORD PTR [EBP],EAX
         JNE       ADDRERR
         SHL       EBP,1
         SHL       ESI,1                    ; next address line
         LOOP      NXADDR
         JMP       PRO_TE

ADDRERR:
         MOV       BP,04H
         JMP       ERRFLSH


;--------------------------------------------------------------+
;                  ENTER PROTECTED MODE  !!                    |
;--------------------------------------------------------------+


PRO_TE:  XOR       DI,DI                    ; mov eprom temp_gdt to RAM addr.: 0
         MOV       ES,DI                    ; at addr. 00000000H
         MOV       SI,OFFSET TEMPGDT
         MOV       CX,ENDTEMPGDT-TEMPGDT
         INC       CX
  REP  MOVS  BYTE PTR ES:[DI],BYTE PTR CS:[SI]
         LGDT      CS:TGDTPWORD


         MOV       SI,OFFSET TSSTEST        ; mov test program to 2000
         MOV       DI,2000H                 ; at addr. 00002000H
         MOV       CX,TSSEND-TSSTEST
  REP  MOVS  BYTE PTR ES:[DI],BYTE PTR CS:[SI]


         MOV       ESI,DWORD PTR IDTEPROM+2 ; setup IDT
         MOV       EDI,IDTBASE              ; at addr. 00000400h
         MOV       CX,WORD PTR IDTEPROM+0
         INC       CX
         SHR       CX,1
         CLD
  REP  MOVS  WORD PTR ES:[EDI],WORD PTR CS:[SI]


         MOV       EAX,CR0
         MOV       EAX,1
         MOV       CR0,EAX
         JMP       FLUSH
FLUSH:                                      ; now entering protect mode
         MOV       BX,08H
         MOV       DS,BX                    ; ds,ss,all point to temp.gdt[1]
         MOV       SS,BX
         MOV       BX,18H
         MOV       ES,BX                    ; es base : FFFF0000
         MOV       ESP,3FF00H               ; setup stack at bottom of 3FF00h

         LIDT      IDTPWORD                 ; set up idt

         DB        0EAH                     ; change base : FFFF0000, selector 18
         DW        OFFSET ROMBASE
         DW        18H


;-----------------------------------------------------------------------------+
;                                                                             !
;   [ TEST 5 ] RTC CHIP INITIAL CHECK & USER RAM TEST                         !
;                                                                             !
;  Purpose: RTC initial date check and RTC's user RAM test                    !
;                                                                             !
;  Description: Using READRTC Macro to check RTC calendar regs. content, if   !
;               not initial, using SETRTC Macro to initialize.                !
;                                                                             !
;   RTC user RAM test result at 303H : 08H pass , 04H error                   !
;                                                                             !
;-----------------------------------------------------------------------------+

ROMBASE: JMP       TIMEOK                   ; skip initial, let OS initial
                                            ; MACRO for read RTC time data
%*DEFINE (READRTC(ADDR,TIME))
      (  MOV       CL,%ADDR                 ; select addr.
         MOV       EBX,RTCLOCK              ; RTC addr. strobe
         MOV       [EBX],CL                 ; point to address
         MOV       EBX,RTCLOC2              ; RTC data strobe, data R/W port
         MOV       AL,[EBX]                 ; read time data
         CMP       AX,%TIME                 ; compare time range is reasonable ?
      )

                                            ; MACRO for set RTC time
%*DEFINE (SETRTC(SETADDR,SETIME))
      (  MOV       CL,0BH                   ; select B reg.
         MOV       AL,10000010B             ; set bit b7=1, b2=0 DM use BCD,
         MOV       EBX,RTCLOCK              ; b1=1 24 hours mode, b0=0 DSE
         MOV       [EBX],CL                 ; point to B reg.
         MOV       EBX,RTCLOC2              ; point to data port
         MOV       [EBX],AL                 ; initialize RTC time

         MOV       CL,%SETADDR              ; set time of RTC addr.
         MOV       AL,%SETIME               ; set time data to RTC
         MOV       EBX,RTCLOCK
         MOV       [EBX],CL
         MOV       EBX,RTCLOC2
         MOV       [EBX],AL
      )

         %READRTC(08,12)                    ; month addr. = 08
         JG        SET_MONTH                ; month > 12 , then set month

         %READRTC(07,31)                    ; data addr. = 07
         JG        SET_DATA                 ; data > 31 , then set data

         %READRTC(04,24)                    ; hour addr. = 04
         JG        SET_HOUR                 ; hour > 24 , then set hour

         %READRTC(02,60)                    ; min. addr. = 02
         JG        SET_MIN                  ; min. > 60 , then set min

         %READRTC(00,60)                    ; sec. addr. = 01
         JG        SET_SEC                  ; sec. > 60 , then set sec

         JMP       TIMEOK

SET_MONTH:
         %SETRTC(08,01)                     ; set month = 01

SET_DATA:
         %SETRTC(07,01)                     ; set date = 01

SET_HOUR:
         %SETRTC(04,00)                     ; set hour = 00

SET_MIN:
         %SETRTC(02,00)                     ; set min = 00

SET_SEC:
         %SETRTC(00,00)                     ; set sec = 00


;---------  TEST RTC USER RAM --------------

TIMEOK:

         MOV       CL,0AH                   ; select A reg.
         MOV       AL,00100000B             ; DV2,DV1,DV0:0,1,0,32.768KHz
         MOV       EBX,RTCLOCK              ;
         MOV       [EBX],CL                 ;
         MOV       EBX,RTCLOC2              ;
         MOV       [EBX],AL                 ;

         MOV       AX,3000H
         CALL      WRITERTC

         JMP       $+2
         JMP       $+2

         MOV       AX,3000H
         CALL      READRTC

         MOV       EDI,303H
         MOV       BYTE PTR [EDI],0         ; clear semaphore byte

         MOV       AL,0EH                   ; select RTC user RAM first addr.
         MOV       CL,0EH                   ; load 0E => (0E)
WR:      MOV       EBX,RTCLOCK
         MOV       [EBX],CL
         JMP       $+2                      ; RTC delay 300ns
         JMP       $+2
         JMP       $+2
         JMP       $+2
         JMP       $+2
         JMP       $+2
         MOV       EBX,RTCLOC2
         MOV       [EBX],AL                 ; write data
         INC       AL                       ; 0F => (0F) ...
         INC       CL                       ; ... 3F => (3F)
         CMP       CL,40H                   ; till last addr. = 3FH
         JNZ       WR                       ; write next data

         MOV       CL,0EH                   ; select 1st addr.
         MOV       DL,0EH
RD:      MOV       EBX,RTCLOCK
         MOV       [EBX],CL
         JMP       $+2                      ; RTC delay 300ns
         JMP       $+2
         JMP       $+2
         JMP       $+2
         JMP       $+2
         JMP       $+2
         MOV       EBX,RTCLOC2
         MOV       AL,[EBX]                 ; read RAM data
         CMP       AL,DL                    ; compare data equal ?
         JNZ       RTCERROR                 ; error if not equal
         INC       DL                       ; point to next addr.
         INC       CL
         CMP       CL,40H                   ; last addr.= 3FH
         JNZ       RD                       ; read next data

         MOV       EDI,303H
         MOV       BYTE PTR [EDI],08H       ; 08H PASS

         JMP       ADDRTST1

RTCERROR:MOV       EDI,303H                 ; 04H ERROR
         MOV       BYTE PTR [EDI],04H


;----------------------------------------------------------------------------+
;                                                                            !
;   [ TEST 6 ]  DRAM ADDRESS LINE TEST : A5 - A9                             !                                     !
;                                                                            !
;        A5 => ADDR. 40 & 10000H ( CAS & RAS signal ), A6 => 80 & 20000      !
;   A7 => 100 & 40000 and so on...                                           !
;        addr. 302h : 10 -> BADSIMM1  20 -> BADSIMM2                         !
;                     40 -> BADSIMM2  80 -> BADSIMM4                         !
;                                                                            !
;----------------------------------------------------------------------------+

ADDRTST1:
         MOV       EBP,40H
         MOV       EBX,EBP
         MOV       ESI,10000H

         MOV       EDI,302H
         MOV       AL,0H                    ; clear
         MOV       [EDI],AL

         MOV       CX,6
RORAD:   MOV       EAX,0
         MOV       DWORD PTR [EBP],0
         MOV       DWORD PTR [ESI],0FFFFFFFFH
         CMP       BYTE PTR [EBP],0
         JE        NXSIM1
         MOV       AL,10H
         OR        [EDI],AL

NXSIM1:  CMP       BYTE PTR [EBP+1],0
         JE        NXSIM2
         MOV       AL,20H
         OR        [EDI],AL

NXSIM2:  CMP       BYTE PTR [EBP+2],0
         JE        NXSIM3
         MOV       AL,40H
         OR        [EDI],AL

NXSIM3:  CMP       BYTE PTR [EBP+3],0
         JE        NXSIM
         MOV       AL,80H
         OR        [EDI],AL
         JMP       NMITEST

NXSIM:   SHL       EBP,1
         SHL       ESI,1
         LOOP      RORAD


;---------------------------------------------------------------------------+
;                                                                           !
;   [ TEST 7 ] NMI & PARITY CHECK TEST                                      !
;                                                                           !
;        Force generate parity bit, clear parity error and then enable      !
;   NMI, if read memory will enter NMI interrupt service routine. The test  !
;   result store semaphore byte at 302H.                                    !
;        Test pass 01 => addr. 302H                                         !
;                                                                           !
;---------------------------------------------------------------------------+

NMITEST:

         MOV       EDI,CPUFLG
         MOV       AX,WORD PTR [EDI]
         AND       AX,0041H                 ; b0=1 force generate parity bit
                                            ; b1=0 clear parity error
                                            ; b6=1 enable NMI
         MOV       WORD PTR [EDI],AX

         MOV       EBP,NMIBIT
         MOV       AL,BYTE PTR [EBP]
         TEST      AL,01H
         JZ        COG

         MOV       AX,1301H                 ; NMI test pass
         CALL      WRITERTC



;----------------------------------------------------------------------------+
;                                                                            |
;   [ TEST 8 ] Detect Interface Control Card Exist and Status                |
;                                                                            |
;    Detect interface control card, set up HSCOFG at addr. 300H              |
;                                                                            |
;           b7   b6   b5   b4   b3   b2   b1   b0                            |
;          4th  3rd  2nd  1st  3rd  2nd  1st                                 |
;          MXF  MXF  MXF  MXF  22C  22C  22C    X                            |
;                                                                            |
;----------------------------------------------------------------------------+


COG:     MOV       CX,0H
         MOV       EBP,HSCOFG               ; 300H,301H
         MOV       [EBP],CX                 ; clear

         MOV       EDI,70000400H            ; 70000400:BOARD-0
         MOV       EBX,'CARD'               ; 400-404
         MOV       [EDI],EBX                ; write
         MOV       EAX,[EDI]                ; read
         XOR       EAX,EBX
         JE        PCB0

CK1:     CALL      EMS
         JE        DISK1

CK2:
         CALL      EMS                      ; Check high speed channel
         JE        DISK2
CK3:
         CALL      EMS
         JE        DISK3
CK4:
         CALL      EMS
         JE        MXF1
CK5:
         CALL      EMS
         JE        MXF2
CK6:
         CALL      EMS
         JE        MXF3
CK7:
         CALL      EMS
         JE        MXF4
         JMP       INTR

PCB0:    OR        CL,01H
         MOV       [EBP],CL
         JMP       CK1

DISK1:
         OR        CL,02H
         MOV       [EBP],CL
         JMP       CK2
DISK2:
         OR        CL,04H
         MOV       [EBP],CL
         JMP       CK3
DISK3:
         OR        CL,08H
         MOV       [EBP],CL
         JMP       CK4
MXF1:
         OR        CL,10H
         MOV       [EBP],CL
         JMP       CK5
MXF2:
         OR        CL,20H
         MOV       [EBP],CL
         JMP       CK6
MXF3:
         OR        CL,40H
         MOV       [EBP],CL
         JMP       CK7
MXF4:
         OR        CL,80H
         MOV       [EBP],CL


;--------------------------------------------------------------------------+
;                                                                          |
;    [ TEST 9 ]  System Interrupt Test                                     |
;                                                                          |
;       CS386 Interrupt 286/controll card, during 286 execute interrupt    |
;    service routine INTERRUPT CS386 again, if success set semaphore at    |
;    addr, 301H.                                                           |
;                                                                          |
;         b7   b6   b5   b4   b3   b2   b1   b0                            |
;        4th  3rd  2nd  1st  3rd  2nd  1st                                 |
;        MXF  MXF  MXF  MXF  22C  22C  22C    X                            |
;                                                                          |
;     During  RUN-IN Mode SKIP interrupt test or no new interface card.    |
;                                                                          |
;--------------------------------------------------------------------------+

INTR:
         MOV       EDI,HSCOFG
         MOV       AL,[EDI]
         CMP       AL,0                     ; without new interface card
         JE        MST                      ; skip int test

         MOV       EDI,CPUST                ; point to CPU status I/O port
         MOV       BP,[EDI]                 ; read  CPU sataus
         TEST      BP,200h                  ; D9=0 loop on BIT is requested
         JZ        FILRUN                   ; Run-in skip this test

         MOV       AX,15                    ; delay time
         CALL      DELAY

         STI                                ; enable int
         MOV       EDI,CPUFLG
         MOV       AX,WORD PTR [EDI]
         AND       AX,00B6H                 ; led off
         MOV       WORD PTR [EDI],AX        ; write word

         MOV       EBP,HSCOFG               ; 300H

         MOV       AL,BYTE PTR [EBP]

         TEST      AL,00000001B
         JZ        NII
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],01H
         POP       EDI

;        MOV       EDI,72000002H            ; 72000002
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL        ; byte strobe to int 286

NII:     TEST      AL,00000010B
         JZ        NI1
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],02H
         POP       EDI

;        MOV       EDI,72100002H            ; 72100002
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL        ; byte strobe to int 286

NI1:     TEST      AL,00000100B
         JZ        NI2
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],04H
         POP       EDI

;        MOV       EDI,72200002H
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL

NI2:     TEST      AL,00001000B
         JZ        NI3
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],08H
         POP       EDI

;        MOV       EDI,72300002H
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL

NI3:     TEST      AL,00010000B
         JZ        NI4
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],10H
         POP       EDI

;        MOV       EDI,72400002H
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL

NI4:     TEST      AL,00100000B
         JZ        NI5
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],20H
         POP       EDI

;        MOV       EDI,72500002H
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL

NI5:     TEST      AL,01000000B
         JZ        NI6
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],40H
         POP       EDI

;        MOV       EDI,72600002H
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL

NI6:     TEST      AL,10000000B
         JZ        HSTEST
         PUSH      EDI
         MOV       EDI,301H
         OR        BYTE PTR [EDI],80H
         POP       EDI

;        MOV       EDI,72700002H
;        MOV       CL,0H
;        MOV       BYTE PTR [EDI],CL

;------------------------------------------------------------------+
;                                                                  !
;    [ TEST 9 ]  Common Memory Test                                |
;          Detect system config. to test control card , if exist   |
;  to write common memory and read back to verify.                 |
;                                                                  !
;------------------------------------------------------------------+

HSTEST:
         MOV       EBP,HSCOFG
         MOV       AL,[EBP]
         TEST      AL,01H
         JZ        MC1
         MOV       ESI,70001000H            ; 1000-3FFFF
         MOV       EBP,70040000H
         MOV       EDI,7000042AH            ; common memory test byte
         CALL      TSTCOM                   ; common memory test

MC1:     TEST      AL,02H
         JZ        MC2
         MOV       ESI,70101000H            ; 1000-3FFFF
         MOV       EBP,70140000H
         MOV       EDI,7010042AH            ; common memory test byte
         CALL      TSTCOM                   ; 42A=11 pass 42A=FF error

MC2:     TEST      AL,04H
         JZ        MC3
         MOV       ESI,70201000H
         MOV       EBP,70240000H
         MOV       EDI,7020042AH
         CALL      TSTCOM

MC3:     TEST      AL,08H
         JZ        MC4
         MOV       ESI,70301000H
         MOV       EBP,70340000H
         MOV       EDI,7030042AH
         CALL      TSTCOM

MC4:     TEST      AL,10H
         JZ        MC5
         MOV       ESI,70401000H
         MOV       EBP,70440000H
         MOV       EDI,7040042AH
         CALL      TSTCOM

MC5:     TEST      AL,20H
         JZ        MC6
         MOV       ESI,70501000H
         MOV       EBP,70540000H
         MOV       EDI,7050042AH
         CALL      TSTCOM

MC6:     TEST      AL,40H
         JZ        MC7
         MOV       ESI,70601000H
         MOV       EBP,70640000H
         MOV       EDI,7060042AH
         CALL      TSTCOM

MC7:     TEST      AL,80H
         JZ        MST
         MOV       ESI,70701000H
         MOV       EBP,70740000H
         MOV       EDI,7070042AH
         CALL      TSTCOM


;-----------------------------------------------------------------------------+
;                                                                             !
;     [ TEST A ]  MEMORY SIZE TEST                                            !
;                                                                             !
;         Test system memory install size , four configurations :             !                      !
;                                                                             !
;             1M SIMM * 4  =>  4M ,  1M SIMM * 8 =>  8M                       !
;             4M SIMM * 4  => 16M ,  4M SIMM * 8 => 32M                       !
;                                                                             !
;-----------------------------------------------------------------------------+

FILRUN:

         MOV       EDI,7040042BH
         MOV       BYTE PTR [EDI],11H

         MOV       AX,5                          ; counter
         CALL      DELAY

MST:     MOV       ESI,10000H
         MOV       EAX,11223344H

LMS:     MOV      [ESI],EAX
         MOV      EBX,[ESI]
         CMP      EAX,EBX
         JNE      M4                               ; 4M
         SHL      ESI,1
         CMP      ESI,400000H
         JBE      LMS

         MOV      ESI,400000H                      ; 4M-8M
         MOV      EAX,4A4B4C4DH
         MOV      [ESI],EAX
         MOV      EBP,ESI
WR4M:    ADD      EBP,10000H
         CMP      EBP,800000H
         JE       M8
         MOV      EBX,[EBP]
         CMP      EAX,EBX
         JNE      WR4M                            ; NOT WRAPE

         SHR      EBP,20                          ; WRAPE
         MOV      AX,BP
         MOV      AH,0EH
         CALL     WRITERTC
         JMP      TSTA

M4:      MOV      AX,0E04H
         CALL     WRITERTC
         JMP      TSTA

M8:      MOV      ESI,0
         MOV      AL,4FH
         MOV      [ESI],AL
         MOV      AH,[EBP]
         CMP      AL,AH
         JNE      M16                           ; NOT WRAPE 16M SIZE
         MOV      AX,0E08H
         CALL     WRITERTC
         JMP      TSTA


M16:     MOV      ESI,800000H                   ; FROM 8M
         MOV      EAX,55667788H
LMS1:    MOV      [ESI],EAX
         MOV      EBX,[ESI]
         CMP      EAX,EBX
         JNE      M16S                            ; 16M SIZE
         SHL      ESI,1
         CMP      ESI,1000000H
         JBE      LMS1

         MOV      ESI,1000000H                          ; 16-32M
         MOV      EAX,99887766H
         MOV      [ESI],EAX                        ;ESI <=1000000H
         MOV      EBP,ESI
WR32M:   ADD      EBP,10000H                      ; NO WRAPE
         CMP      EBP,2000000H
         JE       M32
         MOV      EBX,[EBP]
         CMP      EAX,EBX
         JNE      WR32M

         SHR      EBP,20
         MOV      AX,BP
         MOV      AH,0EH
         CALL     WRITERTC
         JMP      TSTA


M16S:                                            ;16M
         MOV      AX,0E10H
         CALL     WRITERTC
         JMP      TSTA

M32:     MOV      AX,0E20H
         CALL     WRITERTC



;-----------------------------------------------------------------------------+
;                                                                             !
;      [ TEST B ]  DRAM Data Fast Exchange Function TEST                      !
;                                                                             !
;           To test ability of DRAM memory cells to hold '0' or '1' and       !
;    detect dynamic data/addrress faults.                                     !
;                                                                             !
;         BEGIN                                                               !
;             FOR ESI=1st addr. (4000H) TO last address (EDX)                 !                 !
;                  flood 4k bytes                                             !
;                  xchg read 'FF' write '00'                                  !
;                  verify data and inc ESI                                    !
;                  update memory each 4K unit byte                            !
;             NEXT ESI                                                        !
;          END.                                                               !
;                                                                             !
;-----------------------------------------------------------------------------+

TSTA:

         MOV       EBP,0
         MOV       EDI,70400000H
         MOV       BP,WORD PTR DS:[EDI+430H]            ; BP=1600
         ADD       EDI,EBP
         ADD       EDI,2
         MOV       DWORD PTR DS:[MXFSTS],EDI       ; EDI=70401602
         INC       EDI
         MOV       DWORD PTR DS:[MXFABS],EDI       ; EDI=70401603
         ADD       EDI,2
         MOV       DWORD PTR DS:[MXFPAR],EDI       ; EDI=70401605

         MOV       EDI,HSCOFG
         MOV       AL,[EDI]
         TEST      AL,10H                   ; check 1st MXF exist ?
         JNZ       MXF1ST
         CALL      ENABLECRT                ; MXE display

         MOV       EDI,CPUFLG
         MOV       BYTE PTR [EDI],0

         MOV       ESI,OFFSET COPYRIGHT
         MOV       CX,MSG-COPYRIGHT
         CALL      DISPLAYMORE
         JMP       CONT

MXF1ST:

         MOV       ESI,OFFSET COPYRIGHT      ; MXF display
         MOV       CX,MSG-COPYRIGHT
         CALL      MXFDSPY

CONT:    MOV       EDI,303H
         MOV       AL,[EDI]
         CMP       AL,04H                   ; check RTC ok ?
         JNE       CONT1
         MOV       ESI,OFFSET RTCERR         ; RTC error
         MOV       CX,RTCERRL
         CALL      MXEF
         JMP       ERRFLSH

CONT1:   MOV       EDI,302H
         MOV       AL,[EDI]
         TEST      AL,10H
         JZ        LL1
         MOV       ESI,OFFSET MEMF           ; RTC error
         MOV       CX,MEMFL
         CALL      MXEF
         JMP       B1

LL1:     TEST      AL,20H
         JZ        LL2
         MOV       ESI,OFFSET MEMF           ; RTC error
         MOV       CX,MEMFL
         CALL      MXEF
         JMP       B2

LL2:     TEST      AL,40H
         JZ        LL3
         MOV       ESI,OFFSET MEMF           ; RTC error
         MOV       CX,MEMFL
         CALL      MXEF
         JMP       B3

LL3:     TEST      AL,80H
         JZ        CONT2
         MOV       ESI,OFFSET MEMF           ; RTC error
         MOV       CX,MEMFL
         CALL      MXEF
         JMP       B4


CONT2:
         MOV       EDI,CPUST
         MOV       BP,[EDI]
         TEST      BP,200H                  ; D9=1, non-Run In test
         JNZ       NTLOP                    ; normal


         MOV       ESI,OFFSET RUNLOOP       ; display Run-in test
         MOV       CX,RUNLOOPL              ; D9=0
         CALL      MXEF

         MOV       EDI,304H                 ; RUN-IN FLAG
         MOV       BYTE PTR [EDI],01H

         MOV       AX,2000H                 ; RTC RAM 20H contains loop count low byte
         CALL      WRITERTC
         MOV       AX,2100H                 ; RTC RAM 21H contains loop count hi byte
         CALL      WRITERTC

         MOV       AX,2200H                 ; ERROR COUNT contains loop count low byte
         CALL      WRITERTC
         MOV       AX,2300H                 ; ERROR COUNT contains loop count hi byte
         CALL      WRITERTC

NTLOP:   MOV       ESI,OFFSET DRAMSIZE       ; normal mode
         MOV       CX,ENDDRAMSIZE - DRAMSIZE
         CALL      MXEF

         CALL      LASTADDR                 ; upon return, EDX is last DRAM address
         MOV       EDI,40000H               ; set 1st address of DRAM
         MOV       ESI,EDI

         CLD                                ; clear dir flag so string instr incrmnt DI
FILLOP:  MOV       BL,0FFH
         MOV       [EDI],BL
         INC       EDI
         TEST      EDI,0FFFFH
         JNZ       FILLOP

         MOV       CL,0FFH
         MOV       BX,0000H
F64:     MOV       AL,BL
         XCHG      [ESI],AL
         CMP       AL,CL
         JNE       TSTAERR
         MOV       AL,[ESI]
         CMP       AL,BL
         JNE       TSTAERR
         INC       ESI
         TEST      ESI,0FFFFH
         JNZ       F64

         CALL      UPDATESIZE
         CMP       ESI,EDX                  ; do we reach last address?
         JL        FILLOP                   ; no,go on r/w/r test

         MOV       EBX,ESI
         SHR       EBX,16
         MOV       AH,1EH                   ; RTC addr. 1E low byte memory size
         MOV       AL,BL
         CALL      WRITERTC

         MOV       AH,1FH                   ; RTC addr. 1F higt byte memory size
         MOV       AL,BH
         CALL      WRITERTC

         JMP       DIC                      ; end second scan

;-----------------------------------------------------------------------------+
;                                                                             !
;      [ TEST C ]  BAD SIMM DETECT TEST                                       !
;                                                                             !
;          Using LSB of address which is error occurence to detect bad SIMM   !
;    Location, EDX   gs. store memory size, ESI point to error byte.          !
;                                                                             !
;          PCB SIMM     LSB (A0,A1)  MEMORY SIZE COMBINATION                  !
;     -----------------------------------------------------------             !
;           L3              0  0           *    *     @   @                   !
;           L5              0  0                *         @                   !
;                                                               *: 1M SIMM    !
;           L10             0  1           *    *     @   @                   !
;           L15             0  1                *         @     @: 4M SIMM    !
;                                                                             !
;           L18             1  0           *    *     @   @                   !
;           L24             1  0                *         @                   !
;                                                                             !
;           L29             1  1           *    *     @   @                   !
;           L35             1  1                *         @                   !
;                                                                             !
;                                         4M   8M   16M  32M                  !
;                                                                             !
;-----------------------------------------------------------------------------+

TSTAERR:
         PUSH      ESI
         PUSH      EDX

         MOV       ESI,OFFSET TSAER
         MOV       ECX,TSAERR-TSAER
         CALL      MXEF

         POP       EDX
         POP       ESI

         MOV       EBX,ESI
         SHR       EBX,16
         MOV       AH,1EH                   ; RTC addr. 1E low byte memory size
         MOV       AL,BL
         CALL      WRITERTC

         MOV       AH,1FH                   ; RTC addr. 1F higt byte memory size
         MOV       AL,BH
         CALL      WRITERTC

         CMP       EDX,400000H              ; 4M one simm.
         JE        ONESIM
         CMP       EDX,800000H              ; 8M two simm.
         JE        TWOSIM
         CMP       EDX,1000000H             ; 16M one simm.
         JE        ONESIM
TWOSIM:  AND       ESI,11B                  ; 32M two simm.
         CMP       ESI,01B
         JNE       S1
         CALL      L10L15                   ; 01
         JMP       SHFT
S1:      CMP       ESI,10B
         JNE       S2
         CALL      L18L24                   ; 10
         JMP       SHFT
S2:      CMP       ESI,11B
         JNE       S3
         CALL      L29L35                   ; 11
         JMP       SHFT
S3:      CALL      L3L5
         JMP       SHFT

ONESIM:
         AND       ESI,11B
         CMP       ESI,01B
         JNE       S11
B2:      CALL      SIMM                     ; 01
         MOV       AL,31H                   ; L10
         CALL      MXEF1
         MOV       AL,30H
         CALL      MXEF1
         JMP       SHFT

S11:     CMP       ESI,10B
         JNE       S22
B3:      CALL      SIMM                     ; 10
         MOV       AL,31H                   ; L18
         CALL      MXEF1
         MOV       AL,38H
         CALL      MXEF1
         JMP       SHFT

S22:     CMP       ESI,11B                  ; 11
         JNE       B1
B4:      CALL      SIMM
         MOV       AL,32H                   ; L29
         CALL      MXEF1
         MOV       AL,39H
         CALL      MXEF1
         JMP       SHFT

B1:      CALL      SIMM                     ; 00, L3
         MOV       AL,33H
         CALL      MXEF1

SHFT:
         MOV       EDI,304H                 ; #
         CMP       BYTE PTR [EDI],01H       ; #
         JE        RUNER                    ; #

         MOV       ESI,OFFSET KEYSF         ; display shift+reset
         MOV       CX,KEYSFL
         CALL      MXEF

         MOV       EDI,HSCOFG
         MOV       AL,[EDI]
         TEST      AL,10H
         JZ        WAITSK

         MOV       EDI,DWORD PTR DS:[MXFSTS]
WMXF:    MOV       AH,BYTE PTR [EDI]
         CMP       AH,77H
         JNE       WMXF
         MOV       BYTE PTR [EDI],0F0H
         JMP       BOOTS


WAITSK:
         MOV       EDI,CPUFLG
         MOV       BYTE PTR [EDI],0

         MOV       EDI,CPUST
         TEST      WORD PTR [EDI],100H      ; does reset key pressed ?
         JNZ       BOOTS
         JMP       WAITSK

RUNER:
         CALL      ERINC                    ; #
         MOV       ESI,OFFSET LF1
         MOV       CX,LF1L
         CALL      MXEF

;----------------------------------------------------------------+
;                                                                |
;     Display interface control card and diagostic status.       |
;              each card diag. status                            |
;                                                                |
;----------------------------------------------------------------+


DIC:     MOV       EDI,HSCOFG
         MOV       AL,BYTE PTR [EDI]        ; system config.
         MOV       AH,11H                   ; addr. <= 11h
         CALL      WRITERTC

         INC       EDI
         MOV       AL,BYTE PTR [EDI]        ; interrupt test result
         MOV       AH,12H                   ; addr. <= 12h
         CALL      WRITERTC

         MOV       EDI,HSCOFG
         MOV       AL,BYTE PTR [EDI]
         CMP       AL,0                     ; no new interface card
         JE        CAETST

         MOV       ESI,OFFSET SYSMESS        ; display system config.& status
         MOV       CX,SYSLEN
         CALL      MXEF

         MOV       EDI,HSCOFG
         MOV       AL,BYTE PTR [EDI]

         TEST      AL,00000001B
         JZ        NX11
         MOV       ESI,OFFSET PB0
         MOV       CX,PB0L
         CALL      MXEF
         MOV       ESI,70000406H
         CALL      BOARD
         MOV       EBP,70000420H
         CALL      DGST

NX11:    TEST      AL,00000010B
         JZ        NXX1
         MOV       ESI,OFFSET PB1
         MOV       CX,PB1L
         CALL      MXEF
         MOV       ESI,70100406H
         CALL      BOARD
         MOV       EBP,70100420H
         CALL      DGST

NXX1:    TEST      AL,00000100B
         JZ        NXX2
         MOV       ESI,OFFSET PB2
         MOV       CX,PB2L
         CALL      MXEF
         MOV       ESI,70200406H
         CALL      BOARD
         MOV       EBP,70200420H
         CALL      DGST

NXX2:    TEST      AL,00001000B
         JZ        NXX3
         MOV       ESI,OFFSET PB3
         MOV       CX,PB3L
         CALL      MXEF
         MOV       ESI,70300406H
         CALL      BOARD
         MOV       EBP,70300420H
         CALL      DGST

NXX3:    TEST      AL,00010000B
         JZ        NXX4
         MOV       ESI,OFFSET PB1
         MOV       CX,PB1L
         CALL      MXEF
         MOV       ESI,70400406H
         CALL      BOARD
         MOV       EBP,70400420H
         CALL      DGST

NXX4:    TEST      AL,00100000B
         JZ        NXX5
         MOV       ESI,OFFSET PB2
         MOV       CX,PB2L
         CALL      MXEF
         MOV       ESI,70500406H
         CALL      BOARD
         MOV       EBP,70500420H
         CALL      DGST

NXX5:    TEST      AL,01000000B
         JZ        NXX6
         MOV       ESI,OFFSET PB3
         MOV       CX,PB3L
         CALL      MXEF
         MOV       ESI,70600406H
         CALL      BOARD
         MOV       EBP,70600420H
         CALL      DGST

NXX6:    TEST      AL,10000000B
         JZ        NXX7
         MOV       ESI,OFFSET PB4
         MOV       CX,PB4L
         CALL      MXEF
         MOV       ESI,70700406H
         CALL      BOARD
         MOV       EBP,70700420H
         CALL      DGST

NXX7:
         MOV       EDI,CPUST                ; point to CPU status I/O port
         MOV       BP,[EDI]                 ; read  CPU sataus
         TEST      BP,200h                  ; D9=1 loop on BIT is requested
         JZ        CAETST                   ; skip interrupt test

;--------------------------------------------------------------------+
;                                                                    |
;     Display interrupt test result. 386 interrupt 286 and then 286  |
;   will interrupt 386. Address 301H is interrupt test result.       |
;                                                                    |
;     Display High Speed Channel Test Result                         |
;                                                                    |
;--------------------------------------------------------------------+

         MOV       ESI,OFFSET COMT
         MOV       CX,COMTL
         CALL      MXEF

         MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,01H
         JZ        NTN1

         MOV       ESI,OFFSET PB0
         MOV       CX,PB0L
         CALL      MXEF
         MOV       ESI,70000406H
         CALL      BOARD

         MOV       AL,[EDI+1]
         TEST      AL,01H
         JZ        INTERR

         MOV       EBP,7000042AH
         CALL      RESCOM
         JMP       NTN1

INTERR : CALL      ERR

NTN1:    MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,02H
         JZ        NTT1

         MOV       ESI,OFFSET PB1
         MOV       CX,PB1L
         CALL      MXEF
         MOV       ESI,70100406H
         CALL      BOARD

         MOV       AL,[EDI+1]                      ; check int test
         TEST      AL,02H
         JZ        INTERR1

         MOV       EBP,7010042AH                   ; common memory test
         CALL      RESCOM
         JMP       NTT1

INTERR1: CALL      ERR

NTT1:
         MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,04H
         JZ        NTT2

         MOV       ESI,OFFSET PB2
         MOV       CX,PB2L
         CALL      MXEF
         MOV       ESI,70200406H
         CALL      BOARD

         MOV       AL,[EDI+1]
         TEST      AL,04H
         JZ        INTERR2

         MOV       EBP,7020042AH
         CALL      RESCOM

         JMP       NTT2


INTERR2: CALL      ERR


NTT2:    MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,08H
         JZ        NTT3

         MOV       ESI,OFFSET PB3
         MOV       CX,PB3L
         CALL      MXEF
         MOV       ESI,70300406H
         CALL      BOARD

         MOV       AL,[EDI+1]
         TEST      AL,08H
         JZ        INTERR3

         MOV       EBP,7030042AH
         CALL      RESCOM

         JMP       NTT3

INTERR3: CALL      ERR


NTT3:    MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,10H
         JZ        NTT4

         MOV       ESI,OFFSET PB1
         MOV       CX,PB1L
         CALL      MXEF
         MOV       ESI,70400406H
         CALL      BOARD

         MOV       AL,[EDI+1]
         TEST      AL,10H
         JZ        INTERR4

         MOV       EBP,7040042AH
         CALL      RESCOM

         JMP       NTT4

INTERR4: CALL      ERR


NTT4:    MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,20H
         JZ        NTT5

         MOV       ESI,OFFSET PB2
         MOV       CX,PB2L
         CALL      MXEF
         MOV       ESI,70500406H
         CALL      BOARD

         MOV       AL,[EDI+1]
         TEST      AL,20H
         JZ        INTERR5

         MOV       EBP,7050042AH
         CALL      RESCOM

         JMP       NTT5

INTERR5: CALL      ERR


NTT5:    MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,40H
         JZ        NTT6

         MOV       ESI,OFFSET PB3
         MOV       CX,PB3L
         CALL      MXEF
         MOV       ESI,70600406H
         CALL      BOARD

         MOV       AL,[EDI+1]
         TEST      AL,40H
         JZ        INTERR6

         MOV       EBP,7060042AH
         CALL      RESCOM

         JMP       NTT6

INTERR6: CALL      ERR


NTT6:    MOV       EDI,300H
         MOV       AL,[EDI]
         TEST      AL,80H
         JZ        CAETST

         MOV       ESI,OFFSET PB4
         MOV       CX,PB4L
         CALL      MXEF
         MOV       ESI,70700406H
         CALL      BOARD

         MOV       AL,[EDI+1]
         TEST      AL,80H
         JZ        INTERR7

         MOV       EBP,7070042AH
         CALL      RESCOM

         JMP       CAETST


INTERR7: CALL      ERR


;-----------------------------------------------------------------------------+
;                                                                             !
;      [ TEST C ]  CACHE MEMORY TEST                                          !
;                                                                             !
;-----------------------------------------------------------------------------+

CAETST:  MOV       ESI,200000H              ; 2M
         MOV       EAX,11220000H
         MOV       CX,2000H                 ; 8K DW FILL two way associate
P1FIL:   MOV       [ESI],EAX
         ADD       ESI,4
         INC       EAX
         LOOP      P1FIL

         MOV       CX,2000H                 ; fil 8K DW cache two associate
         MOV       EAX,33440000H
P2FIL:   MOV       [ESI],EAX                ; 208000H
         ADD       ESI,4
         INC       EAX
         LOOP      P2FIL

         MOV       CX,2000H
         MOV       EAX,55660000H
P3FIL:   MOV       [ESI],EAX                ; 210000H
         ADD       ESI,4
         INC       EAX
         LOOP      P3FIL

         MOV       CX,2000H
         MOV       EAX,77880000H
P4FIL:   MOV       [ESI],EAX                ; 218000H
         ADD       ESI,4
         INC       EAX
         LOOP      P4FIL

         MOV       AL,0001B                 ; 2M non-cache
         MOV       EDI,NCA                  ;
         MOV       [EDI],AL                 ; enable cache memory

         MOV       ESI,200000H              ; 2M
         MOV       EAX,11220000H
         MOV       CX,2000H                 ; 32K DW FILL two way associate
P1FILL:  MOV       EBX,[ESI]
         CMP       EAX,EBX
         JNE       CACHERR
         ADD       ESI,4
         INC       EAX
         LOOP      P1FILL


         MOV       EAX,33440000H
         MOV       CX,2000H                 ; 32K DW FILL two way associate
P2FILL:  MOV       EBX,[ESI]
         CMP       EAX,EBX
         JNE       CACHERR
         ADD       ESI,4
         INC       EAX
         LOOP      P2FILL

         MOV       EAX,55660000H
         MOV       CX,2000H                 ; 32K DW FILL two way associate
P3FILL:  MOV       EBX,[ESI]
         CMP       EAX,EBX
         JNE       CACHERR
         ADD       ESI,4
         INC       EAX
         LOOP      P3FILL

         MOV       EAX,77880000H
         MOV       CX,2000H                 ; 32K DW FILL two way associate
P4FILL:  MOV       EBX,[ESI]
         CMP       EAX,EBX
         JNE       CACHERR
         ADD       ESI,4
         INC       EAX
         LOOP      P4FILL
         JMP       TSS1

CACHERR:
         MOV       ESI,OFFSET CACERR
         MOV       CX,CACERRL
         CALL      MXEF

         MOV       EDI,304H                 ; #
         CMP       BYTE PTR [EDI],01H       ; #
         JE        RUNER1                   ; #

         MOV       ESI,OFFSET KEYSF
         MOV       CX,KEYSFL
         CALL      MXEF

         MOV       EDI,HSCOFG
         MOV       AL,[EDI]
         TEST      AL,10H
         JZ        WAITK

         MOV       EDI,DWORD PTR DS:[MXFSTS]
WMXF1:   MOV       AH,BYTE PTR [EDI]
         CMP       AH,77H
         JNE       WMXF1
         MOV       BYTE PTR [EDI],0F0H
         JMP       BOOTS

WAITK:
         MOV       EDI,CPUFLG
         MOV       BYTE PTR [EDI],0

         MOV       EDI,CPUST
         TEST      WORD PTR [EDI],100H       ; does reset key pressed ?
         JNZ       BOOTS
         JMP       WAITK

RUNER1:
         CALL      ERINC                    ; #
         MOV       ESI,OFFSET LF2
         MOV       CX,LF2L
         CALL      MXEF

TSS1:
         MOV       EBP,NMIBIT
         MOV       AL,BYTE PTR [EBP]
         TEST      AL,01H
         JZ        JMPRAM

         MOV       AX,1301H
         CALL      WRITERTC

JMPRAM:  MOV       BX,10H                   ; move test program to RAM at 2000H
         MOV       ES,BX

         DB        0EAH
         DW        0000H
         DW        0010H

; Flat_des        desc<0ffffh,0,0,92h,0cfh,0>               08 : DS BASE 0
; CS_DES          DESC<0FFFFH,2000H,0H,9AH,8FH,0H>          10 : CS BASE 00002000
; CS1_DES         DESC<0FFFFH,0000H,0FFH,9AH,8FH,0FFH>      18 : CS BASE FFFF0000


;--------  MOVE TEST PROGRAM TO RAM  ---------------

TSSTEST LABEL BYTE

         MOV       BX,18H                   ; 18 DS SELECTOR
         MOV       ES,BX

         MOV       ESI,OFFSET MSG
         MOV       CX,MSGLEN
         CALL      MXEF

         MOV       ESI,OFFSET DRAMSIZE
         MOV       ECX,ENDDRAMSIZE - DRAMSIZE
         CALL      MXEF

TSTAA:
         MOV       AL,0                     ;
         MOV       EDI,NCA                  ;
         MOV       [EDI],AL                 ; enable all cache memory

         CALL      LASTADDR                 ; upon return, EDX is last DRAM address
         MOV       EDI,40000H               ; set 1st address of DRAM
         MOV       ESI,EDI
         CLD                                ; clear dir flag so string instr incrmnt DI
TSTALOP11:
         MOV       BL,0FFH
         MOV       [EDI],BL                 ; write data to memory
         INC       EDI                      ; point to next address
         TEST      EDI,0FFFFH               ; @@@ do we at 128KB boundry?
         JNZ       TSTALOP11

         MOV       CL,0FFH                  ; EBP,0FFFFFFFFH  ; set last written patt
         MOV       BX,0000H                 ; set revers pattern for write
TSTALOP22:
         MOV       AL,BL                    ; get revers pattern
         XCHG      [ESI],AL                 ; read 'FFFFFFFF',write '00000000'
         CMP       AL,CL                    ; EAX,EBP         ; verify data
         JNE       TSTAERR1                 ; jump if bad (no jump is fast)
         MOV       AL,[ESI]                 ; read '00000000' back

         CMP       AL,BL                    ; EAX,EBX         ; verify data
         JNE       TSTAERR1                 ; jump if bad (no jump is fast)
         INC       ESI                      ; point ESI to next address
         TEST      ESI,0FFFFH                ; @@@ do we at 128KB boundry?
         JNZ       TSTALOP22                ; no, go on r/w/r test

         CALL      UPDATESIZE
         CMP       ESI,EDX                  ; do we reach last address?
         JL        TSTALOP11                ; no,go on r/w/r test

         MOV       EDI,CPUST                ; point to CPU status I/O port
         MOV       BP,[EDI]                 ; read  CPU sataus
         TEST      BP,200h                  ; D9=1 loop on BIT is requested
         JNZ       BOOTS                    ; From here to LOADER


NTLOP1:

         MOV       ESI,OFFSET ERRCONT       ; #
         MOV       CX,ERRCONTL-ERRCONT      ; # count bytes
         CALL      MXEF                     ; # position cursor to least significant digit

         MOV       AX,2200h                 ; #RTC 22h contains loopcount low byte
         CALL      READRTC
         MOV       BL,AL                    ; #save loopcount low byte
         MOV       AX,2300h                 ; #RTC 23h contains loopcount hi byte
         CALL      READRTC
         MOV       BH,AL                    ; #save loopcount hi byte

         MOV       AX,BX                    ; # put loopcount to AX for display
         CALL      UPDATENUM                ; # display loopcount at terminal srceen

         MOV       ESI,OFFSET BLANK         ; #
         MOV       CX,BLANKL                ; #
         CALL      MXEF                     ; #
                                            ; #
         MOV       BP,18                    ; #
CLR:     MOV       ESI,OFFSET BKLINE        ; #
         MOV       CX,BKLINEL               ; #
         CALL      MXEF                     ; #
         DEC       BP                       ; #
         JNZ       CLR                      ; #


         MOV       AX,2000h                 ; RTC 20h contains loopcount low byte
         CALL      READRTC
         MOV       BL,AL                    ; save loopcount low byte
         MOV       AX,2100h                 ; RTC 21h contains loopcount hi byte
         CALL      READRTC
         MOV       BH,AL                    ; save loopcount hi byte

         INC       BX                       ; increase loop count
         MOV       AH,20h                   ; RTC RAM 20h contains loopcount low byte
         MOV       AL,BL                    ; update loopcount
         CALL      WRITERTC
         MOV       AH,21h                   ; RTC RAM 21h contains loopcount hi byte
         MOV       AL,BH                    ; update loopcount
         CALL      WRITERTC

         MOV       ESI,OFFSET LOOPCONT
         MOV       CX,ENDLOOPCONT - LOOPCONT      ;count bytes
         CALL      MXEF                     ; position cursor to least significant digit
         MOV       AX,BX                    ; put loopcount to AX for display
         CALL      UPDATENUM                ; display loopcount at terminal srceen


         MOV       ECX,0FFFFFh              ; time delay in order user
         LOOP      $                        ; can see the last DRAM size on screen

         MOV       ESI,OFFSET DRAMCLEN
         MOV       CX,ENDDRAMCLEN - DRAMCLEN      ;count bytes
         CALL      MXEF                     ; clean msg DRAM ???? KB on screen

         DB        0EAH
         DW        OFFSET NTLOP
         DW        18H

TSTAERR1:

         MOV       ESI,OFFSET CACERR
         MOV       CX,CACERRL
         CALL      MXEF

         MOV       EDI,304H                 ; #
         CMP       BYTE PTR [EDI],01H       ; #
         JE        RUNER2                   ; #

         MOV       ESI,OFFSET KEYSF
         MOV       CX,KEYSFL
         CALL      MXEF

         MOV       EDI,HSCOFG
         MOV       AL,[EDI]
         TEST      AL,10H
         JZ        WAITSK1

         MOV       EDI,DWORD PTR DS:[MXFSTS]
WMXF11:  MOV       AH,BYTE PTR [EDI]
         CMP       AH,77H
         JNE       WMXF11
         MOV       BYTE PTR [EDI],0F0H
         JMP       BOOTS


WAITSK1:

         MOV       EDI,CPUFLG
         MOV       BYTE PTR [EDI],0
         MOV       EDI,CPUST
         TEST      WORD PTR [EDI],100H       ; does reset key pressed ?
         JNZ       BOOTS
         JMP       WAITSK1


BOOTS:
         MOV       ECX,0FFFFFH               ; delay time
         LOOP      $

         DB        0EAH
         DW        OFFSET START_WR
         DW        18H

RUNER2:
         CALL      ERINC                    ; #
         DB        0EAH                     ; #
         DW        OFFSET NTLOP1            ; #
         DW        18H                      ; #

BOTS:
         MOV       AL,0
         MOV       EDI,NCA
         MOV       [EDI],AL                 ; enable all cache memory

;-------------------------------------------------+
;      FLOOD MEMORY                               !
;-------------------------------------------------+

         MOV       EDI,CPUST
         MOV       AX,WORD PTR [EDI]
         MOV       EDI,CPUFLG
         MOV       BYTE PTR [EDI],00H
         MOV       CX,4000H
         LOOP      $

         MOV       EDI,_ABS
         MOV       AL,05H
         MOV       BYTE PTR [EDI],AL


         MOV       ESI,OFFSET FMEM
         MOV       ECX,FMEML
         CALL      MXESF

         CALL      LASTADDR

         MOV       EDI,200000H
AGFL:    MOV       ECX,4000H

FLOD:    MOV       DWORD PTR [EDI],0
         ADD       EDI,4
         LOOP      FLOD
         ADD       EDI,10000H
         CMP       EDI,EDX
         JL        AGFL

         SHR       EDX,16
         MOV       AH,1EH
         MOV       AL,DL
         CALL      WRITERTC

         MOV       AH,1FH
         MOV       AL,DH
         CALL      WRITERTC

         DB        0EAH
         DW        OFFSET START_WR
         DW        18H


;**********************    Call Subroutins    *************************

PASS     PROC     NEAR
         MOV       ESI, OFFSET GSS
         MOV       CX,GSSL
         CALL      MXEF
         RET
PASS     ENDP

;-------------------------------------

ERR      PROC     NEAR
         MOV       ESI,OFFSET GERROR
         MOV       CX,GERRORL
         CALL      MXEF
         RET
ERR      ENDP

;----------------------------------------

BOARD     PROC      NEAR
         PUSH      ES
         PUSH      AX
         MOV       CX,11
         MOV       AX,08H
         MOV       ES,AX
         CALL      MXEF
         POP       AX
         POP       ES
         RET
BOARD     ENDP

;--------------------------------------------------

RESCOM    PROC      NEAR
         PUSH      AX
         MOV       AL,[EBP]                ; RBP=42A
         CMP       AL,11H
         JNE       ERCOM
         CALL      PASS
         JMP       EPS
ERCOM:   CALL      ERR
EPS:     POP       AX
         RET

RESCOM    ENDP

;---------------------------------------------------

MXEF       PROC      NEAR                   ; display MXF or MXE, strings
         PUSH      EDI
         MOV       EDI,HSCOFG               ; semaphore byte
         TEST      BYTE PTR [EDI],10H       ; 1st MXF exist ?
         JZ        MXEDY                    ; MXE display
         CALL      MXFDSPY                  ; MXF display
         JMP       RETN
MXEDY:   CALL      DISPLAYMORE
RETN:    POP       EDI
         RET
MXEF       ENDP

;-------------------------------------------------------------------

MXEF1      PROC      NEAR                   ; display 1 character
         PUSH      EDI
         MOV       EDI,HSCOFG
         TEST      BYTE PTR [EDI],10H
         JZ        MXEDY1                   ; MXE
         CALL      MXFDSPY1                 ; MXF
         JMP       RETN1
MXEDY1:  CALL      DISPLAYONE
RETN1:   POP       EDI
         RET
MXEF1      ENDP

;--------------------------------------------------------------------

DISPLAYMORE     PROC    NEAR
         PUSH    EAX
DISPLAYMLOP:
         MOV       AL,ES:[ESI]               ; ESI point to 1st byte of string
         CALL      DISPLAYONE
         INC       ESI
         LOOP      DISPLAYMLOP
         POP       EAX
         RET
DISPLAYMORE     ENDP

;---------------------------------------------------------------------

DISPLAYONE    PROC    NEAR
         CALL      WAITRESET
         CALL      CHKIORDY                 ; be sure CRT is ready for next send
         PUSH      EDI
         MOV       EDI,_OBS                 ; point to output data bus strobe
         MOV       [EDI],AL                 ; on entry: AL, send out ASCII to be display

         POP       EDI
         RET
DISPLAYONE    ENDP

;---------------------------------------------------------------------------

WAITRESET   PROC   NEAR

        PUSH    EDI
        MOV     EDI,_ABS
        MOV     BYTE PTR [EDI],01H

        PUSH    ECX
        MOV     ECX,0FFH
        LOOP    $
        POP     ECX


        MOV     EDI,CPUST                               ; CPUFLG D4=0 SET
        TEST    WORD PTR [EDI],100H
        JZ      DSP
        JMP     BOTS

DSP:    MOV     EDI,_ABS
        MOV     BYTE PTR [EDI],05H
        POP     EDI

        RET

WAITRESET     ENDP

;-----------------------------------------------------------------------

ENABLECRT       PROC    NEAR
         PUSH      EDI
         MOV       EDI,_ABS                ; point to I/O address bus strobe
         MOV       AL,5                     ; device CRT's address is 05H
         MOV       [EDI],AL
         CALL      CHKIORDY                 ; make sure I/O bus is ready
         POP       EDI
         RET
ENABLECRT       ENDP

;---------------------------------------------------------------------

DELAY           PROC    NEAR

         MOV       CX,AX                    ; set up counter for time delay
WAITMXF:
         PUSH      CX
         MOV       CX,0
L1:      AAM
         AAM
         LOOP      L1
         POP       CX
         LOOP      WAITMXF                  ; 83 ck x .033us x 65536 x 7
         RET

DELAY           ENDP

;----------------------------------------------------------------

CHKIORDY        PROC    NEAR
                                            ; purpose: check CPUST D3=1 as I/O ready
         PUSH      EDI
         MOV       EDI,CPUST
         TEST      BYTE PTR [EDI],08H       ; D3=0,I/O not ready;D3=1,I/O raedy
         POP       EDI
         JZ        CHKIORDY                 ; if not ready, let's wait
         RET
CHKIORDY        ENDP

;------------------------------------------------------------------------

UPDATENUM       PROC    NEAR

;purpose: convert word hex number to decimal and display it.
;on entry: AX contains hex number, cursor points to least signif digit.

         PUSH      EAX
         PUSH      EBX
         PUSH      ECX
         PUSH      EDX

         MOV       EBX,10                   ; set divisor
         MOV       ECX,4                    ; maximun 5 digits, there are 4 times DIV
UPDATENUM1:
         MOV       EDX,0                    ; clear remainder
         DIV       BX                       ; (DX+AX)/BX=AX+DX
         ADD       DL,30h                   ; convert remainder to ascii
         XCHG      AX,DX                    ; put ascii to AX for display
         CALL      MXEF1                    ; display this ascii to screen

         MOV       AL,08h                   ; ascii 08h is back space
         CALL      MXEF1                    ; mov cursor back
         MOV       AL,08h                   ; ascii 08h is back space
         CALL      MXEF1                    ; set cursor to next digit position

         CMP       EDX,0                    ; is quotient zero?
         JE        UPDATENUM2               ; it is zero, no more display

         MOV       AX,DX                    ; restore quotien for next div
         LOOP      UPDATENUM1               ; continue division

         ADD       AL,30h                   ; convert last quotient to ascii
         CALL      MXEF1                    ; display this ascii to screen

UPDATENUM2:

         POP       EDX
         POP       ECX
         POP       EBX
         POP       EAX
         RET

UPDATENUM       ENDP

;--------------------------------------------------------------------

DISPLAYMORE1   PROC    NEAR
         PUSH    EAX
DISPLAYMLOP1:
         MOV       AL,ES:[ESI]               ; ESI point to 1st byte of string
         CALL      DISPLAYONE1
         INC       ESI
         LOOP      DISPLAYMLOP1
         POP       EAX
         RET
DISPLAYMORE1    ENDP

;---------------------------------------------------------------------

DISPLAYONE1    PROC    NEAR
         CALL      CHKIORDY                 ; be sure CRT is ready for next send
         PUSH      EDI
         MOV       EDI,_OBS                 ; point to output data bus strobe
         MOV       [EDI],AL                 ; on entry: AL, send out ASCII to be display
         POP       EDI
         RET
DISPLAYONE1    ENDP

;---------------------------------------------------------------------------

UPDATESIZE      PROC    NEAR

;purpose: show dram memory size on screen in 128KB increasment
;on entry:none on register, but cursor point to least significant digit
;on exit :none, but cursor point to least significsnt digit

         PUSH      ESI
         PUSH      EDX
         PUSH      ECX
         PUSH      EAX

         MOV       EAX,ESI                  ; save surrent address
         MOV       ESI,OFFSET DRAMLOOP       ;
         MOV       ECX,ENDDRAMLOOP - DRAMLOOP      ;count bytes
         CALL      MXEF                     ; reset cursor at least significant digit

         MOV       ECX,1024                 ; divided by 1024
         MOV       EDX,0                    ; clear remainder
         DIV       ECX                      ; (EDX+EAX)/ECX= EAX + EDX
         CALL      UPDATENUM                ; convert size to decimal and display

         POP       EAX
         POP       ECX
         POP       EDX
         POP       ESI
         RET

UPDATESIZE      ENDP

;-----------------------------------------------------------------------

LASTADDR        PROC    NEAR

         PUSH      EBP             ; edx last addr.
         MOV       EBP,RTCLOCK     ; point to RTC address select port
         MOV       EDX,0Eh         ; point to 1st byte on user RAM
         MOV       [EBP],DL        ;
         MOV       EBP,RTCLOC2     ; point to RTC data r/w port
         MOV       DL,[EBP]        ; read DRAM size from RTC RAM
         SHL       EDX,20          ; convert size to flat address
         POP       EBP
         RET

LASTADDR        ENDP


;--------------------------------------------------------------------------+
;The RTC user RAM address starts from 0Eh to FFh, total 50 bytes.          *
;Diagnostic reserved following address:                                    *
;       [0Eh]= the DRAM memory size in unit of MB,bootstrap will read it   *
;       [0Fh]= boot ID high byte                                           *
;       [10H]= boot ID low byte                                            *
;       [11h]= system config           ; 300H semaphore byte               *
;       [12h]= interrupt test result   ; 301H semaphore byte               *
;       [13H]= NMI test result         ; 302H                              *
;       [20h]= loop count low byte                                         *
;       [21h]= loop count hi byte                                          *
;--------------------------------------------------------------------------+

WRITERTC        PROC    NEAR

;on entry: AH= RTC user RAM address (0E-FF),
;          AL= data to be written to RTC user RAM

         PUSH      EBP
         MOV       EBP,RTCLOCK              ; point to RTC address select port
         MOV       [EBP],AH                 ; AH is the user ram address
         JMP       $+2                      ; delay
         JMP       $+2
         JMP       $+2
         JMP       $+2
         JMP       $+2
         JMP       $+2
         MOV       EBP,RTCLOC2              ; point to RTC data r/w port
         MOV       [EBP],AL                 ; write data code to RTC RAM
         POP       EBP
         RET

WRITERTC        ENDP

;------------------------------------------------------------------------

READRTC         PROC    NEAR
;on entry: AH is the address of RTC user RAM
;on exit : AL is the data read from RTC user RAM at address 'AH'

         PUSH      EBP
         MOV       EBP,RTCLOCK              ; point to RTC address select port
         MOV       [EBP],AH                 ; AH is the user ram address
         JMP       $+2                      ; delay
         JMP       $+2
         JMP       $+2
         JMP       $+2                      ; delay
         JMP       $+2
         JMP       $+2
         MOV       EBP,RTCLOC2              ; point to RTC data r/w port
         MOV       AL,[EBP]                 ; read data code from RTC RAM
         POP       EBP
         RET
READRTC         ENDP

;----------------------------------------------------------------------------

ERINC    PROC      NEAR
         MOV       AX,2200h                 ; #RTC 22h contains loopcount low byte
         CALL      READRTC

         MOV       BL,AL                    ; #save loopcount low byte
         MOV       AX,2300h                 ; #RTC 23h contains loopcount hi byte
         CALL      READRTC

         MOV       BH,AL                    ; #save loopcount hi byte

         INC       BX                       ; #increase loop count
         MOV       AH,22h                   ; #RTC RAM 22h contains loopcount low byte
         MOV       AL,BL                    ; #update loopcount
         CALL      WRITERTC
         MOV       AH,23h                   ; #RTC RAM 23h contains loopcount hi byte
         MOV       AL,BH                    ; #update loopcount
         CALL      WRITERTC

         MOV       ESI,OFFSET ERRCONT       ; #
         MOV       CX,ERRCONTL-ERRCONT      ; # count bytes
         CALL      MXEF                     ; # position cursor to least significant digit
         MOV       AX,BX                    ; # put loopcount to AX for display
         CALL      UPDATENUM                ; # display loopcount at terminal srceen
         RET
ERINC    ENDP

;--------------------------------------------

EMS      PROC      NEAR
         ADD       EDI,100000H              ; increase 100000H check next board
         MOV       [EDI],EBX
         MOV       EAX,[EDI]
         XOR       EAX,EBX
         RET
EMS      ENDP

;-------------------------------------------------

SIMM     PROC      NEAR                     ; display bad SIMM
         PUSH      ESI
         PUSH      CX
         MOV       ESI,OFFSET BADSIMM
         MOV       CX,BADSIMML
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
SIMM     ENDP

;------------------------------------------------------

L10L15   PROC      NEAR                     ; L10, L15 error
         PUSH      ESI
         PUSH      CX
         MOV       ESI,OFFSET SM2
         MOV       CX,SM2L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L10L15   ENDP

;-------------------------------------------------

L18L24   PROC      NEAR                     ; L18, L24 bad location
         PUSH      ESI
         PUSH      CX
         MOV       ESI,OFFSET SM3
         MOV       CX,SM3L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L18L24   ENDP

;----------------------------------------------

L29L35   PROC      NEAR                     ; L29, L35
         PUSH      ESI
         PUSH      CX
         MOV       ESI,OFFSET SM4
         MOV       CX,SM4L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L29L35   ENDP

;-------------------------------------------------

L3L5     PROC      NEAR                     ; L3, L5
         PUSH      ESI
         PUSH      CX
         MOV       ESI,OFFSET SM1
         MOV       CX,SM1L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L3L5     ENDP

;-----------------------------------------------------------

DSYCARD  PROC      NEAR                     ; display interface card
         MOV       ECX,31
         CALL      MXEF
         RET
DSYCARD  ENDP

;--------------------------------------------------------

TSTCOM   PROC      NEAR
         PUSH      AX
MM:      MOV       EBX,55AAAA55H
         MOV       [ESI],EBX
         MOV       EAX,[ESI]
         CMP       EAX,EBX
         JNE       COMER
         ADD       ESI,800H
         CMP       ESI,EBP
         JL        MM
         MOV       BYTE PTR [EDI],11H              ; EDI=42A
         JMP       COMOK
COMER:   MOV       BYTE PTR [EDI],0FFH
COMOK:   POP       AX
         RET
TSTCOM   ENDP

;-----------------------------------------------------------

; MXF display format :
;            ADDR.     STS  ABS  CBS  Counter,  String
; at addr. 70401602H => 0F,  07,  FF,  CL,    characters

MXFDSPY  PROC      NEAR
         PUSH      EDI
         PUSH      ESI
         PUSH      EAX
         MOV       EDI,DWORD PTR DS:[MXFSTS]
CHKSTS:  MOV       AH,BYTE PTR [EDI]
         CMP       AH,077H
         JNE       CHKSTS1
         MOV       BYTE PTR [EDI],0F0H
         JMP       BOTS                            ; BOTS      ; jmp to loader
CHKSTS1: AND       AH,0F0H
         CMP       AH,0F0H
         JNE       CHKSTS
         MOV       EDI,DWORD PTR DS:[MXFABS]
         MOV       WORD PTR [EDI],ABS7
         MOV       EDI,DWORD PTR DS:[MXFPAR]
         MOV       BYTE PTR [EDI],CL
         INC       EDI
STRNG:   MOV       AL,ES:[ESI]               ; MOVB
         MOV       [EDI],AL
         INC       ESI
         INC       EDI
         LOOP      STRNG
         MOV       EDI,DWORD PTR DS:[MXFSTS]
         MOV       BYTE PTR [EDI],0FH
         POP       EAX
         POP       ESI
         POP       EDI
         RET
MXFDSPY  ENDP

;---------------------------------------------------

;  MXF display one character

MXFDSPY1   PROC      NEAR
         PUSH      EDI
         PUSH      ESI
         PUSH      EAX
         MOV       EDI,DWORD PTR DS:[MXFSTS]
CHKSTS3: MOV       AH,BYTE PTR [EDI]
         CMP       AH,077H
         JNE       CHKSTS2
         MOV       BYTE PTR [EDI],0F0H
         JMP       BOTS                        ; BOTS            ; jmp to loader
CHKSTS2: AND       AH,0F0H
         CMP       AH,0F0H
         JNE       CHKSTS3
         MOV       EDI,DWORD PTR DS:[MXFABS]
         MOV       WORD PTR [EDI],ABS7
         MOV       EDI,DWORD PTR DS:[MXFPAR]
         MOV       BYTE PTR [EDI],01H
         INC       EDI
         MOV       [EDI],AL
         MOV       EDI,DWORD PTR DS:[MXFSTS]
         MOV       BYTE PTR [EDI],0FH
         POP       EAX
         POP       ESI
         POP       EDI
         RET
MXFDSPY1  ENDP

;------------------------------------------------------

MXESF      PROC       NEAR

         PUSH      EDI
         MOV       EDI,HSCOFG
         TEST      BYTE PTR [EDI],10H
         JZ        MXECRT
         CALL      MXFDSPY
         JMP       RTN
MXECRT:  CALL      DISPLAYMORE1
RTN:     POP       EDI
         RET
MXESF       ENDP


;--------------------------------------------------------------+
;                                                              !
;   Semaphore byte at 0EH of each interface card.              !
;                                                              !
;      '80' : diag PASS        '60' : No print ( 22C11 )       !
;                                   : NO CRT ( MXF )           !
;      '40' : Run-in PASS      '20' : ERROR                    !
;                                                              !
;--------------------------------------------------------------+

DGST       PROC     NEAR
         PUSH      AX
         MOV       AL,[EBP]
         CMP       AL,80H
         JNE       RUINP
         CALL      PASS
         JMP       QR
RUINP:   CMP       AL,60H
         JNE       NOCON
         CMP       EBP,70300420H
         JBE       DKPTR
         MOV       ESI,OFFSET NOCRT          ; NO CRT '60'
         MOV       CX,NOCRTL
         CALL      MXEF
         JMP       QR
DKPTR:   MOV       ESI,OFFSET NOPTR          ; NO PTR '60'
         MOV       CX,NOPTRL
         CALL      MXEF
         JMP       QR
NOCON:   CMP       AL,40H                   ; RUN-IN '40', PASS
         JNE       DKERR
         MOV       ESI,OFFSET GRUN
         MOV       CX,GRUNL
         CALL      MXEF
         JMP       QR
DKERR:
         MOV       EDI,304H
         CMP       BYTE PTR [EDI],01H
         JNE       NORUN

         MOV       AX,2200H
         CALL      READRTC
         MOV       BL,AL
         MOV       AX,2300H
         CALL      READRTC
         MOV       BH,AL
         INC       BX
         MOV       AH,22H
         MOV       AL,BL
         CALL      WRITERTC
         MOV       AH,23H
         MOV       AL,BH
         CALL      WRITERTC

NORUN:
         CALL      ERR
         MOV       AH,[EBP]
         CMP       AH,21H
         JNE       NXER
         MOV       AL,31H
         CALL      MXEF1

NXER:    CMP       AH,22H
         JNE       NXER1
         MOV       AL,32H
         CALL      MXEF1

NXER1:   CMP       AH,23H
         JNE       NXER2
         MOV       AL,33H
         CALL      MXEF1

NXER2:   CMP       AH,24H
         JNE       NXER3
         MOV       AL,34H
         CALL      MXEF1

NXER3:   CMP       AH,25H
         JNE       NXER4
         MOV       AL,35H
         CALL      MXEF1

NXER4:   CMP       AH,26H
         JNE       NXER5
         MOV       AL,36H
         CALL      MXEF1

NXER5:   CMP       AH,27H
         JNE       NXER6
         MOV       AL,37H
         CALL      MXEF1

NXER6:   CMP       AH,28H
         JNE       NXER7
         MOV       AL,38H
         CALL      MXEF1

NXER7:   CMP       AH,29H
         JNE       QR
         MOV       AL,39H
         CALL      MXEF1

QR:      POP       AX
         RET

DGST     ENDP

TSSEND  LABEL  BYTE


;======================  TASK SWITCH TO LOADER =====================

START_WR:
         MOV       BX,08H
         CLD
         XOR       DI,DI
         MOV       ES,BX

         mov       si,offset temp_gdt
         mov       cx,end_temp_gdt - temp_gdt
         inc       cx
         rep     movs byte ptr es:[di],byte ptr cs:[si]
         lgdt      tgdt_pword


         mov       bx,flat_des - temp_gdt
         mov       ds,bx                    ; ds,es,ss all point to
         mov       es,bx                    ; temp. gdt [1]
         mov       ss,bx
         mov       esp,offset end_temp_gdt

         mov       edi,bootid
         mov       bl,byte ptr [edi]        ; bx = boot id
         mov       bh,byte ptr [edi+2]

         mov       edi,rtclock              ; point to RTC address select port
         mov       ah,0fh                   ; point to 2th byte on user RAM
         mov       [edi],ah
         mov       edi,rtcloc2              ; point to RTC data r/w port
         mov       [edi],bh                 ; write high byte to RTC RAM

         mov       edi,rtclock              ; point to RTC address select port
         mov       ah,10h                   ; point to 3th byte on user RAM
         mov       [edi],ah
         mov       edi,rtcloc2              ; point to RTC data r/w port
         mov       [edi],bl                 ; write low byte  to RTC RAM

;
; copy eprom gdt to ram
;
copy2:
         mov       esi,dword ptr gdt_eprom +2
         mov       edi,gdtbase
         mov       cx,word ptr gdt_eprom +0
         inc       cx
         shr       cx,1
         cld
         rep  movs word ptr es:[edi],word ptr ds:[esi]
;
; switch to ram gdt
;
         lgdt      gdt_pword
;
         mov       bx,gdt_alias
         mov       ds,bx
;
; copy eprom tss to ram
;
         mov       bx,init_tss_a
         mov       es,bx
         mov       bx,init_tss
         lar       dx,bx
         mov       [bx].access,ds_access
         mov       fs,bx
         xor       si,si
         xor       di,di
         mov       cx,[bx].lim_0_15
         inc       cx
         rep  movs byte ptr es:[di],byte ptr fs:[si]
         mov       [bx].access,dh

; alter the base address of the initial task's tss

         mov       ax,init_tss_a_offset.bas_0_15
         mov       init_tss_offset.bas_0_15,ax
         mov       al,init_tss_a_offset.bas_16_23
         mov       init_tss_offset.bas_16_23,al
         mov       al,init_tss_a_offset.bas_24_31
         mov       init_tss_offset.bas_24_31,al
;
; change init_tss_a to form a save area for tss on the first task switch
;
         mov       bx,init_tss_a
         mov       word ptr [bx].bas_0_15,1A80h
         mov       [bx].bas_16_23,05h
         mov       [bx].bas_24_31,0
         mov       [bx].access,tss_access
         mov       [bx].gran,0
         ltr       bx
;
; copy eprom ldt to ram
;
         mov       bx,init_ldt_a
         mov       es,bx
         mov       ah,[bx].bas_24_31
         mov       al,[bx].bas_16_23
         shl       eax,16
         mov       ax,[bx].bas_0_15
;
         mov       bx,init_ldt
         lar       dx,bx
         mov       [bx].access,ds_access
         mov       fs,bx
         xor       si,si
         xor       di,di
         mov       cx,[bx].lim_0_15
         inc       cx
         rep       movs byte ptr es:[di],byte ptr fs:[si]
         mov       [bx].access,dh

; alter the base address of ldt's alias
         mov       es:[init_ldt_alias].bas_0_15,ax
         shr       eax,16
         mov       es:[init_ldt_alias].bas_16_23,al
         mov       es:[init_ldt_alias].bas_24_31,ah

; alter the base address of initial task's ldt
         mov       ax,init_ldt_a_offset.bas_0_15
         mov       init_ldt_offset.bas_0_15,ax
         mov       al,init_ldt_a_offset.bas_16_23
         mov       init_ldt_offset.bas_16_23,al
         mov       al,init_ldt_a_offset.bas_24_31
         mov       init_ldt_offset.bas_24_31,al

; now gdt, idt, initial tss and initial ldt are all set up
; start the first task

         jmp       entry_pointer


;-----------------  Interrupt Service Routine   ----------------------


              ORG    0E200H                 ; NMI Service Routine

         PUSH      EDI
         MOV       EDI,CPUFLG
         MOV       AL,02H                   ; b0=0 normal
         MOV       [EDI],AL                 ; b1=1 not clear parity error
                                            ; b6=0 disale NMI
         MOV       EDI,302H
         MOV       AL,01H
         OR        [EDI],AL

         POP       EDI
         IRETD


              ORG    0E240H                 ; INT 28H, MXF-4 INT
         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301H
         MOV       AL,80H                   ; D7=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72700000H
         MOV       WORD PTR [EDI],DX        ; clear 286 INT
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD


              ORG    0E280H                 ; INT 29H, MXF-3 INT

         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301H
         MOV       AL,40H                   ; D6=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72600000H
         MOV       WORD PTR [EDI],DX        ; clear 286 INT
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD


              ORG    0E2C0H                 ; INT 2AH, MXF-2 INT
         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301h
         MOV       AL,20H                   ; D5=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72500000H
         MOV       WORD PTR [EDI],DX        ; clear 286 INT
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD


              ORG    0E300H                 ; INT 2BH, MXF-1 INT
         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301h
         MOV       AL,10H                   ; D4=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72400000H
         MOV       WORD PTR [EDI],DX        ; clear 286 INT
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD

              ORG    0E340H                 ; INT 2CH, 22C11-3 INT
         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301h
         MOV       AL,08H                   ; D3=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72300000H
         MOV       WORD PTR [EDI],DX        ; clear 286 INT
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD


              ORG    0E380H                 ; INT 2DH, 22C11-2 INT
         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301h
         MOV       AL,04H                   ; D2=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72200000H
         MOV       WORD PTR [EDI],DX        ; clear 286 INT
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD


              ORG    0E3C0H                 ; INT 2EH, 22C11-1 INT
         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301h
         MOV       AL,02H                   ; D1=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72100000H
         MOV       WORD PTR [EDI],DX
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD


              ORG    0E400H                 ; INT 2FH, board-0 INT
         PUSH      EDI
         PUSH      EAX
         PUSH      EDX
         MOV       EDI,301H                 ; semaphore byte 301h
         MOV       AL,01H                   ; D1=1
         OR        [EDI],AL
         MOV       DX,0
         MOV       EDI,72000000H
         MOV       WORD PTR [EDI],DX
         POP       EDX
         POP       EAX
         POP       EDI
         IRETD


              ORG       ADJIDT               ; 0E550H

         DB        8 DUP(0)                         ; IDT 0
         DB        8 DUP(0)                         ; IDT 1
         DB        00H,0E2H,18H,00H,00H,8EH,00H,00H ; IDT 2,NMISR

              ORG      0E650H

         DB        8 DUP(0)
         DB        00H,0E2H,18H,00H,00H,8EH,00H,00H   ; INT21H

              ORG      28H*8+0E550H

         DB        40H,0E2H,18H,00H,00H,8EH,00H,00H   ; INT28H,E240H,MXF-4
         DB        80H,0E2H,18H,00H,00H,8EH,00H,00H   ; INT29H,E280H,MXF-3
         DB       0C0H,0E2H,18H,00H,00H,8EH,00H,00H   ; INT2AH,E2C0H,MXF-2
         DB        00H,0E3H,18H,00H,00H,8EH,00H,00H   ; INT2BH,E300H,MXF-1
         DB        40H,0E3H,18H,00H,00H,8EH,00H,00H   ; INT2CH,E340H,22C11-3
         DB        80H,0E3H,18H,00H,00H,8EH,00H,00H   ; INT2DH,E380H,22C11-2
         DB       0C0H,0E3H,18H,00H,00H,8EH,00H,00H   ; INT2EH,E3C0H,22C11-1
         DB        00H,0E4H,18H,00H,00H,8EH,00H,00H   ; INT2FH,board 0


        org     adjust3
        jmp     start_wr

        ORG     ADJUST2
        JMP     START

reset_code      ends
        END

