NAME    CS_SRC
$TITLE ('CS.source')
;PAGE 60,132
;-------------------------------------------------------------------------+
;                                                                         !
;   (c) Copyright, Wang Laboratories, Inc., 1990                          !
;       All rights reserved.                                              !
;                                                                         !
;    Program name: WANG 2200 CS386/2 Power up diagnostics                 !
;    File name: CS.SRC                                                    !
;    Support group: Engineering Service Department                        !
;                   Wang Computer (Taiwan) LTD.                           !
;                   2, Science Road One       ZIP 20077                   !
;                   Hsinchu, Taiwan, Republic of China                    !
;                                                                         !
;    Revision History:                                                    !
;       Date         Programmer      Description            Revision      !
;    04/26/1990     Milton Chen     Rev. CS386/2              59B0        !
;                                                                         !
;    Test Routine:                                                        !
;                                                                         !
;      [ TEST 1 ]  LED TEST                                               !
;      [ TEST 2 ]  CPU 80386 FLAG & REG. TEST                             !
;      [ TEST 3 ]  RTC CHIP INITIAL CHECK & USER RAM TEST                 !
;      [ TEST 3 ]  1M DRAM DATA BUS TEST                                  !
;      [ TEST 4 ]  128K MEMORY TEST                                       !
;              ENTER PROTECTED MODE                                       !
;      [ TEST 5 ]  TEST MEMORY SIZE                                       !
;      [ TEST 6 ]  DETECT INTERFACE CONTROL CARD                          !
;      [ TEST 7 ]  DRAM DATA BUS TEST                                     !
;      [ TEST 8 ]  DRAM BYTE DECODE ADDRESS LINE TEST                     !
;      [ TEST 9 ]  DRAM ADDRESS LINE TEST                                 !
;      [ TEST A ]  DRAM FAST EXCHANGE FUNCTION TEST                       !
;              TASK SWITCH MOV EPROM TEST PROGRAM TO DRAM                 !
;      [ TEST B ]  CACHE & MEMORY TEST                                    !
;      [ TEST C ]  INTERRUPT TEST
;                                                                         !
;-------------------------------------------------------------------------+


;**********************************************************************
;        MEMORY  ALLOCATION MAP
;**********************************************************************

;
;                                                                                                 +----->DS descriptior
;   +--> 00000000 +-------------------------------+ --+              +-------------------------+  !
;   !             !      NULL_DES                 !   !              ! 00 00 00 00 00 00 00 00 !--!
;   !             !-------------------------------!   !              !-------------------------!  !
;   !             !      FLAT_DES                 !   !              ! FF FF 00 00 00 92 CF 00 !--+ base:0 lim:4k X FFFFF = 4G
;   !             !-------------------------------!   !  TEMP_GDT    !-------------------------!
;   !             !      GDT_eprom                !   !=============>! 00 4F FF FF D4 00       !  locate:FFFFD400  lim:4F bytes
;   !             !-------------------------------!   !              !-------------------------!
;   !             !      IDT_eprom                !   !              ! 04 FF FF FF D5 50       !  locate:FFFFD500  lim:FF bytes
;   !             !-------------------------------!   !              !-------------------------!
;   +------------ !      TGDT_pword               !   !              ! 00 2D 00 00 00 00       !  locate:00000000  lim:2D bytes
;                 !-------------------------------!   !              !-------------------------!
;   +------------ !      GDT_pword                !   !              ! 00 4F 00 00 10 00       !  locate:00001000  lim:4F bytes
;   !             !-------------------------------!   !              !-------------------------!
;   !  +--------- !      IDT_pword                !   !              ! 04 FF 00 00 04 00       !  locate:00000400  lim:FF bytes
;   !  !          !-------------------------------! --+              +-------------------------+
;   !  !          !                               !
;   !  !          !            :                  !   +----------------------------------------------------------------------------+
;   !  !          !                               !   !                                                                            !
;   !  ! 00000150 !-------------------------------!   !              +-------------------------+                                   !
;   !  !          !      TSS                      ! ==+        0     ! 00 00 00 00 00 00 00 00 !                                   !
;   !  ! 00000250 !-------------------------------!      LDT         !-------------------------!                                   !
;   !  !          !      LDT                      ! ===========1===> ! 27 00 50 02 00 92 00 00 ! locate:00000250 lim:27H(40bytes)  !
;   !  !          !-------------------------------!                  !-------------------------!                                   !
;   !  !          !            :                  !            2  CS ! FF FF 00 20 00 9A 8F 00 ! base:00002000 lim:FFFFF X 4K      !
;   !  +-00000400 !-------------------------------!                  !-------------------------!                                   !
;   !             !      IDT                      !            3  DS ! FF FF 00 00 FF 92 8F FF ! base:FFFF0000 lim:FFFFF X 4K      !
;   !             !-------------------------------!                  !-------------------------!                                   !
;   !             !                               !            4  SS ! FE FF 00 03 00 96 CF 00 ! base:00000300 lim:FEFFF X 4K      !
;   !             !                               !                  +-------------------------+                                   !
;   !             !                               !                                                                                !
;   +--> 00001000 !-------------------------------!            +-------------------------+                                         !
;                 ! 0    NULL_DES                 !---+        ! 00 00 00 00 00 00 00 00 !                                         !
;                 !-------------------------------!   !        !-------------------------!                                         !
;                 ! 1    FLAT_DESC                !   !        ! 00 10 00 20 00 92 00 00 ! base:00002000 lim:10000H bytes          !
;                 !-------------------------------!   !        !-------------------------!                                         !
;                 ! 2    GDT_ALIAS                !   !        ! 4F 00 00 10 00 92 00 00 ! base:00001000 lim:4FH bytes             !
;                 !-------------------------------!   !        !-------------------------!                                         !
;                 ! 3    IDT_ALIAS                !   !  GDT   ! 00 01 00 00 FF 9A 80 FF ! base:FFFF0000 lim:100H X 4K bytes       !
;                 !-------------------------------!   !=======>!-------------------------!                                         !
;                 ! 4    INIT_TSS   (TSS_eprom)   !   !        ! 67 00 50 D4 FF 89 00 FF ! base:FFFFD450 lim:67H bytes TSS,type:9  !
;                 !-------------------------------!   !        !-------------------------!                                         !
;                 ! 5    INIT_TSS_A (TSS_1)       !   !        ! 67 00 50 01 00 92 00 00 ! base:00000150 lim:67H bytes             !
;                 !-------------------------------!   !        !-------------------------!                                         !
;                 ! 6    INIT_LDT   (LDT_eprom)   !   !        ! 27 00 00 D5 FF 82 00 FF ! base:FFFFD500 lim:27H bytes LDT,type:2  !
;                 !-------------------------------!   !        !-------------------------+                                         !
;                 ! 7    INIT_LDT_A (LDT_1)       !   !        ! 27 00 50 02 00 92 00 00 ! base:00000260 lim:27H bytes             !
;                 !-------------------------------!   !        !-------------------------!                                         !
;                 ! 8    RESERVED FOR TSS_2       !   !        ! 00 00 00 00 00 00 00 00 !                                         !
;                 !-------------------------------! --+        +-------------------------+                                         !
;                 ! 9    RESERVED FOR LDT_2       !            ! 00 00 00 00 00 00 00 00 !                                         !
;        00002000 !-------------------------------!            +-------------------------+                                         !
;                 !                               !                                                                                !
;                 !   TEST PROGRAM LOADING HERE   ! <------+                                                                       !
;                 !                               !        !                                                                       !
;        00040000 !-------------------------------!        !                                                                       !
;                 !                               !        !                                                                       !
;                 !   PROGRAM RUN SPACE           !        !         0             4             8           C          F          !
;                 !                               !        !         +---------------------------------------------------+         !
;                 !                               !        !      00 ! 00 00 00 00 00 00 00 00 - 24 00 00 00 00 00 00 00 !         !
; EPROM: FFFFC000 !-------------------------------!        !      10 ! 00 00 00 00 00 00 00 00 - 00 00 00 00 00 00 00 00 !   TSS   !
;                 !    TEMP_GDT                   !        !      20 ! 00 00 00 00 00 02 00 00 - 00 00 00 00 00 00 00 00 !<--------+
;                 ! - - - - - - - - - - - - - - - !        !      30 ! 00 00 00 00 00 00 00 00 - 00 00 00 00 00 00 00 00 !
;                 !    DIAGNOSTIC PROGRAM         !        !      40 ! 00 00 00 00 00 00 00 00 - 1C 00 00 00 14 00 00 00 !
;                 !- - - - - - - - - - - - - - - -!        !      50 ! 24 00 00 00 1C 00 00 00 - 1C 00 00 00 1C 00 00 00 !
;                 !    TEST PROGRAM (MOV TO DRAM) !--------+      60 ! 30 00 00 00 00 00 00 00 +-------------------------+
;        FFFFCF00 !-------------------------------!                  +-------------------------+
;                 !                               !
;        FFFFD400 !-------------------------------!                         8 : SS0 --> 24      selector:
;                 !    GDT                        !                        48 : ES  --> 1C
;        FFFFD450 !-------------------------------!                        4C : CS  --> 14  -- 00010 100   2 : 2nd LDT's Item
;                 !    TSS                        !                        50 : SS  --> 24
;        FFFFD500 !-------------------------------!                        54 : DS  --> 1C  -- 00011 100   3 : 3rd LDT's Item
;                 !    LDT                        !                        58 : FS  --> 1C           |
;        FFFFD550 !-------------------------------!   IDT                  5C : GS  --> 1C           +---------> 1 : LDT
;                 !    IDT  (20H*8)               !---------+              60 : LDT --> 30  -- 00110 000   6 : 6th GDT
;        FFFFFFF0 !_______________________________!         |                                        |
;                                                           |                                        +---------> 0 : GDT
;                                                           |
;     Interrupt Gate Descriptor                             |
;                                           00H,0D2H,18H,00H,00H,8EH,00H,00H   ; IDT 0
;         00 00  |  8E 00                   00H,0D2H,18H,00H,00H,8EH,00H,00H   ; IDT 1
;      ----------------------                               :                                          Descriptors
;         00 18  |  D2 00                                   :
;                                                           :                                   ## ##  @@ @@ @@  XX  X#  @@
;  Selector : 18  0001 1000  GDT(3)         00H,0D2H,18H,00H,00H,8EH,00H,00H   ; IDT 30                          ||  |
;  Offset : 0000D200 (base:FFFF0000)        00H,0D2H,18H,00H,00H,8EH,00H,00H   ; IDT 111                   +-------+  |
;  INT Gate : 1 000 01110 000 0000 (8E00)                                                                 |        G B 0 AVL
;             |  |    |                                                                                   |          D
;             |  |    |                                                                                   |
;             P DPL   |                                                        DS: descriptor  DPL 1  1 0 E W A
;                     |                                                      EXEC: descriptor           1 C R
;                   01111 ( Trap Gate )                                                               ----------
;                                                                            ## ## # : lim              Type ->2: LDT
;                                                                            @@ @@ @@ @@ : base                9: 386 TSS
;                                                                                                              E: 386 Interrupt Gate
;                                                                                                              F: 386 Trap Gate
;
;
;define the adjustment factor

ADJUST1  EQU       0C000H
ADJUST2  EQU       0fff0h

ADJGDT   EQU       0D400H          ;
ADJTSS   EQU       0D450H          ;
ADJLDT   EQU       0D500H          ;
ADJIDT   EQU       0D550H          ;

GDTBASE EQU        00001000H       ;phisical address for GDT Base

IDTBASE  EQU       00000400H       ;FOR IDT BASE



;I/O PORT
TIMESET EQU          8000H      ;130NS FOR EACH UNIT

CPUST1  EQU         0FFFF8002H  ;READ CPU/IO STATUS
CPUST   EQU          8002H      ;READ CPU/IO STATUS
                                ;D0= 1, 5us pulse of ABS/, OBS/, CBS/
                                ;D1= 1, CPU BUSY
                                ;D2= 1, KEY BOARD FUNCTION
                                ;D3= 1, I/O READY
                                ;D4= 1, TIME OUTME OUT
                                ;D5= 1, CPU HALT
                                ;D6= 1, SYSTEM ERROR
                                ;D7= 1, MEMORY PARITY ERROR
                                ;D8= 1, primary reset in from 2200 bus
                                ;D9= 1, nomal boot; =0, run-in
                                ;D10=1, same as D2, but read cause clear
                                ;D11    not defined
                                ;D12    not defined
                                ;D13=1, same as D5, but read cause clear

CPUFLG1 EQU        0FFFF8004H   ;WRITE IO PORT
CPUFLG  EQU          8004H      ;WRITE IO PORT
                                ;D0= 1, generate even parity bit for diag
                                ;  = 0, clear force even parity condition
                                ;D1= 1, clear parity error if occured anytime
                                ;  = 0, not clear parity error, if occured
                                ;D2= 1, clear 387 error if occured
                                ;  = 0, not clear 387 error if occured
                                ;D3= 1, CPU LED on
                                ;  = 0, CPU LED off

CPURDY  EQU         8006H       ;release CPU BUSY


RTCADDR EQU         8008H       ;RTC address port
RTCDATA EQU         800AH       ;RTC r/w data port

RTCLOCK EQU    0FFFF8008H       ;realtime clock address strobe
RTCLOC2 EQU    0FFFF800AH       ;realtime clock data strobe, data r/w port
                                ;1st byte RTC user rAM = memory size in MB
                                ;17byte user ram = error code in protect mode
                                ;18th byte = MNI flag; 0= no NMI, 1=NMI occured
                                ;19th byte = loop count low byte
                                ;20th byte = loop count hi byte

IBGET   EQU          800CH      ;get input data from bus
_ABS1   EQU     0FFFF8012H      ;address bus strobe
_ABS    EQU          8012H      ;address bus strobe
_OBS    EQU     0FFFF8014H     ;output data to bus
_OBS1   EQU          8014H     ;output data to bus
CBS     EQU          8018H      ;command data bus strobe


DUMMY   SEGMENT RW              ;only for ASM386 MAIN MODULE stack init
        DW      00              ;w/o this dummy segment, you will get
DUMMY   ENDS                    ;Warning #21 during compiler and fatal
;
;*****************************************************************
;
;  RESET_CODE must be USE16 because the 386 initially executes in real mode
;
;*******************************************************************
;

RESET_CODE SEGMENT ER PUBLIC USE16
ASSUME  DS:NOTHING, ES:NOTHING
;
; 386 descriptor template
;

desc    struc
        lim_0_15        dw 0
        bas_0_15        dw 0
        bas_16_23       db 0
        access          db 0
        gran            db 0
        bas_24_31       db 0
desc    ends

; the following is the layout of the real GDT created by BLD386.
; we use the  .bld file to make the following layout.
; it is located in EPROM and will be copied to RAM.

; GDT[0].... null
; GDT[1].... gdt alias
; GDT[2].... idt alias
; GDT[3].... initial task tss ------base address point to ROM from
; GDT[4].... initial task tss alias-base address point to RAM to
; GDT[5].... initial task ldt ------base address ponit to ROM from
; GDT[6].... initial task ldt alias-base address ponit to RAM to
; GDT[7].... reserve for loaded task
; GDT[8].... reserve for loaded task

; the descriptors in gdt total 9

gdt_entries     equ 10
idt_entries     equ 112

FLAT_DESC       EQU 1*SIZE DESC
gdt_alias       equ 2*size desc
idt_alias       equ 3*size desc
init_tss        equ 4*size desc
init_tss_a      equ 5*size desc
init_ldt        equ 6*size desc
init_ldt_a      equ 7*size desc

init_ldt_alias  equ 1*size desc

; access right byte for data and tss descriptor

ds_access       equ 10010010b
tss_access      equ 10001001b

;this temporary gdt will be used to set up the real gdt in ram



            ORG   ADJUST1       ; 0C000H

temp_gdt        label byte
null_des        desc <>

;use this descriptor we can access 4gb memory from physical address 0
flat_des        desc<0ffffh,0,0,92h,0cfh,0>

;the gdt address and limit in eprom
gdt_eprom       label pword
        dw      004Fh
        dd      0FFFFD400h                   ;#

idt_eprom       label pword
        dw      04FFH
        dd      0FFFFD550H

;the temporary gdt address and limit
tgdt_pword      label pword
        dw      end_temp_gdt - 0c000h -1
        dd      0

;the gdt address and limit in ram
gdt_pword      label pword
        dw      gdt_entries * size desc -1
        dd      gdtbase
idt_pword      label pword
        dw      idt_entries * size desc -1
        dd      idtbase

end_temp_gdt    label byte


gdt_des_flat    equ ds:gdt_alias + gdtbase
idt_des_flat    equ ds:idt_alias + gdtbase

init_tss_a_offset       equ ds:init_tss_a
init_tss_offset         equ ds:init_tss

init_ldt_a_offset       equ ds:init_ldt_a
init_ldt_offset         equ ds:init_ldt

; define pointer for first task switch

entry_pointer   label  pword
        dd 0
        dw ds:init_tss



BEGINPUP        LABEL   WORD    ; power-up entry point
                PUBLIC  BEGINPUP

START:  CLI                     ; disable interrupts
        JMP     AFTER_COPYRIGHT

SRAMSIZE        DB      'DRAM', 20H, 20H, '1024',20H,'KB',0DH,0AH,06H
ENDSRAMSIZE:
SRAMLOOP        DB      ' Memory Run-in TEST',40 DUP(20H)
                DB      'LOOP COUNT:', 5 DUP(20H),30H,0DH,0AH,0DH,0AH
ENDSRAMLOOP:

DRAMSIZE        DB      0DH,0AH,'DRAM', 5 DUP(20H),30H,20H,'KB', 08H, 08H, 08H, 08H
ENDDRAMSIZE:

LOOPCONT        DB      01h, 0Ah, 0Ah, 69 DUP(09h),6 DUP(20H)
ENDLOOPCONT:

DRAMCLEN        DB      0Ah, 0Dh, 15 DUP(20h), 0Dh,0Ah     ; #
ENDDRAMCLEN:

DRAMLOOP        DB      0Dh, 5 DUP(09h),4 DUP(20H)
ENDDRAMLOOP:


DSK1     DB        '  * 1st 22C11 card  --  exist',0DH,0AH
DS1:

DSK2     DB        '  * 2nd 22C11 card  --  exist',0DH,0AH
DS2:

DSK3     DB        '  * 3rd 22C11 card  --  exist',0DH,0AH
DS3:

MXF11    DB        '  * 1st  MXF card   --  exist',0DH,0AH
MX1:

MXF22    DB        '  * 2nd  MXF card   --  exist',0DH,0AH
MX2:

MXF33    DB        '  * 3rd  MXF card   --  exist',0DH,0AH
MX3:

MXF44    DB        '  * 4th  MXF card   --  exist',0DH,0AH
MX4:


BADSIMM            DB       0DH,0AH,'    BAD SIMM : L'
BADSIMML:

TS6ER              DB        0DH,0AH,'    DRAM DATA BUS TEST --- ERROR !!'
TS6ERR:

TS8ER              DB        0DH,0AH,'    DRAM BYTE DECODE ADDRESS LINE TEST --- ERROR !!'
TS8ERR:

TS9ER              DB        0DH,0AH,'    DRAM ADDRESS LINE TEST --- ERROR !!',0DH,0AH
TS9ERR:

TSAER              DB        0DH,0AH,'    DRAM DATA FAST EXCHANGE TEST --- ERROR !!',0DH,0AH
TSAERR:

MSG     DB      0DH,0AH,0DH,0AH,0DH,0AH,'TASK SWITCH ---- OK !!!',0DH,0AH
        DB      0DH,0AH,' NOW, Move EPROM Memory Test Program to DRAM,',0DH,0AH
        DB      '  Then EXECUTE !!.....',0DH,0AH,0DH,0AH,00H

INTT               DB        0DH,0AH,0DH,0AH,'Interrupt Test :'
INTT1:

COPYRIGHT       DB      'Copyright, Wang Laboratories, Inc., 1990'
                DB      20H, 20H, 20H, 20H, 20H
REVISION        DB      'CS386/2 Rev 5050', 0DH, 0AH, 0DH, 0AH

                PUBLIC  COPYRIGHT


;-----------------------------------------------------------------------------+
; [ TEST 0 ]  LED TEST                                                        !
;                                                                             !
;  Purpose: To check LED on/off                                               !
;-----------------------------------------------------------------------------+

AFTER_COPYRIGHT:
         MOV       BX,CPUFLG                ; select I/O port, direct memory mapping
         MOV       AX,08H                   ; D3= 1, cpu led on; D3= 0, cpu led off
         MOV       CS:[BX],AX               ; turn CPU LED on
         XOR       ECX,ECX                  ; set up counter for time delay

WITH_CPU_LED:
         AAM                                ; wait so CE can see LED is turned "on".
         AAM
         AAM
         AAM
         AAM
         LOOP      WITH_CPU_LED             ; 83 clk x .033us/clk x 65536 = .679 secnd


         MOV       AX,6                     ; D1=1 clear parity error,D2=1 clr 387
         MOV       CS:[BX],AX               ; BX point to CPUFLG I/O port



;-----------------------------------------------------------------------------+
; [ TEST 1 ]  80386 CPU FLAG & REGS. TEST                                     !
;                                                                             !
;  Purpose: To check 80386 CPU flag on/off and all registers read/write test  !
;-----------------------------------------------------------------------------+


         CLI                                ; disable interrupt
         MOV       AH,0D5H                  ; set SF,CF,ZF,AF flags on
         SAHF                               ; store AH to flag reg.
         JNC       FATAL_ERROR              ; error if CF not set
         JNZ       FATAL_ERROR              ; error if ZF not set
         JNP       FATAL_ERROR              ; error if PF not set
         JNS       FATAL_ERROR              ; error if SF not set
         LAHF                               ; load flag to AH
         MOV       CL,5                     ; shift count
         SHR       AH,CL                    ; shift AF into CARRY bit
         JNC       FATAL_ERROR              ; error if no carry
         MOV       AL,40H                   ; set flag on
         SHL       AL,1                     ; set up for test
         JNO       FATAL_ERROR              ; error if overflow
         XOR       AH,AH                    ; AH=0
         SAHF
         JBE       FATAL_ERROR              ; error if CF on

         JS        FATAL_ERROR              ; error if SIGN flag set
         JP        FATAL_ERROR              ; error if EVEN Parity set
         LAHF                               ; load flag to AH
         MOV       CL,5                     ; shift count
         SHR       AH,CL
         JC        FATAL_ERROR              ; error if CF on
         SHL       AH,1                     ; check OF is clear
         JO        FATAL_ERROR              ; error if OF on

         XOR       EAX,EAX                  ; test reset condition & flags
         JNZ       FATAL_ERROR              ; check zero flag off
         OR        EAX,1
         JZ        FATAL_ERROR              ; check zero flag on
         AND       EAX,EAX
         JC        FATAL_ERROR              ; test carry, error if carry flag not cleared
         STC                                ; set carry flag
         JNC       FATAL_ERROR              ; error if not carry
         CLC                                ; clear carry flag
         JC        FATAL_ERROR              ; error if carry flag is not clear
         CMC                                ; complement carry flag
         JNC       FATAL_ERROR              ; error if no carry

         MOV       EAX,1                    ; store 1 to EAX Reg.
         MOV       CX,7                     ; count times
LP1:     AND       EAX,EAX                  ; set flag
         JZ        FATAL_ERROR              ; error if ZERO flag set
         JS        FATAL_ERROR              ; error if SIGN flag set
         JPE       FATAL_ERROR              ; error if EVEN Parity, b0 - b7
         JC        FATAL_ERROR              ; error if CARRY
         RCL       EAX,1
         LOOP      LP1


;----------- R/W GENERAL AND SEGMENT REGS. WITH 1'S AND 0'S

         MOV       EAX,33CC55AAH            ; pattern data = 55AA
CLSET:   MOV       EBX,EAX                  ; copy to EBX
         MOV       ECX,EBX                  ; copy to ECX
         MOV       EDX,ECX                  ; copy to EDX
         MOV       ESI,EDX                  ; copy to ESI
         MOV       EDI,ESI                  ; copy to EDI
         MOV       EBP,EDI                  ; copy to EBP
         MOV       ESP,EBP                  ; copy to ESP
         CMP       EAX,ESP                  ; equal EAX ?
         JNZ       FATAL_ERROR              ; error if not equal
         XOR       EAX,EAX                  ; EAX <= 0
         CMP       EBX,EAX
         JNZ       CLSET                    ; set all reg. = 0

         JMP       CPUPASS                  ; test pass


FATAL_ERROR:

         MOV       BX,CPUFLG                ; point to I/O port
         MOV       AX,08H                   ; d3=1 CPU LED on
         MOV       CS:[BX],AX               ; turn CPU LED on
         XOR       ECX,ECX                  ; set up counter for time delay
LEDLOOP: AAM
         AAM
         JMP       LEDLOOP
         JMP       $

         JMP       CPUPASS                  ; test pass


;-----------------------------------------------------------------------------+
; [ TEST 2 ] RTC CHIP INITIAL CHECK & USER RAM TEST                           !
;                                                                             !
;  Purpose: RTC initial date check and RTC's user RAM test                    !
;                                                                             !
;  Description: Using READRTC Macro to check RTC calendar regs. content, if   !
;               not initial, using SETRTC Macro to initialize.                !
;-----------------------------------------------------------------------------+

CPUPASS:
                                            ; MACRO for read RTC time data
%*DEFINE (READRTC(ADDR,TIME))
      (  MOV       CL,%ADDR                 ; select addr.
         MOV       BX,RTCADDR               ; RTC addr. strobe
         MOV       CS:[BX],CL               ; point to address
         MOV       BX,RTCDATA               ; RTC data strobe, data R/W port
         MOV       AL,CS:[BX]               ; read time data
         CMP       AX,%TIME                 ; compare time range is reasonable ?
      )

                                            ; MACRO for set RTC time
%*DEFINE (SETRTC(SETADDR,SETIME))
      (  MOV       CL,0BH                   ; select B reg.
         MOV       AL,10000010B             ; set bit b7=1, b2=0 DM use BCD,
         MOV       BX,RTCADDR               ; b1=1 24 hours mode, b0=0 DSE
         MOV       CS:[BX],CL               ; point to B reg.
         MOV       BX,RTCDATA               ; point to data port
         MOV       CS:[BX],AL               ; initialize RTC time

         MOV       CL,%SETADDR              ; set time of RTC addr.
         MOV       AL,%SETIME               ; set time data to RTC
         MOV       BX,RTCADDR
         MOV       CS:[BX],CL
         MOV       BX,RTCDATA
         MOV       CS:[BX],AL
      )

         %READRTC(08,12)                    ; month addr. = 08
         JG        SET_MONTH                ; month > 12 , then set month

         %READRTC(07,31)                    ; data addr. = 07
         JG        SET_DATA                 ; data > 31 , then set data

         %READRTC(04,24)                    ; hour addr. = 04
         JG        SET_HOUR                 ; hour > 24 , then set hour

         %READRTC(02,60)                    ; min. addr. = 02
         JG        SET_MIN                  ; min. > 60 , then set min

         %READRTC(00,60)                    ; sec. addr. = 01
         JG        SET_SEC                  ; sec. > 60 , then set sec

         JMP       TIMEOK

SET_MONTH:
         %SETRTC(08,01)                     ; set month = 01

SET_DATA:
         %SETRTC(07,01)                     ; set date = 01

SET_HOUR:
         %SETRTC(04,00)                     ; set hour = 00

SET_MIN:
         %SETRTC(02,00)                     ; set min = 00

SET_SEC:
         %SETRTC(00,00)                     ; set sec = 00


;---------  TEST RTC USER RAM --------------
TIMEOK:

         MOV       AL,0EH                   ; select RTC user RAM first addr.
         MOV       CL,0EH                   ; load 0E => (0E)
WR:      MOV       BX,RTCADDR
         MOV       CS:[BX],CL
         MOV       BX,RTCDATA
         MOV       CS:[BX],AL               ; write data
         INC       AL                       ; 0F => (0F) ...
         INC       CL                       ; ... 3F => (3F)
         CMP       CL,40H                   ; till last addr. = 3FH
         JNZ       WR                       ; write next data

         MOV       CL,0EH                   ; select 1st addr.
         MOV       DL,0EH
RD:      MOV       BX,RTCADDR
         MOV       CS:[BX],CL
         MOV       BX,RTCDATA
         MOV       AL,CS:[BX]               ; read RAM data
         CMP       AL,DL                    ; compare data equal ?
         JNZ       FATAL_ERROR              ; error if not equal
         INC       DL                       ; point to next addr.
         INC       CL
         CMP       CL,40H                   ; last addr.= 3FH
         JNZ       RD                       ; read next data


;-------------------------------------------------------------------------+
; [  Test 3  ]  1M DRAM data bus Test                                     !                                     !
;                                                                         !
;   Description: Walking '0' & '1' data patterns (double word) are        !
;          written, then read/verified for correct data and NO parity     !
;          error on each bank of 1st addr.                                !
;                                                                         !
;      BEGIN                                                              !
;         FOR current bank(64K) of 1st addr.                              !
;             FOR pattern = FFFFFFFF, 01, 02, 04, ...80000000, 0,         !
;                           7FFFFFFF, BFFFFFFD, ......., FFFFFFFE         !
;                  write current pattern to current address               !
;                  read/verify current address              ERROR 01      !
;                  check for memory parity error            ERROR 02      !
;             NEXT pattern                                                !
;         NEXT bank                                                       !
;                                                                         !
;         FOR current bank(64K) of 1st addr.                              !
;             FOR pattern = 03, 06, 0C, 18, 30,...., C0000000, 80000001   !
;                  write current pattern to current address               !
;                  read/verify current address              ERROR 03      !
;                  check for memory parity error            ERROR 04      !
;             NEXT pattern                                                !
;         NEXT bank                                                       !
;      END                                                                !
;                                                                         !
; ERRORS:                                                                 !
; 01 & 03 Data bus or memory data error                                   !
;         EAX= read data    EBX= expected data     DS:SI= current address !
;         FS= error code                                                  !
;                                                                         !
; 02 & 04 Memory error, may be parity or circuit                          !
;         AX= 4, SRAM parity err                                          !
;         FS= error code                                                  !
;-------------------------------------------------------------------------+

TST3:
         XOR       SI,SI                    ; start from address 0:0000
         MOV       DI,SI
TST3LOP1:
         MOV       AX,1                     ; setup error code
         MOV       FS,AX                    ; when error, FS is the error code
         MOV       AX,DI                    ; start from address 0:0000 to F000:0000
         MOV       DS,AX
         MOV       EBX,0FFFFFFFFH           ; current data pattern
         MOV       EAX,01                   ; next pattern
         MOV       EDX,EAX                  ; save next pattern
         MOV       CX,33                    ; 33 patterns from FFFFFFFF to 01h
         MOV       [SI],EBX                 ; write current pattern to current address
TST3LOP2:
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad (no jump is fast)
         MOV       EBX,EDX                  ; save current pattern
         SHL       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX                  ; ready to write data pattern from EAX reg
         DEC       CX
         JNZ       TST3LOP2                 ; loop for next pattern test

         MOV       EAX,7FFFFFFFH            ; next data pattern 7FFFFFFF
         MOV       EDX,EAX                  ; prepare for shift right
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad (no jump is fast)
         MOV       EBX,EDX                  ; save current pattern for compare later
         ROR       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX
         MOV       CX,32                    ; 32 patterns from 7FFFFFFF to FFFFFFFE
TST3LOP3:
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad (no jump is fast)
         MOV       EBX,EDX                  ; save current pattern for compare later
         ROR       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX
         DEC       CX
         JNZ       TST3LOP3                 ; loop for next pattern test

         MOV       AX,2                     ; setup error code for parrity check
         MOV       FS,AX                    ; error code in FS = 2
         MOV       BX,CPUST                 ; point to CPU status port
         MOV       AX,CS:[BX]               ; read CPU status
         TEST      AX,01000000B             ; keep D6, mask out other bits
         JNZ       TST3_ERR                 ; if D6 not zero; parrity error ocurred

         ADD       EDI,1000H                ; point to next 64KB bank
         CMP       EDI,0F000H               ; last DRAM is F:0000h
         JLE       TST3LOP1                 ; otherwise go on next test


         XOR       ESI,ESI                  ; start from address 0:0000 for
         MOV       EDI,ESI                  ; even number bits patterns rotate
TST3LOP4:
         MOV       AX,3                     ; setup error code
         MOV       FS,AX                    ; when error, FS is the error code
         MOV       AX,DI                    ; start from address 0:0000 to F000:0000
         MOV       DS,AX
         MOV       EBX,03H                  ; current data pattern
         MOV       EAX,06H                  ; next pattern
         MOV       EDX,EAX                  ; save next pattern
         MOV       CX,32                    ; 32 patterns from 03 to 80000001h
         MOV       [SI],EBX                 ; write current pattern to current address
TST3LOP5:
         XCHG      [SI],EAX                 ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST3_ERR                 ; jump if bad
         MOV       EBX,EDX                  ; save current pattern
         ROL       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX                  ; ready to write data pattern from EAX reg
         DEC       CX
         JNZ       TST3LOP5                 ; loop for next pattern test

         MOV       AX,4                     ; setup error code for parrity check
         MOV       FS,AX                    ; error code in FS
         MOV       BX,CPUST                 ; point to CPU status port
         MOV       AX,CS:[BX]               ; read CPU status
         TEST      AX,01000000B             ; keep D6, mask out other bits
         JNZ       TST3_ERR                 ; if D6 not zero; parrity error ocurred

         ADD       EDI,1000H                ; point to next 64KB bank
         CMP       EDI,0F000H               ; last DRAM is F:0000h
         JLE       TST3LOP4                 ; otherwise go on next test

         JMP       TST4

TST3_ERR:
         MOV       DI,CPUFLG                ; select I/O port, direct memory mapping
         MOV       CX,08H                   ; D3= 1, cpu led on; D3= 0, cpu led off
         MOV       CS:[DI],CX               ; turn CPU LED on as an error ocurred

         MOV       DI,CPUST                 ; point to CPU status I/O port
         MOV       BP,CS:[DI]               ; read  CPU sataus
         TEST      BP,200h                  ; D9=0 loop on BIT is requested
         JZ        TST3

         JMP       $                        ; hang here if error

;-------------------------------------------------------------------------+
; [  Test 4  ] 1M DRAM Address Line Test                                  !
;-------------------------------------------------------------------------+

TST4:
;-------------------------------------------------------------------------+
; [ Test 5 ]  1M DRAM byte Decode Address Line Test                       !
;-------------------------------------------------------------------------+

TST5:


;-------------------------------------------------------------------------+
; [ Test 6 ] 1M  DRAM Data Fast Exchange Function TEST                    !
;                                                                         !
;Purpose:  To test ability of all memory cells to hold a zero or a        !
;          one and detec dynamic data/address faults.                     !
;                                                                         !
;Description: The test is very fast because it takes advantage of various !
;          CPU instructions. Three passes from the first to last memory   !
;          address of DRAM are required to verify address and data faults !
;          on both the data and parity DRAM.                              !
;                                                                         !
; BEGIN                                                                   !
;     FOR SRAM first to last by 4's Do                                    !
;         write address = "FFFFFFFF"                                      !
;     END (for DRAM)                                                      !
;                                                                         !
;     FOR SRAM first to last by 4's Do                                    !
;         exchange "FFFFFFFF" with "00000000"                             !
;         verify read data = "FFFFFFFF"                    ERROR 11       !
;         read pattern "00000000"                                         !
;         verify read data = "00000000"                    ERROR 11       !
;                                                                         !
;         exchange "00000000" with "F4F4F4F4"                             !
;         verify read data = "00000000"                    ERROR 11       !
;         read pattern "F4F4F4F4F"                                        !
;         verify read data = "F4F4F4F4"                    ERROR 11       !
;     END (for SRAM)                                                      !
;                                                                         !
;     Verify parity error                                  ERROR 12       !
; END                                                                     !
;                                                                         !
; ERRORS:                                                                 !
; 11  Address Bus or Memory Data error                                    !
;     EAX= read data    EBX= new write data     DS:SI= current or next    !
;      FS= error code   EBP= last written data         address            !
;                                                                         !
; 12  Memory error, may be parity or circuit                              !
;     AX= 4, SRAM parity err                                              !
;     FS= error code                                                      !
;-------------------------------------------------------------------------+

TST6:
         MOV       BX,0                     ; memory range from 0:0000 to F000:FFFF
ADB1_LOOP:
         MOV       ES,BX
         MOV       EDI,0                    ; set 1st address
         MOV       CX,4000H                 ; memory size in dwords
         MOV       EAX,0FFFFFFFFH           ; set flood data pattern
         CLD                                ; clear dir flag so string instr incrmnt DI
         REP       STOSD                    ; flood segment

         ADD       EBX,1000H                ; next bank
         CMP       EBX,0F000H               ; last DRAM is F:FFFFh
         JLE       ADB1_LOOP

         MOV       AX,11H                   ; setup error code
         MOV       FS,AX                    ; when error, FS is the error code
         MOV       EBP,0FFFFFFFFH           ; set last written pattern
         MOV       EBX,0                    ; set revers pattern for write
         MOV       EDX,0                    ; memory size from 0 to F:FFFF

ADB2_LOOP:
         MOV       DS,DX
         MOV       SI,0                     ; set memory address pointer
         MOV       CX,4000H                 ; (1024X64)/4 loop count for double word
         MOV       EAX,EBX                  ; get revers pattern

ADB3_LOOP:                                  ; loop here 16K times
         XCHG      [SI],EAX                 ; read 'FFFFFFFF',write '00000000'
         CMP       EAX,EBP                  ; verify data
         JNE       TST6_ERR                 ; jump if bad (no jump is fast)
         LODSD                              ; read '0000' back and inc SI by 4
         CMP       EAX,EBX                  ; verify data
         LOOPZ     ADB3_LOOP                ; loop as long as CX not 0 and no error.
         JNE       TST6_ERR                 ; jump if bad (no jump is fast)
         MOV       AX,12H                   ; setup error code for parrity check
         MOV       FS,AX                    ; error code in FS = 4
         MOV       DI,CPUST                 ; point to CPU status port
         MOV       AX,CS:[DI]               ; read CPU status
         TEST      AX,01000000B             ; keep D6, mask out other bits
         JNZ       TST6_ERR                 ; if not zero, we have parrity error
         MOV       AX,11H                   ; setup error code for memory test
         MOV       FS,AX                    ; error code in FS = 3
         ADD       EDX,1000H                ; next bank
         CMP       EDX,0F000H               ; last SRAM is F:FFFF
         JLE       ADB2_LOOP                ; jmp if DX < 4000


         MOV       EBP,0                    ; set last written pattern
         MOV       EBX,0F4F4F4F4H           ; after test, all address should be 'F4'
         MOV       EDX,0                    ; DRAM mmry size from 0 to F000:0
ADB4_LOOP:
         MOV       DS,DX
         MOV       ESI,0                    ; set memory address pointer
         MOV       CX,4000H                 ; (1024X64)/4 loop count for double word
         MOV       EAX,EBX                  ; get revers pattern

ADB5_LOOP:                                  ; loop here 16K times
         XCHG      [SI],EAX                 ; read '00000000', write 'F4F4F4F4'
         CMP       EAX,EBP                  ; verify data
         JNE       TST6_ERR                 ; jump if bad (no jump is fast)

         LODSD                              ; read 'F4F4F4F4' back and inc SI by 4
         CMP       EAX,EBX                  ; verify data

         LOOPZ     ADB5_LOOP                ; loop as long as CX not 0 and no error.
         JNE       TST6_ERR                 ; jump if bad (no jump is fast)

         MOV       AX,12H                   ; setup error code for parrity check
         MOV       FS,AX                    ; error code in FS = 4
         MOV       DI,CPUST                 ; point to CPU status port
         MOV       AX,CS:[DI]               ; read CPU status
         TEST      AX,01000000B             ; keep D6, mask out other bits
         JNZ       TST6_ERR                 ; if not zero, we have parrity error

         MOV       AX,11H                   ; setup error code for memory test
         MOV       FS,AX                    ; error code in FS = 3
         ADD       EDX,1000H                ; next bank
         CMP       EDX,0F000H               ; last DRAM is F:FFFF
         JLE       ADB4_LOOP                ; jmp if DX < 4000

         JMP       PRO_TE                   ; enter protected mode

TST6_ERR:
         MOV       DI,CPUFLG                ; select I/O port, direct memory mapping
         MOV       CX,08H                   ; D3= 1, cpu led on; D3= 0, cpu led off
         MOV       CS:[DI],CX               ; turn CPU LED on as an error ocurred

         MOV       DI,CPUST                 ; point to CPU status I/O port
         MOV       BP,CS:[DI]               ; read  CPU sataus
         TEST      BP,200h                  ; D9=0 loop on BIT is requested
         JZ        TST6                     ; retry this test again

         JMP       $                        ; hang here if error

;----------- PROTECTED MODE ----------------

PRO_TE:  XOR       DI,DI
         MOV       ES,DI
         MOV       SI,OFFSET TEMP_GDT
         MOV       CX,END_TEMP_GDT-TEMP_GDT
         INC       CX
         REP  MOVS  BYTE PTR ES:[DI],BYTE PTR CS:[SI]
         LGDT      CS:TGDT_PWORD

         MOV       EAX,CR0
         MOV       EAX,1
         MOV       CR0,EAX
         JMP       FLUSH

FLUSH:                                      ; now entering protect mode
         MOV       BX,08H
         MOV       DS,BX                    ; ds,es,ss,all point to temp.gdt[1]
         MOV       ES,BX
         MOV       SS,BX
         MOV       ESP,40000H               ; setup stack at bottom of SRAM


AGN:     CALL      ENABLECRT                ; now display some messages on MXE ch0
         MOV       AL,03H                   ; ASCII 03 is clean screen
         CALL      DISPLAYONE
         MOV       SI,OFFSET COPYRIGHT
         OR        ESI,0FFFF0000H           ; ESI point to string first address
         MOV       ECX,AFTER_COPYRIGHT-COPYRIGHT
         CALL      DISPLAYMORE              ; display copyright statement

         MOV       DI,CPUST                 ; point to CPU status I/O port
         MOV       BP,CS:[DI]               ; read  CPU sataus
         TEST      BP,200h                  ; D9=1 loop on BIT is requested
         JNZ       COMTEST

         MOV       SI,OFFSET SRAMLOOP
         OR        ESI,0FFFF0000H           ; ESI point to string first address
         MOV       ECX,ENDSRAMLOOP-SRAMLOOP ; count bytes
         CALL      DISPLAYMORE              ; display sram size at 3rd line

         MOV       AX,2000H                 ; RTC RAM 20h contains loopcount low byte
         CALL      WRITERTC
         MOV       AX,2100H                 ; RTC RAM 21h contains loopcount hi byte
         CALL      WRITERTC                 ; reset loopcount

         JMP       MST                      ; without test card

COMTEST:
         MOV       EDI,2100000H             ; 2100000 - 22C11-1
         MOV       EBX,'CARD'

         MOV       [EDI],EBX
         MOV       EAX,[EDI]
         XOR       EAX,EBX

         JE        DISK1
CK2:
         CALL      EMS
         JE        DISK2
CK3:
         CALL      EMS
         JE        DISK3
CK4:
         CALL      EMS
         JE        MXF1
CK5:
         CALL      EMS
         JE        MXF2
CK6:
         CALL      EMS
         JE        MXF3
CK7:
         CALL      EMS
         JE        MXF4
         JMP       MST


DISK1:   MOV       SI,OFFSET DSK1
         CALL      DSYCARD
         JMP       CK2

DISK2:   MOV       SI,OFFSET DSK2
         CALL      DSYCARD
         JMP       CK3

DISK3:   MOV       SI,OFFSET DSK3
         CALL      DSYCARD
         JMP       CK4

MXF1:    MOV       SI,OFFSET MXF11
         CALL      DSYCARD
         JMP       CK5

MXF2:    MOV       SI,OFFSET MXF22
         CALL      DSYCARD
         JMP       CK6

MXF3:    MOV       SI,OFFSET MXF33
         CALL      DSYCARD
         JMP       CK7

MXF4:    MOV       SI,OFFSET MXF44
         CALL      DSYCARD


;-------------------------------------------------------------------------+
; [ TEST 7 ] MEMORY SIZE TEST                                             !
;-------------------------------------------------------------------------+

MST:     MOV       ESI,100000H
         MOV       EAX,11223344H

LMS:     MOV       [ESI],EAX
         XCHG      [ESI],EBX
         CMP       EAX,EBX
         JNE       MSIZE

         ADD       ESI,100000H
         CMP       ESI,800000H
         JL        LMS
         JMP       CHK8M

MSIZE:
         CMP       ESI,400000H             ; 4M ?
         JNE       CHK8M                   ; NO,CHK 8M

         MOV       EAX,ESI
         SHR       EAX,20

         MOV       EDI,RTCLOCK              ; 4M OR 16M
         MOV       BH,0EH
         MOV       [EDI],BH
         MOV       EDI,RTCLOC2
         MOV       [EDI],AL
         JMP       TST7

CHK8M:
         MOV       ESI,30000H               ; FLOOD 5A ,16 OR 32M
         MOV       ECX,16
         MOV       BL,5AH
FLDD:    MOV       [ESI],BL
         INC       ESI
         LOOP      FLDD

         MOV       AL,0                     ; FILL 0,1,...E,F
         MOV       ESI,830000H
LPA1:    MOV       [ESI],AL
         INC       ESI
         INC       AL
         CMP       AL,10H
         JNE       LPA1

         MOV       AL,0H
         MOV       ESI,30000H
LPA2:    MOV       BL,[ESI]
         CMP       AL,BL
         JNZ       CHK32M                   ;-----------> 16 OR 32M
         INC       ESI
         INC       AL
         CMP       AL,10H
         JNZ       LPA2

         MOV       EDI,RTCLOCK
         MOV       BH,0EH
         MOV       [EDI],BH
         MOV       EDI,RTCLOC2
         MOV       AL,8                     ; 8M
         MOV       [EDI],AL
         JMP       TST7

CHK32M:                                     ; ------------->16M OR 32M
         MOV       EDI,RTCLOCK
         MOV       AH,0EH
         MOV       [EDI],AH
         MOV       EDI,RTCLOC2
         MOV       AL,32                    ; 32M
         MOV       [EDI],AL
         JMP       TST7

;-------------------------------------------------------------------------+
;[Test 7]  DRAM Data Bus Test                                             !
;                                                                         !
; Purpose: To verify that no shorts or opens exist on DRAM data bus.      !
;          Some faults with the DRAM parity logic will be detected.       !
;          It also stores memory size at 1st byte of RTC user RAM.        !
;                                                                         !
; Description: Walking one and zero data patterns (32-bit) are written,   !
;          then read/verified for correct data and NO parity error, on    !
;          bank basis. All patterns has an odd number of 'high'(1) bits.  !
;          A double-bit rotating pattern, starting value '00000003'H, will!
;          be written/read/verified for correct data and NO parity. These !
;          patterns have an even number of 'high'(1) bits.                !
;          The routine uses address zero of DRAM of each 64KB bank,       !
;          so chip faults at that location will be flagged as well.       !
;          At each end of MB size, store the memory size to RTC.          !
;                                                                         !
; BEGIN                                                                   !
;     FOR current bank = 40000h, 50000h, ....., 730000h                   !
;         current address = first in current bank                         !
;        FOR current pattern = FFFFFFFF, 01, 02, 04, ...80000000, 0,      !
;                              7FFFFFFF, BFFFFFFF, ......., FFFFFFFE      !
;            write current pattern to current address                     !
;            read/verify current address                ERROR 20          !
;            check for memory parity error              ERROR 21          !
;        NEXT pattern                                                     !
;     NEXT bank                                                           !
;                                                                         !
;     FOR current bank = 40000h, 50000h, ..., 730000h                     !
;         current address = first in current bank                         !
;        FOR current pattern = 03, 06, 0C, 18, 30,...., C0000000, 80000001!
;            write current pattern to current address                     !
;            read/verify current address                ERROR 22          !
;            check for memory parity error              ERROR 23          !
;        NEXT pattern                                                     !
;     NEXT bank                                                           !
; END                                                                     !
;                                                                         !
; ERRORS:                                                                 !
; 20 & 22 Data bus or memory data error                                   !
;         EAX= read data    EBX= expected data     DS:SI= current address !
;         17th byte of RTC user RAM = error code                          !
;                                                                         !
; 21 & 23 Memory error, may be parity or circuit                          !
;         AX= 4, SRAM parity err                                          !
;         17th byte of RTC user RAM = error code                          !
;-------------------------------------------------------------------------+
TST7:

         MOV       ESI,040000H              ; start from address 040000H
TST7LOP1:
         MOV       EDI,RTCLOCK              ; point to RTC address select port
         MOV       AH,1Eh                   ; point to 17th byte on user RAM
         MOV       [EDI],AH
         MOV       EDI,RTCLOC2              ; point to RTC data r/w port
         MOV       AL,20h                   ; setup error code
         MOV       [EDI],AL                 ; write error code to RTC RAM

         CALL      LASTADDR

         MOV       EBX,0FFFFFFFFH           ; current data pattern
         MOV       EAX,01                   ; next pattern
         MOV       EDI,EAX                  ; #save next pattern
         MOV       ECX,33                   ; 33 patterns from FFFFFFFF to 01h
         MOV       [ESI],EBX                ; write current pattern to current address

TST7LOP2:
         XCHG      [ESI],EAX                ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST7ERR                 ; jump if bad (no jump is fast)

         MOV       EBX,EDI                  ; #save current pattern
         SHL       EDI,1                    ; #setup next data pattern
         MOV       EAX,EDI                  ; ready to write data pattern from EAX reg
         DEC       CX
         JNZ       TST7LOP2                 ; loop for next pattern test


         MOV       EAX,7FFFFFFFH            ; next data pattern 7FFFFFFF
         MOV       EDI,EAX                  ; #prepare for shift right
         XCHG      [ESI],EAX                ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST7ERR                 ; jump if bad (no jump is fast)

         MOV       EBX,EDI                  ; #save current pattern for compare later
         ROR       EDI,1                    ; #setup next data pattern
         MOV       EAX,EDI
         MOV       CX,31                    ; #31 patterns from 7FFFFFFF to FFFFFFFE

TST7LOP3:
         XCHG      [ESI],EAX                ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST7ERR                 ; jump if bad (no jump is fast)

         MOV       EBX,EDI                  ; #save current pattern for compare later
         ROR       EDI,1                    ; #setup next data pattern
         MOV       EAX,EDI                  ; #
         DEC       CX
         JNZ       TST7LOP3                 ; loop for next pattern test

         MOV       EDI,RTCLOCK              ; point to RTC address select port
         MOV       AH,1Eh                   ; point to 17th byte on user RAM
         MOV       [EDI],AH
         MOV       EDI,RTCLOC2              ; point to RTC data r/w port
         MOV       AL,21h                   ; setup error code
         MOV       [EDI],AL                 ; write error code to RTC RAM

         MOV       EBX,CPUST                ; point to CPU status port
         MOV       AX,CS:[BX]               ; read CPU status
         AND       AX,11000000B             ; keep D6 & D7, mask out other bits
         CMP       AX,0                     ; D6 & D7 should be all zero
         JNE       TST7ERR                  ; yes, all zero; no parrity error

         ADD       ESI,10000H               ; point to next 64KB bank
         CMP       ESI,EDX                  ; EDX is last byte of DRAM
         JL        TST7LOP1

         JMP       TST8                     ; go on next test

TST7ERR:
         MOV       EDI,CPUFLG               ; select I/O port, direct memory mapping
         MOV       CX,08H                   ; D3= 1, cpu led on; D3= 0, cpu led off
         MOV       CS:[DI],CX               ; turn CPU LED on

         MOV       SI,OFFSET TS6ER
         OR        ESI,0FFFF0000H
         MOV       ECX,TS6ERR-TS6ER
         CALL      DISPLAYMORE

         MOV       DI,CPUST                 ; point to CPU status I/O port
         MOV       BP,CS:[DI]               ; read  CPU sataus
         TEST      BP,200h                  ; D9=0 loop on BIT is requested
         JZ        TST7                     ; retry this test again

         JMP       TST8                     ; hang here if error


;-------------------------------------------------------------------------+
;[Test 8] DRAM Byte Decode Address Line Test                              !
;                                                                         !
; Purpose: To check for shorts and opens on the DRAM byte decode address  !
;          lines.                                                         !
;                                                                         !
; Description: Data is writen and verified to the beginning of each bank  !
;          in byte and word patterns. Doubleword patterns have been tested!
;          in the [Test 6] DRAM Data Line Test. The 80386 stores word data!
;          in memory in reverse byte order i.e. the high-order byte follo-!
;          wing the low-order byte. Therefore the high (most-significant) !
;          byte of a data word is in the highest memory address. The word !
;          read/write can be performed at both even and odd address.      !
;                                                                         !
; BEGIN                                                                   !
;     FOR current bank = DRAM                                             !
;         write 00h, 01h, 02h, 03h for the offset addresses 00h, 01h, 02h,!
;               03h respectively.                                         !
;         read/verify data 00h, 01h, 02h, 03h for the offset addresses    !
;               00h, 01h, 02h, 03h respectively.             ERROR 28     !
;         write 03h, 02h, 01h, 00h for the offset addresses 00h, 01h, 02h,!
;               03h respectively.                                         !
;         read/verify data 03h, 02h, 01h, 00h for the offset addresses    !
;               00h, 01h, 02h, 03h respectively.             ERROR 29     !
;         write AA55h, CC33h for the offset address 00h, 02h respectively.!
;         read/verify AA55h, CC33h for offset addresses 00h, 02h          !
;               respectively.                                ERROR 2A     !
;         write 9966h for offset address 01h.                             !
;         read/verify 6655h, 9966h, CC99h for offset addresses 00h, 01h,  !
;               02h respectively.                            ERROR 2B     !
;     NEXT bank                                                           !
; END                                                                     !
;                                                                         !
; ERRORS:                                                                 !
; 28 & 29,  byte decode address error                                     !
;         AL= read data,   BX & CX=  expected data,  DS:SI= current addres!
;         17th byte of RTC user RAM = error code                          !
; 2A  word decode address error at even boundry                           !
;         AX= read data,   BX & CX=  expected data,  DS:SI= current addres!
;         17th byte of RTC user RAM = error code                          !
; 2B  word decode address error at odd boundry                            !
;         AX= read data,   BX, CX & DX=  expected data,  DS:SI= current   !
;         17th byte of RTC user RAM = error code                address   !
;-------------------------------------------------------------------------+
TST8:
;         CALL      LASTADDR                 ; get last DRAM address back in EDX
;         MOV       EBP,EDX                  ; store it in EBP; EDX has other use
;         MOV       ESI,40000H               ; set 1st address of DRAM

;TST8LOP1:
;         MOV       BX,0100H                 ; byte patterns 00h, 01h, 02h, 03h
;         MOV       CX,0302H
;TST8LOP2:
;         MOV       [ESI],BL                 ; write byte 00h to address 0000h
;         MOV       [ESI+1],BH               ; write byte 01h to address 0001h
;         MOV       [ESI+2],CL               ; write byte 02h to address 0002h
;         MOV       [ESI+3],CH               ; write byte 03h to address 0003h
;         MOV       AL,[ESI]                 ; read byte data from addrs 0000h
;         CMP       AL,BL                    ; verify written byte data
;         JNZ       TST8_ERR
;         INC       ESI                      ; point to next byte address
;         MOV       AL,[ESI]                 ; read byte data from addrs 0001h
;         CMP       AL,BH                    ; verify written byte data
;         JNZ       TST8_ERR
;         INC       ESI                      ; point to next byte address
;         MOV       AL,[ESI]                 ; read byte data from addrs 0002h
;         CMP       AL,CL                    ; verify written byte data
;         JNZ       TST8_ERR
;         INC       ESI                      ; point to next byte address
;         MOV       AL,[ESI]                 ; read byte data from addrs 0003h
;         CMP       AL,CH                    ; verify written byte data
;         JNZ       TST8_ERR

;         MOV       SI,0                     ; reset offset
;         CMP       AL,0                     ; are we first time here?
;         JE        TST80                    ; yes, jump to word patterns test
;         MOV       BX,0203H                 ; byte patterns 03h, 02h, 01h, 00h
;         MOV       CX,0001H

;         JMP       TST8LOP2

;TST8_ERR:
;         MOV       DI,CPUFLG                ; select I/O port, direct memory mapping
;         MOV       CX,08H                   ; D3= 1, cpu led on; D3= 0, cpu led off
;         MOV       CS:[DI],CX               ; turn CPU LED on as an error ocurred

;         MOV       SI,OFFSET TS8ER
;         OR        ESI,0FFFF0000H
;         MOV       ECX,TS8ERR-TS8ER
;         CALL      DISPLAYMORE

;         MOV       DI,CPUST                 ; point to CPU status I/O port
;         MOV       BP,CS:[DI]               ; read  CPU sataus
;         TEST      BP,200h                  ; D9=1 loop on BIT is requested
;         JZ        TST8

;         JMP       TST9                     ; hang here if error

;TST80:
;         ADD       ESI,10000H               ; point to next 64KB bank
;         CMP       ESI,EDX                  ; last DRAM address is in EDX
;         JL        TST8LOP1                 ; otherwise go on next test

;         MOV       ESI,40000H               ; word pattern test from 4000:0
;TST8LOP3:
;         MOV       BX,0AA55H                ; word patterns 55, AA, 33, CC,
;         MOV       CX,0CC33H
;         MOV       SI,0                     ; start from address ?:0000
;         MOV       [ESI],BX                 ; write word 55, AA to addrs 00h
;         MOV       [ESI+2],CX               ; write word 33, CC to addrs 02h
;         MOV       AX,[ESI]                 ; read word data from addrs 0000h
;         CMP       AX,BX                    ; verify written word data
;         JNZ       TST8_ERR
;         ADD       ESI,2                    ; point to next word address
;         MOV       AX,[ESI]                 ; read word data from addrs 0002h
;         CMP       AX,CX                    ; verify written word data
;         JNZ       TST8_ERR

;         MOV       EBX,6655H                ; new data in byte series 55, 66,
;         MOV       ECX,9966H                ; 99, CC; in word serie are 6655,
;         MOV       EDX,0CC99H               ; 9966 and CC99.

;         MOV       SI,0                     ; start from address 0:0000
;         MOV       [ESI+1],CX               ; write word to address 0001h
;         MOV       AX,[ESI]                 ; read word data from addrs 0000h
;         CMP       AX,BX                    ; verify written word data
;         JNZ       TST8_ERR
;         INC       ESI                      ; point to odd address for word
;         MOV       AX,[ESI]                 ; read word data from addrs 0001h
;         CMP       AX,CX                    ; verify written word data
;         JNZ       TST8_ERR
;         INC       ESI                      ; point to even address for word
;         MOV       AX,[ESI]                 ; read word data from addrs 0002h
;         CMP       AX,DX                    ; verify written word data
;         JNZ       TST8_ERR

;         ADD       ESI,10000H               ; point to next 64KB bank
;         CMP       ESI,EBP                  ; last DRAM address is in EBP
;         JL        TST8LOP3                 ; otherwise go on next test


;-------------------------------------------------------------------------+
;[Test 9] DRAM Address Line Test                                          !
;                                                                         !
; Purpose: To verify DRAM address lines and check for shorts and opens on !
;          the DRAM address bus and internal RAM chip addressing.         !
;                                                                         !
; Description:                                                            !
;          Shorted Address Line Test for each bank:                       !
;             A 32-bit data pattern, 'FFFFFFFF'h, is written to the 04h   !
;             base address of SRAM bank. For the next succeeding address  !
;             line, a 32-bit pattern of '00000000'h is written (NOT verif-!
;             ied), scanning the base  address for data integrity FFFFFFFF!
;             h and NO parity errors. The testing proceeds with locations !
;             defined by each succeeding address line (until END of DRAM) !
;             written with the '00000000'h pattern, each time scanning the!
;             base  address for the 'FFFFFFFF'h pattern and verifying NO  !
;             parity errors occurred. In this manner each succeeding addr-!
;             ess line is verified to NOT conflict with the address under !
;             test. The base   address is moved to the next succeeding    !
;             address line, and the 'FFFFFFFF'h data pattern is written to!
;             this address. The algorithm proceeds repeatly until the add-!
;             ress under test reaches END of DRAM.                        !
;                                                                         !
; BEGIN                                                                   !
;     FOR current bank = DRAM                                             !
;        FOR base    address = 04h, 08h, 10h, 20h, 40h, 80h, 100h, 200h,  !
;                              80h, ......., 4000h, 8000h                 !
;            current pattern = FFFFFFFFh                                  !
;            reverse pattern = 00000000h                                  !
;            write current pattern to current address                     !
;                                                                         !
;           FOR succeeding address = 08h, 10h, 20h, 40h, ....., 8000h     !
;               write reverse pattern to succeeding address               !
;               read/verify current address                ERROR 2C       !
;               check for memory no parity error           ERROR 2D       !
;           NEXT succeeding address                                       !
;                                                                         !
;        NEXT base address                                                !
;     NEXT bank                                                           !
;                                                                         !
;     base address = 0000:0000h                                           !
;     write 'FFFFFFFF'h to base address                                   !
;     FOR current bank = DRAM                                             !
;        FOR current address = 04h, 08h, 10h, 20h, ......, 8000h          !
;           FOR current pattern = 00000000h, 01010101h                    !
;               write current pattern to current address                  !
;               read/verify base address                   ERROR 2E       !
;               check for memory no parity error           ERROR 2F       !
;           NEXT pattern                                                  !
;        NEXT address                                                     !
;     NEXT bank                                                           !
; END                                                                     !
;                                                                         !
; ERRORS:                                                                 !
; 2C Address Bus short                                                    !
;         EAX= read data    EBX= reverse pattern   DS:SI= current address !
;                           ES:[BP]= succeeding address                   !
;         17th byte of RTC user RAM = error code                          !
;                                                                         !
; 2E Address Bus Open                                                     !
;         EAX= read data    EBX= expected data     DS:SI= current address !
;         17th byte of RTC user RAM = error code                          !
;                                                                         !
; 2D & 2F Memory error, may be parity or circuit                          !
;         AX= 4, SRAM parity err                   DS:SI= base address    !
;         17th byte of RTC user RAM = error code                          !
;-------------------------------------------------------------------------+
TST9:
        CALL    LASTADDR        ; get last address in EDX
        MOV     EBX,0           ; set reverse pattern for write
        MOV     ESI,40004H      ; set first base address in DRAM
        MOV     EBP,40008H      ; set succeeding address in DRAM
        MOV     ECX,00008H      ; offset of succeeding address

TST9LOP1:
         PUSH    ECX             ; save it as next base address offset
         MOV     EAX,0FFFFFFFFH  ; get current pattern
         MOV     [ESI],EAX       ; write 'FFFFFFFF' to base address
TST9LOP2:
         MOV     [EBP],EBX       ; write reverse data to succeeding address
         MOV     EAX,[ESI]       ; read current data from current address
         CMP     EAX,0FFFFFFFFH  ; verify data
         JNE     TST9_ERR        ; jump if bad (no jump is fast)

         MOV     DI,CPUST        ; point to CPU status port
         MOV     AX,CS:[DI]      ; read CPU status
         TEST    AX,10000000B    ; keep D7, mask out other bits
         JNZ     TST9_ERR        ; if not zero, we have parrity error

         SHL     ECX,1           ; get next succeeding address offset
         CMP     ECX,40000H      ; do we need other algorithem?
         JNL     TST9LOP3        ; yes, ECX = 40000H now
         MOV     EBP,ECX
         OR      EBP,40000H      ; setup next succeeding address
         JMP     TST9LOP2        ; jump if succeeding address < 8000h

TST9LOP3:
         MOV     EBP,ECX         ; get next succeeding address
         CMP     EBP,EDX         ; do we reach the last address?
         JL      TST9LOP2        ; go and write next succeeding address

         POP     ECX             ; get next base address offset back
         MOV     ESI,ECX
         CMP     ESI,40000H      ; do we need other algorithem for ESI?
         JNL     TST9LOP5        ; yes, we do.
         OR      ESI,40000H      ; now we got next base address
         SHL     ECX,1           ; prepare next succeeding address
         MOV     EBP,ECX         ;
         OR      EBP,40000H      ;
         JMP     TST9LOP1        ; go and write next base address

TST9LOP5:
         SHL     ECX,1           ; we will use addr 40000h as open test
         MOV     ESI,ECX         ; This address is new base address
         SHL     ECX,1           ; setup succeeding address
         MOV     EBP,ECX
         CMP     EBP,EDX         ; do we reach the last address?
         JNL     TST9LOP1        ; go and write next base address
                                ; end of address line short test

         MOV     ESI,0           ; point to physical address 0
         MOV     EAX,[ESI]       ; read address 0 of SRAM
         CMP     EAX,0           ; INTEL default this GTD is zero
         JNE     TST9_ERR        ; something wrong, can't identfy

         MOV     DI,CPUST        ; point to CPU status port
         MOV     AX,CS:[DI]      ; read CPU status
         TEST    AX,10000000B    ; keep D7, mask out other bits
         JNZ     TST9_ERR        ; if not zero, we have parrity error

         MOV     ESI,40000H      ; point to first address of DRAM
         MOV     EAX,0FFFFFFFFH
         MOV     [ESI],EAX
         MOV     ESI,0           ; point to physical address 0, SRAM
         MOV     EAX,[ESI]       ; read address 0 of SRAM
         CMP     EAX,0FFFFFFFFH  ; INTEL default this GTD is zero
         JE      TST9_ERR        ; the A18 address line open, the first

         MOV     DI,CPUST        ; point to CPU status port
         MOV     AX,CS:[DI]      ; read CPU status
         TEST    AX,10000000B    ; keep D7, mask out other bits
         JNZ     TST9_ERR        ; if not zero, we have parrity error

         MOV     ESI,40000H      ; it is the only base address
         MOV     EBP,40004H      ; set succeeding address in DRAM
         MOV     ECX,00004H      ; offset of succeeding address

TST9LOP6:
         MOV     [EBP],EBX       ; write reverse data to succeeding address
         MOV     EAX,[ESI]       ; read current data from current address
         CMP     EAX,0FFFFFFFFH  ; verify data
         JNE     TST9_ERR        ; jump if bad (no jump is fast)

         MOV     DI,CPUST        ; point to CPU status port
         MOV     AX,CS:[DI]      ; read CPU status
         TEST    AX,10000000B    ; keep D7, mask out other bits
         JNZ     TST9_ERR        ; if not zero, we have parrity error

         SHL     ECX,1           ; get next succeeding address offset
         CMP     ECX,40000H      ; do we need other algorithem?
         JNL     TST9LOP7        ; ECX = 40000H now, go away
         MOV     EBP,ECX
         OR      EBP,40000H      ; setup next succeeding address
         JMP     TST9LOP6        ; jump if succeeding address < 8000h

TST9_ERR:
         MOV     DI,CPUFLG       ; select I/O port, direct memory mapping
         MOV     CX,08H          ; D3= 1, cpu led on; D3= 0, cpu led off
         MOV     CS:[DI],CX      ; turn CPU LED on as an error ocurred

         MOV       SI,OFFSET TS9ER
         OR        ESI,0FFFF0000H
         MOV       ECX,TS9ERR-TS9ER
         CALL      DISPLAYMORE

         MOV     DI,CPUST        ; point to CPU status I/O port
         MOV     BP,CS:[DI]      ; read  CPU sataus
         TEST    BP,200h         ; D9=1 loop on BIT is requested
         JZ      TST9

         JMP       TSTA

TST9LOP7:
         CMP     ECX,40000H
         JG      TST9LOP8
         SHL     ECX,1           ; by pass base address
TST9LOP8:
         MOV     EBP,ECX
         SHL     EBP,1           ; get next succeeding address
         CMP     EBP,EDX         ; do we reach the last address?
         JL      TST9LOP6        ; go and write next succeeding address

STARTS:  MOV     SI,OFFSET DRAMSIZE ; + 0C000H ;point to string 'DRAM ???? KB'
         OR      ESI,0FFFF0000H  ; convert to flat address
         MOV     ECX,ENDDRAMSIZE - DRAMSIZE
         CALL    DISPLAYMORE     ;

;-------------------------------------------------------------------------+
;[Test A]  DRAM Data Fast Exchange Function TEST                          !
;                                                                         !
;Purpose:  To test ability of all DRAM memory cells to hold a zero or a   !
;          one and detec dynamic data/address faults.                     !
;                                                                         !
;Description: The test is very fast because it takes advantage of various !
;          CPU instructions. Three passes from the first to last memory   !
;          address of DRAM are required to verify address and data faults !
;          on both the data and parity DRAM.                              !
;                                                                         !
; BEGIN                                                                   !
;     FOR DRAM first to last by 4's Do                                    !
;         write address = "FFFFFFFF"                                      !
;     END (for DRAM)                                                      !
;                                                                         !
;     FOR DRAM first to last by 4's Do                                    !
;         exchange "FFFFFFFF" with "00000000"                             !
;         verify read data = "FFFFFFFF"                    ERROR 31       !
;         read pattern "00000000"                                         !
;         verify read data = "00000000"                    ERROR 31       !
;                                                                         !
;         exchange "00000000" with "F4F4F4F4"                             !
;         verify read data = "00000000"                    ERROR 31       !
;         read pattern "F4F4F4F4F"                                        !
;         verify read data = "F4F4F4F4"                    ERROR 31       !
;     END (for DRAM)                                                      !
;                                                                         !
;     Verify parity error                                  ERROR 32       !
; END                                                                     !
;                                                                         !
; ERRORS:                                                                 !
; 31  Address Bus or Memory Data error                                    !
;     EAX= read data    EBX= new write data     DS:SI= current or next    !
;                       EBP= last written data         address            !
;     17th byte of RTC user RAM = error code                              !
;                                                                         !
; 32  Memory error, may be parity or circuit                              !
;     AX= 8, DRAM parity err                                              !
;     17th byte of RTC user RAM = error code                              !
;-------------------------------------------------------------------------+

TSTA:
        CALL    LASTADDR        ; upon return, EDX is last DRAM address
        MOV     EDI,40000H      ; set 1st address of DRAM
        MOV     EBX,0FFFFFFFFH  ; set flood data pattern
        CLD                     ; clear dir flag so string instr incrmnt DI
;---------------US16 can not use this instruction --------
;       REP     STOSD           ; flood segment
;---------------------------------------------------------
TSTALOP1:
        MOV     [EDI],EBX       ; # write data to memory
        ADD     EDI,4           ; # point to next address
        CMP     EDI,EDX         ;
        JL      TSTALOP1
                                ; end first scan
;       MOV     AX,1E31H        ; setup error code 31h to RTC RAM
;       CALL    WRITERTC        ; point to 17th byte on user RAM
;       MOV     AX,0F00H        ; reset update counter in RTC RAM
;       CALL    WRITERTC        ; point to addr 0Fh on user RAM

        MOV     CL,0FFH         ; # EBP,0FFFFFFFFH  ; set last written pattern
        MOV     BX,0000H        ; # set revers pattern for write
        MOV     ESI,40000H      ; start from 1st byte of DRAM

TSTALOP2:
        MOV     AL,BL           ; # get revers pattern
        XCHG    [ESI],AL        ; # read 'FFFFFFFF',write '00000000'
        CMP     AL,CL           ; # EAX,EBP         ; verify data
        JNE     TSTAERR         ; jump if bad (no jump is fast)
;----------------------------------------------------------------------+
;       LODSD                   ; read '00000000' back and inc SI by 4 !
;       LOOPZ   TSTALOP4        ; loop as long as CX not 0 and no error!
;       JNE     TSTAERR         ; jump if bad (no jump is fast)        !
;----------------------------------------------------------------------+
        MOV     AL,[ESI]        ; # read '00000000' back

        CMP     AL,BL           ; # EAX,EBX         ; verify data
        JNE     TSTAERR         ; jump if bad (no jump is fast)
        INC     ESI             ; # point ESI to next address
        TEST    ESI,1FFFFH      ; do we at 128KB boundry?
        JNZ     TSTALOP2        ; no, go on r/w/r test

        CALL    UPDATESIZE
        CMP     ESI,EDX         ; do we reach last address?
        JL      TSTALOP2        ; no,go on r/w/r test
                                ; end second scan

;----------------------------------------------------------------------+
;       LODSD                   ; read 'F4F4' back and inc SI by 4     !
;       LOOPZ   TSTALOP3        ; loop as long as CX not 0 and no error!
;       JNE     TSTAERR         ; jump if bad (no jump is fast)        !
;----------------------------------------------------------------------+

;---------------US16 can not use this instruction --------
;       REP     STOSD           ; flood segment
;---------------------------------------------------------

        MOV     DI,CPUST        ; point to CPU status I/O port
        MOV     BP,CS:[DI]      ; read  CPU sataus
        TEST    BP,200h         ; D9=1 loop on BIT is requested

        JNZ     TSS             ; From here to LOADER

        MOV     AX,2000h        ; RTC 20h contains loopcount low byte
        CALL    READRTC
        MOV     BL,AL           ; save loopcount low byte
        MOV     AX,2100h        ; RTC 21h contains loopcount hi byte
        CALL    READRTC
        MOV     BH,AL           ; save loopcount hi byte

        INC     BX              ; increase loop count
        MOV     AH,20h          ; RTC RAM 20h contains loopcount low byte
        MOV     AL,BL           ; update loopcount
        CALL    WRITERTC
        MOV     AH,21h          ; RTC RAM 21h contains loopcount hi byte
        MOV     AL,BH           ; update loopcount
        CALL    WRITERTC

        MOV     SI,OFFSET LOOPCONT ; + 0C000H
        OR      ESI,0FFFF0000H  ; ESI point to string first address
        MOV     ECX,ENDLOOPCONT - LOOPCONT      ;count bytes
        CALL    DISPLAYMORE     ; position cursor to least significant digit
        MOV     AX,BX           ; put loopcount to AX for display
        CALL    UPDATENUM       ; display loopcount at terminal srceen

        MOV     ECX,0FFFFFFFh   ; time delay in order user
        LOOP    $               ; can see the last DRAM size on screen

        MOV     SI,OFFSET DRAMCLEN
        OR      ESI,0FFFF0000H  ; ESI point to string first address
        MOV     ECX,ENDDRAMCLEN - DRAMCLEN      ;count bytes
        CALL    DISPLAYMORE     ; clean msg DRAM ???? KB on screen
        JMP     TST7            ; let's go back in loop

TSTAERR:

        MOV     EDI,CPUFLG      ; select I/O port, direct memory mapping
        MOV     CX,08H          ; D3= 1, cpu led on; D3= 0, cpu led off
        MOV     CS:[DI],CX      ; turn CPU LED on

         MOV       SI,OFFSET TSAER
         OR        ESI,0FFFF0000H
         MOV       ECX,TSAERR-TSAER
         CALL      DISPLAYMORE

        MOV     DI,CPUST
        MOV     BP,CS:[DI]      ; read  CPU sataus
        TEST    BP,200h         ; D9=1 loop on BIT is requested
        JZ      TSTA

        JMP     $               ; hang here if error


;********* all these subroutins are vailid only in Protect Mode *******

;-------->>>>>------------------+
DISPLAYMORE     PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: ESI is point to the first byte of string                     ;
;          ECX is the number of bytes in the string                     ;
;on exit : none                                                         ;
;reg destroyed: none                                                    ;
;-----------------------------------------------------------------------+
        PUSH    EAX
DISPLAYMLOP:
        MOV     AL,[ESI]        ;
        CALL    DISPLAYONE
        INC     ESI
        LOOP    DISPLAYMLOP
        POP     EAX
        RET
;-------------------------------+
DISPLAYMORE     ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
DISPLAYONE      PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AL is the ASCII code to be displayed                         ;
;on exit : none                                                         ;
;reg destroyed: none                                                    ;
;-----------------------------------------------------------------------+
        CALL    CHK5USPULS      ; delay 5useconds before next character
        CALL    CHKIORDY        ; be sure CRT is ready for next send
        PUSH    EDI
        MOV     EDI,_OBS        ; point to output data bus strobe
        MOV     [EDI],AL        ; send out ascii
        POP     EDI
        RET
;-------------------------------+
DISPLAYONE      ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
ENABLECRT       PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AL is the device address to be enabled, CRT=05, PRT=         ;
;on exit : none
;reg destroyed: none                                                    ;
;-----------------------------------------------------------------------+
        PUSH    DI
        MOV     DI,_ABS         ;point to I/O address bus strobe
        MOV     AL,5            ;device CRT's address is 05H
        MOV     CS:[DI],AL      ;#
        CALL    CHK5USPULS      ;wait for two 5usec pulses
        CALL    CHKIORDY        ;make sure I/O bus is ready
        POP     DI
        RET
;-------------------------------+
ENABLECRT       ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHK5USPULS      PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: check CPUST D0= 1 twice in order to get min dealy 5usec       ;
;on entry: none                                                         ;
;on exit : none                                                         ;
;regs destroy: none                                                     ;
;-----------------------------------------------------------------------+
        PUSH    DI
        PUSH    AX
        MOV     DI,8002H        ;point tp CPU I/O status port
CHK5USPULSA:
        MOV     AL,BYTE PTR CS:[DI]   ;#
        TEST    AL,01H          ;D0=1, 5usec pulse for ABS/, OBS/, CBS/
        JZ      CHK5USPULSA     ;try again, be sure got first pulse
CHK5USPULSB:
        MOV     AL,BYTE PTR CS:[DI]   ;#
        TEST    AL,01H          ;D0=1, 5usec pulse for ABS/, OBS/, CBS/
        JZ      CHK5USPULSB     ;try again in order to get second pulse
        POP     AX
        POP     DI
        RET
;-------------------------------+
CHK5USPULS      ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHKIORDY        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: check CPUST D3=1 as I/O ready                                 ;
;on entry: none                                                         ;
;on exit : none                                                         ;
;regs destroy: none                                                     ;
;-----------------------------------------------------------------------+
        PUSH    DI
        MOV     DI,CPUST
        TEST    BYTE PTR CS:[DI],08H ; # D3=0,I/O not ready;D3=1,I/O raedy
        POP     DI
        JZ      CHKIORDY        ; if not ready, let's wait
        RET
;-------------------------------+
CHKIORDY        ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHKPARTY        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: verify whether parity err occurred or not. if occurred, loop. !
;on entry: AH is the RTC RAM address, AL is error code                  !
;on exit : none, if no parity err. otherwise loop within this subroutine!
;regs destroy: none                                                     !
;-----------------------------------------------------------------------+
        CALL    WRITERTC        ; write err code first
        PUSH    EAX
        PUSH    EBX
        MOV     BX,CPUST        ; point to CPU status port
        MOV     AX,CS:[BX]      ; # read CPU status
        AND     AX,11000000B    ; keep D6 & D7, mask out other bits
        CMP     AX,0            ; D6 & D7 should be all zero
        JE      CHKPRTYA        ; if no parity err, go exit
        JMP     $               ; if parity err, stop here
CHKPRTYA:
        POP     EBX
        POP     EAX
        RET
;-------------------------------+
CHKPARTY        ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATENUM       PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: convert word hex number to decimal and display it.            !
;on entry: AX contains hex number, cursor points to least signif digit. !
;on exit : none                                                         !
;regs destroy: none                                                     !
;-----------------------------------------------------------------------+
        PUSH    EAX
        PUSH    EBX
        PUSH    ECX
        PUSH    EDX

        MOV     EBX,10          ; set divisor
        MOV     ECX,4           ; maximun 5 digits, there are 4 times DIV
UPDATENUM1:
        MOV     EDX,0           ; clear remainder
        DIV     BX              ; (DX+AX)/BX=AX+DX
        ADD     DL,30h          ; convert remainder to ascii
        XCHG    AX,DX           ; put ascii to AX for display
        CALL    DISPLAYONE      ; display this ascii to screen

        MOV     AL,08h          ; ascii 08h is back space
        CALL    DISPLAYONE      ; mov cursor back
        MOV     AL,08h          ; ascii 08h is back space
        CALL    DISPLAYONE      ; set cursor to next digit position

        CMP     EDX,0           ; is quotient zero?
        JE      UPDATENUM2      ; it is zero, no more display

        MOV     AX,DX           ; restore quotien for next div
        LOOP    UPDATENUM1      ; continue division

        ADD     AL,30h          ; convert last quotient to ascii
        CALL    DISPLAYONE      ; display this ascii to screen

UPDATENUM2:

        POP     EDX
        POP     ECX
        POP     EBX
        POP     EAX
        RET

;-------------------------------+
UPDATENUM       ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATESIZE      PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: show dram memory size on screen in 128KB increasment          ;
;on entry:none on register, but cursor point to least significant digit ;
;on exit :none, but cursor point to least significsnt digit             ;
;regs destroy: none                                                     ;
;-----------------------------------------------------------------------+
        PUSH    ESI
        PUSH    EDX
        PUSH    ECX
        PUSH    EAX

        MOV     EAX,ESI         ; save surrent address
        MOV     SI,OFFSET DRAMLOOP ;  + 0C000H
        OR      ESI,0FFFF0000H  ; ESI point to string first address
        MOV     ECX,ENDDRAMLOOP - DRAMLOOP      ;count bytes
        CALL    DISPLAYMORE     ; reset cursor at least significant digit

        MOV     ECX,1024        ; divided by 1024
;       SUB     EAX,40000H      ; deduct sram size
        MOV     EDX,0           ; clear remainder
        DIV     ECX             ; (EDX+EAX)/ECX= EAX + EDX
        CALL    UPDATENUM       ; convert size to decimal and display

        POP     EAX
        POP     ECX
        POP     EDX
        POP     ESI
        RET
;-------------------------------+
UPDATESIZE      ENDP            ;
;--------<<<<<------------------+

;-------->>>>>------------------+
LASTADDR        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: none                                                         !
;on exit : EDX is the byte number of DRAM, or the last DRAM address + 1 !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+

        PUSH    EBP
        MOV     EBP,RTCLOCK     ; point to RTC address select port
        MOV     EDX,0Eh         ; point to 1st byte on user RAM
        MOV     [EBP],DL        ;
        MOV     EBP,RTCLOC2     ; point to RTC data r/w port
        MOV     DL,[EBP]        ; read DRAM size from RTC RAM
        SHL     EDX,20          ; convert size to flat address
;       ADD     EDX,40000H      ; ##  plus 1st byte offset
        POP     EBP
        RET
;-------------------------------+
LASTADDR        ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
WRITERTC        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AH= RTC user RAM address (0E-FF),                            !
;          AL= data to be written to RTC user RAM                       !
;on exit : none                                                         !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+
        PUSH    EBP             ;
        MOV     EBP,RTCLOCK     ; point to RTC address select port
        MOV     [EBP],AH        ; AH is the user ram address
        MOV     EBP,RTCLOC2     ; point to RTC data r/w port
        MOV     [EBP],AL        ; write data code to RTC RAM
        POP     EBP             ;
        RET                     ;
;-------------------------------+
WRITERTC        ENDP            ;
;--------<<<<<------------------+---------------------------------------+
;The RTC user RAM address starts from 0Eh to FFh, total 50 bytes.       *
;Diagnostic reserved following address:                                 *
;       [0Eh]= the DRAM memory size in unit of MB,bootstrap will read it*
;       [0Fh]= external counter for update DRAM size shown on screen    *
;       [1Eh]= error code store here during protect mode                *
;       [1Fh]= MNI flag; 0= no NMI, 1= NMI occured during diagnostics   *
;       [20h]= loop count low byte                                      *
;       [21h]= loop count hi byte                                       *
;-------->>>>>------------------+---------------------------------------+
READRTC         PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AH is the address of RTC user RAM,                           !
;on exit : AL is the data read from RTC user RAM at address 'AH'        !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+
        PUSH    EBP             ;
        MOV     EBP,RTCLOCK     ; point to RTC address select port
        MOV     [EBP],AH        ; AH is the user ram address
        MOV     EBP,RTCLOC2     ; point to RTC data r/w port
        MOV     AL,[EBP]        ; read data code from RTC RAM
        POP     EBP             ;
        RET                     ;
;-------------------------------+
READRTC         ENDP            ;
;--------<<<<<------------------+

;--------------------------------------------
SIMM     PROC      NEAR
         PUSH      ESI
         MOV       SI,OFFSET BADSIMM
         OR        ESI,0FFFF0000H
         MOV       ECX,BADSIMML-BADSIMM
         CALL      DISPLAYMORE
         POP       ESI
         RET
SIMM     ENDP
;----------------------------------------------
EMS      PROC      NEAR
         ADD       EDI,100000H
         MOV       [EDI],EBX
         MOV       EAX,[EDI]
         XOR       EAX,EBX
         RET
EMS      ENDP
;-------------------------------------------------
DSYCARD  PROC      NEAR
         OR        ESI,0FFFF0000H
         MOV       ECX,31
         CALL      DISPLAYMORE
         RET
DSYCARD  ENDP
;-----------------------------------------

TSS:
;
; copy eprom gdt to ram
;
        mov     esi,dword ptr gdt_eprom +2
        mov     edi,gdtbase
        mov     cx,word ptr gdt_eprom +0
        inc     cx
        shr     cx,1
        cld
        rep     movs word ptr es:[edi],word ptr ds:[esi]

; copy eprom idt to ram

        mov     esi,dword ptr idt_eprom +2
        mov     edi,idtbase
        mov     cx,word ptr idt_eprom +0
        inc     cx
        shr     cx,1
        cld
        rep     movs word ptr es:[edi],word ptr ds:[esi]
;
; switch to ram gdt
;
        lidt    idt_pword
        lgdt    gdt_pword

        mov     bx,gdt_alias
        mov     ds,bx
;
; copy eprom tss to ram
;
        mov     bx,init_tss_a
        mov     es,bx
        mov     bx,init_tss
        lar     dx,bx
        mov     [bx].access,ds_access
        mov     fs,bx
        xor     si,si
        xor     di,di
        mov     cx,[bx].lim_0_15
        inc     cx
        rep     movs byte ptr es:[di],byte ptr fs:[si]
        mov     [bx].access,dh

; alter the base address of the initial task's tss
        mov     ax,init_tss_a_offset.bas_0_15
        mov     init_tss_offset.bas_0_15,ax
        mov     al,init_tss_a_offset.bas_16_23
        mov     init_tss_offset.bas_16_23,al
        mov     al,init_tss_a_offset.bas_24_31
        mov     init_tss_offset.bas_24_31,al
;
; change init_tss_a to form a save area for tss on the first task switch
;
        mov     bx,init_tss_a
        mov     word ptr [bx].bas_0_15,1A80h
        mov     [bx].bas_16_23,05h
        mov     [bx].bas_24_31,0
        mov     [bx].access,tss_access
        mov     [bx].gran,0
        ltr     bx
;
; copy eprom ldt to ram
;
        mov     bx,init_ldt_a
        mov     es,bx
        mov     ah,[bx].bas_24_31
        mov     al,[bx].bas_16_23
        shl     eax,16
        mov     ax,[bx].bas_0_15
;
        mov     bx,init_ldt
        lar     dx,bx
        mov     [bx].access,ds_access
        mov     fs,bx
        xor     si,si
        xor     di,di
        mov     cx,[bx].lim_0_15
        inc     cx
        rep     movs byte ptr es:[di],byte ptr fs:[si]
        mov     [bx].access,dh
;
; alter the base address of ldt's alias
        mov     es:[init_ldt_alias].bas_0_15,ax
        shr     eax,16
        mov     es:[init_ldt_alias].bas_16_23,al
        mov     es:[init_ldt_alias].bas_24_31,ah
;
; alter the base address of initial task's ldt
        mov     ax,init_ldt_a_offset.bas_0_15
        mov     init_ldt_offset.bas_0_15,ax
        mov     al,init_ldt_a_offset.bas_16_23
        mov     init_ldt_offset.bas_16_23,al
        mov     al,init_ldt_a_offset.bas_24_31
        mov     init_ldt_offset.bas_24_31,al
;
; now gdt, idt, initial tss and initial ldt are all set up
; start the first task

;------------ MOV TEST PROGRAM -------------------------

        MOV     bx,08h
        mov     es,bx

        MOV     ESI,OFFSET TSSTEST                       ; MOV DS:SI ==> ES:DI
        MOV     EDI,0
        MOV     CX,TSSEND-TSSTEST                        ; COUNT BYTE

REP     MOVS BYTE  PTR ES:[EDI],BYTE PTR CS:[ESI]


        jmp     entry_pointer


;--------  TASK SWITCH TEST PROGRAM  ---------------

TSSTEST LABEL BYTE

        MOV     DI,_ABS
        MOV     BYTE PTR DS:[DI],5            ; ENABLE CRT OUTPUT

DISPLAY_1:
        MOV     DI,CPUST
        TEST    BYTE PTR DS:[DI],08H          ; TEST IF I/O READY
        JZ      DISPLAY_1
        MOV     ESI,OFFSET MSG

DISPLAY_2:
        MOV     AL,[ESI]
        CMP     AL,00H
        JZ      DISY
        MOV     DI,_OBS1
        MOV     BYTE PTR DS:[DI],AL
        INC     ESI

        MOV     ECX,4000H
        LOOP    $
        JMP     DISPLAY_2

DISY:   MOV     ESI,OFFSET DRAMSIZE ; + 0C000H ;point to string 'DRAM ???? KB'
        OR      ESI,0FFFF0000H  ; convert to flat address
        MOV     ECX,ENDDRAMSIZE - DRAMSIZE
        CALL    DISPLAYMORE_1   ;

TSTAA:
        CALL    LASTADDR_1      ; upon return, EDX is last DRAM address

        MOV     EDI,40000H      ; set 1st address of DRAM
        MOV     EBX,0FFFFFFFFH  ; set flood data pattern
        CLD                     ; clear dir flag so string instr incrmnt DI
TSTALOP11:
        MOV     [EDI],EBX       ; # write data to memory
        ADD     EDI,4           ; # point to next address
        CMP     EDI,EDX         ; max address is 83FFFFh
        JL      TSTALOP11

        MOV     CL,0FFH         ; # EBP,0FFFFFFFFH  ; set last written patt
        MOV     BX,0000H        ; # set revers pattern for write
        MOV     ESI,40000H      ; start from 1st byte of DRAM

TSTALOP22:
        MOV     AL,BL           ; # get revers pattern
        XCHG    [ESI],AL        ; # read 'FFFFFFFF',write '00000000'
        CMP     AL,CL           ; # EAX,EBP         ; verify data
        JNE     TSTAERR1        ; jump if bad (no jump is fast)
        MOV     AL,[ESI]        ; # read '00000000' back

        CMP     AL,BL           ; # EAX,EBX         ; verify data
        JNE     TSTAERR1        ; jump if bad (no jump is fast)
        INC     ESI             ; # point ESI to next address
        TEST    ESI,1FFFFH      ; do we at 128KB boundry?
        JNZ     TSTALOP22       ; no, go on r/w/r test

        MOV     AX,1E32H        ; write error code 32H in RTC RAM
        CALL    CHKPARTY_1      ; and check parity err every 256KB
        CALL    UPDATESIZE_1
        CMP     ESI,EDX         ; do we reach last address?
        JL      TSTALOP22       ; no,go on r/w/r test

        MOV     CL,0            ; # set last written pattern
        MOV     BL,0F4H         ; # after test,all address have F4
        MOV     ESI,40000H      ; start from 1st byte DRAM

TSTALOP33:
        MOV     AL,BL           ; # get revers pattern
        XCHG    [ESI],AL        ; # read '0000',write 'F4F4F4F4'
        CMP     AL,CL           ; # verify data
        JNE     TSTAERR1        ; jump if bad (no jump is fast)

        MOV     AL,[ESI]        ; # read 'F4F4F4F4' back
        CMP     AL,BL           ; # verify data
        JNE     TSTAERR1        ; jump if bad (no jump is fast)
        INC     ESI             ; # point ESI to next address
        TEST    ESI,1FFFFH      ; do we at 128KB boundry?
        JNZ     TSTALOP33       ; no, go on r/w/r test

        MOV     AX,1E33H        ; write error code 33H in RTC RAM
        CALL    CHKPARTY_1      ; and check parity err every 256KB

        CALL    UPDATESIZE_1
        CMP     ESI,EDX         ; do we reach last address?
        JL      TSTALOP33       ; no,go on r/w/r test
                                ; end third scan

        JMP     INTTEST

 TSTAERR1:

        PUSH       ESI          ; #
        MOV     EDI,CPUFLG      ; select I/O port, direct memory mapping
        MOV     CX,08H          ; D3= 1, cpu led on; D3= 0, cpu led off
        MOV     DS:[DI],CX      ; turn CPU LED on

        MOV       SI,OFFSET TSAER
        OR        ESI,0FFFF0000H
        MOV       ECX,TSAERR-TSAER
        CALL      DISPLAYMORE

        MOV     DI,CPUST
        MOV     BP,DS:[DI]      ; read  CPU sataus
        TEST    BP,200h         ; D9=1 loop on BIT is requested
        JZ      TSTAA

        JMP     $               ; hang here if error

INTTEST:MOV     ESI,OFFSET INTT
        OR      ESI,0FFFF0000H  ; convert to flat address
        MOV     ECX,INTT1 - INTT
        CALL    DISPLAYMORE_1   ;

         MOV       ESI,OFFSET NMIMESS
         INT       02H

         MOV       ESI,OFFSET ISR21H
         INT       21H

         MOV       ESI,OFFSET ISR6FH
         INT       6FH

         JMP       DISY


NMIMESS   DB        ' This is NMI Service Routine  !!',00H
ISR21H    DB        0DH,0AH,16 DUP(20H),' Enter INT 21H Service Routine !',00H
ISR6FH    DB        0DH,0AH,16 DUP(20H),' Enter INT 6FH Service Routine !',00H


;--------------------------------------------------
LASTADDR_1      PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: none                                                         !
;on exit : EDX is the byte number of DRAM, or the last DRAM address + 1 !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+

        PUSH    EBP
        MOV     BP,RTCADDR     ; point to RTC address select port
        MOV     DL,0Eh         ; point to 1st byte on user RAM
        MOV     DS:[BP],DL        ;

        MOV     BP,RTCDATA     ; point to RTC data r/w port
        MOV     DL,DS:[BP]        ; read DRAM size from RTC RAM
        SHL     EDX,20          ; convert size to flat address
;       ADD     EDX,40000H      ; ##  plus 1st byte offset
        POP     EBP
        RET
;-------------------------------+
LASTADDR_1      ENDP            ;
;--------<<<<<------------------+

DISPLAYMORE_1   PROC    NEAR    ;
        PUSH    EAX
DISPLAYMLOP_1:
        MOV     AL,[ESI]        ;
        CALL    DISPLAYONE_1
        INC     ESI
        LOOP    DISPLAYMLOP_1
        POP     EAX
        RET
DISPLAYMORE_1   ENDP            ;

;--------------------------------------------

 DISPLAYONE_1    PROC    NEAR    ;
        CALL    CHK5USPULS_1    ; delay 5useconds before next character

        CALL    CHKIORDY_1      ; be sure CRT is ready for next send
        PUSH    EDI
        MOV     DI,_OBS1        ; point to output data bus strobe
        MOV     DS:[DI],AL        ; send out ascii
        POP     EDI
        RET
 DISPLAYONE_1    ENDP            ;

 ;---------------------------------------------

 CHK5USPULS_1    PROC    NEAR    ;
        PUSH    DI
        PUSH    AX
        MOV     DI,CPUST        ;point tp CPU I/O status port
 CHK5USPULSAA:
        MOV     AL,BYTE PTR DS:[DI]   ;#
        TEST    AL,01H          ;D0=1, 5usec pulse for ABS/, OBS/, CBS/
        JZ      CHK5USPULSAA    ;try again, be sure got first pulse
 CHK5USPULSBB:
        MOV     AL,BYTE PTR DS:[DI]   ;#
        TEST    AL,01H          ;D0=1, 5usec pulse for ABS/, OBS/, CBS/
        JZ      CHK5USPULSBB    ;try again in order to get second pulse
        POP     AX
        POP     DI
        RET
 CHK5USPULS_1    ENDP            ;

 ;---------------------------------------------

 CHKIORDY_1      PROC    NEAR    ;
        PUSH    DI
        MOV     DI,CPUST
        TEST    BYTE PTR DS:[DI],08H ; # D3=0,I/O not ready;D3=1,I/O raedy
        POP     DI
        JZ      CHKIORDY_1      ; if not ready, let's wait
        RET
CHKIORDY_1       ENDP            ;

;----------------------------------------------

CHKPARTY_1      PROC    NEAR    ;
        PUSH    EAX
        PUSH    EBX
        MOV     BX,CPUST         ; point to CPU status port
        MOV     AX,DS:[BX]      ; # read CPU status
        AND     AX,11000000B    ; keep D6 & D7, mask out other bits
        CMP     AX,0            ; D6 & D7 should be all zero
        JE      CHKPRTYAA       ; if no parity err, go exit
        JMP     $               ; if parity err, stop here
CHKPRTYAA:
        POP     EBX
        POP     EAX
        RET
CHKPARTY_1      ENDP            ;

;----------------------------------------------------

UPDATENUM_1     PROC    NEAR    ;

        PUSH    EAX
        PUSH    EBX
        PUSH    ECX
        PUSH    EDX

        MOV     EBX,10          ; set divisor
        MOV     ECX,4           ; maximun 5 digits, there are 4 times DIV
UPDATENUM11:
        MOV     EDX,0           ; clear remainder
        DIV     BX              ; (DX+AX)/BX=AX+DX
        ADD     DL,30h          ; convert remainder to ascii
        XCHG    AX,DX           ; put ascii to AX for display
        CALL    DISPLAYONE_1    ; display this ascii to screen

        MOV     AL,08h          ; ascii 08h is back space
        CALL    DISPLAYONE_1    ; mov cursor back
        MOV     AL,08h          ; ascii 08h is back space
        CALL    DISPLAYONE_1    ; set cursor to next digit position

        CMP     EDX,0           ; is quotient zero?
        JE      UPDATENUM22     ; it is zero, no more display

        MOV     AX,DX           ; restore quotien for next div
        LOOP    UPDATENUM11     ; continue division

        ADD     AL,30h          ; convert last quotient to ascii
        CALL    DISPLAYONE_1    ; display this ascii to screen

UPDATENUM22:
        POP     EDX
        POP     ECX
        POP     EBX
        POP     EAX
        RET

 UPDATENUM_1     ENDP            ;

 ;---------------------------------------------------

 UPDATESIZE_1    PROC    NEAR    ;

        PUSH    ESI
        PUSH    EDX
        PUSH    ECX
        PUSH    EAX

        MOV     EAX,ESI         ; save surrent address
        MOV     SI,OFFSET DRAMLOOP ;  + 0C000H
        OR      ESI,0FFFF0000H  ; ESI point to string first address


        MOV     ECX,ENDDRAMLOOP - DRAMLOOP      ;count bytes
        CALL    DISPLAYMORE_1   ; reset cursor at least significant digit

        MOV     ECX,1024        ; divided by 1024
;       SUB     EAX,40000H      ; deduct sram size
        MOV     EDX,0           ; clear remainder
        DIV     ECX             ; (EDX+EAX)/ECX= EAX + EDX
        CALL    UPDATENUM_1     ; convert size to decimal and display

        POP     EAX
        POP     ECX
        POP     EDX
        POP     ESI
        RET
   UPDATESIZE_1    ENDP            ;

TSSEND  LABEL  BYTE

;-------------------Interrupt Service Routine --------------------------

            ORG       0D200H

         MOV       BX,CPUFLG
         MOV       AX,8H
         MOV       DS:[BX],AX

         MOV       BYTE PTR DS:[_ABS],5
DY_1:
         TEST      BYTE PTR DS:[CPUST],08H
         JZ        DY_1
DY_2:
         MOV       AL,[ESI]
         CMP       AL,00H
         JZ        DY_RET
         MOV       BYTE PTR DS:[_OBS1],AL
         INC       ESI
         MOV       ECX,4000H
         LOOP      $
         JMP       DY_2

DY_RET:  IRETD


             ORG       ADJGDT                   ; 0D400H

     DB       00H,00H,00H,00H,00H,00H,00H,00H    ; NULL_DES
     DB       00H,10H,00H,20H,00H,92H,00H,00H
     DB       4FH,00H,00H,10H,00H,92H,00H,00H    ; GDT_ALIAS
     DB       00H,01H,00H,00H,0FFH,9AH,80H,0FFH    ; IDT_ALIAS
     DB       67H,00H,50H,0D4H,0FFH,89H,00H,0FFH ; INIT_TSS
     DB       67H,00H,50H,01H,00H,92H,00H,00H    ; INIT_TSS_A
     DB       27H,00H,00H,0D5H,0FFH,82H,00H,0FFH ; INIT_LDT
     DB       27H,00H,50H,02H,00H,92H,00H,00H    ; INIT_LDT_A
     DB       00H,00H,00H,00H,00H,00H,00H,00H    ; RESERVE
     DB       00H,00H,00H,00H,00H,00H,00H,00H    ; RESERVE


              ORG       ADJTSS                   ; 0D450H

    DB       00H,00H,00H,00H,00H,00H,00H,00H,24H,00H,00H,00H,00H,00H,00H,00H
    DB       00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
    DB       00H,00H,00H,00H,00H,02H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
    DB       00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
    DB       00H,00H,00H,00H,00H,00H,00H,00H,1CH,00H,00H,00H,14H,00H,00H,00H
    DB       24H,00H,00H,00H,1CH,00H,00H,00H,1CH,00H,00H,00H,1CH,00H,00H,00H
    DB       30H,00H,00H,00H,00H,00H,00H,00H


               ORG       ADJLDT                   ; 0D500H

    DB       00H,00H,00H,00H,00H,00H,00H,00H
    DB       27H,00H,50H,02H,00H,92H,00H,00H
    DB       0FFH,0FFH,00H,20H,00H,9AH,8FH,00H
    DB       0FFH,0FFH,00H,00H,0FFH,92H,8FH,0FFH
    DB       0FEH,0FFH,00H,03H,00H,96H,0CFH,00H


               ORG        ADJIDT                   ; 0D550H

    DB       8 DUP(0)                          ; IDT 0, NULL
    DB       8 DUP(0)                          ; IDT 1, NULL
    DB       00H,0D2H,18H,00H,00H,8FH,00H,00H  ; IDT 2, NMISR


               ORG        0D650H

    DB       8 DUP(0)                          ; IDT 32,NULL
    DB       00H,0D2H,18H,00H,00H,8FH,00H,00H  ; IDT 33, INT 21H
                                               ;  :
                                               ;  :
               ORG        0D850H               ;  :
    DB   32 DUP(00H,0D2H,18H,00H,00H,8FH,00H,00H)  ; IDT 111, INT6F


        ORG     ADJUST2
        JMP     START

reset_code      ends
        END     CS:START,SS:DUMMY,DS:DUMMY


