

   .386P
   .LIST
   PAGE 60,132
;--------------------------------------------------------------------------+
;                                                                          !
;   (c) Copyright, Wang Laboratories, Inc., 1991                           !
;          All rights reserved.                                            !
;                                                                          !
;    Program name: WANG 2200/II CS386 Diagnostics                          !
;    File name: @DG2.ASM                                                   !
;    Support group: Engineering Service Department                         !
;                   Wang Computer (Taiwan) LTD.                            !
;                   2, Science Road One       ZIP 20077                    !
;                   Hsinchu, Taiwan, Republic of China                     !
;                                                                          !
;     Revision History:                                                    !
;           Date         Programmer      Description      Revision         !
;         08/07/91      Milton Chen       Creation          1180           !
;                                                                          !
;     Assumptions upon download this program to CS386 CPU memory:          !
;        1. Base of CS, DS, ES, SS are 160H. FS's base is FFFF8000H.       !
;        2. Upon exit, use GDT at 0H, jump to 0FFE90034H (EPROM)           !
;                                                                          !
;     TEST ROUTINE:                                                        !
;        1. Detect Interface Card                                          !
;        2. System configuration and diagnostic status                     !
;        3. System interrupt test result                                   !
;        4. High Speed Channel test                                        !
;        5. Parity check and NMI test result                               !
;        6. DRAM data bus test                                             !
;        7. DRAM memory double word pattern test                           !
;        8. DRAM memory word pattern test                                  !
;        9. DRAM memory byte pattern test, bad SIMM detect                 !
;                                                                          !
;--------------------------------------------------------------------------+

GDTBASE EQU     00001000H       ;phisical address for GDT Base

;I/O PORT
TIMESET EQU          8000H      ;120NS FOR EACH UNIT
CPUST   EQU             2H      ;READ CPU/IO STATUS
                                ;D0= 1, 5us pulse of ABS/, OBS/, CBS/
                                ;D1= 1, CPU BUSY
                                ;D2= 1, KEY BOARD FUNCTION
                                ;D3= 1, I/O READY
                                ;D4= 1, TIME OUTME OUT
                                ;D5= 1, CPU HALT
                                ;D6= 1, SRAM PARITY ERROR
                                ;D7= 1, DRAM PARITY ERROR
                                ;D8= 1, primary reset in from 2200 bus
                                ;D9= 1, nomal boot; =0, run-in
                                ;D10=1, same as D2, but read cause clear
                                ;D11    not defined
                                ;D12    not defined
                                ;D13=1, same as D5, but read cause clear

CPUFLG  EQU             4H      ;WRITE IO PORT
                                ;D0= 1, generate even parity bit for diag
                                ;  = 0, clear force even parity condition
                                ;D1= 1, clear parity error if occured anytime
                                ;  = 0, not clear parity error, if occured
                                ;D2= 1, clear 387 error if occured
                                ;  = 0, not clear 387 error if occured
                                ;D3= 1, CPU LED on
                                ;  = 0, CPU LED off

CPURDY  EQU            6H       ;release CPU BUSY

RTCLOCK EQU            8H       ;realtime clock address strobe
RTCLOC2 EQU           0AH       ;realtime clock data strobe, data r/w port
                                ;1st byte RTC user RAM = memory size in MB
                                ;17byte user ram = error code in protect mode
                                ;18th byte = MNI flag; 0= no NMI, 1=NMI occured
                                ;19th byte = loop count low byte
                                ;20th byte = loop count hi byte

IBGET   EQU            0CH      ;get input data from bus
_ABS    EQU            12H      ;address bus strobe
_OBS    EQU            14H      ;output data to bus
CBS     EQU            18H      ;command data bus strobe

NCA      EQU          108H                  ; non-cache

ABS7     EQU       0FF07H
MXFMEM   EQU       400004H
MXFSTS   EQU       401602H
MXFABS   EQU       401603H
MXFPAR   EQU       401605H


ldt_alias       equ     0ch
data_offset     equ     5*8+2
data_sel        equ     2ch


; LDT(0)  selector        null         lim     G
; LDT(1)     04       base=000000A0   00037    0    B=0   E=0   W=1
; LDT(2)     0C       base=00000960   020FF    1    D=1   C=0   R=1
; LDT(3)     14       base=80000000   00032    0    B=1   E=0   W=1
; LDT(4)     1C       base=00000960   020FF    1    B=1   E=0   W=1
; LDT(5)     24       base=00000960   00032    1    B=1   E=0   W=1
; LDT(6)     2C       base=FFFF0000   020FF    0    D=0   C=0   R=1
; LDT(7)     34       base=70000000   60FFF    1    B=1   E=0   W=1


CODE    SEGMENT USE32
        ASSUME CS:CODE,DS:CODE,ES:CODE

START:
         CLD
         MOV       EDX,OFFSET STACK_BUFFER
         ADD       EDX,200H                 ; extend minimun 200H spac for stack
         AND       EDX,0FFFFFF00H           ; get first address to test
         MOV       FIRSTRAM,EDX             ; save this address
         MOV       ESP,EDX                  ; and set up stack point

         PUSH      DS                       ; save current ds
         MOV       EAX,0                    ; clear its
         MOV       EBX,EAX
         MOV       AX,0CH                   ; tempery point to LDT
         MOV       DS,AX                    ;
         MOV       AX,CS                    ; calculate which one discriptor we use now
         AND       AX, 0FFFbH               ; mask out privalige
         MOV       BX,[EAX+2]               ; get base address
         POP       DS                       ; restore original ds

         MOV       BASE,EBX                 ; save this ds base offset

         MOV       AH,12H
         CALL      READRTC
         MOV       INTRES,AL

         MOV       AX,2000H                 ; RTC RAM 20h contains loopcount low byte
         CALL      WRITERTC
         MOV       AX,2100H                 ; RTC RAM 21h contains loopcount hi byte
         CALL      WRITERTC                 ; reset loopcount

SHOWLOOP:

         MOV       [HSCOFG],0

         MOV       EDI,400400H
         MOV       EBX,'@DG1'

         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNE       MXE
         MOV       [HSCOFG],10H
         JMP       MXF

MXE:     MOV       [HSCOFG],0H

MXF:     MOV       ESI,OFFSET COPYRIGHT
         MOV       CX,COPYRIGHTL
         CALL      MXEF                     ; display copyright statement

         MOV       ESI,OFFSET REVISION
         MOV       CX,REVISIONL
         CALL      MXEF                     ; display copyright statement


;------------------------------------------------------------------------+
;                                                                        |
;    [ Test 1 ] Detect Interface Card                                    |
;                                                                        |
;         To test system interface control card MXF, 22C11-HS exist      |
;                                                                        |
;------------------------------------------------------------------------+

SYS:

         XOR       DL,DL
         MOV       EDI,000400H
         MOV       EBX,'@DG1'
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       DK1

         OR        DL,01H            ; board 0


DK1:     ADD       EDI,100000H
         MOV       EBX,'@DG1'
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       DK2

         OR        DL,02H

DK2:     ADD       EDI,100000H
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       DK3

         OR        DL,04H

DK3:     ADD       EDI,100000H
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       MXF1

         OR        DL,08H

MXF1:    ADD       EDI,100000H
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       MXF2

         OR        DL,10H

MXF2:    ADD       EDI,100000H
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       MXF3

         OR        DL,20H

MXF3:    ADD       EDI,100000H
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       MXF4

         OR        DL,40H

MXF4:    ADD       EDI,100000H
         MOV       GS:[EDI],EBX
         MOV       EAX,GS:[EDI]
         CMP       EAX,EBX
         JNZ       WARMBOOT

         OR        DL,80H

;----------------------------------------------------------------------+
;                                                                      |
;    [ Test 2 ] Sent to Interface card warm boot                       |
;                                                                      |
;         Execulisive 1st MXF board for display                        |
;                                                                      |
;----------------------------------------------------------------------+

WARMBOOT:
         MOV       AL,DL

         TEST      AL,01H
         JZ        PCB1
         MOV       EBP,00042BH                     ; sent warmboot code
         MOV       BYTE PTR GS:[EBP],80H                    ; 80H
         CALL      WBCOM1

PCB1:
         TEST      AL,02H
         JZ        PCB2
         MOV       EBP,10042BH                     ; sent warmboot code
         MOV       BYTE PTR GS:[EBP],80H                    ; 80H
         CALL      WBCOM1

PCB2:
         TEST      AL,04H
         JZ        PCB3
         MOV       EBP,20042BH                     ; sent warmboot code
         MOV       BYTE PTR GS:[EBP],80H                    ; 80H
         CALL      WBCOM1

PCB3:
         TEST      AL,08H
         JZ        PCB5                            ; skip 1st MXF
         MOV       EBP,30042BH                     ; sent warmboot code
         MOV       BYTE PTR GS:[EBP],80H                    ; 80H
         CALL      WBCOM1

PCB5:
         TEST      AL,20H
         JZ        PCB6
         MOV       EBP,50042BH                     ; sent warmboot code
         MOV       BYTE PTR GS:[EBP],80H                    ; 80H
         CALL      WBCOM2

PCB6:
         TEST      AL,40H
         JZ        PCB7
         MOV       EBP,60042BH                     ; sent warmboot code
         MOV       BYTE PTR GS:[EBP],80H                    ; 80H
         CALL      WBCOM2

PCB7:
         TEST      AL,80H
         JZ        DSPDG
         MOV       EBP,70042BH                     ; sent warmboot code
         MOV       BYTE PTR GS:[EBP],80H                    ; 80H
         CALL      WBCOM2

;----------------------------------------------------------------------+
;                                                                      |
;    [ Test 3 ] System interface card exist and Diagnostic status      |
;                                                                      |
;          To display system interface control card and each card      |
;    diagnostic test result                                            |                    |
;                                                                      |
;----------------------------------------------------------------------+

DSPDG:
         CALL      ITEM

         MOV       ESI,OFFSET SYSCARD
         MOV       CX,SYSCARDL             ; max 40 bytes in string
         CALL      MXEF                    ; display test 1 name

         TEST      AL,01H
         JZ        PCB11

         MOV       SI,OFFSET PB0
         MOV       CX,PB0L
         CALL      MXEF
         MOV       EBP,000405H
         CALL      BOARD
         MOV       EBP,000420H
         CALL      DGST

PCB11:
         TEST      AL,02H
         JZ        PCB22

         MOV       SI,OFFSET DSK1
         MOV       CX,DS1L
         CALL      MXEF
         MOV       EBP,100405H
         CALL      BOARD
         MOV       EBP,100420H
         CALL      DGST

PCB22:
         TEST      AL,04H
         JZ        PCB33

         MOV       SI,OFFSET DSK2
         MOV       CX,DS2L
         CALL      MXEF
         MOV       EBP,200405H
         CALL      BOARD
         MOV       EBP,200420H
         CALL      DGST

PCB33:
         TEST      AL,08H
         JZ        PCB44

         MOV       SI,OFFSET DSK3
         MOV       CX,DS3L
         CALL      MXEF
         MOV       EBP,300405H
         CALL      BOARD
         MOV       EBP,300420H
         CALL      DGST

PCB44:
         TEST      AL,10H
         JZ        PCB55

         MOV       SI,OFFSET MXF11
         MOV       CX,MX1L
         CALL      MXEF
         MOV       EBP,400405H
         CALL      BOARD
         MOV       EBP,400420H
         CALL      DGST

PCB55:
         TEST      AL,20H
         JZ        PCB66

         MOV       SI,OFFSET MXF22
         MOV       CX,MX2L
         CALL      MXEF
         MOV       EBP,500405H
         CALL      BOARD
         MOV       EBP,500420H
         CALL      DGST

PCB66:
         TEST      AL,40H
         JZ        PCB77

         MOV       SI,OFFSET MXF33
         MOV       CX,MX3L
         CALL      MXEF
         MOV       EBP,600405H
         CALL      BOARD
         MOV       EBP,600420H
         CALL      DGST

PCB77:
         TEST      AL,80H
         JZ        TSTINT

         MOV       SI,OFFSET MXF44
         MOV       CX,MX4L
         CALL      MXEF
         MOV       EBP,700405H
         CALL      BOARD
         MOV       EBP,700420H
         CALL      DGST

;-----------------------------------------------+
;                                               |
;    [ Test 4 ] Interrupt test result           |
;                                               |
;-----------------------------------------------+


TSTINT:
         MOV       BYTE PTR [HSCOFG],AL
         CALL      DELAYSEC

         jmp       hstest

         MOV       ESI,OFFSET CLEAN
         MOV       CX,CLEANL
         CALL      MXEF

         MOV       CX,8
CLN1:    MOV       ESI,OFFSET CLNR
         PUSH      CX
         MOV       CX,CLNRL
         CALL      MXEF
         POP       CX
         LOOP      CLN1

         CALL      ITEM

         MOV       ESI,OFFSET INTRT
         MOV       CX,INTRTL               ; max 40 bytes in string
         CALL      MXEF                     ; display test 1 name

         MOV       ESI,OFFSET CLEAN
         MOV       CX,CLEANL
         CALL      MXEF

         MOV       AL,BYTE PTR [HSCOFG]

         TEST      AL,00000001B
         JZ        NTT11
         MOV       SI,OFFSET PB0
         MOV       CX,PB0L
         CALL      MXEF
         MOV       EBP,000405H
         CALL      BOARD

         MOV       EBP,00041FH
         CALL      RESINT

NTT11:   TEST      AL,00000010B
         JZ        NTT1
         MOV       SI,OFFSET DSK1
         MOV       CX,DS1L
         CALL      MXEF
         MOV       EBP,100405H
         CALL      BOARD

         MOV       EBP,10041FH
         CALL      RESINT

NTT1:    TEST      AL,00000100B
         JZ        NTT2
         MOV       SI,OFFSET DSK2
         MOV       CX,DS2L
         CALL      MXEF
         MOV       EBP,200405H
         CALL      BOARD

         MOV       EBP,20041FH
         CALL      RESINT

NTT2:    TEST      AL,00001000B
         JZ        NTT3
         MOV       SI,OFFSET DSK3
         MOV       CX,DS3L
         CALL      MXEF
         MOV       EBP,300405H
         CALL      BOARD

         MOV       EBP,30041FH
         CALL      RESINT

NTT3:    TEST      AL,00010000B
         JZ        NTT4
         MOV       SI,OFFSET MXF11
         MOV       CX,MX1L
         CALL      MXEF
         MOV       EBP,400405H
         CALL      BOARD

         MOV       EBP,40041FH
         CALL      RESINT

NTT4:    TEST      AL,00100000B
         JZ        NTT5
         MOV       SI,OFFSET MXF22
         MOV       CX,MX2L
         CALL      MXEF
         MOV       EBP,500405H
         CALL      BOARD

         MOV       EBP,50041FH
         CALL      RESINT

NTT5:    TEST      AL,01000000B
         JZ        NTT6
         MOV       SI,OFFSET MXF33
         MOV       CX,MX3L
         CALL      MXEF
         MOV       EBP,600405H
         CALL      BOARD

         MOV       EBP,60041FH
         CALL      RESINT

NTT6:    TEST      AL,10000000B
         JZ        HSTEST
         MOV       SI,OFFSET MXF44
         MOV       CX,MX4L
         CALL      MXEF
         MOV       EBP,700405H
         CALL      BOARD

         MOV       EBP,70041FH
         CALL      RESINT


;----------------------------------------------------------------------+
;                                                                      |
;    [ Test 5 ] High Speed Channel Bus Test                            |
;                                                                      |
;       Display the test result record -> 42A=>                        |
;                                                                      |
;----------------------------------------------------------------------+

HSTEST:  CALL      DELAYSEC

         MOV       ESI,OFFSET CLEAN
         MOV       CX,CLEANL
         CALL      MXEF

         MOV       CX,8
CLN31:   MOV       ESI,OFFSET CLNR
         PUSH      CX
         MOV       CX,CLNRL
         CALL      MXEF
         POP       CX
         LOOP      CLN31

         CALL      ITEM

         MOV       ESI,OFFSET COMST
         MOV       CX,COMSTL               ; max 40 bytes in string
         CALL      MXEF                     ; display test 1 name

         MOV       ESI,OFFSET CLEAN
         MOV       CX,CLEANL
         CALL      MXEF

         MOV       AL,BYTE PTR [HSCOFG]

         TEST      AL,00000001B
         JZ        T11
         MOV       SI,OFFSET PB0
         MOV       CX,PB0L
         CALL      MXEF
         MOV       EBP,000405H             ; board ID
         CALL      BOARD

         CALL      COMMTEST


T11:     TEST      AL,00000010B
         JZ        T1
         MOV       SI,OFFSET DSK1
         MOV       CX,DS1L
         CALL      MXEF
         MOV       EBP,100405H             ; board ID
         CALL      BOARD

         CALL      COMMTEST

T1:      TEST      AL,00000100B
         JZ        T2
         MOV       SI,OFFSET DSK2
         MOV       CX,DS2L
         CALL      MXEF
         MOV       EBP,200405H
         CALL      BOARD

         CALL      COMMTEST


T2:      TEST      AL,00001000B
         JZ        T4
         MOV       SI,OFFSET DSK3
         MOV       CX,DS3L
         CALL      MXEF
         MOV       EBP,300405H
         CALL      BOARD

         CALL      COMMTEST

T4:      TEST      AL,00100000B
         JZ        T5
         MOV       SI,OFFSET MXF22
         MOV       CX,MX2L
         CALL      MXEF
         MOV       EBP,500405H
         CALL      BOARD

         CALL      COMMTEST

T5:      TEST      AL,01000000B
         JZ        T6
         MOV       SI,OFFSET MXF33
         MOV       CX,MX3L
         CALL      MXEF
         MOV       EBP,600405H
         CALL      BOARD

         CALL      COMMTEST

T6:      TEST      AL,10000000B
         JZ        NMITEST
         MOV       SI,OFFSET MXF44
         MOV       CX,MX4L
         CALL      MXEF
         MOV       EBP,700405H
         CALL      BOARD

         CALL      COMMTEST

;-----------------------------------------------------------------------+
;                                                                       |
;    [ Test 6 ] Parity check and NMI test result                        |
;                                                                       |
;-----------------------------------------------------------------------+

NMITEST: CALL      DELAYSEC

         CALL      ITEM

         MOV       ESI,OFFSET NMITST
         MOV       CX,40                   ; max 40 bytes in string
         CALL      MXEF                     ; display test 1 name

         MOV       ESI,OFFSET CLEAN
         MOV       CX,CLEANL
         CALL      MXEF

         MOV       CX,8
CLN3:    MOV       ESI,OFFSET CLNR
         PUSH      CX
         MOV       CX,CLNRL
         CALL      MXEF
         POP       CX
         LOOP      CLN3

         MOV       AH,13H
         CALL      READRTC

         TEST      AL,01H
         JNZ       NMIOK

         MOV       ESI,OFFSET FAILMSG
         MOV       CX,FAILMSGL
         CALL      MXEF
         JMP       RAMTEST

NMIOK:   MOV       ESI,OFFSET PASSMSG
         MOV       CX,PASSMSGL
         CALL      MXEF


;-------------------------------------------------------------------------+
;                                                                         |
;   [Test 7]  RAM Data Bus Test                                           !
;                                                                         |
;-------------------------------------------------------------------------+

RAMTEST: CALL      DELAYSEC

         CALL      ITEM

         MOV       ESI,OFFSET TST1NAME
         MOV       CX,40                   ; max 40 bytes in string
         CALL      MXEF                     ; display test 1 name

         MOV       ESI,OFFSET CLEAN
         MOV       CX,CLEANL
         CALL      MXEF

         MOV       CX,8
CLN4:    MOV       ESI,OFFSET CLNR
         PUSH      CX
         MOV       CX,CLNRL
         CALL      MXEF
         POP       CX
         LOOP      CLN4

         MOV       ESI,FIRSTRAM             ; start from lowest 64KB boundry
TST1LOP1:
         MOV       EBX,0FFFFFFFFH           ; current data pattern
         MOV       EAX,01                   ; next pattern
         MOV       EDX,EAX                  ; save next pattern
         MOV       ECX,33                   ; 33 patterns from FFFFFFFF to 01h
         MOV       [ESI],EBX                ; write current pattern to current address

TST1LOP2:
         XCHG      [ESI],EAX                ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST1ERR                  ; jump if bad (no jump is fast)

         MOV       EBX,EDX                  ; save current pattern
         SHL       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX                  ; ready to write data pattern from EAX reg
         LOOP      TST1LOP2                 ; loop for next pattern test

         MOV       EAX,7FFFFFFFH            ; next data pattern 7FFFFFFF
         MOV       EDX,EAX                  ; prepare for shift right
         XCHG      [ESI],EAX                ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST1ERR                  ; jump if bad (no jump is fast)

         MOV       EBX,EDX                  ; save current pattern for compare later
         ROR       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX
         MOV       CX,31                    ; 31 patterns from 7FFFFFFF to FFFFFFFE

TST1LOP3:
         XCHG      [ESI],EAX                ; read data from currnt addr & write next
         CMP       EAX,EBX                  ; verify read data against current pattern
         JNE       TST1ERR                  ; jump if bad (no jump is fast)

         MOV       EBX,EDX                  ; save current pattern for compare later
         ROR       EDX,1                    ; setup next data pattern
         MOV       EAX,EDX
         LOOP      TST1LOP3                 ; loop for next pattern test


         ADD       ESI,10000H               ; point to next 64KB bank
         CALL      LASTADDR                 ; get last dram address in edx
         CMP       ESI,EDX
         JL        TST1LOP1

         MOV       ESI,OFFSET PASSMSG
         MOV       CX,PASSMSGL
         CALL      MXEF              ; display pass message

         CALL      DELAYSEC

         JMP       TST2                     ; go on next test


TST1ERR:
         MOV       EBP,EAX                  ; save read data, pattern
         MOV       ECX,8
TST1ERR1:
         AND       EAX,0FH                  ; mask out all hi nibble
         LEA       EDI,ASCIIFONT            ; point to ascii string
         MOV       AH,[EDI+EAX]             ; exchange hex to ascii
         MOV       READDATA[ECX],AH         ; put it to display string
         SHR       EBP,4                    ; take low nibble out
         MOV       EAX,EBP                  ; convert next nibble
         LOOP      TST1ERR1


         MOV       EBP,EBX                  ; save expected data
         MOV       ECX,8
TST1ERR2:
         AND       EBX,0FH                  ; mask out all hi nibble
         LEA       EDI,ASCIIFONT            ; point to ascii string
         MOV       AH,[EDI+EBX]             ; exchange hex to ascii
         MOV       EXPECDATA[ECX],AH        ; put it to display string
         SHR       EBP,4                    ; take low nibble out
         MOV       EBX,EBP                  ; convert next nibble
         LOOP      TST1ERR2


         MOV       EBX,ESI                  ; save address
         MOV       ECX,8
TST1ERR3:
         AND       EBX,0FH                  ; mask out all hi nibble
         LEA       EDI,ASCIIFONT            ; point to ascii string
         MOV       AH,[EDI+EBX]             ; exchange hex to ascii
         MOV       ADDRESS[ECX],AH          ; put it to display string
         SHR       ESI,4                    ; take low nibble out
         MOV       EBX,ESI                  ; convert next nibble
         LOOP      TST1ERR3

         MOV       ESI,OFFSET ERRMSG
         MOV       CX,ERRMSGL
         CALL      MXEF

         MOV       ESI,OFFSET ADDRESS
         MOV       CX,ADDRESSL
         CALL      MXEF

         MOV       ESI,OFFSET READDATA
         MOV       CX,READDATAL
         CALL      MXEF

         MOV       ESI,OFFSET EXPECDATA
         MOV       CX,EXPECDATAL
         CALL      MXEF              ; display error message

TST1ERR9:
         MOV       ESI,OFFSET SHIFTKEY
         MOV       CX,RESETKEY - SHIFTKEY
         CALL      MXEF              ; memtion user by blinking words

         CALL      DELAYSEC

         MOV       ESI,OFFSET RESETKEY
         MOV       CX,RESETKEYEND - RESETKEY
         CALL      MXEF              ; memtion user by blinking words

         CALL      DELAYSEC

         JMP       TST1ERR9                 ; loop here wait for reset key pressed


;-------------------------------------------------------------------------+
;                                                                         |
;    [Test 8] DRAM Memory Test --- DW ,55AAAA55                           |
;                                                                         |
;-------------------------------------------------------------------------+

TST2:
         CALL      ITEM

         MOV       ESI,OFFSET TSTDW
         MOV       CX,40                   ; max 40 bytes in string
         CALL      MXEF              ; display test 2 name

         MOV       ESI,OFFSET CLEAN                ; clear
         MOV       CX,CLEANL
         CALL      MXEF

         CALL      LASTADDR

         MOV       EBX,55AAAA55H
         SUB       EDX,FIRSTRAM
         MOV       ECX,EDX
         SHR       ECX,2
         MOV       ESI,FIRSTRAM
DWOD:    MOV       [ESI],EBX
         MOV       EAX,[ESI]
         ADD       ESI,4
         CMP       EAX,EBX
         JNE       TST1ERR
         LOOP      DWOD

         MOV       ESI,OFFSET PASSMSG
         MOV       CX,PASSMSGL
         CALL      MXEF              ; display error message

         CALL      DELAYSEC

;-------------------------------------------------------------------------+
;         DRAM Memory Test --- Word,3CC3                                  !
;-------------------------------------------------------------------------+

TST3:
         CALL      ITEM

         MOV       ESI,OFFSET TSTW
         MOV       CX,40                   ; max 40 bytes in string
         CALL      MXEF              ; display test 2 name

         MOV       ESI,OFFSET CLEAN                ; clear
         MOV       CX,CLEANL
         CALL      MXEF

         CALL      LASTADDR

         MOV       EBX,3CC3H
         SUB       EDX,FIRSTRAM
         MOV       ECX,EDX
         SHR       ECX,1
         MOV       ESI,FIRSTRAM
WOD:     MOV       [ESI],BX
         MOV       AX,[ESI]
         ADD       ESI,2
         CMP       AX,BX
         JNE       TST1ERR
         LOOP      WOD

         MOV       ESI,OFFSET PASSMSG
         MOV       CX,PASSMSGL
         CALL      MXEF              ; display error message

         CALL      DELAYSEC

;-------------------------------------------------------------------------+
;          DRAM memory test - byte                                        !
;-------------------------------------------------------------------------+

TST5:

         CALL      ITEM

         MOV       ESI,OFFSET TST5NAME
         MOV       CX,SHOWSIZEND-TST5NAME
         CALL      MXEF              ; display test 5 name



         mov       ax,ldt_alias
         mov       ds,ax                   ;ds point to ldt alias
         mov       esi,data_offset         ;esi point to base 15-0 of data segment
         mov       word ptr [esi],0        ;change the base to 0
         mov       ax,data_sel
         mov       ES,ax                   ;reset ds to data segment
         MOV       BX,24H
         MOV       DS,BX
         MOV       EDX,0
         MOV       AX,0E00H
         CALL      READRTC
         MOV       DL,AL
         SHL       EDX,20                   ; last addr.
         MOV       EDI,BASE
         ADD       EDI,FIRSTRAM
         MOV       ESI,EDI

         CLD
FILL1:   MOV       BL,0FFH
         MOV       ES:[EDI],BL
         INC       EDI
         TEST      EDI,1FFFH
         JNZ       FILL1

         MOV       CL,0FFH
         MOV       BX,0000H
FILL2:   MOV       AL,BL
         XCHG      ES:[ESI],AL
         CMP       AL,CL
         JNE       TST5ERR                  ; jump if bad (no jump is fast)

         MOV       AL,ES:[ESI]
         CMP       AL,BL
         JNE       TST5ERR
         INC       ESI
         TEST      ESI,1FFFH
         JNZ       FILL2

         CALL      UPDATESIZE
         CMP       ESI,EDX                  ; do we reach last address?
         JL        FILL1                    ; no,go on r/w/r test


         MOV     AX,2000h        ; RTC 20h contains loopcount low byte
         CALL    READRTC
         MOV     BL,AL           ; save loopcount low byte
         MOV     AX,2100h        ; RTC 21h contains loopcount hi byte
         CALL    READRTC
         MOV     BH,AL           ; save loopcount hi byte

         INC     BX              ; increase loop count
         MOV     AH,20h          ; RTC RAM 20h contains loopcount low byte
         MOV     AL,BL           ; update loopcount
         CALL    WRITERTC
         MOV     AH,21h          ; RTC RAM 21h contains loopcount hi byte
         MOV     AL,BH           ; update loopcount
         CALL    WRITERTC

         MOV     ESI,OFFSET SHOWCOUNT
         MOV     CX,83          ; count bytes
         CALL    MXEF     ; position cursor to least significant digit
         MOV     AX,BX           ; put loopcount to AX for display
         CALL    UPDATENUM       ; display loopcount at terminal srceen

         CALL    DELAYSEC
         CALL    DELAYSEC        ; delay for user to see the msg on screen
         JMP     SYS             ; go on loop


TST5ERRA:
         PUSH      EDX

TST5ERR:
         MOV     EBP,EAX         ; save read data
         MOV     ECX,8
TST5ERR1:
         AND     EAX,0FH         ; mask out all hi nibble
         LEA     EDI,ASCIIFONT   ; point to ascii string
         MOV     AH,[EDI+EAX]    ; exchange hex to ascii
         MOV     READDATA[ECX],AH; put it to display string
         SHR     EBP,4           ; take low nibble out
         MOV     EAX,EBP         ; convert next nibble
         LOOP    TST5ERR1

         MOV     EBP,EBX         ; save expected data
         MOV     ECX,8
TST5ERR2:
         AND     EBX,0FH         ; mask out all hi nibble
         LEA     EDI,ASCIIFONT   ; point to ascii string
         MOV     AH,[EDI+EBX]    ; exchange hex to ascii
         MOV     EXPECDATA[ECX],AH; put it to display string
         SHR     EBP,4           ; take low nibble out
         MOV     EBX,EBP         ; convert next nibble
         LOOP    TST5ERR2

         MOV     EBX,ESI         ; save address data
         PUSH    ESI             ; #
         MOV     ECX,8
TST5ERR3:
         AND     EBX,0FH         ; mask out all hi nibble
         LEA     EDI,ASCIIFONT   ; point to ascii string
         MOV     AH,[EDI+EBX]    ; exchange hex to ascii
         MOV     ADDRESS[ECX],AH ; put it to display string
         SHR     ESI,4           ; take low nibble out
         MOV     EBX,ESI         ; convert next nibble
         LOOP    TST5ERR3

         MOV       ESI,OFFSET ERRMSG
         MOV       CX,ERRMSGL
         CALL      MXEF

         MOV       ESI,OFFSET ADDRESS
         MOV       CX,ADDRESSL
         CALL      MXEF

         MOV       ESI,OFFSET READDATA
         MOV       CX,READDATAL
         CALL      MXEF

         MOV       ESI,OFFSET EXPECDATA
         MOV       CX,EXPECDATAL
         CALL      MXEF              ; display error message


         POP       ESI
         POP       EDX

         CMP       EDX,400000H              ; 4M one simm.
         JE        ONESIM
         CMP       EDX,800000H              ; 8M two simm.
         JE        TWOSIM
         CMP       EDX,1000000H             ; 16M one simm.
         JE        ONESIM
TWOSIM:  AND       ESI,11B                  ; 32M two simm.
         CMP       ESI,01B
         JNE       S1
         CALL      L10L15                   ; 01
         JMP       TSTAR
S1:      CMP       ESI,10B
         JNE       S2
         CALL      L18L24                   ; 10
         JMP       TSTAR
S2:      CMP       ESI,11B
         JNE       S3
         CALL      L29L35                   ; 11
         JMP       TSTAR
S3:      CALL      L3L5
         JMP       TSTAR

ONESIM:
         AND       ESI,11B
         CMP       ESI,01B
         JNE       S11
         CALL      SIMM                     ; 01
         MOV       AL,31H                   ; L10
         CALL      MXEF1
         MOV       AL,30H
         CALL      MXEF1
         JMP       TSTAR

S11:     CMP       ESI,10B
         JNE       S22
         CALL      SIMM                     ; 10
         MOV       AL,31H                   ; L18
         CALL      MXEF1
         MOV       AL,38H
         CALL      MXEF1
         JMP       TSTAR

S22:     CMP       ESI,11B                  ; 11
         JNE       S33
         CALL      SIMM
         MOV       AL,32H                   ; L29
         CALL      MXEF1
         MOV       AL,39H
         CALL      MXEF1
         JMP       TSTAR

S33:     CALL      SIMM                     ; 00, L3
         MOV       AL,33H
         CALL      MXEF1

TSTAR:
        MOV     ESI,OFFSET SHIFTKEY
        MOV     CX,RESETKEY - SHIFTKEY
        CALL    MXEF     ; memtion user by blinking words
        CALL    DELAYSEC
        MOV     ESI,OFFSET RESETKEY
        MOV     CX,RESETKEYEND - RESETKEY
        CALL    MXEF     ; memtion user by blinking words
        CALL    DELAYSEC
        JMP     TSTAR           ; loop here wait for reset key pressed

;********* all these subroutins are vailid only in Protect Mode *******

;-------->>>>>------------------+
WAITRESET       PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: none                                                         !
;on exit : return to loader in EPROM upon reset, otherwise nomal return !
;regs destroied: none                                                   !
;purpose : this proc can check the console reset key pressed or not     !
;-------------------------------+---------------------------------------+

        PUSH    EDX
        MOV     EDX,_ABS
        MOV     BYTE PTR FS:[EDX],01H   ;enable KEYBOARD (device address 01)

        PUSH    ECX
        MOV     ECX,0FFH
        LOOP    $               ; time delay
        POP     ECX

        MOV     EDX,CPUST
        TEST    WORD PTR FS:[EDX],100H  ;does reset key pressed?
        JNZ     EXIT

        MOV     EDX,_ABS
        MOV     BYTE PTR FS:[EDX],05H   ;enable CRT (device address 05)
        POP     EDX
        RET

EXIT:
        CALL    RELEASE
        MOV     EDI,CPUFLG      ; get i/o port
        MOV     DX,02H
        MOV     FS:[EDI],DX     ; clear parity error condition

        CALL    LASTADDR        ; upon return, EDX is last DRAM address
        MOV     EDI,FIRSTRAM    ; get 1st address of unused memory

        MOV     EAX,0F4F4F4F4H  ; set flood data pattern
        MOV     ECX,EDX         ;
        SUB     ECX,EDI         ; the bytes need write
        SHR     ECX,2           ; divid bytes by 4, convert to dword size
        CLD                     ; clear dir flag so string instr incrmnt DI
        REP     STOSD           ; flood segment

        MOV     EDX,_ABS
        MOV     BYTE PTR FS:[EDX],05H   ;enable CRT (device address 05)
        POP     EDX
        POP     EDX             ; balance the stack
                                ; end house keeping
        LGDT    TGDT            ; restore the GDT
        JMP     PWORD PTR IJUMP ; jump to bootstrap on EPROM, exit from @DG

;-------------------------------+
WAITRESET       ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
DELAYSEC        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: none                                                         !
;on exit : none                                                         !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+

        PUSH    ECX
        MOV     ECX,5FFFFFH
        LOOP    $               ; time delay
        POP     ECX
        RET
;-------------------------------+
DELAYSEC        ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
LASTADDR        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: none                                                         !
;on exit : EDX is the byte number of DRAM, or the last DRAM address + 1 !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+

         PUSH      EBP
         MOV       EBP,RTCLOCK     ; point to RTC address select port
         MOV       EDX,0Eh         ; point to 1st byte on user RAM
         MOV       FS:[EBP],DL     ;
         MOV       EBP,RTCLOC2     ; point to RTC data r/w port
         MOV       DL,FS:[EBP]     ; read DRAM size from RTC RAM
         SHL       EDX,20          ; convert size to flat address
         SUB       EDX,BASE
         AND       EDX,0FFFFF000H
         POP       EBP
         RET
;-------------------------------+
LASTADDR        ENDP            ;
;--------<<<<<------------------+

;-------->>>>>------------------+
WRITERTC        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AH= RTC user RAM address (0E-FF),                            !
;          AL= data to be written to RTC user RAM                       !
;on exit : none                                                         !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+
        PUSH    EBP             ;
        MOV     EBP,RTCLOCK     ; point to RTC address select port
        MOV     FS:[EBP],AH     ; AH is the user ram address
        MOV     EBP,RTCLOC2     ; point to RTC data r/w port
        MOV     FS:[EBP],AL     ; write data code to RTC RAM
        POP     EBP             ;
        RET                     ;
;-------------------------------+
WRITERTC        ENDP            ;

;--------<<<<<------------------+---------------------------------------+
;The RTC user RAM address starts from 0Eh to FFh, total 50 bytes.       *
;Diagnostic reserved following address:                                 *
;       [0Eh]= the DRAM memory size in unit of MB,bootstrap will read it*
;       [0Fh]= external counter for update DRAM size shown on screen    *
;       [1Eh]= error code store here during protect mode                *
;       [1Fh]= MNI flag; 0= no NMI, 1= NMI occured during diagnostics   *
;       [20h]= loop count low byte                                      *
;       [21h]= loop count hi byte                                       *
;-------->>>>>------------------+---------------------------------------+

READRTC         PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AH is the address of RTC user RAM,                           !
;on exit : AL is the data read from RTC user RAM at address 'AH'        !
;regs destroied: none                                                   !
;-------------------------------+---------------------------------------+
        PUSH    EBP             ;
        MOV     EBP,RTCLOCK     ; point to RTC address select port
        MOV     FS:[EBP],AH     ; AH is the user ram address
        MOV     EBP,RTCLOC2     ; point to RTC data r/w port
        MOV     AL,FS:[EBP]     ; read data code from RTC RAM
        POP     EBP             ;
        RET                     ;
;-------------------------------+
READRTC         ENDP            ;
;--------<<<<<------------------+

;-------->>>>>------------------+
DISPLAYMORE PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: ESI is point to the first byte of string                     ;
;          CX is the number of bytes in the string                     ;
;on exit : none                                                         ;
;reg destroyed: none                                                    ;
;-----------------------------------------------------------------------+
        PUSH    EAX
DISPLAYMLOP:
        MOV     AL,[ESI]        ;
        CALL    DISPLAYONE
        INC     ESI
        LOOP    DISPLAYMLOP
        POP     EAX
        RET
;-------------------------------+
DISPLAYMORE     ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
DISPLAYONE      PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AL is the ASCII code to be displayed                         ;
;on exit : none                                                         ;
;reg destroyed: none                                                    ;
;-----------------------------------------------------------------------+
        CALL    WAITRESET       ; does reset key pressed? if, yes, end diagnos
        CALL    CHKIORDY        ; be sure CRT is ready for next send
        PUSH    EDI
        MOV     EDI,_OBS        ; point to output data bus strobe
        MOV     FS:[EDI],AL     ; send out ascii
        POP     EDI
        RET
;-------------------------------+
DISPLAYONE      ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
ENABLE          PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;on entry: AL is the device address to be enabled, CRT=05, PRT=         ;
;on exit : none                                                         ;
;reg destroyed: none                                                    ;
;-----------------------------------------------------------------------+
        PUSH    EDI
        MOV     EDI,_ABS        ;point to I/O address bus strobe
        MOV     FS:[EDI],AL
        CALL    CHKIORDY        ;make sure I/O bus is ready
        POP     EDI
        RET
;-------------------------------+
ENABLE          ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHKIORDY        PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: check CPUST D3=1 as I/O ready                                 ;
;on entry: none                                                         ;
;on exit : none                                                         ;
;regs destroy: none                                                     ;
;-----------------------------------------------------------------------+
        PUSH    EDI             ; watch your push-pop pair in this procedure
        MOV     EDI,CPUST
        TEST    BYTE PTR FS:[EDI],08H ;D3=0,I/O not ready;D3=1,I/O raedy
        POP     EDI             ; this push-pop should inside next jump loop
        JZ      CHKIORDY        ; if not ready, let's wait
        RET
;-------------------------------+
CHKIORDY        ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATENUM       PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: convert word hex number to decimal and display it.            !
;on entry: AX contains hex number, cursor points to least signif digit. !
;on exit : none                                                         !
;regs destroy: none                                                     !
;-----------------------------------------------------------------------+
        PUSH    EAX
        PUSH    EBX
        PUSH    ECX
        PUSH    EDX

        MOV     EBX,10          ; set divisor
        MOV     ECX,4           ; maximun 5 digits, there are 4 times DIV
UPDATENUM1:
        MOV     EDX,0           ; clear remainder
        DIV     BX              ; (DX+AX)/BX=AX+DX
        ADD     DL,30h          ; convert remainder to ascii
        XCHG    AX,DX           ; put ascii to AX for display
        CALL    MXEF1      ; display this ascii to screen

        MOV     AL,08h          ; ascii 08h is back space
        CALL    MXEF1      ; mov cursor back
        MOV     AL,08h          ; ascii 08h is back space
        CALL    MXEF1      ; set cursor to next digit position

        CMP     EDX,0           ; is quotient zero?
        JE      UPDATENUM2      ; it is zero, no more display

        MOV     AX,DX           ; restore quotien for next div
        LOOP    UPDATENUM1      ; continue division

        ADD     AL,30h          ; convert last quotient to ascii
        CALL    MXEF1      ; display this ascii to screen

UPDATENUM2:
        POP     EDX
        POP     ECX
        POP     EBX
        POP     EAX
        RET
;-------------------------------+
UPDATENUM       ENDP            ;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATESIZE      PROC    NEAR    ;
;-------------------------------+---------------------------------------+
;purpose: show dram memory size on screen in  64KB increasment          ;
;on entry:none on register, but cursor point to least significant digit ;
;on exit :none, but cursor point to least significsnt digit             ;
;regs destroy: none                                                     ;
;-----------------------------------------------------------------------+
         PUSH      ESI
         PUSH      EDX
         PUSH      ECX
         PUSH      EAX

         MOV       EAX,ESI         ; save surrent address

         MOV       ESI,OFFSET DRAMSIZE
         MOV       CX,DRAMSIZEND-DRAMSIZE ;count bytes
         CALL      MXEF     ; point cursor to 'DRAM 0 KB'

         MOV       ECX,1024
         MOV       EDX,0           ; clear remainder
         DIV       ECX             ; (EDX+EAX)/ECX= EAX + EDX
         CALL      UPDATENUM       ; convert size to decimal and display

UPDATESIZEB:
         POP       EAX
         POP       ECX
         POP       EDX
         POP       ESI
         RET
;-------------------------------+
UPDATESIZE      ENDP            ;
;--------<<<<<------------------+

;-------------------------------------------------

MXEF       PROC      NEAR                   ; display MXF or MXE, strings
         PUSH      EDI
         TEST      HSCOFG,10H               ; 1st MXF exist ?
         JZ        MXEDY                    ; MXE display
         CALL      MXFDSPY                  ; MXF display
         JMP       RETN
MXEDY:   CALL      DISPLAYMORE
RETN:    POP       EDI
         RET
MXEF       ENDP

;-------------------------------------------------------------------

MXEF1      PROC      NEAR                   ; display 1 character
         PUSH      EDI
         TEST      HSCOFG,10H
         JZ        MXEDY1                   ; MXE
         CALL      MXFDSPY1                 ; MXF
         JMP       RETN1
MXEDY1:  CALL      DISPLAYONE
RETN1:   POP       EDI
         RET
MXEF1      ENDP

;---------------------------------------------------------------------------

COMMTEST   PROC     NEAR
         PUSH      EAX
         MOV       EAX,55AAAA55H
         MOV       BP,800H
LOP:     MOV       GS:[EBP],EAX
         XCHG      GS:[EBP],EBX
         CMP       EAX,EBX
         JNE       COMERR
         ADD       EBP,08H
         TEST      EBP,40000H
         JZ        LOP
         MOV       SI,OFFSET GSS
         MOV       CX,GSSL
         CALL      MXEF
         JMP       TOK
COMERR:
         MOV       SI,OFFSET GERROR
         MOV       CX,GERRORL
         CALL      MXEF
TOK:     POP       EAX
         RET
COMMTEST   ENDP

;-------------------------------------------------------------------

MXFDSPY  PROC      NEAR                     ; MXF display strings interface
         PUSH      EDI
         PUSH      ESI
         PUSH      EAX
         MOV       EDI,MXFSTS               ; addr.:401602H, MXF function spec.
CHKSTS:  MOV       AH,BYTE PTR GS:[EDI]
         CMP       AH,077H                  ; shift-reset key code
         JNE       CHKSTS1
         MOV       BYTE PTR GS:[EDI],0F0H
         CALL      RELEASE
         LGDT      TGDT
         JMP       PWORD PTR IJUMP

CHKSTS1: AND       AH,0F0H                  ; store high nibble
         CMP       AH,0F0H
         JNE       CHKSTS
         MOV       EDI,MXFABS               ; ABS 401603
         MOV       WORD PTR GS:[EDI],ABS7      ; fill FF07
         MOV       EDI,MXFPAR               ; 401605
         MOV       BYTE PTR GS:[EDI],CL        ; display counter
         INC       EDI
STRNG:   MOV       AL,DS:[ESI]               ; MOVB, display character
         MOV       GS:[EDI],AL
         INC       ESI
         INC       EDI
         LOOP      STRNG
         MOV       EDI,MXFSTS               ; 401602
         MOV       BYTE PTR GS:[EDI],0FH       ; fill 0FH to tell MXFOS to display
         POP       EAX
         POP       ESI
         POP       EDI
         RET
MXFDSPY  ENDP

;---------------------------------------------------

MXFDSPY1   PROC      NEAR                   ; MXF display one character
         PUSH      EDI
         PUSH      ESI
         PUSH      EAX
         MOV       EDI,MXFSTS
CHKSTS3: MOV       AH,BYTE PTR GS:[EDI]
         CMP       AH,077H
         JNE       CHKSTS2
         MOV       BYTE PTR GS:[EDI],0F0H
         CALL      RELEASE
         LGDT      TGDT
         JMP       PWORD PTR IJUMP

CHKSTS2: AND       AH,0F0H
         CMP       AH,0F0H
         JNE       CHKSTS3
         MOV       EDI,MXFABS
         MOV       WORD PTR GS:[EDI],ABS7
         MOV       EDI,MXFPAR
         MOV       BYTE PTR GS:[EDI],01H       ; one character
         INC       EDI
         MOV       GS:[EDI],AL
         MOV       EDI,MXFSTS
         MOV       BYTE PTR GS:[EDI],0FH
         POP       EAX
         POP       ESI
         POP       EDI
         RET
MXFDSPY1  ENDP

;-----------------------------------------------------------

RESINT    PROC     NEAR
         PUSH      AX
         MOV       AL,GS:[EBP]
         CMP       AL,11H
         JE        INTPASS
         MOV       SI,OFFSET GERROR
         MOV       CX,GERRORL
         CALL      MXEF
         JMP       SKP
INTPASS: MOV       SI,OFFSET GSS
         MOV       CX,GSSL
         CALL      MXEF
SKP:     POP       AX
         RET
RESINT    ENDP

;---------------------------------------------

BOARD    PROC     NEAR
         PUSH      AX
         MOV       AL,GS:[EBP]
         CMP       AL,00H
         JE        MXF2236
         CMP       AL,01H
         JE        HS22C11
         CMP       AL,02H
         JE        HSSCSI
         JMP       CTLB

MXF2236: MOV       SI,OFFSET M2236
         MOV       CX,M2236L
         CALL      MXEF
         JMP       BD

HS22C11: MOV       SI,OFFSET HS
         MOV       CX,HSL
         CALL      MXEF
         JMP       BD

HSSCSI:
         MOV       SI,OFFSET SCSI
         MOV       CX,SCSIL
         CALL      MXEF
         JMP       BD

CTLB:
         MOV       SI,OFFSET CTLB1
         MOV       CX,CTLB1L
         CALL      MXEF

BD:      POP       AX
         RET
BOARD    ENDP

;---------------------------------------------------

DGST       PROC     NEAR
         PUSH      AX
         MOV       AL,GS:[EBP]
         CMP       AL,80H
         JNE       RUINP
         MOV       SI,OFFSET GSS            ; PASS '80'
         MOV       CX,GSSL
         CALL      MXEF
         JMP       QR
RUINP:   CMP       AL,60H
         JNE       NOCON
         CMP       EBP,300420H
         JBE       DKPTR
         MOV       SI,OFFSET NOCRT          ; NO CRT '60'
         MOV       CX,NOCRTL
         CALL      MXEF
         JMP       QR
DKPTR:   MOV       SI,OFFSET NOPTR          ; NO PTR '60'
         MOV       CX,NOPTRL
         CALL      MXEF
         JMP       QR
NOCON:   CMP       AL,40H                   ; RUN-IN '40', PASS
         JNE       DKERR
         MOV       SI,OFFSET GRUN
         MOV       CX,GRUNL
         CALL      MXEF
         JMP       QR
DKERR:   MOV       SI,OFFSET GERROR         ; ERROR '20'
         MOV       CX,GERRORL
         CALL      MXEF

         MOV       AH,GS:[EBP]
         CMP       AH,21H
         JNE       NXER
         MOV       AL,31H
         CALL      MXEF1

NXER:    CMP       AH,22H
         JNE       NXER1
         MOV       AL,32H
         CALL      MXEF1

NXER1:   CMP       AH,23H
         JNE       NXER2
         MOV       AL,33H
         CALL      MXEF1

NXER2:   CMP       AH,24H
         JNE       NXER3
         MOV       AL,34H
         CALL      MXEF1

NXER3:   CMP       AH,25H
         JNE       NXER4
         MOV       AL,35H
         CALL      MXEF1

NXER4:   CMP       AH,26H
         JNE       NXER5
         MOV       AL,36H
         CALL      MXEF1

NXER5:   CMP       AH,27H
         JNE       NXER6
         MOV       AL,37H
         CALL      MXEF1

NXER6:   CMP       AH,28H
         JNE       NXER7
         MOV       AL,38H
         CALL      MXEF1

NXER7:   CMP       AH,29H
         JNE       QR
         MOV       AL,39H
         CALL      MXEF1

QR:      POP       AX
         RET

DGST     ENDP

;--------------------------------------------------

WBCOM1   PROC      NEAR

         MOV       BP,0C00H
         INC       BP
         MOV       BYTE PTR GS:[EBP],1FH
         JMP       $+2
         JMP       $+2
         DEC       BP
         MOV       BYTE PTR GS:[EBP],80H
         RET

WBCOM1    ENDP

;-------------------------------------------------------

WBCOM2   PROC      NEAR
         MOV       BP,1603H
         MOV       BYTE PTR GS:[EBP],06H    ; 1603
         INC       BP
         MOV       BYTE PTR GS:[EBP],31H    ; 1604
         JMP       $+2
         JMP       $+2
         SUB       BP,2
         MOV       BYTE PTR GS:[EBP],0FH    ; 1602
         RET

WBCOM2    ENDP

;----------------------------------------------------

RELEASE  PROC     NEAR

         PUSH      EDI
         PUSH      AX
         MOV       AL,[HSCOFG]
         TEST      AL,01H
         JZ        DK11
         MOV       EDI,00042BH
         MOV       BYTE PTR GS:[EDI],88H

DK11:    TEST      AL,02H
         JZ        DK22
         MOV       EDI,10042BH
         MOV       BYTE PTR GS:[EDI],88H

DK22:    TEST      AL,04H
         JZ        DK33
         MOV       EDI,20042BH
         MOV       BYTE PTR GS:[EDI],88H

DK33:    TEST      AL,08H
         JZ        MX2                             ; skip 1st MXF
         MOV       EDI,30042BH
         MOV       BYTE PTR GS:[EDI],88H

MX2:     TEST      AL,20H
         JZ        MX3
         MOV       EDI,50042BH
         MOV       BYTE PTR GS:[EDI],88H

MX3:     TEST      AL,40H
         JZ        MX4
         MOV       EDI,60042BH
         MOV       BYTE PTR GS:[EDI],88H

MX4:     TEST      AL,80H
         JZ        MXRT
         MOV       EDI,70042BH
         MOV       BYTE PTR GS:[EDI],88H

MXRT:    POP       AX
         POP       EDI
         RET

RELEASE   ENDP

;------------------------------------------------------

ITEM     PROC      NEAR

         MOV       ESI,OFFSET TST0NAME
         MOV       CX,TST0NAMEL             ; count bytes
         CALL      MXEF                     ; point cursor to test name

         MOV       CX,8
CLN5:    MOV       ESI,OFFSET CLNR
         PUSH      CX
         MOV       CX,CLNRL
         CALL      MXEF
         POP       CX
         LOOP      CLN5

         MOV       ESI,OFFSET TST0NAMEND
         MOV       CX,TST0NAMENDL
         CALL      MXEF

         RET

ITEM     ENDP

;----------------------------------------------------------

SIMM     PROC      NEAR
         PUSH      ESI
         PUSH      CX

         MOV       SI,OFFSET BADSIMM
         MOV       CX,BADSIMML
         CALL      MXEF

         POP       CX
         POP       ESI
         RET
SIMM     ENDP

;------------------------------------------------------

L10L15   PROC      NEAR
         PUSH      ESI
         PUSH      CX
         MOV       SI,OFFSET SM2
         MOV       CX,SM2L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L10L15   ENDP

;-------------------------------------------------

L18L24   PROC      NEAR
         PUSH      ESI
         PUSH      CX
         MOV       SI,OFFSET SM3
         MOV       CX,SM3L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L18L24   ENDP

;----------------------------------------------

L29L35   PROC      NEAR
         PUSH      ESI
         PUSH      CX
         MOV       SI,OFFSET SM4
         MOV       CX,SM4L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L29L35   ENDP

;-------------------------------------------------

L3L5     PROC      NEAR
         PUSH      ESI
         PUSH      CX
         MOV       SI,OFFSET SM1
         MOV       CX,SM1L
         CALL      MXEF
         POP       CX
         POP       ESI
         RET
L3L5     ENDP

;---------------------------------------------------------

TGDT    LABEL   PWORD
        DW      0030h
        DD      0h

IJUMP   LABEL   PWORD
        DD      0FFE9h
        DW      034h


;===========================  DISPLAY MESSAGE ============================


COPYRIGHT       DB      05H,03H,06H,0AH, 20 DUP(20H), 0EH, '****', 20H         ;05,03,06
                DB      'CS386 TURBO CUSTOMER DIAGNOSTIC ****', 0FH, 0AH, 0AH, 0DH
                DB      19 DUP(20H),'Copyright, Wang Laboratories, Inc., 1991'
COPYRIGHTL         EQU       $-COPYRIGHT

REVISION        DB      8 DUP(20H),'Rev 1170', 0DH, 0AH, 0AH, 0AH, 0AH

                DB      7 DUP(20H),'TEST ITEM:', 44 DUP(20H),'COUNT:'
                DB      5 DUP(20H), 30H, 0DH, 0AH, 0AH, 8 DUP(20H),'MESSAGES:'
                DB      0DH, 14 dup(0AH), 20 DUP(20H),0FH, 'Press ', 0Eh
                DB      'SHIFT+RESET',0Fh,' key to terminate Diagnostics'
REVISIONL          EQU       $-REVISION


AFTER_COPYRIGHT:
SHOWCOUNT       DB      01H, 7 DUP(0AH), 69 DUP(09H), 5 DUP(20H), 0EH


TST0NAME        DB      01H, 11 DUP(0AH), 80 DUP(20H)
TST0NAMEL          EQU       $-TST0NAME


CLNR     DB        0DH,0AH,80 DUP(20H)
CLNRL    EQU       $-CLNR


TST0NAMEND       DB      01H, 7 DUP(0AH), 20 DUP(09H), 0FH
TST0NAMENDL        EQU       $-TST0NAMEND

CLEAN    DB        01H,11 DUP(0AH), 74 DUP(20H)
CLEANL   EQU       $-CLEAN


TST1NAME        DB      '      CPU Memory Data Bus Test',10 DUP(20H)
TSTDW           DB      '    DRAM Double word pattern Test       '
TSTW            DB      '      DRAM Word pattern Test',12 DUP(20H)
NMITST          DB      '  Parity Generator Check & NMI Test     '

SYSCARD         DB      '      System Interface Card Test',9 DUP(20H)
                DB      01H, 10 DUP(0AH)
SYSCARDL        EQU     $-SYSCARD

TST4NAME        DB      '     CPU Memory Address Bus Test', 8 DUP(20H)
TST5NAME        DB      '   DRAM Data Byte Fast Exchange Test    '

SHOWSIZE        DB      0DH, 5 DUP(0AH),30 DUP(20H),'DRAM'
                DB      5 DUP(20H), 30H, 20H, 'KB',4 DUP(08H)
SHOWSIZEND:

DRAMSIZE        DB      0DH, 34 DUP(09H), 5 DUP(20H)
DRAMSIZEND:

PASSMSG         DB        01H, 12 DUP(0AH), 29 DUP(20H), 0EH, '  TEST PASS !!'
PASSMSGL        EQU       $-PASSMSG

FAILMSG         DB        01H, 12 DUP(0AH), 29 DUP(20H), 0EH, '  TEST FAIL !!'
FAILMSGL        EQU       $-FAILMSG

ERRMSG          DB      01H, 10 DUP(0AH), 21 DUP(20H), 0EH, 'Error was found:'
                DB      0FH, 0DH, 0AH, 0AH, 25 DUP(20H), 'at address ='
ERRMSGL         EQU     $-ERRMSG

ADDRESS         DB      20H, 8 DUP(30H), 20 DUP(20H)
                DB      0DH,0AH,0DH,0AH,26 DUP(20H),'read data ='
ADDRESSL        EQU     $-ADDRESS

READDATA        DB      20H, 8 DUP(30H), 20 DUP(20H)
                DB      0DH,0AH,0DH,0AH,22 DUP(20H)
                DB      'expected data ='
READDATAL       EQU     $-READDATA

EXPECDATA       DB      20H, 8 DUP(30H), 20 DUP(20H)
                DB      0DH, 0AH, 0DH, 0AH, 21 DUP(20H)
EXPECDATAL      EQU     $-EXPECDATA

ERRMSGEND:
TST2MSG         DB      'Unexpect parity error occurred'
TST2MSGA        DB      'Force parity bit generation failure'
TST2MSGB        DB      'Can not clear parity error'
TST2MSGEND:
ASCIIFONT       DB      30H, 31H, 32H, 33H, 34H, 35H, 36H, 37H, 38H, 39H
                DB      41H, 42H, 43H, 44H, 45H, 46H
SHIFTKEY        DB      01H, 23 DUP(0AH), 26 DUP(09H), 'SHIFT+RESET'
RESETKEY        DB      01H, 23 DUP(0AH), 26 DUP(09H), 0EH, 'SHIFT+RESET', 0FH
RESETKEYEND:

BADSIMM         DB      0DH,27 DUP(20H),0EH,' BAD SIM : L'
BADSIMML           EQU   $-BADSIMM

SM1      DB      0DH,27 DUP(20H),0EH,' BAD SIMM : L3 or L5'
SM1L     EQU       $-SM1

SM2      DB      0DH,27 DUP(20H),0EH,' BAD SIMM : L10 or L15'
SM2L     EQU       $-SM2

SM3      DB      0DH,27 DUP(20H),0EH,' BAD SIMM : L18 or L24'
SM3L     EQU       $-SM3

SM4      DB      0DH,27 DUP(20H),0EH,' BAD SIMM : L29 or L35'
SM4L     EQU       $-SM4

PB0      DB        0DH,0AH,24 DUP(20H),' * '
PB0L     EQU       $-PB0

DSK1     DB        0DH,0AH,24 DUP(20H),' * 1st '
DS1L     EQU       $-DSK1

DSK2     DB        0DH,0AH,24 DUP(20H),' * 2nd '
DS2L     EQU       $-DSK2

DSK3     DB        0DH,0AH,24 DUP(20H),' * 3rd '
DS3L     EQU       $-DSK3

MXF11    DB        0DH,0AH,24 DUP(20H),' * 1st '
MX1L     EQU       $-MXF11

MXF22    DB        0DH,0AH,24 DUP(20H),' * 2nd '
MX2L     EQU       $-MXF22

MXF33    DB        0DH,0AH,24 DUP(20H),' * 3rd '
MX3L     EQU       $-MXF33

MXF44    DB        0DH,0AH,24 DUP(20H),' * 4th '
MX4L     EQU       $-MXF44

M2236    DB        '2236-MXF  -> '
M2236L   EQU       $-M2236

HS       DB        '22C11-HS  -> '
HSL      EQU       $-HS

SCSI     DB        '22C11-SCSI ->'
SCSIL    EQU       $-SCSI

CTLB1    DB        'Peripheral Card ->'
CTLB1L   EQU       $-CTLB1

GSS      DB        'PASS                          '
GSSL     EQU       $-GSS

GRUN     DB        'Run-in mode: PASS             '
GRUNL    EQU       $-GRUN

NOCRT    DB        'Run-in mode: CRT NOT connect  '
NOCRTL   EQU       $-NOCRT

NOPTR    DB        'Run-in mode: Printer NOT ready'
NOPTRL   EQU       $-NOPTR

GERROR   DB        'ERROR                         ',24 DUP(08H)       ;need test
GERRORL  EQU       $-GERROR


INTRT    DB        '  Peripheral Card Interrupt Test ',7 DUP(20h)
         DB      01H, 10 DUP(0AH), 22 DUP(20H)
INTRTL   EQU     $-INTRT

COMST    DB      4 DUP(20H),'High Speed Channel Bus Test  '
         DB      01H, 10 DUP(0AH), 22 DUP(20H)
COMSTL   EQU     $-COMST

FIRSTRAM        DD      0H
BASE            DD      0H
HSCOFG          DB      0H
INTRES          DB      0H

STACK_BUFFER    LABEL   PWORD
CODE    ENDS
        END


